Package: FrF2
Title: Fractional Factorial Designs with 2-Level Factors
Version: 2.3-5
Depends: R(>= 2.13.0), DoE.base(>= 0.25)
Imports: sfsmisc(>= 1.0-26), utils, scatterplot3d, igraph(>= 0.7),
        methods
Suggests: FrF2.catlg128, BsMD, DoE.wrapper
Date: 2026-05-25
Authors@R: person("Ulrike", "Groemping", role = c("aut", "cre"), email =
        "ulrike.groemping@bht-berlin.de")
Description: Regular and non-regular Fractional Factorial 2-level designs 
        can be created. Furthermore, analysis tools for Fractional
        Factorial designs with 2-level factors are offered (main
        effects and interaction plots for all factors simultaneously,
        cube plot for looking at the simultaneous effects of three
        factors, full or half normal plot, alias structure in a more
        readable format than with the built-in function alias). 
License: GPL (>= 2)
LazyLoad: yes
Encoding: UTF-8
URL: https://prof.bht-berlin.de/groemping/DoE/,
        https://prof.bht-berlin.de/groemping/
NeedsCompilation: no
Packaged: 2026-05-24 23:53:17 UTC; groemping
Author: Ulrike Groemping [aut, cre]
Maintainer: Ulrike Groemping <ulrike.groemping@bht-berlin.de>
Repository: CRAN
Date/Publication: 2026-05-25 05:10:19 UTC
Built: R 4.6.0; ; 2026-05-25 07:27:04 UTC; unix

     import(DoE.base,igraph,scatterplot3d)
     importFrom(utils,combn) 
     importFrom(sfsmisc,AsciiToInt,as.intBase,digitsBase)
     importFrom("grDevices", "grey", "rgb", "xyz.coords")
     importFrom("graphics", "abline", "axis", "box", "lines", "mtext",
             "par", "plot", "plot.default", "plot.new", "plot.window",
             "points", "segments", "strwidth", "symbols", "text",
             "title")
     importFrom("methods", "is")
     importFrom("stats", "aggregate", "as.formula", "coef", "contrasts",
             "contrasts<-", "formula", "model.matrix", "ppoints",
             "predict", "qnorm", "qqnorm", "rnorm", "terms", "setNames")
     importFrom("utils", "as.roman", "packageVersion", "sessionInfo")

     export("DanielPlot", "aliases", "aliasprint", "IAPlot", "MEPlot", 
            "cubePlot", "cubedraw", "cubecorners", "cubelabel",
            "FrF2","pb","FrF2Large","nrunsV","fold.design","add.center",
            "blockpick","blockpick.big","splitpick","leftadjust",
            "nruns", "nfac", "nclear.2fis",
            "clear.2fis","all.2fis.clear.catlg",
            "res","WLP","dominating",
            "CIG","CIGstatic","catlg","Yates","BsProb.design",
            "block.catlg","pb.list","compromise","gen2CIG",
            "FrF2.currentlychecked","blockgencreate","phimax",
            "Xcalc", "FF_from_X", "X_from_profile",
            "X_from_parts", "colpick", "makecatlg")

     S3method("[",catlg)
     S3method(print,catlg)
     S3method(print,aliases)

     S3method(MEPlot,default)
     S3method(MEPlot,design)
     S3method(IAPlot,default)
     S3method(IAPlot,design)
     S3method(DanielPlot,default)
     S3method(DanielPlot,design)

     S3method(nruns,catlg)
     S3method(nfac,catlg)
     S3method(res,catlg)
     S3method(nclear.2fis,catlg)
     S3method(clear.2fis,catlg)
#     S3method(all.2fis.clear,catlg)
     S3method(WLP,catlg)
     S3method(dominating,catlg)

     S3method(nruns,character)
     S3method(nfac,character)
     S3method(res,character)
     S3method(nclear.2fis,character)
     S3method(clear.2fis,character)
#     S3method(all.2fis.clear,character)
     S3method(WLP,character)
     S3method(dominating,character)

#     S3method(nruns,default)
#     S3method(nfac,default)
#     S3method(res,default)
#     S3method(nclear.2fis,default)
#     S3method(clear.2fis,default)
#     S3method(all.2fis.clear,default)
#     S3method(WLP,default)
#     S3method(dominating,default)

`intfind` <-
function(i,j,mat){
     ## function for finding the correct interaction coefficient 
     ## to be used in function iap
     sp <- NULL
     for (k in 1:ncol(mat)){
          if (!all(mat[,k]==c(i,j))) next 
          else sp <- k
          break
          }
     sp
    }

makecatlg <- function(k, gen){
    ## May 29 2020: let the WLP start with position 1
    ## May 29 2020: 
    if (!is.numeric(gen)) stop("gen must be a numeric vector of generators")
    if (!all(gen%%1==0)) stop("gen must be integer")
    if (any(gen < 1 | gen >=2^k)) stop("gen must be a positive integer of at most ", 2^k)
    if (any(gen %in% 2^(0:(k-1)))) stop("gen must not refer to any base columns")
    p <- length(gen)
    allwords <- words.all(k, gen, max.length=4)$words.up.to.length.4
    clear2fis <- DoE.base:::nchoosek(k+p, 2)  ## initialize to all clear
    ## extract the words
    if (length(allwords[[1]])==0)  WLP <- c(0,0,0,0,NA) else{
      ## error for resolution 3 (could also allow this for other purposes)
      if (any(lengths(allwords)<=3)) stop("design has resolution less than IV")
      WLP <- c(0, 0, 0, length(allwords), NA)
      ## remove non-clear words of length 2
      notclear <- sapply(1:ncol(clear2fis),
        function(obj) any(sapply(allwords, function(obj2) all(clear2fis[,obj] %in% obj2))))
#        function(obj) any(sapply(allwords[[1]], function(obj2) all(obj %in% obj2))))
      clear2fis <- clear2fis[,!notclear]
    }
    nclear2fis <- ncol(clear2fis)
    aus <- list(list(nruns=2^k, nfac=k+length(gen), gen=gen, res=ifelse(length(allwords[[1]])>0, 4, 5),
           WLP=WLP,
           clear.2fis=clear2fis,
           nclear.2fis=nclear2fis))
           ## more is not needed, resolution is set to 5 (might in fact be larger)
    names(aus) <- paste0(k+p, "-", p, ".custom")
    class(aus) <- c("catlg","list")
    aus
}

## Plackett Burman generating vectors and matrices, according to 1946 article
## are in pb.list (in workspace sysdata.rda)
## most functions in here are used for extending the pb designs with entries 0 in this list
## (double.des, Williamson etc.)

## function for doubling designs
## normalize.row.last is applied later to ensure that last row is "-1"
double.des <- function(des){
   zeil <- nrow(des)
   ### +/- column and reflected columns unconventionally moved to the front !!!
   ### benefit: first columns are resolution IV
   rbind(cbind(rep(1,zeil),des,des),cbind(rep(-1,zeil),-des,des))
}

circ.mat <- function(gen){
   n <- length(gen)+1
   sel <- gen
   for (i in 1:(n-2))
    sel <- c(sel,gen[(n-i):(n-1)],gen[1:(n-i-1)])
    matrix(sel,n-1,n-1,byrow=1)
}

williamson <- function(A,B,C,D){rbind(
  cbind(A, B, C, D),
  cbind(-B, A, D, -C),
  cbind(-C, -D, A, B),
  cbind(-D, C, -B, A))
}

normalize.col.first <- function(mat){
    hilf <- mat[,1]==-1
    mat[hilf,] <- -mat[hilf,]
    mat[,-1]
}

normalize.row.last <- function(mat){
    hilf <- mat[nrow(mat),]==1
    mat[,hilf] <- -mat[,hilf]
    mat
}

des.recode <- function (var, recodes, as.factor.result, char) 
{
    recode.list <- rev(strsplit(recodes, ";")[[1]])
    is.fac <- is.factor(var)
    if (missing(as.factor.result)) 
        as.factor.result <- is.fac
    if (missing(char)) char <- FALSE
    result <- var
    if (is.fac) 
        result <- as.character(result)

    for (term in recode.list) {
            set <- eval(parse(text = strsplit(term, "=")[[1]][1]))
        if (!char)
            target <- eval(parse(text = strsplit(term, "=")[[1]][2]), 
                envir=parent.frame(), enclos=sys.frame(0))
        else 
        target <- strsplit(term, "=")[[1]][2]
        for (val in set) {
                if (is.na(val)) 
                  result[is.na(var)] <- target
                else result[var == val] <- target
        }
    }
    if (as.factor.result) result <- as.factor(result)
    result
}

cols <- function(k, gen){
    main <- c(2^(0:(k-1)),gen)
    hilf <- combn(k+length(gen),2)
    fi2s <- rep(NA,ncol(hilf))
    for (i in 1:length(fi2s)){
          hilf2 <- which(sapply(Yates[1:(2^k-1)],function(obj2){ 
                                     obj <- mult.gen(Yates[main[hilf[,i]]])
                                     hh <- FALSE 
                                     if (length(obj2)==length(obj)) 
                                          if (all(obj2==obj)) hh <- TRUE
                                     hh}))
          if (length(hilf2)>0)
          fi2s[i]<-hilf2
          }
   # fi2s <- sort(table(fi2s))
    list(main=main,fi2s=fi2s,freecols=setdiff(1:(2^k-1),c(main,fi2s)))
}

blockpick.big <- function(k, gen, k.block, design=NULL, show=10, alias.block.2fis=FALSE, select.catlg=catlg){
  ## function that picks design right number of blocks (power of 2, 2^k.block) 
  ## it may be possible to find an appropriate design within 
  ##    the same base design with modified generator columns
  ##  This is currently not pursued (and may never be).

  ## k.block is the number of independent block factors needed for block construction
  ##   (log2 of number of blocks)
  ## usual assumption: their interactions with treatment factors can be neglected

  ## 2^k.block-1 first columns are used exclusively for blocks

  ## if requested (default), alias.block.2fis=FALSE stops confounding of treatment 2fis with blocks
  ## if this is not requested, no attempt is made to reduce the number of aliased 2fis, 
  ##       but they are reported

  if (!is.logical(alias.block.2fis)) stop("alias.block.2fis must be a logical.")
  if (!is.null(design)){ 
     if (!is.character(design)) stop("If specified, design must be a character string.")
     if (!design %in% names(select.catlg)) stop("If specified, design must be the name of a design in select.catlg.")
     if (!missing(gen)) warning("If design is specified, gen is ignored.")
     }
  if (is.null(design) & missing(gen)) stop("One of gen or design must be given.")
  if (!"catlg" %in% class(select.catlg)) stop("select.catlg must be a catalogue of designs of class catlg.")

  if (is.character(design)) {
       if (!design %in% names(select.catlg)) stop ("Design must be a valid design name.")
       if (select.catlg[[design]]$res == 3 & !alias.block.2fis)
           stop("Resolution III design. Package FrF2 does not allow to request all 2fis to be clear of blocks.")
       gen <- select.catlg[[design]]$gen
  }
  g <- length(gen)

  hilf <- c(k,gen,k.block,show)
  if (!is.numeric(hilf))
      stop ("k, gen, k.block and show must be numeric.")
  if (!all(hilf == floor(hilf) & hilf > 0))
      stop ("k, gen, k.block and show must contain positive integer numbers.")
  if (!k >= 3) stop ("blockpick requires k>=3.")
  if (!k.block < k) stop ("blockpick requires k.block < k.")
  if (any(gen %in% 2^(0:(k-1)))) 
        stop ("gen must not contain column numbers of base factors.")
  if (any(!gen %in% 3:(2^k-1))) 
        stop ("Column numbers in gen must be smaller than ", 2^k,".")
  perm <- permutations(k)

  hilf <- digitsBase(gen,ndigits=k) ## always k rows
         ## generating columns as binary numbers
  ergeb <- matrix(0,nrow=nrow(perm),ncol=g)
         ## will contain equivalent generating columns after permuting base factors
         ## block columns are the first k.block base factors
         ## are related to other factors in the same way they originally were
         ## in their position before permuting
  nfacs.block <- rep(NA, nrow=nrow(perm))
         ## will contain number of factors aliased with block generators
  n2fis.block <- rep(NA, nrow=nrow(perm))
         ## will contain check whether 2fis aliased with block generators
  if (g>1) hilfc <- combn(g,2)
         ## interactions among generated factors
  count <- 0 ## number of possibilities found
  
  ## loop through permutations of base factors
  for (i in 1:factorial(k)){
      ergeb[i,] <- as.integer(reord(hilf,perm[i,]))
      nfacs.block[i] <- sum(ergeb[i,] < 2^k.block) ## generated main effects confounded with blocks

      if (nfacs.block[i]==0 & !alias.block.2fis){
           ## if in principle eligible, check for aliasing of 2fis with blocks
           hilf1 <- digitsBase(perm[i,-(1:k.block)],ndigits=k)

           n2fis.block[i] <- sum(colSums(digitsBase(ergeb[i,])[-((k-k.block+1):k),,drop=FALSE])<=1)>0
                ## non-block rows of digitsBase(ergeb[i,]) 
                ## if these contain at most one "1", 
                ## interactions of blocks with 2fis between base factors and generated factors occur
                ## only for clean rows of ergeb, interactions AMONG generated factors are checked
       if (g > 1 & !n2fis.block[i]){
             ## check interactions among generated factors
             hilf2 <- digitsBase(ergeb[i,],ndigits=k)
             for (j in 1:choose(g,2)){
                    if (as.intBase(paste((hilf2[,hilfc[1,j]]+hilf2[,hilfc[2,j]])%%2,collapse="")) < 2^k.block) {
                           n2fis.block[i]<-TRUE
                           break
                    }
             }
             }
             if (!n2fis.block[i]) {
                 count <- count + 1
                 if (count>=show) break
             }
    }
  }  ## end of loop over permutations
  
  if (!alias.block.2fis) pick <- which(nfacs.block==0 & !n2fis.block) else pick <- which(nfacs.block==0)
      ### sometimes length of the two vectors in the first which seems to differ
      ### have not yet found the reason
  if (length(pick)<show) show <- length(pick) 
  if (show==0 & !alias.block.2fis) stop("no adequate block design found with 2fis unconfounded with blocks\n")
  if (show==0) stop("no adequate block design found\n")
  else {
    ntreat=k+length(gen)-k.block
    blocks=2^k.block
    blockcols <- 1:(blocks-1)
    if (alias.block.2fis){
        alias.2fis.block <- vector("list",show)
        nam2fis <- outer(Letters[1:ntreat],Letters[1:ntreat],function(X,Y) paste(X,Y,sep=":"))
        nam2fis <- nam2fis[upper.tri(nam2fis)]
        for (i in 1:show) {
                       treats <- c(2^(k.block:(k-1)),ergeb[pick[i],])
                       hilf <- outer(treats,treats,
                                      function(X,Y) as.integer((digitsBase(X,ndigits=k) + 
                                                                digitsBase(Y,ndigits=k)) %% 2)) 
                       hilf <- hilf[upper.tri(hilf)]
                       alias.2fis.block[[i]] <- nam2fis[hilf<blocks]
        }
    }
    else alias.2fis.block <- "none"
    gens <- ergeb[pick[1:show],,drop=FALSE]  
    list(orig=gen, basics = c(nruns=2^k,nblocks=blocks, ntreat=ntreat), 
           perms=perm[pick[1:show],,drop=FALSE],blockcols=blockcols,
           alias.2fis.block=alias.2fis.block, gen=gens)
  }
}

require(FrF2)
### tests for Godolphin methodology

### colpick
## colpick for full factorial
colpick(6, 2)   ## default
## character estimability requirement
colpick(6, 3, estimable=compromise(6, 4)$requirement)
## matrix estimability requirement
colpick(6, 3, estimable=rbind(4, c(1:3,5:6)))
## character design specification
colpick("6-1.1", 3, estimable=compromise(6, 4)$requirement)
## catlg design specification
colpick(catlg["6-1.1"], 3, estimable=compromise(6, 4)$requirement)
## impossible request
colpick(catlg["8-2.1"], 1)
## request that requires to suppress dominating filter
colpick("9-3.2", 2)  ## (implemented with version 2.1)
## request with more than 50 factors (don't try to create the design!)
tail(colpick(55, 2)$clear2fis)

## phimax
phimax(8, 2)            ## maximum possible
phimax(8, 2, c(4,3,1))  ## fewer
phimax(9, 4)            ## all

## blockgencreate
X <- rbind(c(1,1,1,0,0,0,1),
           c(1,1,0,1,1,1,0))
blockgencreate(X, 2)

## Xcalc
## gen character
Xcalc(rbind(c(1,1,0,0),c(0,1,1,1)), 
       gen=c("AB","BCD"))
## gen list
Xcalc(rbind(c(1,1,0,0),c(0,1,1,1)), 
             gen=list(c(1,2),2:4))
## gen Yates columns
Xcalc(rbind(c(1,1,0,0),c(0,1,1,1)), 
             gen=c(3,14))
## gen catlg
Xcalc(rbind(c(1,1,0,0),c(0,1,1,1)), 
             gen=catlg[nruns(catlg)==16 & 
                         res(catlg)==4])

plangodolphin <- FF_from_X(colpick(4, 2)$X, randomize=FALSE)
cbind(run.order(plangodolphin), plangodolphin)

nrunsV <- function(nfactors){
   ## make sure to include the latest knowledge
   ## Ryan and Bulutoglu 2010 Technometrics:
   ## Xu 25 to 28 in 1024 runs are MA
   ## Xu 24 to 27 in 2048 runs are MA
   ## Xu 28 to 32 in 2048 runs have been improved by Ryan and Bulutoglu 
       ### change bound in fiveFrF2MA as soon as incorporated in catalogue
   ## Xu 25 factors in 4096 runs is MA
   ## Xu 26 factors has been improved by Ryan and Bulutoglu
       ### change bound in fiveFrF2MA as soon as incorporated in catalogue
   ## 
   fiveFrF2MA <- data.frame(nfactors=c(2,3,5,6,8,11,17,23,28,27,25),nruns=2^(2:12))
   fiveFrF2 <- data.frame(nfactors=c(2,3,5,6,8,11,17,23,33,47,65),nruns=2^(2:12))
   fiveSanchezSanchez <- data.frame(nfactors=c(2,3,5,6,8,11,17,21,29,38,52,69,92,120),nruns=2^(2:15))
   oldwarn <- options("warn")
   options(warn=-1)
   FrF2MA <- min(fiveFrF2MA$nruns[which(fiveFrF2MA$nfactors >= nfactors)])
   FrF2 <- min(fiveFrF2$nruns[which(fiveFrF2$nfactors >= nfactors)])
   SanchezSanchez <- min(fiveSanchezSanchez$nruns[which(fiveSanchezSanchez$nfactors >= nfactors)])
   options(warn=oldwarn$warn)
   if (SanchezSanchez == Inf) message("no regular resolution V design can be created with this software")
   else if (SanchezSanchez<FrF2) message("Only function FrF2Large with nruns=", SanchezSanchez, " creates a regular resolution V design.")
         else if (FrF2MA<=FrF2) message("Function FrF2 with nruns=", FrF2, " creates a minimum aberration regular resolution V design.")
              else message("Function FrF2 with nruns=", FrF2, " creates a regular resolution V design (good, but not necessarily MA).")
   invisible(min(FrF2MA, FrF2, SanchezSanchez))
}

FrF2Large <- function(nruns, nfactors=NULL,
                 factor.names = if(!is.null(nfactors)) {if(nfactors<=50) Letters[1:nfactors]
                                else paste("F",1:nfactors,sep="")} else NULL,
                 default.levels = c(-1,1), ncenter=0, center.distribute=NULL,
                 generators=NULL, replications=1, repeat.only=FALSE,
                 randomize=TRUE, seed=NULL, alias.info=2, ...){
     creator <- sys.call()
     ## provide the SanchezSanchez column numbers
     ## which are NOT a recipe for MA
     ## these can be used with function YatesFly
      SanchezSanchez <- data.frame(nfactors=1:120,
          gen=c(1, 2, 4, 8, 15, 16, 32, 51, 64, 85, 106, 128,
               150, 171, 219, 237, 247, 256, 279, 297, 455, 512, 537,
               557, 594, 643, 803, 863, 998, 1024, 1051, 1070, 1112,
               1169, 1333, 1345, 1620, 1866, 2048, 2076, 2085, 2185,
               2372, 2456, 2618, 2800, 2873, 3127, 3284, 3483, 3557,
               3763, 4096, 4125, 4135, 4174, 4435, 4459, 4469, 4497,
               4752, 5255, 5732, 5804, 5915, 6100, 6369, 6907, 7069,
               8192, 8263, 8351, 8422, 8458, 8571, 8750, 8858, 9124,
               9314, 9500, 10026, 10455, 10556, 11778, 11885, 11984,
               13548, 14007, 14514, 14965, 15125, 15554, 16384, 16457,
               16517, 16609, 16771, 16853, 17022, 17453, 17891, 18073,
               18562, 18980, 19030, 19932, 20075, 20745, 21544, 22633,
               23200, 24167, 25700, 26360, 26591, 26776, 28443, 28905,
               29577, 32705))
    ##check nruns
    k <- round(log2(nruns))
       if (!2^k==nruns) stop("nruns must be a power of 2.")
## check validity of center point options
    if (!is.numeric(ncenter)) stop("ncenter must be a number")
    if (!length(ncenter)==1) stop("ncenter must be a number")
    if (!ncenter==floor(ncenter)) stop("ncenter must be an integer number")
    if (is.null(center.distribute)){
      if (!randomize) center.distribute <- min(ncenter, 1)
         else center.distribute <- min(ncenter, 3)}
    if (!is.numeric(center.distribute)) stop("center.distribute must be a number")
    if (!center.distribute==floor(center.distribute)) stop("center.distribute must be an integer number")
    if (center.distribute > ncenter)
       stop("center.distribute can be at most ncenter")
    if (randomize & center.distribute==1) warning("running all center point runs together is usually not a good idea.")
## check that no incompatible options are used together
    if (ncenter>0) if (center.distribute > nruns + 1)
        stop("center.distribute must not be larger than nruns+1")
## simple checks for individual option entries
    if (default.levels[1]==default.levels[2]) stop("Both default levels are identical.")
    if (!(is.logical(repeat.only) & is.logical(randomize)))
         stop("repeat.only and randomize must be logicals (TRUE or FALSE).")
    if (!is.numeric(replications)) stop("replications must be an integer number.")

    if (!alias.info %in% c(2,3))
         stop("alias.info can be 2 or 3 only.")
    if (!(is.numeric(default.levels) | is.character(default.levels)))
         stop("default.levels must be a numeric or character vector of length 2")
    if (!length(default.levels) ==2)
         stop("default.levels must be a numeric or character vector of length 2")

## check factor specifications
    if (is.null(factor.names) & is.null(nfactors) & is.null(generators))
         stop("The number of factors must be specified via nfactors, via factor.names, or via generators.")
    if (!is.null(factor.names) & !(is.character(factor.names) | is.list(factor.names)) )
         stop("factor.names must be a character vector or a list.")
    if (is.null(nfactors)) {if (!is.null(factor.names)) nfactors <- length(factor.names)
                          else if (!is.null(generators)) nfactors <- length(generators)+k
                          }
    ## from here on, nfactors is known
    if (!nfactors==floor(nfactors))
         stop("nfactors must be an integer number.")
    if (!is.null(factor.names) & !length(factor.names)==nfactors)
         stop("There must be nfactors factor names, if any.")
    if (is.null(factor.names))
         if(nfactors<=50) factor.names <- Letters[1:nfactors] else factor.names <- paste("F",1:nfactors,sep="")

## warnings to prevent users from using inferior designs
#    if ((nruns >= 4 & nruns <= 512) | (nruns == 1024 & nfactors <= 24)
#        | (nruns == 2048 & nfactors <= 23)  | (nruns == 4096 & nfactors <= 24))
#        stop("Please use function FrF2, it will guarantee an optimal design.")
#    if (nruns >= 1024 & nruns <= 4096)
#        warning("Usage of function FrF2 might yield a better design \n(up to 33 factors in 1024 runs, \nup to 47 factors in 2048 runs, \nup to 65 factors in 4096 runs)")

    if (nruns <= 4096) stop("Please use function FrF2; it does everything that FrF2Large can do and more.") 

## check factor level specifications
    if (!((is.character(default.levels) | is.numeric(default.levels)) & length(default.levels)==2) )
                 stop("default.levels must be a vector of 2 levels.")
    if (is.list(factor.names)){
        if (is.null(names(factor.names))){
            if (nfactors<=50) names(factor.names) <- Letters[1:nfactors]
            else names(factor.names) <- paste("F", 1:nfactors, sep="")
        }
        if (any(factor.names=="")) factor.names[which(factor.names=="")] <- list(default.levels)}
        else {hilf <- vector("list",nfactors)
              names(hilf) <- factor.names
              hilf[1:nfactors]<-list(default.levels)
              factor.names <- hilf}
    ## from here on, factor.names is a named list
    ## make all names valid R names
      names(factor.names) <- make.names(names(factor.names), unique=TRUE)

    if (ncenter > 0) if(any(is.na(sapply(factor.names,"is.numeric"))))
       stop("Center points are implemented for experiments with all factors quantitative only.")

    ### prepare generators case
    if (!is.null(generators)){
      generators <- gen.check(k, generators)   ## check entries for validity
      hilf <- YatesFly(c(2^((1:k)-1), gencalc(generators)), k=k)
      }
    else hilf <- YatesFly(SanchezSanchez$gen[1:nfactors], k=k)

    gen <- hilf$gen    ## vector of column numbers for non-base factors
    if (is.null(generators)) generators <- hilf$Yates.brief[-(1:k)]
                       ## list of column combinations

    ## number of generators
    g <- nfactors - k
    if (!length(generators)== g)
       stop("This design in ", nruns, " runs with ", nfactors," factors requires ", g, " generators.")

    res <- NA; nclear.2fis<-NA; clear.2fis<-NULL;all.2fis.clear<-NA
    if (g<10) wl <- words.all(k, generators,max.length=6)
    else if (g<15) wl <- words.all(k, generators,max.length=5)
    else if (g<20) wl <- words.all(k, generators,max.length=4)
    else if (g>=20) wl <- alias3fi(k, generators, order=2)  ## not a word list
    WLP <- NULL
    if (g < 20){
        WLP <- wl$WLP
        res <- min(as.numeric(names(WLP)[which(WLP>0)]))
        if (res==Inf) {if (g<10) res="7+"
                   else if (g<15) res="6+"
                   else if (g<20) res="5+" }
                 }
               else{
                   if (!is.list(wl)) res="5+"
                   else{
                   if (length(wl$"main")>0) res="3"
                      else if (length(wl$"fi2")>0) res="4"
                           else res="5+"}
               }

               gen <- gencalc(generators)
               gen <- gen*sapply(generators, function(obj) sign(obj[1]))

               cand <- list(custom=list(res=res, nfac=nfactors, nruns=nruns,
                    gen=gen,
                    WLP=WLP, nclear.2fis=nclear.2fis, clear.2fis=clear.2fis, all.2fis.clear=all.2fis.clear))
                   ## needs to be list of list, because access later is always via cand[1]
               class(cand) <- c("catlg","list")
      ## prepare output
      desmat <- hilf$mat
      rownames(desmat) <- 1:nruns
      if (randomize & !is.null(seed)) set.seed(seed)
      rand.ord <- rep(1:nruns,replications)
      if (replications > 1 & repeat.only) rand.ord <- rep(1:nruns,each=replications)
      if (randomize & !repeat.only) for (i in 1:replications)
                  rand.ord[((i-1)*nruns+1):(i*nruns)] <- sample(nruns)
      if (randomize & repeat.only) rand.ord <- rep(sample(1:nruns), each=replications)
        orig.no <- rownames(desmat)
        orig.no <- orig.no[rand.ord]
    ## row added 27 01 2011 (for proper ordering of design)
    orig.no.levord <- sort(as.numeric(orig.no),index=TRUE)$ix
        rownames(desmat) <- NULL
        desmat <- desmat[rand.ord,]
        
      ## order information for output
      orig.no.rp <- orig.no
      if (replications > 1){
                if (repeat.only)
                orig.no.rp <- paste(orig.no.rp, rep(1:replications,nruns),sep=".")
                else
                orig.no.rp <- paste(orig.no.rp, rep(1:replications,each=nruns),sep=".")
           }

      ## prepare generators to be character for output
      if (nfactors <=50)
        generators <- paste(Letters[(k+1):nfactors],colnames(desmat)[(k+1):nfactors],sep="=")
      else
        generators <- paste(paste("F",1:nfactors,sep=""),colnames(desmat)[1:nfactors],sep="=")
      ## factor names for design matrix
      colnames(desmat) <- names(factor.names)

      ## create output data frame
      desdf <- as.data.frame(desmat)
      quant <- rep(FALSE,nfactors)
      for (i in 1:nfactors) {
        ## make all variables into factors
        desdf[,i] <- des.recode(desdf[,i],"-1=factor.names[[i]][1];1=factor.names[[i]][2]")
        quant[i] <- is.numeric(desdf[,i])
        desdf[,i] <- factor(desdf[,i],levels=factor.names[[i]])
        contrasts(desdf[,i]) <- contr.FrF2(2)
        }
      attr(desdf, "desnum") <- desmat
      ## change 27 Jan 2011: leave orig.no as a factor, but with better-ordered levels
      orig.no <- factor(orig.no, levels=unique(orig.no[orig.no.levord]))
      attr(desdf, "run.order") <- data.frame("run.no.in.std.order"=orig.no,
           "run.no"=1:nrow(desmat),"run.no.std.rp"=orig.no.rp, stringsAsFactors=TRUE)
      design.info <- list(type="FrF2.large", nruns=nruns,
          nfactors=nfactors, factor.names=factor.names, generators=generators,
          aliased = alias3fi(k,cand[1][[1]]$gen,order=alias.info),
          replications=replications,
          repeat.only=repeat.only, randomize=randomize, seed=seed, creator=creator, 
          FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
      if (nfactors<=50) design.info$aliased <- c(list(legend=paste(Letters[1:nfactors],names(factor.names),sep="=")),design.info$aliased)
          else design.info$aliased <- c(list(legend=paste(paste("F",1:nfactors,sep=""),names(factor.names),sep="=")),design.info$aliased)
      attr(desdf, "design.info") <- design.info
      class(desdf) <- c("design", "data.frame")
      ## add center points, if requested
      if (ncenter>0) desdf <- add.center(desdf, ncenter, distribute=center.distribute)
      ## hier weiter: sicherstellen, dass dieses alle Voraussetzungen an FrF2-artige Objekte erfuellt
      desdf
}

fold.design <- function(design, columns="full", ...){
## currently does not cover splitplot, block and parameter designs
## splitplot should be easiest to include as well

## full foldover:
## standard order for mirror runs is actually mirrored, i.e. e.g. for 8 to 16, mirror of 1 is 16, mirror of 2 is 15 etc.
## base factors remain base factors
## some generators are previous generators plus nruns, others remain unchanged, depending on the number of 
##    factors involved

## any other foldover
## analogous

   ## error checks
   if (!"design" %in% class(design))
      stop("design must be of class design")
   di <- design.info(design)
    if (!(substr(di$type,1,4)=="FrF2" | substr(di$type,1,2)=="pb" | 
             (length(grep("full factorial",di$type))>0 & all(di$nlevels==2))))
      stop("fold.design is applicable for FrF2 or pb designs only")
#   if (di$type %in% c("FrF2.blocked", "FrF2.param", "FrF2.splitplot"))
#      stop("blocked, split-plot and parameter designs cannot be treated with fold.design")
   if (di$type %in% c("FrF2.blocked", "FrF2.param"))
      stop("blocked designs and parameter designs in long format cannot be treated with fold.design")
   if (length(grep("center", di$type)) > 0)
      stop("currently, designs with center points cannot be treated with fold.design")
   if (length(grep("full factorial",di$type))>0)
      stop("currently, full factorial designs cannot be treated with fold.design")
   if (!(identical(columns, "full") | all(columns %in% names(di$factor.names)) | is.numeric(columns)))
      stop("columns must be full or a vector with factor position numbers or factor names")
   methcall <- columns
   if (is.character(columns) & !identical(columns, "full"))
      columns <- which(di$factor.names %in% columns)
   if (identical(columns, "full")) columns <- 1:di$nfactors
   if (is.numeric(columns))
      if (any(columns < 1 | columns>di$nfactors)) stop("elements of columns must between 1 and nfactors")
   ## now, columns is a vector of column numbers for columns to be mirrored
   fn <- names(factor.names(design))
   on <- setdiff(colnames(design),fn)  ## columns other than experimental factors
   k <- round(log2(di$nruns))  ## meaningful for FrF2 only, but for convenience generally available
   hilf <- undesign(design)[fn]
   hilf.mirror <- hilf
   for (i in 1:length(columns))
      hilf.mirror[[columns[i]]][1:nrow(design)] <- as.character(factor(3-as.numeric(hilf.mirror[[columns[i]]]), 
           labels=levels(hilf.mirror[[columns[i]]])))
   ## create new design matrix
   ## place fold factor at the end of the factors (pb), 
   ##       at the beginning of the factors (FrF2.splitplot), or after the base factors (other FrF2)
     if (substr(di$type,1,2)=="pb" | di$nfactors <= k){
        if (length(on)>0){
           hilf.mirror <- cbind(hilf.mirror,fold=rep("mirror",nrow(design)), matrix(NA,ncol=length(on),nrow=nrow(design)), stringsAsFactors=TRUE)
           hilf <- cbind(hilf, fold=rep("original",nrow(design)), design[,on], stringsAsFactors=TRUE)
           }
        else{
           hilf.mirror <- cbind(hilf.mirror,fold=rep("mirror",nrow(design)), stringsAsFactors=TRUE)
           hilf <- cbind(hilf, fold=rep("original",nrow(design)), stringsAsFactors=TRUE)
         }
        colnames(hilf.mirror) <- colnames(hilf)
        di$factor.names <- c(di$factor.names,list(fold=c("original","mirror")))
     }
     else{ 
       ## FrF2 type designs with more factors than base ones
       
       ## splitplot special, because WP-factors in front even if not base factors
       ## Fold factor after the WP factors because it is WP factor!!!
       if (di$type=="FrF2.splitplot"){
         if (length(on)>0){
           hilf.mirror <- cbind(fold=rep("mirror",nrow(design)),hilf.mirror,matrix(NA,ncol=length(on),nrow=nrow(design)), stringsAsFactors=TRUE)
           hilf <- cbind(fold=rep("original",nrow(design)), hilf, design[,on], stringsAsFactors=TRUE)
         }
         else{
           hilf.mirror <- cbind(fold=rep("mirror",nrow(design)),hilf.mirror, stringsAsFactors=TRUE)
           hilf <- cbind(fold=rep("original",nrow(design)),hilf, stringsAsFactors=TRUE)
           }
          di$factor.names <- c(list(fold=c("original","mirror")),di$factor.names)
           } 
       else {
         if (length(on)>0){
             ## further variables to be accomodated
             ## more than k factors
             hilf.mirror <- cbind(hilf.mirror[,1:k],
                                  fold=rep("mirror",nrow(design)),
                                  hilf.mirror[,(k+1):ncol(hilf.mirror),drop=FALSE],
                                  matrix(NA,ncol=length(on),nrow=nrow(design)), stringsAsFactors=TRUE)
             hilf <- cbind(hilf[,1:k], fold=rep("original",nrow(design)), hilf[,(k+1):ncol(hilf),drop=FALSE], design[,on], 
             stringsAsFactors=TRUE)
         }
         else{ hilf.mirror <- cbind(hilf.mirror[,1:k],fold=rep("mirror",nrow(design)),
                             hilf.mirror[,(k+1):ncol(hilf.mirror),drop=FALSE], stringsAsFactors=TRUE)
             hilf <- cbind(hilf[,1:k], fold=rep("original",nrow(design)), hilf[,(k+1):ncol(hilf),drop=FALSE], stringsAsFactors=TRUE)
         }
         if (!di$nfactors>k)
            di$factor.names <- c(di$factor.names,list(fold=c("original","mirror")))
         else di$factor.names <- c(di$factor.names[1:k],list(fold=c("original","mirror")),di$factor.names[(k+1):di$nfactors])

         }
         colnames(hilf.mirror) <- colnames(hilf)
     }
#   if (length(on)>0)
#     hilf <- cbind(hilf, fold=rep("original",nrow(design)), design[,on])
#   else
#     hilf <- cbind(hilf, fold=rep("original",nrow(design)))
     hilf <- rbind(hilf, hilf.mirror, stringsAsFactors=TRUE)
     for (nam in fn){ 
        hilf[[nam]] <- factor(hilf[[nam]])
        contrasts(hilf[[nam]]) <- contr.FrF2(2)
        }
     contrasts(hilf$fold) <- contr.FrF2(2)
   
   gen <- NULL

   ## extract and remove old generator information
   ## also in form catlg.entry (or base.design)
   if (!is.null(di$catlg.entry)){
      gen <- di$catlg.entry[[1]]$gen
      if (is.null(di$base.design) & !is.null(di$catlg.entry)) di$base.design <- names(di$catlg.entry)
      di$catlg.entry <- NULL
   }
   else if (!is.null(di$generators)) 
        gen <- sapply(strsplit(di$generators,"="), 
        function(obj) if (substr(obj[2],1,1)=="-") -1*which(names(Yates)==substring(obj[2],2)) 
                      else which(names(Yates) == obj[2]))

  ## now gen is a vector of signed columns 
  ## as it was valid for the original design
  
  ## walk through all generated columns to determine whether they are moved to the right or not
  if (!is.null(gen)){
  for (i in 1:length(gen)){
     ## decide about signs of generators
     ## decide is 0 for words of even length
     ##   and 1 for words of odd length
     decide <- length(intersect(
                   c(round( log2( Yates[[abs(gen[i])]] )) + 1 , k+i), columns
                   ))%%2
     if (decide == 1){ 
            ## odd word
            gen[i] <- -sign(gen[i])*(abs(gen[i]) + di$nruns)  ## nruns columns to the right, reverse sign
            }
  }
  }
  
   ## below, numbering includes fold column
   di$nruns <- 2*di$nruns
   di$nfactors <- 1+di$nfactors
   di$nfac.WP <- 1+di$nfac.WP
   di$nWPs <- 2*di$nWPs
   di$res.WP <- "unknown"  ### ??? have to work on this
   ## new k would be k+1, not changed

  if (!is.null(gen)){
   ## create new generator information for display
   minus <- sign(gen)
   cminus <- rep("", length(gen))
   cminus[minus<0] <- "-"
   if (di$nfactors <=50)
      di$generators <- paste(Letters[(k+2):di$nfactors],paste(cminus,names(Yates)[abs(gen)],sep=""),sep="=")
   else di$generators <- paste(paste("F",(k+2):di$nfactors, sep=""),
               paste(cminus, sapply(gen,function(obj) paste(paste("F",abs(obj),sep=""),collapse=":")),sep=""), sep="=")
   }
   
   ## process alias information
   alorder <- length(di$aliased) - 1
   if (!alorder %in% c(2,3)) alorder <- 2 ## set for the case that it was unconfounded already
    if (di$nfactors<=50) legend <- paste(Letters[1:di$nfactors],names(di$factor.names),sep="=")
       else legend <- paste(paste("F",1:di$nfactors,sep=""),names(di$factor.names),sep="=")
   
   ### easy for generators, other scenarios more difficult
   if (!is.null(gen)) di$aliased <- c(legend=list(legend), alias3fi(k+1, gen, order=alorder))
   else{ 
      ## estimable is handled here
          ## split-plot designs might also be treatable here later
          ## blocked designs with generators should also be possible later
          ## (current priority for extending is low)
      dat <- hilf[,names(di$factor.names)]
      stupid.workaround <- rnorm(nrow(dat))
      if (alorder==2) fit <- lm(stupid.workaround~(.)^2,data=dat)
      if (alorder==3) fit <- lm(stupid.workaround~(.)^3,data=dat)
      if (!substr(di$type,1,2)=="pb") di$aliased <- aliases(fit, code=TRUE, condense=TRUE)
   } 
   

   desnum <- model.matrix(as.formula(paste("~",paste(names(di$factor.names),collapse="+"))), hilf)[,-1]
    if (length(on)>0){
      desnumo <- desnum(design)[,on,drop=FALSE]
      desnumo2 <- desnumo*NA
      desnum <- cbind(desnum, rbind(desnumo, desnumo2))
      }

   ro <- run.order(design)
   run.no <- c(ro$run.no, ro$run.no+max(ro$run.no))
   ## this should be adjusted for regular fractional factorials
   if (substr(di$type,1,2)=="pb"){ 
     run.no.in.std.order <- c(paste(ro$run.no.in.std.order,"orig",sep="."),paste(ro$run.no.in.std.order,"mirror",sep="."))
     run.no.std.rp <- c(paste(ro$run.no.std.rp,"orig",sep="."),paste(ro$run.no.std.rp,"mirror",sep="."))}
   else{
     if (any(1:k %in% columns)){
        ll <- rep(list(c(-1,1)), k)
        for (i in 1:k) if (i %in% columns) ll[[i]] <- -ll[[i]]
        v <- ord(expand.grid(ll)[,k:1]) 
     }
     else v <- 1:k
     ### now the reordering of standard order runs for the mirror portion has been calculated
     ### must be accomodated including replicates
     ## is easy to do as v is a permutation that solely swaps pairs of runs 
     
     ## so complicated because of split-plot designs
     run.no.in.std.order <- sapply(strsplit(as.character(ro$run.no.in.std.order),".",fixed=TRUE),function(obj) as.numeric(obj[1]))
     reordered <- v[run.no.in.std.order]
     
     ## create the replaced rp version for run order
     run.no.std.rp2 <- mapply("sub",run.no.in.std.order,reordered+round(di$nruns/2),ro$run.no.std.rp)
     run.no.in.std.order <- c(run.no.in.std.order, reordered+round(di$nruns/2))
     run.no.std.rp <- c(as.character(ro$run.no.std.rp), run.no.std.rp2)
   }
     
   ro <- data.frame(run.no.in.std.order,run.no, run.no.std.rp, stringsAsFactors = TRUE)
   di$type <- paste(di$type,"folded",sep=".")
   aus <- hilf
   class(aus) <- c("design","data.frame")
   desnum(aus) <- desnum
   run.order(aus) <- ro
   di$creator <- list(di$creator, fold=methcall)
   if (length(grep("splitplot",di$type))>0){
         hilfresp <- 1:di$nruns
         hilf <- aliases(lm(hilfresp~(.)^2,aus[,names(di$factor.names)[1:di$nfac.WP]]),code=TRUE,condense=TRUE)
         if(!is.null(hilf$main)) di$res.WP <- 3
         else if (!is.null(hilf$fi2)) di$res.WP <- 4
         if (di$res.WP=="unknown"){
             hilf <- aliases(lm(hilfresp~(.)^3,aus[,names(di$factor.names)[1:di$nfac.WP]]),code=TRUE,condense=TRUE)
             if(!is.null(hilf$fi2)) di$res.WP <- 5
             else if (!is.null(hilf$fi3)) di$res.WP <- 6
                  else {
                     di$res.WP <- 7
                     if (!di$nfac.WP < di$res.WP) 
                     message("resolution of whole plot portion of design is at least 7,
                     \nres.WP element of design.info attribute has been set to Inf")
                     di$res.WP <- Inf
                  }
         } 
      }
   design.info(aus) <- di
   aus
}

splitpick <- function(k, gen, k.WP, nfac.WP, show=10){
  ## function that picks design with right number of factors within first 
  ## so many columns as long as the current factors remain generators
  ## it may be possible to find an appropriate design within 
  ##    the same base design with modified generator columns
  ##  ?? how can these be chosen ??
  ##  ?? is this worth the extra effort ??

  ## k.WP is the dimension for WP factors
  ## 2^k.WP - 1 first columns are used exclusively for whole plots
  ## and should not contain SP factors

  ## k.WP <= nfac.WP is needed
  ##                  if not true in the subject matter situation,
  ##                  extra whole plot generation factors are required
  
  ## potential future improvement:
  ## enhance output by alias structure, if requested
  ## alias for whole plot separate as well
  
  hilf <- c(k,abs(gen),k.WP,nfac.WP,show)
  if (!is.numeric(hilf))
      stop ("All inputs to splitpick must be numeric.")
  if (!all(hilf == floor(hilf) & hilf > 0))
      stop ("All inputs to splitpick must contain integer numbers.")
  minus <- which(gen<0)
  gen <- abs(gen)
  if (!k >= 3) stop ("splitpick requires k>=3.")
  if (!k.WP < k) stop ("splitpick requires k.WP < k.")
  if (!nfac.WP < 2^k.WP) stop ("nfac.WP >= 2^k.WP is not permitted.")
  if (!nfac.WP >= k.WP) stop ("nfac.WP < k.WP is not permitted. 
       You must increase nfac.WP by ", k.WP - nfac.WP, 
       " in order to support generation of ", 2^k.WP, " whole plots.")
  if (any(gen %in% 2^(0:(k-1)))) 
        stop ("gen must not contain column numbers of base factors.")
  if (any(!gen %in% 3:(2^k-1))) 
        stop ("Column numbers in gen must be smaller than ", 2^k,".")
  g <- length(gen)
  if (!nfac.WP < k+g) stop ("nfac.WP must be smaller than the total number of factors!")
  
  perm <- permutations(k)
  hilf <- digitsBase(gen,ndigits=k) ## always k rows
  ergeb <- matrix(0,nrow=nrow(perm),ncol=length(gen))
  nfacs.WP <- rep(NA, nrow=nrow(perm))
  for (i in 1:factorial(k)){
    ergeb[i,] <- sort(as.integer(reord(hilf,perm[i,])))
    nfacs.WP[i] <- sum(ergeb[i,]<2^k.WP)
  }
  pick <- which(nfacs.WP==nfac.WP-k.WP)
  if (length(pick)<show) show <- length(pick) 
  if (show==0) stop("no adequate split-plot design found\n")
  else {
    gens <- ergeb[pick[1:show],,drop=FALSE]
    if (nfac.WP==k.WP) res.WP <- rep(Inf,show)
    else {
         if (nfac.WP > 2^(k.WP-1)) res.WP <- rep(3,show)
         else {
         hilf <- gens[,1:(nfac.WP-k.WP),drop=FALSE]
         res.WP <- apply(hilf,1,function(obj) min(sapply(words.all(k.WP,obj)[[2]],length)))
         }
    } 
    reorder <- sort(res.WP,index.return=TRUE,decreasing=TRUE)$ix
    ## if > 7, res.WP=Inf

    gen[minus] <- -gen[minus]
    ## for documentation purposes only
    ## does not affect the resulting design whether or not there is a minus there
    
    list(orig=gen, basics=c(nruns=2^k, nWPs=2^k.WP, nfac.WP=nfac.WP, nfac.SP=k+g-nfac.WP),
         perms=perm[pick[1:show][reorder],,drop=FALSE],
         res.WP=res.WP[reorder], gen=gens[reorder,,drop=FALSE])
  }
}

DanielPlot <- function(fit, ...){
    UseMethod("DanielPlot")
}
DanielPlot.design <- function(fit, ..., response=NULL){
    if (!"design" %in% class(fit)) 
        stop("DanielPlot.design works for obj from class design only.")
    di <- design.info(fit)
    if (is.null(di$response.names)) 
        stop("The design fit must have at least one response.")
    if (!(is.null(response))) 
      if (!response %in% di$response.names)
        stop("Requested response is not a response variable in fit.")
    if (!(length(grep("FrF2",di$type))>0 | 
           length(grep("pb",di$type))>0)) { 
           if (!(di$type=="full factorial" & all(di$nlevels==2)))
        stop("The design obj must be of a type containing FrF2 or pb.")
    if (length(grep("blocked",di$type))>0)
        stop("Function DanielPlot does not handle blocked designs. \nUse function halfnormal from package DoE.base instead.")
       }
    grad <- 1
    if (length(grep("pb",di$type)) > 0 & di$nfactors < di$nruns-1)
          warning("Effects plots for Plackett-Burman designs must be done with nruns-1 effects! The error effects are missing!")
    ## make sure there are as many effects as possible in the plots, redundant ones will not be shown
    if (length(grep("FrF2",di$type)) > 0 | (di$type=="full factorial" & all(di$nlevels==2))){
       grad <- 2
       hilf <- lm(fit, degree=grad)
       ncoef <- sum(!is.na(coef(hilf)))
       nhilf <- di$nrun
       if (!is.null(di$ncenter)) nhilf <- nhilf - di$ncenter
       while (ncoef < nhilf){
          grad <- grad+1
          hilf <- lm(fit, degree=grad)
          ncoef <- sum(!is.na(coef(hilf)))
          }
    }
    subtext <- ""
    if (length(grep("splitplot", di$type)) > 0){
        ## splitplot situation
        ## make sure to use different plot symbols for whole plot and split plot effects
        ## important to distinguish wbreps and bbreps
               mm <- model.matrix(hilf)
               coefs <- coef(hilf)[-1] ## only non-missing coefficients
               nms <- names(coefs)
               hilf <- apply(mm[,1+(1:di$nfac.WP),drop=FALSE],1,paste,collapse="")
               pchs <- rep("*", length(coefs))
               pchs[1:di$nfac.WP] <- "o"
               
               for (j in setdiff(1:(length(coefs)),1:di$nfac.WP)){
                  if (!length(table(paste(hilf,mm[,nms[j]],sep="")))>di$nWPs) pchs[j] <- "o"
                }
          subtext <- "WARNING: whole plot effects (marked by o) may have larger variation than split-plot effects"
          DanielPlot(lm(fit, degree=grad, response=response), pch=pchs, subtitle=subtext, ...)
          }
          else DanielPlot(lm(fit, degree=grad, response=response), ...)
}

DanielPlot.default <-
function (fit, code = FALSE, autolab = TRUE, alpha=0.05,
         faclab = NULL, 
         block = FALSE, datax = TRUE, 
         half = FALSE, pch = "*", 
         cex.fac = par("cex.lab"), cex.lab = par("cex.lab"), 
         cex.pch = par("cex"), cex.legend = par("cex.lab"), 
         main = NULL, subtitle=NULL, ...) 
{
   if (! ("lm" %in% class(fit) | "aov" %in% class(fit))) 
      stop("fit must be a linear model object (lm or aov), or a design of class design")
    ## transform into -1 and 1 coded model
    fit <- remodel(fit)$model
    ## check whether of appropriate type
    if (!check(fit))
     stop("This routine is applicable for 2-level factorial designs without partial aliasing only.")
   
    
    if (any(names(coef(fit)) == "(Intercept)")) {
        factor.effects <- 2 * coef(fit)[-1]
        }
    else {
        factor.effects <- 2 * coef(fit)
    }
    respnam <- colnames(fit$model)[attr(attr(fit$model,"terms"),"response")]
    names(factor.effects) <- attr(fit$terms, "term.labels")
    terms.ord <- attr(fit$terms, "order")[!is.na(factor.effects)] ##moved here
    factor.effects <- factor.effects[!is.na(factor.effects)]
    plotmain <- paste("Normal Plot for", respnam)
    if (autolab) {
        ### take simulated critical values from package DoE.base, if available
        crit <- ME.Lenth(factor.effects,alpha=alpha)$ME
        if (!code)
        faclab <- list(idx = which(crit<=abs(factor.effects)),
               lab = names(factor.effects)[which(crit<=abs(factor.effects))])
        plotmain <- paste(plotmain, ", ", "alpha=", alpha, sep="") 
        }
    if (half) {
        n <- length(factor.effects)
        ## qnorm(0.5 + ppoints(n, a=1/2)/2) in halfnormal
        tn <- list(x = qnorm(0.5 + ppoints(n, a=1/2)/2)[rank(abs(factor.effects))], 
            y = abs(factor.effects))
        xlab <- "half-normal scores"
        ylab <- "absolute effects"
        plotmain <- paste("Half", plotmain)
    }
    else {
        tn <- qqnorm(factor.effects, plot = FALSE)
        xlab <- "normal scores"
        ylab <- "effects"
    }
    names(tn$x) <- names(factor.effects)  ## moved here
    names(tn$y) <- names(factor.effects)  ## added
    if (datax) {
        tmp <- tn$x
        tn$x <- tn$y
        tn$y <- tmp
        tmp <- xlab
        xlab <- ylab
        ylab <- tmp
    }
    labx <- names(factor.effects)
    laby <- 1:length(tn$y)
    points.labels <- names(factor.effects)
    if (is.null(main)) main <- plotmain
    plot.default(tn, xlim = c(min(tn$x), max(tn$x) + diff(range(tn$x))/5), 
        pch = pch, xlab = xlab, ylab = ylab, cex=cex.pch, cex.lab = cex.lab, 
        mgp=c(2,1,0), main = main, ...)
    ## at the top below main title, mainly for warning in case of split-plot
    if (!is.null(subtitle)) mtext(subtitle, cex=cex.lab)  ## added cex.lab July 2019
    if (is.null(faclab)) {
        if (!code) {
            effect.code <- labx
        }
        else {
            max.order <- max(terms.ord)
            no.factors <- length(terms.ord[terms.ord == 1])
            factor.label <- attr(fit$terms, "term.labels")[terms.ord == 1]
            faclet <- c(LETTERS[-9],letters[-9])
            factor.code <- faclet[1:no.factors]
            if (block) 
                factor.code <- c("BK", factor.code)
            texto <- paste(factor.code[1], "=", factor.label[1])
            for (i in 2:no.factors) {
                texto <- paste(texto, ", ", factor.code[i], "=", 
                  factor.label[i])
            }
            mtext(side = 1, line = 3.5, texto, cex = cex.legend)
            get.sep <- function(string, max.order) {
                k <- max.order - 1
                get.sep <- rep(0, k)
                j <- 1
                for (i in 2:(nchar(string)-1)) {
                  if (substring(string, i, i) == ":") {
                    get.sep[j] <- i
                    if (j == k) 
                      break
                    j <- j + 1
                  }
                }
                get.sep
            }
            labeling <- function(string, get.sep, max.order, 
                factor.code, factor.label) {
                labeling <- ""
                sep <- get.sep(string, max.order)
                sep <- sep[sep > 0]
                n <- length(sep) + 1
                if (n > 1) {
                  sep <- c(0, sep, nchar(string) + 1)
                  for (i in 1:n) {
                    labeling <- paste(labeling, sep = "", factor.code[factor.label == 
                      substring(string, sep[i] + 1, sep[i + 1] - 
                        1)][1])
                  }
                }
                else labeling <- paste(labeling, sep = "", factor.code[factor.label == 
                  string][1])
                labeling
            }
            effect.code <- rep("", length(terms.ord))
            for (i in 1:length(terms.ord)) {
                effect.code[i] <- labeling(names(tn$x)[i], get.sep, 
                  max.order, factor.code, factor.label)
            }
        }
        ## changed text position to be farther from the points July 2019
        if (autolab){ 
               faclab <- list(idx = which(crit<=abs(factor.effects)),
                  lab = effect.code[which(crit<=abs(factor.effects))])
               if (length(faclab$idx) > 0)
               text(as.data.frame(tn)[faclab$idx,], paste(" ", faclab$lab), cex = cex.fac, pos = 4, 
                  xpd = NA)
           }
        else
        text(tn, paste("   ", effect.code), cex = cex.fac, pos=4, 
            xpd = NA)
    }
    else {
        if (!is.list(faclab)) 
            stop("* Argument 'faclab' has to be NULL or a list with idx and lab objects")
        if (length(faclab$lab)>0) text(tn$x[faclab$idx], tn$y[faclab$idx], labels = faclab$lab, 
            cex = cex.fac, pos=4)
    }
    if (!length(pch)==length(factor.effects)) pchs <- rep(pch, length(factor.effects))
    if (code) aus <- cbind(as.data.frame(tn), no = 1:length(tn$x), effect=names(factor.effects), coded=effect.code, pchs=pch)
      else aus <- cbind(as.data.frame(tn), no = 1:length(tn$x), effect=names(factor.effects), pchs=pch)
    aus$effect <- as.character(aus$effect)
    invisible(aus)
}

#removed August 15 2013
#halfnormal <- function(effects, labs, codes=labs, alpha=0.05, xlab="absolute effects", ...){
#    effects <- abs(effects)
#    labord <- order(effects)
#    effects <- sort(effects)
#    if (!identical(codes, labs)){ 
#         haupteff <- setdiff(1:length(labs), grep(":", labs)) 
#         legende <- paste(codes[haupteff], labs[haupteff], sep="=",collapse=", ")
#     }
#     else legende <- ""
#    n <- length(effects)
#    ui <- qnorm(0.5 + (0:(n-1)+0.5)/(2*n))  ## "+0.5" in parentheses added Aug 14 2013
#    codes <- paste(rep("  ",n), codes, sep="")
#    
#    crit <- LenthPlot(effects,alpha=alpha,plt=FALSE)["ME"]
#    nlab <- sum(effects>crit)
#    plot(effects, ui, ylab = "Half-normal scores", xlab = xlab, sub=legende, ...)
#    if (nlab < n) 
#        points(effects[1:(n - nlab)],ui[1:(n - nlab)])
#    text(effects[(n - nlab + 1):n], ui[(n - nlab + 1):n], codes[labord][(n - 
#        nlab + 1):n], adj=0, xpd=NA)
#}

gen2clear.2fis <- function(k, gen){
    ncand <- choose(k+length(gen),2)
    hilf <- combn(k+length(gen),2)
    sel <- 1:ncand
    ww <- words.all(k, gen, max.length=4)
    if (length(ww$WLP)==0) return(list(nclear.2fis=ncand, clear.2fis=hilf))
    else{
                woerter <- ww[[2]]
                nclear.2fis <- 0
                spalte <- numeric(0)
                for (i in 1:ncol(hilf))
                    if (all(sapply(woerter, function(obj) sum(hilf[,i] %in% obj)<2))){
                      nclear.2fis <- nclear.2fis+1
                      spalte <- c(spalte,sel[i])
                    }
                return(list(nclear.2fis=nclear.2fis, clear.2fis=hilf[,spalte]))
    }
}

gen2CIG <- function(nruns, gen){
    if (!is.numeric(nruns)) stop("nruns must be a number")
    if (!length(nruns)==1) stop("nruns must be a single number")
    k <- round(log2(nruns))
    if (!nruns==2^k) stop("nruns must be a power of 2")
    if (!is.numeric(gen)) stop("gen must be a numeric vector")
    if (!all(gen==round(gen))) stop("gen must contain integer numbers only")
    hilf <- gen2clear.2fis(k, gen)
    if (hilf$nclear.2fis==0) return(graph(numeric(0), n=k+length(gen)))
    else{
    return(graph(hilf$clear.2fis, n=k+length(gen),directed=FALSE))
    }
}

cubelabel <- function(cdobj, lablev=list(c(-1,1),c(-1,1),c(-1,1)),labs=NULL,
          pos="outab", offs=1, adj=c(0,0),col="blue",cex=1.5){
    ### function to write labels onto the corners of the cube
    ### lablev is a list of factor levels that is automatically expanded to the right order
    ###       for the eight cube points; lablev is ignored, if labs is given
    ### labs is a vector of eight labels in the order of the points
    ###       for labelling only part of the points, give empty labels for unlabelled points

    ### pos can be NULL, 1(below),2(left),3(above),4(right),
    ###       and special words outab (outer above/below), in ab (inner above/below)
    ###                         outlr (outer left/right),  in lr (inner left/right)
    ### for labelling the corners inside plot symbols, use pos=NULL

    if(!is.null(pos)){
      if (pos=="outab") pos <- c(1,1,1,1,3,3,3,3)
      else if (pos=="inab") pos <- c(3,3,3,3,1,1,1,1)
      else if (pos=="outlr") pos <- c(2,4,2,4,2,4,2,4)
      else if (pos=="inlr") pos <- c(4,2,4,2,4,2,4,2)
    }
    x <- cdobj$res3d$xyz.convert(cdobj$cub)$x
    y <- cdobj$res3d$xyz.convert(cdobj$cub)$y
    if (is.null(labs)){
        lc <- expand.grid(lablev)  ## labels for corners
        if (all(sapply(lc,is.numeric)))
           labs <- paste(lc[,1],lc[,2], lc[,3],sep="/")
        else
           labs <- paste(as.character(lc[,1]),as.character(lc[,2]),
               as.character(lc[,3]),sep="")
    }
    text(cdobj$res3d$xyz.convert(cdobj$cub), labels=labs, col=col,
         cex=cex, pos=pos, offset=offs, adj=adj)
}

colpickIV <- function(design, q, all=FALSE, 
                    select.catlg=catlg, 
                    estimable=NULL, method="VF2", sort="natural", 
                    res3=FALSE, all0=FALSE, quiet=FALSE,
                    firsthit=is.numeric(design)){
  ## design is a character string that identifies a catalogued design
  ##    or a length 1 catlg object
  ##    of the number of factors for a full factorial
  ## q is log2 of the block size
  ## all = TRUE requests a list of results
  ## select.catlg is a design catalogue of class catlg
  ## estimable is a character vector of estimable 2fis in letter notation, 
  ##    or a two row numeric matrix with factor numbers (each column indicates a clear 2fi)
  ## method is the method to use for subgraph isomorphism checking
  ## sort is the method to use for pre-sorting in the subgraph isomorphism check
  ## all0 = TRUE also outputs X matrices with zero columns
  ## for all=FALSE, the function returns a list with an X matrix 
  ## the clear2fis, 
  ## and possibly a mapping (if estimable is specified)
  ## for all=TRUE, the function returns a list of three or four lists (X matrices, estimable 2fis, profiles)
  ## if estimable is specified, 
  ## the fourth list indicates a re-mapping with which the requirement can be accommodated
  catlg <- select.catlg
  if (!("catlg" %in% class(design) || is.character(design) || is.numeric(design)))
    stop("design must be the number of factors of a full factorial", 
         " or a character string that refers to an element of select.catlg", 
         " or of class catlg")
  if (is.numeric(design)){
    ## full factorial
    element <- list(list(nfac  = design,
                         nruns = 2^design,
                         res = Inf,
                         nclear.2fis = choose(design, 2),
                         clear.2fis = combn(design, 2),
                         gen = numeric(0), 
                         WLP = rep(0, 4),
                         dominating = TRUE))
    class(element) <- c("catlg", "list")
  }
  if (is.character(design)){
    element <- catlg[design]    ## class catlg
  }
  if ("catlg" %in% class(design)) 
    element <- design[1]       ## more than one element is ignored without warning
  N <- element[[1]]$nruns
  n <- element[[1]]$nfac
    #if (n>50) stop("colpickIV works for up to 50 factors only")
  gen <- element[[1]]$gen         ## Yates column numbers
  p <- length(gen)
  k <- n - p
  map <- NULL        ## overwrite any map object from global environment

  ## select adequate element of catlg in case of estimable
  if (!is.null(estimable)){
    ## stricter checks than with estimable.check
    ## as there need not be factor.names etc.
    if (!is.numeric(estimable) && !is.character(estimable))
      stop("estimable must be an integer-valued two row matrix",
           " or a character vector")
    if (is.numeric(estimable) && !is.matrix(estimable))
      stop("if numeric, estimable must be a matrix with two rows")
    if (is.numeric(estimable) && !nrow(estimable)==2)
      stop("if numeric, estimable must be a matrix with two rows")
    if (is.character(estimable)){
      colons <- grep(":", estimable)
      if (!(length(colons)==0 || length(colons)==length(estimable)))
          stop("All elements of estimable must have the same format")
      if (length(colons)==0)
      estimable <- sapply(estimable, 
                          function(obj) which(Letters %in% unlist(strsplit(obj, "", fixed=TRUE))))
      else   estimable <- sapply(estimable, 
                          function(obj) as.numeric(gsub( "F", "", unlist(strsplit(obj, ":", fixed=TRUE)) )))
      if (!is.matrix(estimable)) stop("invalid estimable")
     }
    ## estimable is now a two-row matrix
    ## obtain map with which the requirement set can be accommodated in the design
    if (p==0) map <- 1:n else
    map <- mapcalc(estimable, n, N, select.catlg=element, method=method, sort=sort, 
                   res3=res3, ignore.dom=TRUE)[[1]]
    ## adapt estimable to the mapping
    estimable <- matrix(map[estimable], nrow=2)  
    ## sort columns in ascending order
    for (i in 1:ncol(estimable)) estimable[,i] <- sort(estimable[,i])
    if (n<=50) estimchar <- sapply(1:ncol(estimable), 
                        function(obj) 
                          paste0(Letters[estimable[,obj]], collapse=""))
    else estimchar <- sapply(1:ncol(estimable), function(obj) 
      paste("F",estimable[,obj], sep="", collapse = ":"))
    ## character representation for later success checking
  } 

  ## clear before blocking
  clear2fis <- clear.2fis(element)
  if (length(clear2fis[[1]])==0) 
    clear2fis <- character(0) 
  else{
    if (n <= 50) 
    clear2fis <- sapply(1:ncol(clear2fis[[1]]), function(obj) 
      paste(Letters[clear2fis[[1]][,obj]], collapse = ""))
    else
    clear2fis <- sapply(1:ncol(clear2fis[[1]]), function(obj) 
      paste("F",clear2fis[[1]][,obj], sep="", collapse = ":"))
    }
  nclear2fis <- length(clear2fis)
  
  Z <- t(sapply(gen, function(obj) rev(digitsBase(obj, 2, k))))
  div <- 2^q-1

  ## list of possible X column vectors
  Xcands <- lapply(1:div, function(obj) digitsBase(obj, 2, ndigits=q))

  success <- FALSE
  Xlist <- vector(mode="list")
  tablist <- vector(mode="list")
  clearlist <- vector(mode="list")
  ## each row of picks provides a possible XI matrix
  ## lazy expanding in order to facilitate large problems
  ## modify Godolphin as follows:
  ##    only use up to the first e elements in position e
  poscands <- lapply(1:k, function(obj) 
    if (obj <= div) 
      1:obj
    else 1:div
  )
  nxt <- do.call(lazyExpandGrid, poscands)
  nr <- prod(lengths(poscands))
  # picks <- as.matrix(expand.grid(rep(list(1:div), k-1)))
  # picks would have nr rows
  message("checking up to ", nr, " matrices")
  
  ## initialize to a heavily balanced first matrix
  ## not yet using nxt()
  ## this makes full factorials without estimability requirements immediate
  ##     and may also be beneficial for fractions with easy structure
  ## nxt() is used at the end of the loop
  jetzt <- rep(1:div, (k%/%div+1))[1:k]
  i <- 0
  while (i <= nr){
    ## nr + 1, because the first jetzt is extra
    XI <- do.call(cbind, Xcands[jetzt])
        if (length(Z)>0){ 
          XII <- (XI%*%t(Z))%%2
          X <- cbind(XI, XII)
        }
        else X <- XI   ## full factorial
        
        ## skip settings with less than q different columns
        ## i.e. with row rank less than q
        hilf <- lapply(1:n, function(obj) X[,obj])
        if (length(unique(hilf)) < q){
          jetzt <- unlist(nxt())
          i <- i+1
          next 
        }
        
    if (all(colSums(X) > 0) || all0){ 
      ## remove direct aliases from blocking
      clearcur <- clear2fis
      if (length(clear2fis) > 0){
        ingroup <- character(0)
        for (ii in 1:(n-1))
          for (jj in (ii+1):n)
            if (all(X[,ii]==X[,jj]))
              ingroup <- c(ingroup,ifelse(n<=50,
                                  paste0(Letters[ii],Letters[jj]),
                                  paste0("F",ii,":F",jj)))
        clearcur <- setdiff(clearcur, ingroup)
      }
      
      if (is.null(estimable))
        success <- TRUE 
      else{
        if (all(estimchar %in% clearcur)) {
          success <- TRUE
          if (firsthit) 
            return(list(X=X, clear2fis=clearcur, map=map))
        }
        else{
          ## skip matrices that violate estimability requirement
          jetzt <- unlist(nxt())
          i <- i+1
          if (is.logical(jetzt)){
            if (!jetzt) break
          }
          next 
        }
      }  
    }
    else {
      ## skip matrices with all-zero column
      jetzt <- unlist(nxt())
      i <- i+1
      if (is.logical(jetzt)){
        if (!jetzt) break
      }
      next 
    }
    tab <- sort(table(apply(X, 2, paste0, collapse="")), decreasing = TRUE)
    tab <- unname(tab)
    ## optimal in terms of maximum number
    if (length(clearcur) == min(phimax(n,q), nclear2fis) && !all){
      ## maximum possible with this candidate and this q
      if (is.null(estimable)) 
        return(list(X=X, clear2fis=clearcur))
      else 
        return(list(X=X, clear2fis=clearcur, map=map))
    } else{
      if (is.null(estimable)){
        Xlist <- c(Xlist, list(X))
        tablist <- c(tablist, list(tab))
        clearlist <- c(clearlist, list(clearcur))
      }
      else{
        if (!is.null(map)){
          Xlist <- c(Xlist, list(X))
          tablist <- c(tablist, list(tab))
          clearlist <- c(clearlist, list(clearcur))
        }
      }
    }
    ## obtain next entry (will be FALSE, if no further entry available)
    jetzt <- unlist(nxt())  ## should get the next entry
    i <- i+1
    if (is.logical(jetzt)){
      if (!jetzt) break
      }
  } ## end of loop
  if (!success) {
    if (!quiet) message("no suitable block arrangement was found")
    return(invisible(NULL))
  }
  if (all) {
    if (is.null(estimable)) 
      return(list(X_matrices=Xlist, clearlist=clearlist, profiles=tablist))
    else
      return(list(X_matrices=Xlist, clearlist=clearlist, profiles=tablist, map=map))
  } ## else
  
  lens <- lengths(clearlist)
  tablist <- tablist[lens==max(lens)]
  Xlist <- Xlist[lens==max(lens)]
  clearlist <- clearlist[lens==max(lens)]
  diffs <- sapply(tablist, function(obj) diff(range(obj)))
  pos <- which.min(diffs)
  if (is.null(estimable))
    return(list(X=Xlist[[pos]], clear2fis=clearlist[[pos]]))
  else
    return(list(X=Xlist[[pos]], clear2fis=clearlist[[pos]], map=map))
}

IAPlot <- function(obj, ...){
    UseMethod("IAPlot")
}
IAPlot.design <- function(obj, ..., response=NULL){
    if (!"design" %in% class(obj)) 
        stop("IAPlot.design works for obj from class design only.")
    di <- design.info(obj)
    if (is.null(di$response.names)) 
        stop("The design obj must have at least one response.")
    if (is.null(response)) response <- di$response.names[1] else{
      if (!response %in% di$response.names)
        stop("Requested response is not a response variable in obj.")
        }
    if (!(length(grep("FrF2",di$type))>0 | 
           length(grep("pb",di$type))>0)) { 
           if (!(di$type=="full factorial" & all(di$nlevels==2)))
        stop("The design obj must be of a type containing FrF2 or pb.")
       }
    if (!length(grep("blocked",di$type))>0)
        IAPlot(lm(obj, degree=2, response=response), ...)
    else
        IAPlot(lm(as.formula(paste(response, "~ (", 
                      paste(names(di$factor.names), collapse="+"), 
                                                ")^2")),
               data=obj), ...)
}

IAPlot.default <-
function(obj, main=paste("Interaction plot matrix for",respnam), pch=c(15,17), 
    cex.lab=par("cex.lab"), cex=par("cex"), cex.xax=par("cex.axis"), 
    cex.yax=cex.xax, cex.title = 1.5, lwd=par("lwd"), las=par("las"), abbrev=4, select=NULL, 
    show.alias=FALSE, ...){
    # obj     a linear model
    # pch     plot characters used
    # cex     plot character size
    # cex.lab size of variable labels in diagonal panels
    # cex.xax size of annotation for x-axis
    # cex.yax size of annotation for y-axis
    # cex.title
    # lwd
    # las
    # abbrev  number of characters for factor levels in diagonal panel
    # show.alias  show number of effect in each panel, 
    #         in order to allow immediate judgment which effects are aliased

   if (! ("lm" %in% class(obj) | "aov" %in% class(obj))) 
      stop("obj must be a linear model object (lm or aov), or a design of class design")
    
   ### base everything on model recoded to -1 and 1 numerics
   obj <- remodel(obj)
   ## extract labels for factor levels to desired length (abbrev)
   labs <- lapply(obj$labs,function(sp) substr(sp,1,abbrev))
   ## reduce obj list to its linear model component
   obj <- obj$model
   mod <- obj$model
   respnam <- colnames(mod)[attr(attr(mod,"terms"),"response")]
   ## for some reason, this does not work when using obj instead of mod
   ## it neither works if respnam is something like rnorm(16)
      ## how do I fix or at least report this ?
  
   if (!check(obj))
     stop("This routine is applicable for 2-level factorial designs without partial aliasing only.")
     
   ## prepare for simple access of important quantities
   term.ord <- attr(terms(obj),"order")
   mains <- which(term.ord==1)
   if (!any(term.ord==2)) stop("no 2-factor interaction terms in model")
   ints <- which(term.ord==2)
   nmain <- length(mains)
   nint <- length(ints)
   intcol <- attr(terms(obj),"intercept")
   termnames <- attr(terms(obj),"term.labels")
   
   if (!is.null(select)){
       ## take care of selecting only part of the interactions
       if (!is.numeric(select)) stop("select must be numeric")
       if (!all(floor(select)==select)) stop("select must contain integer numbers")
       if (any(select<1 | select>nmain)) stop("select must contain numbers betweeen 1 and ", nmain, " only")
       select <- unique(select)
       if (length(select)<2) stop("at least 2 effects must be selected for an interaction plot matrix")
       selnam <- termnames[select]
       labs <- labs[select]
       hilf <- strsplit(termnames[ints],":")
       selected <- sapply(hilf, function(.obj1) all(.obj1 %in% selnam))
       intselnam <- termnames[ints[selected]]
       if (!is.name(respnam)) formulasel <- paste(respnam, "~", paste(c(selnam,intselnam),collapse="+"))
       formulasel <- paste(respnam, "~", paste(c(selnam,intselnam),collapse="+"))
       obj <- lm(formulasel, data=mod)
       term.ord <- attr(terms(obj),"order")
       mains <- which(term.ord==1)
       ints <- which(term.ord==2)
       nmain <- length(mains)
       nint <- length(ints)
       intcol <- attr(terms(obj),"intercept")
   }
   mm <- model.matrix(obj)
   ## omit intercept, if present
   if (intcol > 0) mm <- mm[,-intcol]
   
   coefs <- coef(obj)
   if (intcol > 0) coefs <- coefs[-intcol] 

 
   terms1 <- colnames(mm)[mains]
   terms2 <- colnames(mm)[ints]
   ## used for easy prediction
   addnam <- setdiff(colnames(obj$model), terms1)
   names <- c(terms1, addnam)
   
   ## create matrix for plots
   predmat <- matrix(rep(0,4*nint),4,nint)
   varnums <- matrix(rep(0,2*nint),2,nint)
   colnames(predmat) <- terms2
   
   ## extract aliasing information
   ## for optional annotation of plots
   al <- aliases(obj)$aliases
   if (is.null(al)) exist <- names(coefs) 
   else exist <- sapply(al, function(sp) sp[1])
   alnum <- rep(0,length(coefs))
   names(alnum) <- names(coefs)
   
   ## calculate alias numbers for aliasing information
   ## and replace NA coefficients for aliased effects 
   ## with the appropriately signed replacement from "master"
   for (i in 1:length(coefs)){
      if (!is.na(coefs[i])) alnum[i] <- which(exist==names(coefs)[i])
      else{ 
          hilf <- grep(names(coefs)[i],al)
          coefs[i] <- coefs[exist[hilf]]
          alnum[i] <- hilf
          if (substr(al[[hilf]][grep(names(coefs[i]),al[[hilf]])],1,1)=="-")
             coefs[i] <- -coefs[i]
      }
   }
   ## re-include intercept coefficient
   if (intcol==1) coefs <- c(obj$coefficients[intcol], coefs)
   else if (intcol < length(coefs)) 
       coefs <- c(coefs[1:(intcol-1)],obj$coefficients[intcol])
   else coefs <- c(coefs, obj$coefficients[intcol])
   
   ## calculate model-based predicted values for all interactions
   for (i in 1:nint){
      dat <- matrix(rep(0,4*nmain),4,nmain)
      inam <- strsplit(terms2[i],":")[[1]]
      ml <- inam[1]
      mr <- inam[2]
      ## first in terms1
      m1 <- min(grep(ml,terms1,fixed=TRUE),grep(mr,terms1,fixed=TRUE))
      ## second in terms1
      m2 <- max(grep(ml,terms1,fixed=TRUE),grep(mr,terms1,fixed=TRUE))
      ## file in varnums matrix for later use
      varnums[,i] <- c(m1,m2)  
      dat[,m1] <- c(-1,1,-1,1)
      dat[,m2] <- c(-1,-1,1,1)
      colnames(dat) <- terms1
      ## allow for things like variables excluded by "-" etc.
      dat <- cbind(dat, matrix(rep(0,length(addnam)*4),4,length(addnam)))
      colnames(dat) <- names
      dat <- as.data.frame(dat)
      modmat <- model.matrix(lm(terms(obj),dat))
      predmat[,i] <- modmat %*% coefs
   }

   ## save old parameters for restoring them later
   omfrow <- par("mfrow")
   omar <- par("mar")
   ooma <- par("oma")
   ## determine axis limits and graphics parameters
   ax <- pretty(c(min(predmat),max(predmat)))
   par(mfcol=c(nmain,nmain), mar=c(1, 1, 1, 1) + 0.1, oma=c(3,5,4,0.1))
   # i is column index, j is row index
   for (i in 1:nmain){
     for (j in 1:nmain){
         ## interaction terms (regardless whether in model or not)
         ww <- c(paste(terms1[i],terms1[j],sep=":"), 
               paste(terms1[j],terms1[i],sep=":"))
         ## column of alnum corresponding to this ww 
         ## (if any, otherwise integer(0))
         hilf <- which(names(alnum) %in% ww)
         
         ## handle diagonal panels
          if (i==j){ plot(c(-1),min(ax)+5/6*(max(ax)-min(ax)),
                          ylim=c(min(ax),max(ax)),xlim=c(-1.1,1.1),
                          axes=FALSE,xlab="",ylab=respnam,
                          col="red", pch=pch[1], cex=cex, las=las)
                box(which="figure")
                points(c(-1),min(ax)+1/6*(max(ax)-min(ax)),pch=pch[2], cex=cex, lwd=lwd)
                text(c(-1,-1),min(ax)+c(5,1)*(max(ax)-min(ax))/6,
                      labs[[i]],pos=4, xpd=NA, cex=cex.lab)
                text(0,(max(ax)+min(ax))/2, terms1[i],adj=0.5,xpd=TRUE, 
                    cex=cex.lab, col="blue")
                ## axes for corner panels
                if (i==1) axis(2, at = ax, labels = ax, cex.axis=cex.yax, 
                      outer=TRUE, lwd=lwd, las=las)
                if (j==nmain) axis(1, at = c(-1,1), labels = labs[[j]], 
                    cex.axis=cex.xax, outer=TRUE, lwd=lwd, las=las)
                }
          if (i<j) {
              sp <- intfind(i,j,varnums)
              if (is.null(sp)) {
                    plot(c(-10,10),c(-1,1), axes=FALSE, xlab="",
                       xlim=c(-1.1,1.1),ylim=c(min(ax),max(ax)), las=las)
                       ## skip panel
                    box(which="figure")
                    }
              else {plot(c(-1,1),predmat[c(1,2),sp],ylab=respnam,xlab=terms1[j],
                     type="b", xlim=c(-1.1,1.1),ylim=c(min(ax),max(ax)), 
                     axes=FALSE, col="red",
                     lty=3, pch=pch[1], cex=cex, lwd=lwd, las=las)
                    box(which="figure")
                    lines(c(-1,1),predmat[c(3,4),sp],lwd=lwd)
                    points(c(-1,1),predmat[c(3,4),sp],pch=pch[2], cex=cex, lwd=lwd)
                    }       
              ## axes for border panels
                if (i==1) axis(2, at = ax, labels = ax, cex.axis=cex.yax, outer=TRUE, lwd=lwd, las=las)
                if (j==nmain) axis(1, at = c(-1,1), labels = labs[[i]], 
                    cex.axis=cex.xax, outer=TRUE, lwd=lwd, las=las)
          }
          if (i>j) {
               sp <- intfind(j,i,varnums)
              if (is.null(sp)) {
                    plot(c(-10,10),c(-1,1), axes=FALSE, xlab="",
                       xlim=c(-1.1,1.1),ylim=c(min(ax),max(ax)), las=las)## skip panel
                    box(which="figure")
                    }
              else {plot(c(-1,1),predmat[c(1,3),sp],ylab=respnam,xlab=terms1[j],
                       type="b", xlim=c(-1.1,1.1),ylim=c(min(ax),max(ax)), axes=FALSE, 
                       col="red", lty=3, pch=pch[1], cex=cex, lwd=lwd, las=las)
                    box(which="figure")
                    lines(c(-1,1),predmat[c(2,4),sp],lwd=lwd)
                    points(c(-1,1),predmat[c(2,4),sp],pch=pch[2], cex=cex, lwd=lwd)}
         }
         if (show.alias) numact <- if (length(hilf)>0)
             text(0,max(ax),alnum[hilf],xpd=TRUE,cex=cex.lab)
   }
   }
   title(main, line=1.5, outer=TRUE, cex.main=cex.title*par("cex.main"))
   par(mfrow=omfrow,mar=omar,oma=ooma)
   rownames(predmat) <- c("-:-","+:-","-:+","+:+")
   if (show.alias) attr(predmat, "aliasgroups") <- 
        lapply(unique(alnum), function(obj) names(alnum)[alnum==obj])
   invisible(predmat)
}

colpick <- function(design, q, all=FALSE, 
                    select.catlg=catlg, estimable=NULL,
                    method="VF2", sort="natural", res3=FALSE, all0=FALSE, 
                    quiet=FALSE, firsthit=is.numeric(design)){
  ## design is a character string that identifies a catalogued design
  ##    or a length 1 catlg object
  ##    of the number of factors for a full factorial
  ## q is log2 of the block size
  ## all = TRUE requests a list of results
  ## select.catlg is a design catalogue of class catlg
  ## estimable is a character vector of estimable 2fis in letter notation, 
  ##    or a two row numeric matrix with factor numbers (each column indicates a clear 2fi)
  ## all0 = TRUE also outputs X matrices with zero columns
  ## for all=FALSE, the function returns a list with an X matrix 
  ## the clear2fis, 
  ## and possibly a mapping (if estimable is specified)
  ## for all=TRUE, the function returns a list of three or four lists (X matrices, estimable 2fis, profiles)
  ## if estimable is specified, the fourth list indicates a re-mapping with which the requirement can be accommodated
  catlg <- select.catlg
  if (!("catlg" %in% class(design) || is.character(design) || is.numeric(design)))
    stop("design must be the number of factors of a full factorial", 
         " or a character string that refers to an element of select.catlg", 
         " or of class catlg")
  if (is.numeric(design)){
    element <- list(list(nfac  = design,
                         nruns = 2^design,
                         res = Inf,
                         nclear.2fis = choose(design, 2),
                         clear.2fis = combn(design, 2),
                         gen = numeric(0), 
                         WLP = rep(0, 4)))
    class(element) <- c("catlg", "list")
  }
  if (is.character(design)){
    element <- catlg[design]    ## class catlg
  }
  if ("catlg" %in% class(design)) element <- design[1]       ## more than one element is ignored without warning
  N <- element[[1]]$nruns
  n <- element[[1]]$nfac
  gen <- element[[1]]$gen         ## Yates column numbers
  p <- length(gen)
  k <- n - p
  
  ## clear before blocking
  clear2fis <- clear.2fis(element)
  nclear2fis <- nclear.2fis(element)
  if (length(clear2fis[[1]])==0) 
    clear2fis <- character(0) 
  else{
    if (n <= 50) 
    clear2fis <- sapply(1:ncol(clear2fis[[1]]), function(obj) 
      paste(Letters[clear2fis[[1]][,obj]], collapse = ""))
    else
    clear2fis <- sapply(1:ncol(clear2fis[[1]]), function(obj) 
      paste("F",clear2fis[[1]][,obj], sep="", collapse = ":"))
    }
  
  if (!is.null(estimable)){
    if (!is.numeric(estimable) && !is.character(estimable))
      stop("estimable must be an integer-valued two row matrix",
           " or a character vector")
    if (is.numeric(estimable) && !is.matrix(estimable))
      stop("if numeric, estimable must be a matrix with two rows")
    if (is.numeric(estimable) && !nrow(estimable)==2)
      stop("if numeric, estimable must be a matrix with two rows")
    if (is.character(estimable)){
      colons <- grep(":", estimable)
      if (!(length(colons)==0 || length(colons)==length(estimable)))
          stop("All elements of estimable must have the same format")
      if (length(colons)==0)
      estimable <- sapply(estimable, 
                          function(obj) which(Letters %in% unlist(strsplit(obj, "", fixed=TRUE))))
      else   estimable <- sapply(estimable, 
                          function(obj) as.numeric(gsub( "F", "", unlist(strsplit(obj, ":", fixed=TRUE)) )))
      if (!is.matrix(estimable)) stop("invalid estimable")
     }
    #graph_requirement <- make_empty_graph(n = n, directed = FALSE)
    #graph_requirement <- add_edges(graph_requirement, estimable)
  } 
  Z <- t(sapply(gen, function(obj) rev(digitsBase(obj, 2, k))))
  div <- 2^q-1
  divbase <- 2^(0:(q-1))
  
  ## list of possible X column vectors
  Xcands <- lapply(1:div, function(obj) digitsBase(obj, 2, ndigits=q))
    
  success <- FALSE
  Xlist <- vector(mode="list")
  tablist <- vector(mode="list")
  clearlist <- vector(mode="list")
  if (!is.null(estimable)) maplist <- vector(mode="list")
  ## each row of picks (=instance produced by nxt) provides a possible XI matrix
  ## fix May 2023: for k < 2^(q-1), XI and hence X did not have full row rank
  ##      because of the choice of columns in poscands
  poscands <- lapply(1:k, function(obj){ 
    if (k >= 2^(q-1)){
      if (obj <= div) 
        return(1:obj)
      else return(1:div)
    }
      else{
        if (obj <= q) 
           return(2^(0:(obj-1)))
        else 
           return(c(2^(0:(q-1)), setdiff(1:obj, 2^(0:(obj-1)))[1:(obj-q)]))
    }
    }
  )
  nxt <- do.call(lazyExpandGrid, poscands)
  #picks <- as.matrix(expand.grid(poscands))
  #for storage reasons, avoid expand.grid
  nr <- prod(lengths(poscands))
  if (!quiet) message("checking up to ", nr, " matrices")
  
  ## initialize to a heavily balanced first matrix
  ## not yet using nxt()
  ## this makes full factorials without estimability requirements immediate
  ##     and may also be beneficial for fractions with easy structure
  ## nxt() is used at the end of the loop
  jetzt <- rep(1:div, (k%/%div+1))[1:k]
  i <- 0
  while (i <= nr){
    XI <- do.call(cbind, Xcands[jetzt])
    if (length(Z)>0){ 
      XII <- (XI%*%t(Z))%%2
      X <- cbind(XI, XII)
    }
    else X <- XI   ## full factorial
    ## rank check added May 29 2020
    rankdefect <- "try-error" %in% class(try(solve(tcrossprod(X)), silent=TRUE))
    if ((all(colSums(X) > 0) || all0) && !rankdefect){ 
      ## remove direct aliases from blocking
      if (length(clear2fis) > 0){
        ingroup <- character(0)
        for (i in 1:(n-1))
          for (j in (i+1):n)
            if (all(X[,i]==X[,j])) 
              ingroup <- c(ingroup,
                           ifelse(n<=50,
                                  paste0(Letters[i],Letters[j]),
                                  paste0("F",i,":F",j)))
            clearcur <- setdiff(clear2fis, ingroup)
      }
      else clearcur <- clear2fis
      if (is.null(estimable)){
        ### added April 4th 2020, return 
        ### blocking for full factorial as early as possible
        if (length(Z)==0 && !all)         
            return(list(X=X, clear2fis=clearcur))
        success <- TRUE 
        }
      else{
        map <- mapcalc.block(estimable, n, 
                             sapply(clearcur,
                                    function(obj) which(Letters %in% 
                                                          unlist(strsplit(obj, "", 
                                                        fixed=TRUE)))),
                             method=method, sort=sort)[[1]]
        if (!is.null(map)) {
          success <- TRUE
          if (success && !all && firsthit) 
              return(list(X=X, clear2fis=clearcur, map=map))
        }
        else{
          jetzt <- unlist(nxt())
          i <- i+1
          if (is.logical(jetzt)){
            if (!jetzt) break
          }
          next 
        }
      }  
    }
    else {
      jetzt <- unlist(nxt())
      i <- i+1
      if (is.logical(jetzt)){
        if (!jetzt) break
      }
      next
      }## 
    tab <- sort(table(apply(X, 2, paste0, collapse="")))
    tab <- unname(tab)
    ## optimal in terms of maximum number
    if (length(clearcur)== min(phimax(n,q), nclear2fis) && !all){
      if (is.null(estimable)) 
        return(list(X=X, clear2fis=clearcur))
      else  {## if firsthit=FALSE
        return(list(X=X, clear2fis=clearcur, map=map))
      }
    } else{
      if (is.null(estimable)){
        Xlist <- c(Xlist, list(X))
        tablist <- c(tablist, list(tab))
        clearlist <- c(clearlist, list(clearcur))
      }
      else{
        if (!is.null(map)){
          Xlist <- c(Xlist, list(X))
          tablist <- c(tablist, list(tab))
          clearlist <- c(clearlist, list(clearcur))
          maplist <- c(maplist, list(map))
        }
      }
    }
    ## obtain next entry (will be FALSE, if no further entry available)
    jetzt <- unlist(nxt())  ## should get the next entry
    i <- i+1
    if (is.logical(jetzt)){
      if (!jetzt) break
    }
  } ## end of loop
  if (!success) {
    if (!quiet) message("no suitable block arrangement was found")
    return(NULL)
  }
  if (all) {
    if (is.null(estimable)) 
      return(list(X_matrices=Xlist, clearlist=clearlist, profiles=tablist))
    else   ### firstcalc=FALSE, otherwise was already returned
      return(list(X_matrices=Xlist, clearlist=clearlist, profiles=tablist, maplist=maplist))
  } ## else
  
  lens <- lengths(clearlist)
  tablist <- tablist[lens==max(lens)]
  Xlist <- Xlist[lens==max(lens)]
  clearlist <- clearlist[lens==max(lens)]
  diffs <- sapply(tablist, function(obj) diff(range(obj)))
  pos <- which.min(diffs)
  if (!is.null(estimable))
    return(list(X=Xlist[[pos]], clear2fis=clearlist[[pos]], map=maplist[[pos]]))
  list(X=Xlist[[pos]], clear2fis=clearlist[[pos]])
}

contr.FrF2 <- function (n) 
{
    ## CAUTION: for more than 4 levels, levels need to be in correct order
    ## for the FrF2 structure to hold!
    if (length(n) <= 1) {
        if (is.numeric(n) && length(n) == 1 && n > 1) 
            levels <- 1:n
        else stop("invalid choice for n in contr.FrF2")
    }
    else levels <- n
    lenglev <- length(levels)
    if (!2^round(log2(lenglev))==lenglev) 
        stop("contr.FrF2 requires that the number of levels is a power of 2.")

    ## definition of contrast matrix
       if (lenglev==2) destxt <- "matrix(c(-1,1),ncol=1)"
       else {
       destxt <- "expand.grid(c(-1,1)"
       for (i in 2:round(log2(lenglev))) 
                destxt <- paste(destxt,",c(-1,1)",sep="")
       destxt <- paste("as.matrix(",destxt,"))",sep="")
       }
       cont <- eval(parse(text=destxt))
       cont <- sapply(Yates[1:(lenglev-1)], function(obj) (apply(cont[,obj,drop=FALSE],1,prod)))
       rownames(cont) <- levels
       colnames(cont) <- NULL
    cont
}

myscatterplot3d <- function (x, y = NULL, z = NULL, color = par("col"), pch = NULL, 
    main = NULL, sub = NULL, xlim = NULL, ylim = NULL, zlim = NULL, 
    xlab = NULL, ylab = NULL, zlab = NULL, scale.y = 1, angle = 40, 
    axis = TRUE, tick.marks = TRUE, label.tick.marks = TRUE, 
    x.ticklabs = NULL, y.ticklabs = NULL, z.ticklabs = NULL, zline=1, 
    y.margin.add = 0, grid = TRUE, box = TRUE, lab = par("lab"), 
    lab.z = mean(lab[1:2]), zadj = c(0.8,0.9), type = "p", highlight.3d = FALSE, 
    mar = c(5, 3, 4, 3) + 0.1, col.axis = par("col.axis"), col.grid = "grey", 
    col.lab = par("col.lab"), cex.symbols = par("cex"), cex.axis = par("cex.axis"), 
    cex.lab = 0.8 * par("cex.lab"), font.axis = par("font.axis"), 
    font.lab = par("font.lab"), lty.axis = par("lty"), lty.grid = par("lty"), 
    lty.hide = NULL, log = "", ...) 
{
    mem.par <- par(mar = mar)
    x.scal <- y.scal <- z.scal <- 1
    xlabel <- if (!missing(x)) 
        deparse(substitute(x))
    ylabel <- if (!missing(y)) 
        deparse(substitute(y))
    zlabel <- if (!missing(z)) 
        deparse(substitute(z))
    if (highlight.3d && !missing(color)) 
        warning("color is ignored when highlight.3d = TRUE")
    if (!is.null(d <- dim(x)) && (length(d) == 2) && (d[2] >= 
        4)) 
        color <- x[, 4]
    else if (is.list(x) && !is.null(x$color)) 
        color <- x$color
    xyz <- xyz.coords(x = x, y = y, z = z, xlab = xlabel, ylab = ylabel, 
        zlab = zlabel, log = log)
    if (is.null(xlab)) {
        xlab <- xyz$xlab
        if (is.null(xlab)) 
            xlab <- ""
    }
    if (is.null(ylab)) {
        ylab <- xyz$ylab
        if (is.null(ylab)) 
            ylab <- ""
    }
    if (is.null(zlab)) {
        zlab <- xyz$zlab
        if (is.null(zlab)) 
            zlab <- ""
    }
    if (length(color) == 1) 
        color <- rep(color, length(xyz$x))
    else if (length(color) != length(xyz$x)) 
        stop("length(color) ", "must be equal length(x) or 1")
    angle <- (angle%%360)/90
    yz.f <- scale.y * abs(if (angle < 1) angle else if (angle > 
        3) angle - 4 else 2 - angle)
    yx.f <- scale.y * (if (angle < 2) 
        1 - angle
    else angle - 3)
    if (angle > 2) {
        temp <- xyz$x
        xyz$x <- xyz$y
        xyz$y <- temp
        temp <- xlab
        xlab <- ylab
        ylab <- temp
        temp <- xlim
        xlim <- ylim
        ylim <- temp
    }
    angle.1 <- (1 < angle && angle < 2) || angle > 3
    angle.2 <- 1 <= angle && angle <= 3
    dat <- cbind(as.data.frame(xyz[c("x", "y", "z")]), col = color)
    if (!is.null(xlim)) {
        xlim <- range(xlim)
        dat <- dat[xlim[1] <= dat$x & dat$x <= xlim[2], , drop = FALSE]
    }
    if (!is.null(ylim)) {
        ylim <- range(ylim)
        dat <- dat[ylim[1] <= dat$y & dat$y <= ylim[2], , drop = FALSE]
    }
    if (!is.null(zlim)) {
        zlim <- range(zlim)
        dat <- dat[zlim[1] <= dat$z & dat$z <= zlim[2], , drop = FALSE]
    }
    n <- nrow(dat)
    if (n < 1) 
        stop("no data left within (x|y|z)lim")
    y.range <- range(dat$y[is.finite(dat$y)])
    if (type == "p" || type == "h") {
        y.ord <- rev(order(dat$y))
        dat <- dat[y.ord, ]
        if (length(pch) > 1) 
            if (length(pch) != length(y.ord)) 
                stop("length(pch) ", "must be equal length(x) or 1")
            else pch <- pch[y.ord]
        daty <- dat$y
        daty[!is.finite(daty)] <- mean(daty[is.finite(daty)])
        if (highlight.3d && !(all(diff(daty) == 0))) 
            dat$col <- rgb(seq(0, 1, length = n) * (y.range[2] - 
                daty)/diff(y.range), green = 0, blue = 0)
    }
    p.lab <- par("lab")
    y.range <- range(dat$y[is.finite(dat$y)], ylim)
    y.prty <- pretty(y.range, n = lab[2], 
        min.n = max(1, min(0.5 * lab[2], p.lab[2])))
    y.scal <- round(diff(y.prty[1:2]), digits = 12)
    y.add <- min(y.prty)
    dat$y <- (dat$y - y.add)/y.scal
    y.max <- (max(y.prty) - y.add)/y.scal
    if (!is.null(ylim)) 
        y.max <- max(y.max, ceiling((ylim[2] - y.add)/y.scal))
    if (angle > 2) 
        dat$y <- y.max - dat$y
    x.range <- range(dat$x[is.finite(dat$x)], xlim)
    x.prty <- pretty(x.range, n = lab[1], min.n = max(1, min(0.5 * 
        lab[1], p.lab[1])))
    x.scal <- round(diff(x.prty[1:2]), digits = 12)
    dat$x <- dat$x/x.scal
    x.range <- range(x.prty)/x.scal
    x.max <- ceiling(x.range[2])
    x.min <- floor(x.range[1])
    if (!is.null(xlim)) {
        x.max <- max(x.max, ceiling(xlim[2]/x.scal))
        x.min <- min(x.min, floor(xlim[1]/x.scal))
    }
    x.range <- range(x.min, x.max)
    z.range <- range(dat$z[is.finite(dat$z)], zlim)
    z.prty <- pretty(z.range, n = lab.z, min.n = max(1, min(0.5 * 
        lab.z, p.lab[2])))
    z.scal <- round(diff(z.prty[1:2]), digits = 12)
    dat$z <- dat$z/z.scal
    z.range <- range(z.prty)/z.scal
    z.max <- ceiling(z.range[2])
    z.min <- floor(z.range[1])
    if (!is.null(zlim)) {
        z.max <- max(z.max, ceiling(zlim[2]/z.scal))
        z.min <- min(z.min, floor(zlim[1]/z.scal))
    }
    z.range <- range(z.min, z.max)
    plot.new()
    if (angle.2) {
        x1 <- x.min + yx.f * y.max
        x2 <- x.max
    }
    else {
        x1 <- x.min
        x2 <- x.max + yx.f * y.max
    }
    plot.window(c(x1, x2), c(z.min, z.max + yz.f * y.max))
    temp <- strwidth(format(rev(y.prty))[1], cex = cex.lab/par("cex"))
    if (angle.2) 
        x1 <- x1 - temp - y.margin.add
    else x2 <- x2 + temp + y.margin.add
    plot.window(c(x1, x2), c(z.min, z.max + yz.f * y.max))
    if (angle > 2) 
        par(usr = par("usr")[c(2, 1, 3:4)])
    usr <- par("usr")
    title(main, sub, ...)
    if (grid) {
        i <- x.min:x.max
        segments(i, z.min, i + (yx.f * y.max), yz.f * y.max + 
            z.min, col = col.grid, lty = lty.grid)
        i <- 0:y.max
        segments(x.min + (i * yx.f), i * yz.f + z.min, x.max + 
            (i * yx.f), i * yz.f + z.min, col = col.grid, lty = lty.grid)
    }
    if (axis) {
        xx <- if (angle.2) 
            c(x.min, x.max)
        else c(x.max, x.min)
        if (tick.marks) {
            xtl <- (z.max - z.min) * (tcl <- -par("tcl"))/50
            ztl <- (x.max - x.min) * tcl/50
            mysegs <- function(x0, y0, x1, y1) segments(x0, y0, 
                x1, y1, col = col.axis, lty = lty.axis)
            i.y <- 0:y.max
            mysegs(yx.f * i.y - ztl + xx[1], yz.f * i.y + z.min, 
                yx.f * i.y + ztl + xx[1], yz.f * i.y + z.min)
            i.x <- x.min:x.max
            mysegs(i.x, -xtl + z.min, i.x, xtl + z.min)
            i.z <- z.min:z.max
            mysegs(-ztl + xx[2], i.z, ztl + xx[2], i.z)
            if (label.tick.marks) {
                las <- par("las")
                mytext <- function(labels, side, at, ...) mtext(text = labels, 
                  side = side, at = at, col = col.lab, 
                  cex = cex.lab, font = font.lab, ...)
                if (is.null(x.ticklabs)) 
                  x.ticklabs <- format(i.x * x.scal)
                mytext(x.ticklabs, side = 1, at = i.x, las=0, line=1)
                if (is.null(z.ticklabs)) 
                  z.ticklabs <- format(i.z * z.scal)
                mytext(z.ticklabs, side = 2, at = i.z, adj = if (0 < las && las < 
                  3) 
                  zadj[1]
                else zadj[2], las=1, line=zline)
                temp <- if (angle > 2) 
                  rev(i.y)
                else i.y
                if (is.null(y.ticklabs)) 
                  y.ticklabs <- format(y.prty)
                else if (angle > 2) 
                  y.ticklabs <- rev(y.ticklabs)
                text(i.y * yx.f + xx[1], i.y * yz.f + z.min, 
                  y.ticklabs, pos = if (angle.1) 
                    2
                  else 4, offset = 1.8, col = col.lab, cex = cex.lab/par("cex"), 
                  font = font.lab, las=1)
            }
        }
        mytext2 <- function(lab, side, line, at, las = 0) mtext(lab, side = side, 
            line = line, at = at, col = col.lab, cex = cex.axis, 
            font = font.axis, las = las)
        lines(c(x.min, x.max), c(z.min, z.min), col = col.axis, 
            lty = lty.axis)
        mytext2(xlab, 1, line = 0, at = mean(x.range))
        lines(xx[1] + c(0, y.max * yx.f), c(z.min, y.max * yz.f + 
            z.min), col = col.axis, lty = lty.axis)
        mytext2(ylab, if (angle.1) 
            2
        else 4, line = -3.5, at = z.min + y.max * yz.f/2,las=1)
        lines(xx[c(2, 2)], c(z.min, z.max), col = col.axis, lty = lty.axis)
        mytext2(zlab, if (angle.1) 
            4
        else 2, line = -0.5, at = mean(z.range),las=1)
        if (box) {
            if (is.null(lty.hide)) 
                lty.hide <- lty.axis
            temp <- yx.f * y.max
            temp1 <- yz.f * y.max
            lines(c(x.min + temp, x.max + temp), c(z.min + temp1, 
                z.min + temp1), col = col.axis, lty = lty.hide)
            lines(c(x.min + temp, x.max + temp), c(temp1 + z.max, 
                temp1 + z.max), col = col.axis, lty = lty.axis)
            temp <- c(0, y.max * yx.f)
            temp1 <- c(0, y.max * yz.f)
            lines(temp + xx[2], temp1 + z.min, col = col.axis, 
                lty = lty.hide)
            lines(temp + x.min, temp1 + z.max, col = col.axis, 
                lty = lty.axis)
            temp <- yx.f * y.max
            temp1 <- yz.f * y.max
            lines(c(temp + x.min, temp + x.min), c(z.min + temp1, 
                z.max + temp1), col = col.axis, lty = if (!angle.2) 
                lty.hide
            else lty.axis)
            lines(c(x.max + temp, x.max + temp), c(z.min + temp1, 
                z.max + temp1), col = col.axis, lty = if (angle.2) 
                lty.hide
            else lty.axis)
        }
    }
    x <- dat$x + (dat$y * yx.f)
    z <- dat$z + (dat$y * yz.f)
    col <- as.character(dat$col)
    if (type == "h") {
        z2 <- dat$y * yz.f + z.min
        segments(x, z, x, z2, col = col, cex = cex.symbols, ...)
        points(x, z, type = "p", col = col, pch = pch, cex = cex.symbols, 
            ...)
    }
    else points(x, z, type = type, col = col, pch = pch, cex = cex.symbols, 
        ...)
    if (axis && box) {
        lines(c(x.min, x.max), c(z.max, z.max), col = col.axis, 
            lty = lty.axis)
        lines(c(0, y.max * yx.f) + x.max, c(0, y.max * yz.f) + 
            z.max, col = col.axis, lty = lty.axis)
        lines(xx[c(1, 1)], c(z.min, z.max), col = col.axis, lty = lty.axis)
    }
    ob <- ls()
    rm(list = ob[!ob %in% c("mar", "usr", "x.scal", "y.scal", 
        "z.scal", "yx.f", "yz.f", "y.add", "z.min", "z.max", 
        "x.min", "x.max", "y.max", "x.prty", "y.prty", "z.prty")])
    rm(ob)
    invisible(list(xyz.convert = function(x, y = NULL, z = NULL) {
        xyz <- xyz.coords(x, y, z)
        y <- (xyz$y - y.add)/y.scal
        return(list(x = xyz$x/x.scal + yx.f * y, y = xyz$z/z.scal + 
            yz.f * y))
    }, points3d = function(x, y = NULL, z = NULL, type = "p", 
        ...) {
        xyz <- xyz.coords(x, y, z)
        y2 <- (xyz$y - y.add)/y.scal
        x <- xyz$x/x.scal + yx.f * y2
        y <- xyz$z/z.scal + yz.f * y2
        mem.par <- par(mar = mar, usr = usr)
        on.exit(par(mem.par))
        if (type == "h") {
            y2 <- z.min + yz.f * y2
            segments(x, y, x, y2, ...)
            points(x, y, type = "p", ...)
        } else points(x, y, type = type, ...)
    }, plane3d = function(Intercept, x.coef = NULL, y.coef = NULL, 
        lty = "dashed", lty.box = NULL, ...) {
        if (!is.null(coef(Intercept))) Intercept <- coef(Intercept)
        if (is.null(lty.box)) lty.box <- lty
        if (is.null(x.coef) && length(Intercept) == 3) {
            x.coef <- Intercept[2]
            y.coef <- Intercept[3]
            Intercept <- Intercept[1]
        }
        mem.par <- par(mar = mar, usr = usr)
        on.exit(par(mem.par))
        x <- x.min:x.max
        ltya <- c(lty.box, rep(lty, length(x) - 2), lty.box)
        x.coef <- x.coef * x.scal
        z1 <- (Intercept + x * x.coef + y.add * y.coef)/z.scal
        z2 <- (Intercept + x * x.coef + (y.max * y.scal + y.add) * 
            y.coef)/z.scal
        segments(x, z1, x + y.max * yx.f, z2 + yz.f * y.max, 
            lty = ltya, ...)
        y <- 0:y.max
        ltya <- c(lty.box, rep(lty, length(y) - 2), lty.box)
        y.coef <- (y * y.scal + y.add) * y.coef
        z1 <- (Intercept + x.min * x.coef + y.coef)/z.scal
        z2 <- (Intercept + x.max * x.coef + y.coef)/z.scal
        segments(x.min + y * yx.f, z1 + y * yz.f, x.max + y * 
            yx.f, z2 + y * yz.f, lty = ltya, ...)
    }, box3d = function(...) {
        mem.par <- par(mar = mar, usr = usr)
        on.exit(par(mem.par))
        lines(c(x.min, x.max), c(z.max, z.max), ...)
        lines(c(0, y.max * yx.f) + x.max, c(0, y.max * yz.f) + 
            z.max, ...)
        lines(c(0, y.max * yx.f) + x.min, c(0, y.max * yz.f) + 
            z.max, ...)
        lines(c(x.max, x.max), c(z.min, z.max), ...)
        lines(c(x.min, x.min), c(z.min, z.max), ...)
        lines(c(x.min, x.max), c(z.min, z.min), ...)
    }))
}

add.center <- function(design, ncenter, distribute=NULL, ...){
    if (!"design" %in% class(design)) stop("design must be of class design")
    di <- design.info(design)
    if (missing(ncenter)) stop("ncenter must be specified")
    if (!(substr(di$type,1,4)=="FrF2" | substr(di$type,1,2)=="pb" | (substr(di$type,1,14)=="full factorial" & all(di$nlevels==2))))
       stop("center points only available for FrF2 and pb type designs")
    if (di$type=="FrF2.splitplot") 
       stop("currently, center points for splitplot designs are not supported")
    if (di$type=="FrF2.param") 
       stop("currently, center points for long version parameter designs are not supported")
    if (length(grep("center",di$type))>0)
       stop("design has center points already")
    if (!is.numeric(ncenter)) stop("ncenter must be a number")
    if (!length(ncenter)==1) stop("ncenter must be a number")
    if (!ncenter==floor(ncenter)) stop("ncenter must be an integer number")
    if (is.null(distribute)){
      if (ncenter==0 || !di$randomize) distribute <- 1
         else distribute <- min(ncenter, 3)
         if (di$type=="FrF2.blocked") distribute <- min(distribute, di$blocksize+1)
         }
    if (!is.numeric(distribute)) stop("distribute must be a number")
    if (!distribute==floor(distribute)) stop("distribute must be an integer number")
    if (distribute < 1 || distribute > max(1, min(di$nruns+1, ncenter)))
       stop("distribute must be at least 1 and at most min(ncenter, nruns+1 of the design)")
    if (di$type=="FrF2.blocked") 
       if (distribute>di$blocksize+1) stop("distribute must not exceed the block size by more than one")
    if (di$randomize && distribute==1 && ncenter > 1) 
       warning("running all center point runs together is usually not a good idea.")
    
    if (any(is.na(sapply(factor.names(design),"is.numeric"))))
       stop("Center points are implemented for experiments with all factors quantitative only.")
       

    design <- qua.design(design, quantitative="all")
    di <- design.info(design)
    clevels <- sapply(factor.names(design), "mean")
    di$ncube <- di$nruns
    di$ncenter <- ncenter
    if (!is.null(di$blocksize)) di$nruns <- di$nruns + di$nblocks*ncenter else
    di$nruns <- di$nruns+ncenter
    
    desnum <- desnum(design)
    ro <- run.order(design)
    if (!all(ro$run.no == sort(ro$run.no)) & (di$replications>0 | !is.null(di$blocksize))) 
        stop("center points cannot be added to designs that are not in original order")
    hilf <- undesign(design)

    ### 15 Feb 2013: removed restriction to bbreps==1 (as now always factor)
    if (!is.null(di$blocksize)) block.contrasts <- contrasts(hilf[,di$block.name])
    
    ## positions for adding center points (added after these positions)
    ## !!! blocks, replications and repeated measurements to be treated correctly
    if (!is.null(di$blocksize)){
         ## distribute within blocks
        if (distribute==1)
            addpos <- di$blocksize 
        else
            addpos <- c(0,round((1:(distribute-1))*di$blocksize/(distribute-1)))
    }
    else{
        if (distribute==1) 
             addpos <- di$ncube 
        else
          addpos <- c(0,round((1:(distribute-1))*di$ncube/(distribute-1)))
    }
    ## determine numbers of center points to be added at each position
        nrest <- ncenter%%distribute
        if (nrest==0) nadd <- rep(round(ncenter/distribute),distribute)
        if (nrest==1) nadd <- c(1,rep(0,distribute-1))+rep(floor(ncenter/distribute),distribute)
        if (nrest==2) nadd <- c(1,rep(0,distribute-2),1)+rep(floor(ncenter/distribute),distribute)
        if (nrest>2) nadd <- rep(floor(ncenter/distribute),distribute) + sample(c(rep(1,nrest),rep(0,distribute-nrest)))
        ## repeated measurements also for center points (assuming they are done for measurement accuracy)
        if (di$repeat.only) if (!is.null(di$blocksize)) nadd <- nadd*di$wbreps else nadd <- nadd*di$replications

    ## split vector for design and run order data frame
        if (!is.null(di$blocksize)){
          if (di$wbreps==1 | di$repeat.only)
             together <- paste(rep(1:(di$nblocks*di$bbreps),each=di$blocksize*di$wbreps),
                          rep(cut(1:di$blocksize,unique(c(0,addpos))),each=di$wbreps,di$nblocks*di$bbreps),sep=".")
          else
             together <- paste(rep(1:(di$nblocks*di$bbreps*di$wbreps),each=di$blocksize),
                          rep(cut(1:di$blocksize,unique(c(0,addpos))),di$nblocks*di$bbreps*di$wbreps),sep=".")
             }
        else {if (di$replications==1 | (di$replications > 1 & di$repeat.only)) 
                together <- cut(rep(1:di$ncube,each=di$replications), unique(c(0,addpos)))
              else together <- paste(rep(1:di$replications,each=di$ncube),rep(cut(1:di$ncube,unique(c(0,addpos))),di$replications),sep=".")
                }
        ## the following command prevents split from reordering the data
        together <- factor(together, levels=unique(together))
   ## split the data frame
        getrennt <- split(hilf,together)
          ## added 15 Feb 2013
          ## obtaine levels for the new factors including the center points
          rostdlev <- levels(ro$run.no.in.std.order)
          rostdrplev <- levels(ro$run.no.std.rp)
          levgleich <- identical(ro$run.no.in.std.order,ro$run.no.std.rp)
          
          ## extend levels to include all center point related ones
          ## for run.no.in.std.order
          if (all(sapply(rostdlev, function(obj) length(grep(".", obj, fixed=TRUE)) > 0 ) )){
              punkt1 <- sapply(rostdlev, function(obj) regexpr(".",obj,fixed=TRUE))
              if (all(sapply(substr(rostdlev,punkt1+1,999), function(obj) length(grep(".", obj, fixed=TRUE))>0)))
              punkt2 <- sapply(substr(rostdlev, punkt1+1, 999), function(obj) regexpr(".", obj, fixed=TRUE))+punkt1
              else punkt2 <- 1000  ## repeated measurements or replications only
              if (punkt2[1] < 1000) rostdlev <- c(unique(paste("0", substr(rostdlev, punkt1+1, punkt2-1), "0", sep=".")),
                  rostdlev) else rostdlev <- c(unique(paste("0", substr(rostdlev, punkt1+1, punkt2-1), sep=".")),
                  rostdlev)         
          } 
          else rostdlev <- c("0", rostdlev)
          ## for run.no.std.rp
          if (levgleich) rostdrplev <- rostdlev else{
          if (all(sapply(rostdrplev, function(obj) length(grep(".", obj, fixed=TRUE))>0))){
              punkt1 <- sapply(rostdrplev, function(obj) regexpr(".",obj,fixed=TRUE))
              if (all(sapply(substr(rostdrplev,punkt1+1,999), function(obj) length(grep(".", obj, fixed=TRUE))>0))){
              punkt2 <- sapply(substr(rostdrplev, punkt1+1, 999), function(obj) regexpr(".", obj, fixed=TRUE))+punkt1
              punkt3 <- sapply(substr(rostdrplev, punkt2+1, 999), function(obj) regexpr(".", obj, fixed=TRUE))+punkt2
              if (all(sapply(substr(rostdrplev,punkt3+1,999), function(obj) length(grep(".", obj, fixed=TRUE))>0)))
                 punkt4 <- sapply(substr(rostdrplev, punkt3+1, 999), function(obj) regexpr(".", obj, fixed=TRUE))+punkt3 else
                 punkt4 <- 1000
              if (punkt4[1] == 1000) rostdrplev <- c(unique(paste("0", 
                   substr(rostdrplev, punkt1+1, punkt2-1), "0", 
                   substr(rostdrplev, punkt3+1, punkt4-1), sep=".")), rostdrplev)
                 else {if (di$repeat.only) rostdrplev <- c(unique(paste("0", 
                   substr(rostdrplev, punkt1+1, punkt2-1), "0", 
                   substr(rostdrplev, punkt3+1, punkt4-1), 1:di$wbreps, sep=".")), rostdrplev)
                   else rostdrplev <- c(unique(paste("0", 
                   substr(rostdrplev, punkt1+1, punkt2-1), "0", 
                   substr(rostdrplev, punkt3+1, punkt4-1), substr(rostdrplev, punkt4+1, 999), sep=".")), rostdrplev)
              }
              }
              else {punkt2 <- 1000 ## repeated measurements or replications only
              rostdrplev <- c(unique(paste("0", substr(rostdrplev, punkt1+1, punkt2-1), 
                     sep=".")), rostdrplev)
              }            
          } 
          else rostdrplev <- c("0", rostdrplev)
          }
          ## end of added 15 Feb 2013
        ro.getrennt <- split(ro,together)
        ## wird das ueberhaupt gebraucht???
        ro$run.no.in.std.order <- as.character(ro$run.no.in.std.order)
        ro$run.no.std.rp <- as.character(ro$run.no.std.rp)
        
        blockid <- rep("1", length(getrennt))

        von <- cumsum(c(1,lapply(getrennt, "nrow")))[-(length(getrennt)+1)]
        bis <- cumsum(lapply(getrennt, "nrow"))
        
        ## added 15 Feb 2013
        std <- lapply(ro.getrennt, function(obj) {
           hilf <- t(as.matrix(sapply(obj$run.no.in.std.order, function(obj2) 
                      unlist(strsplit(as.character(obj2),".",fixed=TRUE)))))
#           if (nrow(hilf)==1) hilf <- t(hilf) ## commented out April 2020
                                               ## caused trouble with blocked case
           hilf
           })
        stdrp <- lapply(ro.getrennt, function(obj) {
            hilf <- t(as.matrix(sapply(obj$run.no.std.rp, function(obj2) unlist(strsplit(as.character(obj2),".",fixed=TRUE)))))
            if (nrow(hilf)==1) hilf <- t(hilf)
            hilf
            })
        ## get rid of rp-entry
        if (is.null(di$blocksize) & di$replications > 1 & di$repeat.only) 
            stdrp <- lapply(stdrp, function(obj) obj[,1:(ncol(obj)-1)])   
        if (!is.null(di$blocksize)){ 
            blockid <- sapply(getrennt, function(obj) as.character(obj[1,di$block.name]))
            ## get rid of rp-entry in stdrp for blocked designs
            if (di$wbreps > 1 & di$repeat.only) 
                stdrp <- lapply(stdrp, function(obj) obj[,1:(ncol(obj)-1)])
            }
        ## end of added 15 Feb 2013
        if (!is.null(di$blocksize)) 
            blockid <- sapply(getrennt, function(obj) as.character(obj[1, 
               di$block.name]))
        if (di$replications > 1 & is.null(di$blocksize) & !di$repeat.only){ 
            blockid <- rep(1:di$replications, each=max(1,distribute-1))
            }
        if (!is.null(di$blocksize)) if (di$wbreps > 1 & !di$repeat.only){
            blockid <- paste(blockid, rep(1:di$wbreps, 
                    each=max(1,distribute-1), 
                    times=di$nblocks*di$bbreps),sep=".")
            }
    ## create center point data frames to be interleaved
    #    if (!is.null(di$nblocks)) no.center.groups <- distribute*di$nblocks*di$bbreps else 
    #        no.center.groups <- distribute*di$replications
        centers <- vector("list")
        ros <- lapply(1:length(std), function(obj) vector("list"))

        ## centers and ros for WITHIN each block
        more <- setdiff(colnames(hilf), c(di$block.name, names(di$factor.names)))
            ## columns other than design factors, e.g. responses
      if (ncenter>0){
        for (i in 1:distribute){
              cnext <- data.frame(matrix(clevels, ncol=di$nfactors, nrow=nadd[i], 
                                                 byrow=TRUE, dimnames=list(NULL, names(di$factor.names))))
              if (length(more)>0) cnext <- cbind(cnext, matrix(NA,nrow=nadd[i],ncol=length(more),dimnames=list(NULL,more)))
              centers <- c(centers, list(cnext))
              
              ## simultaneous creation of inserts for all blocks via lapply
              if (!is.null(di$blocksize)){
                 stdnext <- lapply(1:length(std), function(obj) rep(paste(c(0,std[[obj]][1,2],0),collapse="."), nadd[i]))
                 stdrpnext <- stdnext
                 if (di$bbreps>1 | (di$wbreps>1 & !di$repeat.only)) stdrpnext <- 
                       mapply(paste, stdrpnext, lapply(stdrp, function(obj) paste(obj[1,4], collapse=".")), 
                       sep=".", SIMPLIFY=FALSE)
                 if (di$bbreps>1 & di$wbreps>1 & !di$repeat.only) stdrpnext <- 
                       mapply(paste, stdrpnext, lapply(stdrp, function(obj) paste(obj[1,5], collapse=".")), 
                       sep=".", SIMPLIFY=FALSE)
                 if (di$wbreps>1 & di$repeat.only) 
                       stdrpnext <- lapply(stdrpnext, function(obj) paste(obj, rep(1:di$wbreps, round(nadd[i]/di$wbreps)),sep="."))
                 }
              else {
                stdnext <- stdrpnext <- lapply(1:length(std), function(obj) rep(0, nadd[i]) )
                if (di$replications > 1){ 
                if (!di$repeat.only) 
                  stdrpnext <- mapply(paste, stdrpnext, lapply(stdrp, function(obj) obj[1,2]),sep=".", SIMPLIFY=FALSE)
                  else stdrpnext <- lapply(stdrpnext, function(obj) paste(obj, rep(1:di$replications,round(nadd[i]/di$replications)),sep="."))
                }
                }
                stdnext <- lapply(stdnext, function(obj) factor(obj, levels=rostdlev))
                stdrpnext <- lapply(stdrpnext, function(obj) factor(obj, levels=rostdrplev))
                bothnext <- mapply(function(...) list(data.frame(...)), stdnext, lapply(1:length(std), function(obj) rep(0,nadd[i])),
                    stdrpnext, SIMPLIFY=FALSE)
                bothnext <- lapply(bothnext, function(obj){colnames(obj[[1]]) <- c("run.no.in.std.order","run.no","run.no.std.rp"); obj})
                ros <- mapply(c,ros, bothnext, SIMPLIFY=FALSE)
              }
    ## interleave
         ## first center point entry of first (replication) block
          if (!is.null(di$blocksize)) 
              new <- cbind(matrix(hilf[1,di$block.name],nrow=nadd[1],ncol=1, dimnames=list(NULL, di$block.name)),centers[[1]])
          else new <- centers[[1]]
          ronew <- ros[[1]][[1]] 
          blockids <- unique(blockid)
          if (distribute==1){
            ## center points appended at end of each (replication) block
            if (length(blockids)==1){
               ## one (replication) block only
               new <- rbind(hilf, new)
               ronew <- rbind(ro, ronew, stringsAsFactors=TRUE)
               }
            else{
               ## more than one (replication) block
               ## as distribute equals 1, blockids == blockid
               new <- rbind(getrennt[[1]], new)
               ronew <- rbind(ro.getrennt[[1]], ronew, stringsAsFactors=TRUE)
               for (i in 2:length(blockids)){
                  if (!is.null(di$blocksize)) 
                  new <- rbind(new, getrennt[[i]], 
                      cbind(matrix(getrennt[[i]][1,di$block.name],nrow=nadd[1],ncol=1, dimnames=list(NULL, di$block.name)),
                            centers[[1]]))
                  else new <- rbind(new, getrennt[[i]], centers[[1]])
                  ronew <- rbind(ronew, ro.getrennt[[i]], ros[[i]][[1]], stringsAsFactors=TRUE)
               }
              }
            }
            else{
              ### distribute > 1
              for (i in 1:length(blockids)){
              iblock <- which(blockid==blockids[i])
              if (i>1){ 
                  ## append first center points series of new block
                  ## for first block done already
                  if (!is.null(di$blocksize)) 
                  new <- rbind(new, 
                      cbind(matrix(getrennt[[iblock[1]]][1,di$block.name],nrow=nadd[1],ncol=1, dimnames=list(NULL, di$block.name)),centers[[1]]))
                  else new <- rbind(new, centers[[1]])
                  ronew <- rbind(ronew, ros[[iblock[1]]][[1]], stringsAsFactors=TRUE)
                  }
              for (j in 1:length(iblock)){
                  ## append next data with subsequent center points for the current (replication) block 
                  if (!is.null(di$blocksize)) 
                  new <- rbind(new, getrennt[[iblock[j]]],
                      cbind(matrix(getrennt[[iblock[j]]][1,di$block.name],nrow=nadd[j+1],ncol=1, dimnames=list(NULL, di$block.name)),centers[[j+1]]))
                  else new <- rbind(new, getrennt[[iblock[j]]], centers[[j+1]])
                  ronew <- rbind(ronew, ro.getrennt[[iblock[j]]], ros[[iblock[1]]][[j+1]], stringsAsFactors=TRUE)
                  }
              }
              }
        if (!is.null(di$blocksize)){
          new[[di$block.name]] <- factor(new[[di$block.name]], levels(design[[di$block.name]]))
          if (exists("block.contrasts")) contrasts(new[,di$block.name]) <- block.contrasts
          desnum <- model.matrix(formula(paste("~",paste(c(di$block.name,names(di$factor.names)),collapse="+"))),data=new)[,-1]
        }
        else
        desnum <- model.matrix(formula(paste("~",paste(names(di$factor.names),collapse="+"))),data=new)[,-1]
        if (length(setdiff(colnames(new),c(di$block.name, names(di$factor.names))))>0){ 
               anhaeng <- as.matrix(new[,setdiff(colnames(new),c(di$block.name, names(di$factor.names))),drop=FALSE])
               storage.mode(anhaeng) <- "numeric"
               desnum <- cbind(desnum, anhaeng)
               }
     
     rownames(new) <- rownames(desnum) <- ronew$run.no <- 1:nrow(new)
     class(new) <- c("design","data.frame")
     desnum(new) <- desnum
     
     run.order(new) <- ronew
     di$type <- paste(di$type,"center",sep=".")
     ## added in order to support usage of function code.design and steepest ascent analysis
     di$coding <- make.formulas(paste("x", 1:length(di$factor.names), sep=""), di$factor.names)
     design.info(new) <- di
     }
     else {
     ### ??? was wird noch benoetigt fuer ccd.augment???
        new <- design
        ronew <- run.order(design)
        if (!any(is.na(as.numeric(as.character(ronew$run.no.in.std.order)))))
            ronew$run.no.in.std.order <- as.numeric(as.character(ronew$run.no.in.std.order))
        design.info(new) <- c(design.info(new), ncenter=0, ncube=nrow(new))
        run.order(new) <- ronew
        }
     new
}

## utility functions in support of valid and readable 
## alias info in the summary for block and splitplot designs
## old versions (before FrF2 2.0) at the top
## new versions at the bottom of the file

recalc.alias.block <- function(dia,leg){
## dia is the aliased element of a blocked or split plot design without the legend element
## leg is the separate legend element
    hilf <- lapply(dia, function(obj) strsplit(obj, ":", fixed=TRUE))
    leg <- lapply(leg, function(obj) unlist(strsplit(obj, "=", fixed=TRUE)))
    leg.c <- sapply(leg, function(obj) obj[1])
    leg.n <- sapply(leg, function(obj) obj[2])
    hilf <- lapply(hilf, function(obj) lapply(obj, function(obj2) sort(leg.c[which(leg.n %in% obj2)])))
    hilf <- lapply(hilf, function(obj) obj[ord(data.frame(sapply(obj, length)))])
    unlist(sort(unlist(sapply(hilf, function(obj) {
        ## vector of elements to be equated
        hilf <- paste(sapply(obj, function(obj2) paste(obj2, collapse="")), collapse="=")
        hilf[ord(data.frame(nchar(hilf), hilf))]
        }
        ))))
}

struc.aliased <- function(struc, nk, order){
        ## nk distinguishes Letters (nk<=50) or F1, F2, ...
        struc <- sapply(struc, function(obj) {
            if (length(grep("^-", obj)) == 1) {
                obj <- gsub("-", "~", obj)
                obj <- gsub("=", "=-", obj)
                obj <- gsub("=-~", "=", obj)
                obj <- gsub("~", "", obj)
            }
            obj
        })
        names(struc) <- NULL
        if (nk <= 50) {
            wme <- grep("^[[:alpha:]]=[[:alpha:][:punct:]]*", 
                struc)
            wme2 <- grep("^[[:alpha:]]{2}=[[:alpha:][:punct:]]*", 
                struc)
            if (order == 3) 
                wme3 <- grep("^[[:alpha:]]{3}=[[:alpha:][:punct:]]*", 
                  struc)
        }
        else {
            wme <- grep("^F[[:digit:]]+=F[[:digit:][:punct:]]*", 
                struc)
            wme2 <- grep("^F[[:digit:]]+:F[[:digit:]]+=F[[:digit:][:punct:]]*", 
                struc)
            if (order == 3) 
                wme3 <- grep("^F[[:digit:]]+:F[[:digit:]]+:F[[:digit:]]+=F[[:digit:][:punct:]]*", 
                  struc)
        }
        if (order == 2) 
            aus <- list(main = sort(struc[wme]), fi2 = sort(struc[wme2]))
        else aus <- list(main = sort(struc[wme]), fi2 = sort(struc[wme2]), 
            fi3 = sort(struc[wme3]))
    aus
}

recalc.alias.block.new <- function (dia, leg=NULL) 
{
  ## dia was in factor names before aliases was replaced by blockfull
  ## leg is not needed here any more
  ## blockfull so far only treats up to 50 factors
  ## 30 July 2019
  ## April 2020: restriction to 50 factors removed;
  ##             return with colons, remove colons later
  ##             otherwise, struc.aliased.new does not work
  if (is.list(dia)){ 
    return(unname(sapply(dia, function(obj){
      #obj <- gsub(":","", obj)
      paste(obj, collapse="=")
    })))
  }
  else return(dia) #return(gsub(":", "", dia))
}

struc.aliased.new <- function (struc, nk, order) 
{
  ## struc is the outcome of recalc.alias.block.new
  if (nk <= 50) {
    struc <- unname(sapply(struc, function(obj) gsub(":", "", obj)))
    wme <- grep("^[[:alpha:]]=[[:alpha:][:punct:]]*", 
                struc)
    wme2 <- grep("^[[:alpha:]]{2}=[[:alpha:][:punct:]]*", 
                 struc)
    if (order == 3) 
      wme3 <- grep("^[[:alpha:]]{3}=[[:alpha:][:punct:]]*", 
                   struc)
  }
  else {
    wme <- grep("^F[[:digit:]]+=F[[:digit:][:punct:]]*", 
                struc)
    wme2 <- grep("^F[[:digit:]]+:F[[:digit:]]+=F[[:digit:][:punct:]]*", 
                 struc)
    if (order == 3) 
      wme3 <- grep("^F[[:digit:]]+:F[[:digit:]]+:F[[:digit:]]+=F[[:digit:][:punct:]]*", 
                   struc)
  }
  if (order == 2) 
    aus <- list(main = sort(struc[wme]), fi2 = sort(struc[wme2]))
  else aus <- list(main = sort(struc[wme]), fi2 = sort(struc[wme2]), 
                   fi3 = sort(struc[wme3]))
  ## remove ':'
  #if (nk <= 50) aus <- lapply(aus, function(obj) gsub(":","",obj))
  aus
}

BsProb.design <- function(design, mFac = NULL, response=NULL, select=NULL, mInt = 2, p = 0.25, g = 2,
    ng = 1, nMod = 10){
 ## accessor function for BsProb from package BsMD
    if (!requireNamespace("BsMD", quiet=TRUE)) stop("Package BsMD is not available. Please install it before using this function.")
    if (!"design" %in% class(design)) stop("This function is for class design objects only.")
    di <- design.info(design)
    if (!(length(grep("FrF2",di$type))>0 |
           length(grep("pb",di$type))>0)) {
           if (!(di$type=="full factorial" & all(di$nlevels==2)))
        stop("The design obj must be of a type containing FrF2 or pb.")
        }
   if (is.null(di$response.names)) stop("The design must have at least one response.")
   if (is.null(mFac)) mFac <- di$nfactors   ## modified later, if select is used
    
   if (!is.null(select)){
       ## take care of selecting only part of the factors
       if (!is.numeric(select)) stop("select must be numeric")
       if (!all(floor(select)==select)) stop("select must contain integer numbers")
       if (any(select<1 | select>di$nfactors)) stop("select must contain numbers betweeen 1 and ", di$nfactors, " only")
       select <- unique(select)
       if (length(select)<2) stop("at least 2 factors must be selected")
       faclab <- names(di$factor.names)[select]
       if (mFac>length(select)) mFac <- length(select)
       }
    else faclab <- names(di$factor.names)
    
    X <- desnum(design)[,faclab]
    if (!is.null(response)){
      if (!response %in% di$response.names)
        stop("Requested response is not a response variable in design.")
        }
    else response <- di$response.names[1] ## use first response variable per default
    y <- unlist(design[,response])
    
    BsMD::BsProb(X, y, mFac=mFac, mInt=mInt, p=p, g=g, ng=ng, nMod=nMod)
}

require(FrF2)

summary(FrF2(16,7),brief=TRUE)
summary(FrF2(32,7,blocks=4),brief=TRUE)
summary(FrF2(32,9,blocks=4,alias.block.2fis=TRUE),brief=TRUE)
summary(FrF2(32,9,blocks=c("AB","AC"),alias.block.2fis=TRUE),brief=TRUE)
summary(FrF2(64,7,blocks=4),brief=TRUE)
summary(plan1 <- FrF2(32,7,estimable=c("AB","CD")),brief=TRUE)
summary(plan2 <- FrF2(nruns= 64 ,nfactors= 10 , estimable= c( "AB","AC","AD","AE","AF",
       "AG","AH","AJ","AK","BC","BD","BE","BF","BG","BH","BJ","BK" ) ), brief=TRUE)

C1 <- compromise(10, 1:2, msg=FALSE)
summary(plan3 <- FrF2(nruns= 64 ,nfactors= 10 , 
    estimable= C1$requirement, perms=C1$perms.full, clear=FALSE ), brief=TRUE)

### blockpick.big, a design that uses map (old version)
plan <- FrF2(64,15,blocks=16,alias.block.2fi=TRUE,block.old=TRUE)
summary(plan,brief=TRUE)

### various split-plot setups
planfull <-FrF2(32,5,WPs=4,nfac.WP=2,factor.names=Letters[21:25])
summary(planfull, brief=TRUE)
design.info(planfull)[-which(names(design.info(planfull))=="FrF2.version")]
try(planfull <-FrF2(32,5,WPs=4,nfac.WP=3,factor.names=Letters[21:25]))

plan0 <-FrF2(32,7,gen=c(6,15),WPs=4,WPfacs=c("B","C","F"),factor.names=Letters[19:25])
summary(plan0,brief=TRUE)
design.info(plan0)[-which(names(design.info(plan0))=="FrF2.version")]

plan <-FrF2(32,7,WPs=4,nfac.WP=3,factor.names=Letters[19:25])
summary(plan, brief=TRUE)
generators(plan)
design.info(plan)[-which(names(design.info(plan))=="FrF2.version")]

plan2 <- FrF2(32,7,gen=c(6,15),WPs=4,nfac.WP=3,WPfacs=c(2,3,6),factor.names=Letters[19:25])
summary(plan2, brief=TRUE)
generators(plan2)
design.info(plan2)[-which(names(design.info(plan2))=="FrF2.version")]

`check` <-
function(obj){
     check<-TRUE
     ## check whether completely aliased only
     ## works for numeric -1 and 1 coded data only
     mm <- model.matrix(remodel(obj)$model)
     if (any(colnames(mm)=="(Intercept)")) 
             mm <- mm[,-which(colnames(mm)=="(Intercept)")]
     if (!all(round(t(mm)%*%mm, 10) %in% c(0,nrow(mm),-nrow(mm)))) check <- FALSE 
     check
   }

FrF2old <- function(nruns=NULL, nfactors=NULL, 
                 factor.names = if(!is.null(nfactors)) {if(nfactors<=50) Letters[1:nfactors] 
                                else paste("F",1:nfactors,sep="")} else NULL, 
                 default.levels = c(-1,1), ncenter=0, center.distribute=NULL,
                 generators=NULL, design=NULL, resolution=NULL, select.catlg=catlg, 
                 estimable=NULL, clear=TRUE, method="VF2", sort="natural", 
                 res3=FALSE, max.time=60, 
                 perm.start=NULL, perms=NULL, MaxC2=FALSE, 
                 replications=1, repeat.only=FALSE, 
                 randomize=TRUE, seed=NULL, alias.info=2, 
                 blocks=1, block.name="Blocks", bbreps=replications, wbreps=1, alias.block.2fis = FALSE,
                 hard=NULL, check.hard=10, WPs=1, nfac.WP=0, WPfacs=NULL, check.WPs=10, ...){
## Version 1.7: added the LAD method and made it the default for clear designs
## method option allows to change it to VF2
creator <- sys.call()
catlg.name <- deparse(substitute(select.catlg))
    nichtda <- "try-error" %in% class(try(eval(parse(text=paste(catlg.name,"[1]",sep=""))),silent=TRUE))
    if (nichtda){
      ## provide valid catalogue names from FrF2.catlg128 (added for version 1.6-6
      
      catlgs128 <- c("catlg128.8to15","catlg128.26to33",paste("catlg128",16:25,sep="."))
      
      if (catlg.name %in% catlgs128){
             if (!requireNamespace("FrF2.catlg128", quietly=TRUE, character.only=TRUE)) 
                  stop("Package FrF2.catlg128 is not available")
             if (packageVersion("FrF2.catlg128") < numeric_version("1.2")){ 
                  if (catlg.name %in% catlgs128[c(1,3:11)])
                  stop("For this version of package FrF2.catlg128,\n",
                       "load ", catlg.name, " with the command data(", catlg.name,")\n",
                       "and then rerun the FrF2 command.\n",
                       "Alternatively, install the latest version of package FrF2.catlg128.")
                  else stop("You need to get the latest version of package FrF2.catlg128 for using ", catlg.name)
                       }
       }
     else stop(catlg.name, " not available")
     }
    if (!"catlg" %in% class(select.catlg)) stop("invalid choice for select.catlg")
## check validity of center point options
    if (!is.numeric(ncenter)) stop("ncenter must be a number")
    if (!length(ncenter)==1) stop("ncenter must be a number")
    if (!ncenter==floor(ncenter)) stop("ncenter must be an integer number")
    if (is.null(center.distribute)){
      if (!randomize) center.distribute <- min(ncenter, 1)
         else center.distribute <- min(ncenter, 3)}
    if (!is.numeric(center.distribute)) stop("center.distribute must be a number")
    if (!center.distribute==floor(center.distribute)) stop("center.distribute must be an integer number")
    if (center.distribute > ncenter)
       stop("center.distribute can be at most ncenter")
    if (randomize & center.distribute==1) warning("running all center point runs together is usually not a good idea.")
    block.name <- make.names(block.name) ## make block.name a valid R name

## check that no incompatible options are used together
if (ncenter>0 & !identical(WPs,1)) stop("center points for split plot designs are not supported")
if (!(is.null(generators) | (identical(WPs,1) | !is.null(WPfacs)))) stop("generators can only be used with split-plot designs, if WPfacs are specified.")
if (!is.null(nruns)) if (ncenter>0) if (center.distribute > nruns + 1) 
    stop("center.distribute must not be larger than nruns+1")
if (!(is.null(design) | is.null(estimable))) stop("design and estimable must not be specified together.")
if (!(is.null(generators) | is.null(design))) stop("generators and design must not be specified together.")
if (is.null(nruns) & !(is.null(generators))) stop("If generators is specified, nruns must be given.")
if (!(is.null(generators) | is.null(estimable))) stop("generators and estimable must not be specified together.")
if (!(identical(blocks,1) | is.null(estimable))) stop("blocks and estimable must not be specified together.")
    ## 1.7 it might be desirable to alleviate this constraint, and it might also be possible!
if (!(identical(WPs,1) | is.null(estimable))) stop("WPs and estimable must not be specified together.")
if (!(is.null(hard) | is.null(estimable))) stop("hard and estimable must not be specified together.")
if (!(identical(blocks,1) | identical(WPs,1))) stop("blocks and WPs must not be specified together.")
if (!(identical(blocks,1) | is.null(hard))) stop("blocks and hard must not be specified together.")
if (!(identical(WPs,1) | is.null(hard))) stop("WPs and hard must not be specified together.")
if (identical(blocks,1) & !identical(wbreps,1)) stop("wbreps must not differ from 1, if blocks = 1.")
if (!(is.null(WPfacs) | identical(WPs,1)) & is.null(design) & is.null(generators))
          stop("WPfacs requires explicit definition of a design via design or generators.")
if (identical(nfac.WP,0) & is.null(WPfacs) & !identical(WPs,1)) 
          stop("WPs whole plots require specification of whole plot factors through nfac.WP or WPfacs!")
    ## thus, wbreps = 1 can always be assumed for non-split-plot situations,
    ## which reduces necessary case distinctions
#if ((!identical(WPs,1)) & is.null(nruns)) stop("WPs only works if nruns is specified.")
    ### ??? or is that needed for the automatic version only ???
    if (!(is.null(resolution) | is.null(estimable))) stop("You can only specify resolution OR estimable.")
    if (!(is.null(resolution) | is.null(nruns))) warning("resolution is ignored, if nruns is given.")

## simple checks for individual option entries
    if (default.levels[1]==default.levels[2]) stop("Both default levels are identical.")
    if (!(is.logical(clear) & is.logical(res3) & is.logical(MaxC2) & is.logical(repeat.only) 
         & is.logical(randomize) & is.logical(alias.block.2fis) ))
         stop("clear, res3, MaxC2, repeat.only, randomize, and alias.block.2fis must be logicals (TRUE or FALSE).")
    if (!is.numeric(max.time)) 
         stop("max.time must be a positive maximum run time for searching a design with estimable given and clear=FALSE.")
    if (!is.numeric(check.hard)) stop("check.hard must be an integer number.")
    if (!is.numeric(check.WPs)) stop("check.WPs must be an integer number.")
    check.hard <- floor(check.hard)  ## avoid stupid errors
    check.WPs <- floor(check.WPs)  ## avoid stupid errors
    if (!is.numeric(bbreps)) stop("bbreps must be an integer number.")
    if (!is.numeric(wbreps)) stop("wbreps must be an integer number.")
    if (!is.numeric(replications)) stop("replications must be an integer number.")
    if (bbreps > 1  & identical(blocks,1) & !replications > 1) 
        stop("Use replications, not bbreps, for specifying replications for unblocked designs.")

    if (!alias.info %in% c(2,3)) 
         stop("alias.info can be 2 or 3 only.")
    if (!(is.numeric(default.levels) | is.character(default.levels))) 
         stop("default.levels must be a numeric or character vector of length 2")
    if (!length(default.levels) ==2) 
         stop("default.levels must be a numeric or character vector of length 2")
    if (!(is.null(hard) | is.numeric(hard))) 
         stop("hard must be numeric.")
    if (!(is.null(resolution) | is.numeric(resolution))) 
         stop("resolution must be numeric.")
    if (is.numeric(resolution)) if(!(resolution == floor(resolution) & resolution>=3)) 
         stop("resolution must be an integer number (at least 3), if specified.")
    res.WP <- NULL   ## for simple way of checking whether split-plot design later on

## treatment of one specifically-chosen design entry
    if (!is.null(design)){ 
        ## design is looked for in the catalogue specified in select.catlg
        if (!is.character(design)) stop("design must be a character string.")
        if (!length(design)==1) stop("design must be one character string.")
           if (design %in% names(select.catlg)){ 
                cand <- select.catlg[design]
                if (!is.null(nruns)) {if (!nruns==cand[[1]]$nruns) 
                          stop("selected design does not have the desired number of runs.")}
                       else nruns <- cand[[1]]$nruns
                if (!is.null(factor.names)) {if (!length(factor.names)==cand[[1]]$nfac) 
                          stop("selected design does not have the number of factors specified in factor.names.")}
                if (!is.null(nfactors)) {if (!nfactors==cand[[1]]$nfac) 
                          stop("selected design does not have the number of factors specified in nfactors.")}
                       else nfactors <- cand[[1]]$nfac
                }
                else stop("invalid entry for design")
    }
    
    ## should occur after special treatment for specific design
    if (!is.null(nruns)){
       k <- round(log2(nruns))
       if (!2^k==nruns) stop("nruns must be a power of 2.")
       if (nruns < 4 | nruns > 4096) stop("less than 4 or more than 4096 runs are not covered by FrF2.")
       }

    ## check factor specifications
    if (is.null(factor.names) & is.null(nfactors) & (is.null(nruns) | is.null(generators)) & is.null(estimable)) 
         stop("The number of factors must be specified via nfactors, via factor.names, via estimable, through selecting 
         one specific catalogued design or via nruns together with generators.")
    if (!is.null(factor.names) & !(is.character(factor.names) | is.list(factor.names)) ) 
         stop("factor.names must be a character vector or a list.")
    if (is.null(nfactors)) {if (!is.null(factor.names)) nfactors <- length(factor.names)
                          else if (!is.null(generators)) nfactors <- length(generators)+k 
                          }
    if (!is.null(estimable)) {
              if (!is.character(sort)) stop("option sort must be a character string")
              if (!is.character(method)) stop("option method must be a character string")
              if (!sort %in% c("natural","high","low")) stop("invalid choice for option sort")
              if (clear && !method %in% c("LAD","VF2")) stop("invalid choice for option method")
              estimable <- estimable.check(estimable, nfactors, factor.names)
              if (is.null(nfactors)) nfactors <- estimable$nfac
              estimable <- estimable$estimable
              if (is.null(nruns)) {
                 ## make even
                 nruns <- nfactors+ncol(estimable)+1 + (nfactors+ncol(estimable)+1)%%2
                 if (!isTRUE(all.equal(log2(nruns) %% 1,0))) nruns <- 2^(floor(log2(nruns))+1)
                 k <- round(log2(nruns))
                 if (k<3) stop("Please specify nruns and/or nfactors. Calculated values are unreasonable.")
           }
                if (is.null(perm.start)) perm.start <- 1:nfactors
                    else if (!is.numeric(perm.start)) 
                         stop ("perm.start must be NULL or a numeric permutation vector of length nfactors.")
                if (!all(sort(perm.start)==1:nfactors)) 
                         stop ("perm.start must be NULL or a numeric permutation vector of length nfactors.")
                if (!is.null(perms)) {
                   if (!is.matrix(perms) | !is.numeric(perms)) stop("perms must be a numeric matrix.")
                   if (!ncol(perms)==nfactors) stop ("matrix perms must have nfactors columns.")
                   if (any(apply(perms,1,function(obj) any(!sort(obj)==1:nfactors)))) 
                        stop("Each row of perms must be a permutation of 1:nfactors.")
                   }
    }
          ## from here on, estimable is a numeric matrix with two rows
          ## and nfactors is known
    if (!nfactors==floor(nfactors)) 
         stop("nfactors must be an integer number.")
    if (!is.null(factor.names) & !length(factor.names)==nfactors) 
         stop("There must be nfactors factor names, if any.")
    if (is.null(factor.names)) 
         if(nfactors<=50) factor.names <- Letters[1:nfactors] else factor.names <- paste("F",1:nfactors,sep="")
    
    ## check factor level specifications
    if (!((is.character(default.levels) | is.numeric(default.levels)) & length(default.levels)==2) ) 
                 stop("default.levels must be a vector of 2 levels.")
    if (is.list(factor.names)){ 
        if (is.null(names(factor.names))){
            if (nfactors<=50) names(factor.names) <- Letters[1:nfactors]
            else names(factor.names) <- paste("F", 1:nfactors, sep="")
        } 
        if (any(factor.names=="")) factor.names[which(factor.names=="")] <- list(default.levels)}
        else {hilf <- vector("list",nfactors)
              names(hilf) <- factor.names
              hilf[1:nfactors]<-list(default.levels)
              factor.names <- hilf}
    ## from here on, factor.names is a named list
    ## make all names valid R names
      names(factor.names) <- make.names(names(factor.names), unique=TRUE)
    
    if (ncenter > 0) if(any(is.na(sapply(factor.names,"is.numeric"))))
       stop("Center points are implemented for experiments with all factors quantitative only.")

    ### prepare generators case
    if (!is.null(generators)){
               ### nruns and k are always given for this situation
              generators <- gen.check(k, generators)
                 g <- nfactors - k
                 if (!length(generators)== g) 
                 stop("This design in ", nruns, " runs with ", nfactors," factors requires ", g, " generators.")
              
               res <- NA; nclear.2fis<-NA; clear.2fis<-NULL;all.2fis.clear<-NA
               if (g<10) wl <- words.all(k, generators,max.length=6)
               else if (g<15) wl <- words.all(k, generators,max.length=5)
               else if (g<20) wl <- words.all(k, generators,max.length=4)
               else if (g>=20) wl <- alias3fi(k, generators, order=2)  ## not a word list
               WLP <- NULL
               if (g < 20){
               WLP <- wl$WLP
                 res <- min(as.numeric(names(WLP)[which(WLP>0)]))
                 if (res==Inf) {if (g<10) res="7+"
                   else if (g<15) res="6+"
                   else if (g<20) res="5+" }
                 }
               else{
                   if (!is.list(wl)) res="5+"
                   else{
                   if (length(wl$"main")>0) res="3"
                      else if (length(wl$"fi2")>0) res="4"
                           else res="5+"}
               }

               gen <- sapply(generators,function(obj) which(sapply(Yates[1:(nruns-1)],
                             function(obj2) isTRUE(all.equal(sort(abs(obj)),obj2)))))
               gen <- gen*sapply(generators, function(obj) sign(obj[1]))

               cand <- list(custom=list(res=res, nfac=nfactors, nruns=nruns, 
                    gen=gen, 
                    WLP=WLP, nclear.2fis=nclear.2fis, clear.2fis=clear.2fis, all.2fis.clear=all.2fis.clear))
                   ## needs to be list of list, because access later is always via cand[1]
               class(cand) <- c("catlg","list")
      }
    ## prepare blocks
    ## 1.8: if blocks and estimable go together, does this preparation need to accomodate estimable ?
    ## 1.8: so far, estimable is a two-row matrix, and nfactor and nruns are known (lines 135 to 170)
    ## 1.8: provisionally, it looks as though no adjustments are needed
    ## 1.8: only perhaps checks against using block generators with estimable (???)
    if (!identical(blocks,1)) {
             blocks <- block.check(k, blocks, nfactors, factor.names)  
             if (is.list(blocks)) k.block <- length(blocks)
             block.auto=FALSE   ## default for block.auto, set to TRUE below, if TRUE
             map <- NULL   ## initialize, will be filled for blockpick.big
             }
             ## blocks is now list of generators or number of blocks
    if (!is.list(blocks)){
    if (blocks>1){
       block.auto=TRUE
       if (is.null(nruns)) 
            stop("blocks>1 only works if nruns is specified.")
       k.block <- round(log2(blocks))
       if (blocks > nruns/2) 
            stop("There cannot be more blocks than half the run size.")
       if (nfactors+blocks-1>=nruns) 
            stop(paste(nfactors, "factors cannot be accomodated in", nruns, "runs with", blocks, "blocks."))
       ntreat <- nfactors
#       nfactors <- nfactors + k.block     #### omitted for new version, that alone is probably not yet the solution
    }
    }
    ### treat hard before WPs, so that it can be handled by split-plot approach
    if (!is.null(hard)){
      if (is.null(generators)){ 
           ## otherwise, cand has already been specified
           if (is.null(nruns)){
              ## resolution requested, k not yet defined
                 cand <- select.catlg[which(res.catlg(select.catlg)>=resolution & nfac.catlg(select.catlg)==nfactors)]
                 if (length(cand)==0) {
                    message("full factorial design needed for achieving requested resolution")
                    k <- nfactors
                    nruns <- 2^k
                    cand <- list(list(gen=numeric(0)))
                    }
                 else {
                     nruns <- min(nruns.catlg(cand))
                     k <- round(log2(nruns))
                     cand <- cand[which(nruns.catlg(cand)==nruns)]
                    }
           }
           else {
               ## nruns specified
               if (nfactors > k)
               cand <- select.catlg[which(nfac.catlg(select.catlg)==nfactors & nruns.catlg(select.catlg)==nruns)]
               else cand <- list(list(gen=numeric(0)))
           }
      }
      if (hard == nfactors) stop("It does not make sense to choose hard equal to nfactors.")
      if (hard >= nruns/2) 
          warning ("Do you really need to declare so many factors as hard-to-change ?")
      nfac.WP <- hard
         ## determine WPs (implying k.WP)
         if (hard < nruns/2){
         WPs <- NA
         ## for full factorials, WPs <- 2^hard, 
         ## otherwise achievable WPs is determined using leftadjust 
         if (length(cand[[1]]$gen) > 0)
         for (i in 1:min(length(cand),check.hard)){
             leftadjust.out <- leftadjust(k,cand[[i]]$gen, early=hard, show=1)
             if (is.na(WPs) | WPs > 2^leftadjust.out$k.early) 
                 WPs <- 2^leftadjust.out$k.early
             }
             else WPs <- 2^hard
         }
      if (hard>=nruns/2 | WPs==nruns) {
           warning("There are so many hard-to-change factors that no good special design could be found.
                Randomization has been switched off.")
           randomize <- FALSE
           WPs <- 1
           ## desmat will be the special slow-change matrix
           leftadjust.out <- leftadjust(k,cand[[1]]$gen,early=hard,show=1)
           generators <- leftadjust.out$gen
           ## with this generator and the hard-to-change factors in the earliest columns
      }
    }
    ## prepare WPs
    if (!identical(WPs,1)) {
             if (is.null(nruns)) stop("WPs>1 only works if nruns is specified.")
             if (WPs > nruns/2) stop("There cannot be more whole plots (WPs) than half the run size.")
             k.WP <- round(log2(WPs))
             if (!WPs == 2^k.WP) stop("WPs must be a power of 2.")
             if (!is.null(WPfacs) & nfac.WP==0){
                    nfac.WP <- length(WPfacs)
                    if (nfac.WP < k.WP) stop("WPfacs must specify at least log2(WPs) whole plot factors.")
                 }
             if (nfac.WP==0) stop("If WPs > 1, a positive nfac.WP or WPfacs must also be given.")
             if (nfac.WP < k.WP) {
                 add <- k.WP - nfac.WP
                 names.add <- rep(list(default.levels),add)
                 names(names.add) <- paste("WP",(nfac.WP+1):(nfac.WP+add),sep="")
                 nfactors <- nfactors + add
                 factor.names <- c(factor.names[1:nfac.WP],names.add,factor.names[-(1:nfac.WP)])
                 nfac.WP <- k.WP
                 warning("There are fewer factors than needed for a full factorial whole plot design. ", add, 
                   " dummy splitting factor(s) have been introduced.")
              }
             if (!is.null(WPfacs)) {
                WPfacs <- WP.check(k, WPfacs, nfac.WP, nfactors, factor.names)  
                ## WPfacs is now character vector of the form Fno or single number of whole plots
                WPsnum <- as.numeric(chartr("F", " ", WPfacs))
                ## the factor positions within the factor names list from which the design is generated
                WPsorig <- WPsnum 
             }
             else {WPsorig <- WPsnum <- 1:nfac.WP }  ## first nfac.WP factors are whole plot factors
             
             ## here, the original WPsorig has been documented
             ## this must not be overwritten later!
    }
    
    if (!is.null(nruns)){
       ## find FrF2 for given number of runs 
       ## or one specific design (because nruns has already been calculated if necessary)
       ## or estimable given (because nruns has already been calculated if necessary)
       if (nfactors<=k & identical(blocks,1) & identical(WPs,1)) {
                    ### full factorial without blocks or WPs finally treated here
                    ### result immediately returned, i.e. function ends here in this case
                    if (nfactors==k) aus <- fac.design(2, k, factor.names=factor.names, 
                             replications=replications, repeat.only=repeat.only, 
                             randomize=randomize, seed=seed)
                    else aus <- fac.design(2, nfactors, factor.names=factor.names, 
                             replications=replications*2^(k-nfactors), repeat.only=repeat.only, 
                             randomize=randomize, seed=seed)
                    aus <- qua.design(aus, quantitative="none", contrasts=rep("contr.FrF2",nfactors))
                    ## add center points, if requested
                    if (ncenter>0) aus <- add.center(aus, ncenter, distribute=center.distribute)
                    di <- design.info(aus)
                    di$creator <- creator
                    di <- c(di, list(FrF2.version = "1.7.2"))
                    design.info(aus) <- di
                    return(aus)
              }
       else {
       if (nfactors < k) stop("A full factorial for nfactors factors requires fewer than nruns runs. 
           Please reduce the number of runs and introduce replications instead.")
       if (nfactors == k) {
           generators <- as.list(numeric(0))
               cand <- list(custom=list(res=Inf, nfac=nfactors, nruns=nruns, 
                    gen=numeric(0), 
                    WLP=c(0,0,0,0), nclear.2fis=choose(k,2), clear.2fis=combn(k,2), all.2fis.clear="all"))
                   ## needs to be list of list, because access later is always via cand[1]
               class(cand) <- c("catlg","list")
           }
             ### fractional factorial
         if (nfactors > nruns - 1) 
               stop("You can accomodate at most ",nruns-1," factors in a FrF2 design with ",nruns," runs." )
         g <- nfactors - k  ## number of generators needed
            if (!is.null(estimable)) {
                ## determine design that satisfies estimability requests
                      desmat <- estimable(estimable, nfactors, nruns, 
                           clear=clear, res3=res3, max.time=max.time, select.catlg=select.catlg, 
                           method=method,sort=sort,
                           perm.start=perm.start, perms=perms, order=alias.info)
                      design.info <- list(type="FrF2.estimable", 
                             nruns=nruns, nfactors=nfactors, factor.names=factor.names, catlg.name = catlg.name,
                             map=desmat$map, aliased=desmat$aliased, clear=clear, res3=res3, 
                             FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
                      desmat <- desmat$design
                      desmat <- as.matrix(sapply(desmat,function(obj) as.numeric(as.character(obj))))
                      rownames(desmat) <- 1:nrow(desmat)
                  }
            else if (is.null(generators) & is.null(design)) 
                 cand <- select.catlg[nruns.catlg(select.catlg)==nruns & nfac.catlg(select.catlg)==nfactors]
               ## candidate designs for further steps
               ## resolution not here, because ignored for given nruns

            ## treating blocked designs
            ## 1.7 here is where the main changes are needed for allowing to block 
            ## 1.7    designs with estimable 2fis;
            ## 1.7    probably restrict choose(nruns - 1 - nfactors - length(estimable), k.block)
            ## 1.7    also generally probably choose(nruns - 1 - nfactors - ncol2fis, k.block)
            ## 1.7     if alias.block.2fis = FALSE; but do I get ncol2fis?
            block.gen <- NULL    ## for later checks
            if (!is.list(blocks)){
                if (blocks > 1) {
                    ### small case
                    if (g==0 | choose(nruns - 1 - nfactors, k.block) < 100000){
                    for (i in 1:length(cand)){
                      ### loop through possible generator designs from best to worse overall
                      ### break stops the loop as soon as design has been found
                      if (g==0) {blockpick.out <- try(blockpick(k, gen=0, 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
                            }
                      else {
                      if (is.null(generators))
                      blockpick.out <- try(blockpick(k, design=names(cand[i]), 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
                      else  blockpick.out <- try(blockpick(k, gen=cand[[i]]$gen, 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
                      }

                      if (!"try-error" %in% class(blockpick.out)) {
                         blocks <- blockpick.out$blockcols   ## column numbers in Yates matrix
                         block.gen <- blocks ## for design.info
                         cand <- cand[i]
                         cand[[1]]$gen <- c(cand[[1]]$gen,blocks)
                         blocks <- nfactors+(1:k.block) ## now in terms of factors
                      #### nfactors changed, will be reduced again later
                         nfactors <- nfactors+k.block
                         g <- g+k.block
                      ### adjust factor.names
                         hilf <- factor.names
                         factor.names <- vector("list", nfactors)
                         factor.names[-blocks] <- hilf
                         factor.names[blocks] <- list(default.levels)
                         names(factor.names) <- c(names(hilf),paste("b",1:k.block,sep=""))
                         blocks <- as.list(blocks)   ### rest is treated with the manual routine
                         break
                      }
                    }## else treats big case
                    }
                else{
                    nfactors <- nfactors+k.block
                    g <- g+k.block
                    hilf <- factor.names
                    factor.names <- vector("list", nfactors)
                    factor.names[(k.block+1):nfactors] <- hilf
                    factor.names[1:k.block] <- list(default.levels)
                    names(factor.names) <- c(paste("b",1:k.block,sep=""),paste(names(hilf)))
                    cand <- select.catlg[nfac.catlg(select.catlg)==nfactors & nruns.catlg(select.catlg)==nruns]
                    if (!(is.null(generators) & is.null(design))) 
                         warning("For this request, generator or design specifications have been ignored, 
                              because the block allocation procedure for big problems was used.")
                    for (i in 1:length(cand)){
                      blockpick.out <- try(blockpick.big(k, gen=cand[[i]]$gen, 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
                      if (!"try-error" %in% class(blockpick.out)) {
                         cand <- cand[i]
                         cand[[1]]$gen <- blockpick.out$gen[1,]  ## includes block generators
                         map <- blockpick.out$perms[1,]
                         blocks <- as.list(1:k.block) ## first k.block generator columns
                                                      ### rest is treated with the manual routine
                         block.gen <- 2^(0:(k.block-1))
                         break
                      }
                    }
                    } ## end else (for big case)
                if (alias.block.2fis & !is.list(blocks)) 
                         stop("no adequate block design found")
                if ((!alias.block.2fis) & !is.list(blocks)) 
                         stop("no adequate block design found with 2fis unconfounded with blocks")
                }
                }
            if (is.list(blocks)) {
                # can be a pre-treated automatic situation or a manual specification
                hilf.gen <- c(2^(0:(k-1)), cand[[1]]$gen)    ### cand[[1]] exists ?
                hilf.block.gen <- sapply(blocks, function(obj) 
                       as.intBase(paste(rowSums(do.call(cbind,lapply(obj, function(obj2) digitsBase(hilf.gen[obj2],2,k))))%%2,collapse="")))
                k.block.add <- length(intersect(hilf.block.gen, hilf.gen))
                if (is.null(block.gen)) {
                   ## manual allocation 
                   ntreat <- nfactors - k.block.add
                   block.gen <- hilf.block.gen
                   }
                if (k.block > 1){
                ### check that manual choices for block entries contain independent entries only
                hilf <- hilf.block.gen
                for (i in 2:k.block){
                       sel <- combn(k.block,i)
                       for (j in 1:ncol(sel)){
                          neu <- as.intBase(paste(rowSums(do.call(cbind,lapply(sel[,j], function(obj) digitsBase(hilf.block.gen[obj],2,k))))%%2,collapse=""))
                          if (neu %in% hilf) stop("specified blocks is invalid (dependencies)")
                          else hilf <- c(hilf, neu)
                          }
                   }
                rm(hilf)
                }
            }
            ## automatic treatment of split-plot designs
            if (WPs > 1){ 
               WP.auto <- FALSE
               map <- 1:k
               orignew <- WPsorig  ## orignew will be changed according to map later
                                   ## for cases without WPfacs
                                   ## this is for orig.fac.order element of design.info, 
                                   ## not for re-arranging columns in design
               if (is.null(WPfacs)){
                    WP.auto <- TRUE
                    ## max.res.5 is the max number of factors for resolution 5, 
                    ##    if k (or k.WP is of size ...
                    max.res.5 <- c(1,2,3, 5, 6, 8, 11, 17)
                    for (i in 1:length(cand)){
                      ## prevent wasting search time on impossible setups
                      if (is.null(generators)){
                        if (cand[[i]]$res>=5 & nfac.WP > max.res.5[k.WP]) next
                        if (cand[[i]]$res>=4 & nfac.WP > WPs/2) next
                      }
                      if (nfac.WP > WPs/2 | nfac.WP <= k.WP)
                      splitpick.out <- try(splitpick(k, cand[[i]]$gen, k.WP=k.WP, nfac.WP=nfac.WP, show=1),TRUE)
                      else  splitpick.out <- try(splitpick(k, cand[[i]]$gen, k.WP=k.WP, nfac.WP=nfac.WP, 
                             show=check.WPs),TRUE)
                      if (!"try-error" %in% class(splitpick.out)) {
                         WPfacs <- 1:k.WP    
                         if (nfac.WP > k.WP) WPfacs <- c(WPfacs, (k + 1):(k+nfac.WP-k.WP))
                         cand <- cand[i]
                         cand[[1]]$gen <- splitpick.out$gen[1,]
                         res.WP <- splitpick.out$res.WP[1]
                         map <- splitpick.out$perms[1,]
                         break
                      }
                    }
                if (is.null(res.WP)){
 #                     if (nruns >= 2^nfactors) stop("currently, splitplot designs for full factorials are not covered.")
                      if (nruns >= 2^nfactors) {
                          res.WP <- Inf
                          WP.auto <- TRUE
                          WPfacs <- 1:k.WP
                          if (!k.WP == nfac.WP) stop(nfac.WP, " whole plot factors cannot be accomodated in ", (2^k.WP), 
                              " whole plots for a full factorial. Please request smaller design with replication instead.")
                          cand <- list(list(gen=numeric(0)))  ## in order to be treatable below
                      }
                      else stop("no adequate splitplot design found")
                }
  #             WPsorig <- WPsnum <- WPfacs    ## this was a mistake, leading to other than the first nfac.WP factors being WP factors
                orignew <- WPsnum <- WPfacs
                WPfacs <- paste("F",WPfacs,sep="")  ## treat with manual below
                }
              }
              ## next closing brace for !(nfactors<=k & identical(blocks,1) & identical(WPs,1))
              }
              ### next closing brace for !is.null(nruns)
              } 
else {
       ## nruns not given and not already assigned for estimable
       ## find smallest FrF2 that fulfills the requirements regarding resolution/estimability
       if (is.null(resolution) & is.null(estimable)) 
                 stop("At least one of nruns or resolution or estimable must be given.")
       if (!is.null(resolution)){ 
                 cand <- select.catlg[which(res.catlg(select.catlg)>=resolution & nfac.catlg(select.catlg)==nfactors)]
                 if (length(cand)==0) {
                    message("full factorial design needed")
                    aus <- fac.design(2, nfactors, factor.names=factor.names, 
                             replications=replications, repeat.only=repeat.only, 
                             randomize=randomize, seed=seed)
                    for (i in 1:nfactors) 
                           if (is.factor(aus[[i]])) contrasts(aus[[i]]) <- contr.FrF2(2)
                    ## add center points, if requested
                    if (ncenter>0) aus <- add.center(aus, ncenter, distribute=center.distribute)
                    return(aus)
                    }
            }
       else{ cand <- select.catlg[which(nfac.catlg(select.catlg)==nfactors)]   
           ## no prior restriction by resolution
         if (length(cand)==0) 
             stop("No design listed in catalogue ", deparse(substitute(select.catlg)), " fulfills all requirements.")
           }
    }
    ## standard FrF2 situations
    ## maximize clear 2fis among maximum resolution designs (res3=FALSE)
    ##     or among all designs (res3=TRUE)
    ##   changed 28 01 2011 from always maximum resolution
    if (MaxC2 & is.null(estimable) & is.null(generators) ) {
           if (!res3)
           cand <- cand[which.max(sapply(cand[which(sapply(cand, 
                   function(obj) obj$res)==max(sapply(cand, function(obj) obj$res)))], 
                   function(obj) obj$nclear.2fis))]
           else
           cand <- cand[which.max(sapply(cand, 
                   function(obj) obj$nclear.2fis))]
            }
    ### creation of actual design 
    if (is.null(nruns)) {nruns <- cand[[1]]$nruns 
                         k <- round(log2(nruns))
                         g <- nfactors - k}
    if (is.null(estimable)){
       destxt <- "expand.grid(c(-1,1)"
       for (i in 2:k) destxt <- paste(destxt,",c(-1,1)",sep="")
       destxt <- paste("as.matrix(",destxt,"))",sep="")
       desmat <- eval(parse(text=destxt))
       if (is.character(WPfacs) | is.list(blocks)) {
          desmat <- desmat[,k:1]  ## slow first rather than fast first
          ##rownames(desmat) <- slowfast(k)  ## 17 Feb 2013; this would make the 
          ##   run numbers refer to the Yates matrix row order with A changing fastest
          ##   but would also change row order of the resulting design
          ##   therefore not done; 
          }
       if (!is.null(hard)) {
             desmat <- rep(c(-1,1),each=nruns/2)
             for (i in 2:k) desmat <- cbind(desmat,rep(c(1,-1,-1,1),times=(2^i)/4,each=nruns/(2^i)))
             }
       if (g>0)
       for (i in 1:g) 
       desmat <- cbind(desmat, sign(cand[[1]]$gen[i][1])*apply(desmat[,unlist(Yates[abs(cand[[1]]$gen[i])])],1,prod))
       if (WPs > 1) {
          if (!WP.auto) {
            hilf <- apply(desmat[,WPsorig,drop=FALSE],1,paste,collapse="")
            if (!length(table(hilf))==WPs) 
            stop("The specified design creates ",
                 length(table(hilf)), " and not ", WPs, " whole plots.")
                 for (j in setdiff(1:nfactors,WPsorig))
                     if (!length(table(paste(hilf,desmat[,j],sep="")))>WPs) 
                         stop("Factor ", names(factor.names)[j], " is also a whole plot factor.")
            ## added 19 Feb 2013
            if (nfac.WP<3) res.WP <- Inf
            else res.WP <- GR((3-desmat[,WPsorig,drop=FALSE])%/%2)$GR%/%1           
          }
          }
       
       ## slow changing order, if required
      # if ((!is.character(WPfacs)) & !is.null(hard) ) 
      #     desmat <- desmat[,order(c(2^(0:(k-1)),abs(cand[[1]]$gen)))]

      ## make sure matrix desmat has rownames (14 Feb 2013)
      if (is.null(rownames(desmat))) rownames(desmat) <- 1:nruns

       if (is.list(blocks)) {
           ## manually blocked designs and continuation of automatically blocked designs
            if (is.null(block.gen)) block.gen <- blocks
            hilf <- blocks
            for (i in 1:k.block) hilf[[i]] <- apply(desmat[,hilf[[i]],drop=FALSE],1,prod)
            Blocks <- factor(as.numeric(factor(apply(matrix(as.character(unlist(hilf)),ncol=k.block),
                        1,paste,collapse=""))))
            hilf <- order(Blocks, as.numeric(rownames(desmat)))   ## rownames(desmat) added 17 Feb 2013
            desmat <- desmat[hilf,]
            Blocks <- Blocks[hilf]
            contrasts(Blocks) <- contr.FrF2(levels(Blocks))
            nblocks <- 2^length(blocks)
            blocksize <- nruns / nblocks
            block.no <- paste(Blocks,rep(1:blocksize,nblocks),sep=".")
            }
       if (is.character(WPfacs)) {
            ## make WP factor columns the first ones
            ## make factor.names match this order 
            #### for automatic selection the first WPs factors are WP factors !!!
            #if (nfac.WP > k.WP){
            
            ## are WPsnum is the current factor numbering 
            ## WPsorig is the original numbering before column permutations 
               desmat <- desmat[,c(WPsnum,setdiff(1:nfactors,WPsnum))]
               factor.names <- factor.names[c(WPsorig,setdiff(1:nfactors,WPsorig))]

               plotsize <- round(nruns/WPs)
               
               if (is.null(hard)){
                 hilf <- ord(cbind(desmat[,1:nfac.WP],as.numeric(rownames(desmat))))  ## as.numeric(rownames) added 17 Feb 2013
                 desmat <- desmat[hilf,]
               }
               wp.no <- paste(rep(1:WPs,each=plotsize),rep(1:plotsize,WPs),sep=".")
               
            }
    }

    ## until Feb 14 2013, rownames(desmat) were defined here, 
    ##    which did not make sense, especially since they should be available 
    ##    in already resorted form above for block and wp
    ## 14 Feb 2013: auskommentiert
    ## rownames(desmat) <- 1:nruns

    ## handle randomization and replication
    if (randomize & !is.null(seed)) set.seed(seed)
    if (!(is.list(blocks) | WPs > 1)){
      ## simple randomization situations
      rand.ord <- rep(1:nruns,replications)
      if (replications > 1 & repeat.only) rand.ord <- rep(1:nruns,each=replications)
      if (randomize & !repeat.only) for (i in 1:replications) 
                  rand.ord[((i-1)*nruns+1):(i*nruns)] <- sample(nruns)
      if (randomize & repeat.only) rand.ord <- rep(sample(1:nruns), each=replications)
      }
      else {   
          if (is.list(blocks)){
          #### implement randomization and replication for blocks
          ## bbreps=replications, wbreps=1
          ## if ((!repeat.only) & !randomize)
              rand.ord <- rep(1:nruns, bbreps * wbreps)
              if ((!repeat.only) & !randomize)
                 for (i in 0:(nblocks-1))
                    for (j in 1:wbreps)
                    rand.ord[(i*blocksize*wbreps+(j-1)*blocksize+1):((i+1)*blocksize*wbreps+j*blocksize)] <- 
                         (i*blocksize+1):((i+1)*blocksize)
                    rand.ord <- rep(rand.ord[1:(nruns*wbreps)],bbreps)
              if (repeat.only & !randomize)
                    for (j in 1:wbreps)
                    rand.ord[(i*blocksize*wbreps + (j-1)*blocksize + 1) : 
                          (i*blocksize*wbreps + j*blocksize)] <- 
                               sample((i%%nblocks*blocksize+1):(i%%nblocks*blocksize+blocksize))
              if (wbreps > 1 & repeat.only) rand.ord <- rep(1:nruns,bbreps, each=wbreps)
              if ((!repeat.only) & randomize)
                 for (i in 0:(nblocks*bbreps-1))
                    for (j in 1:wbreps)
                    rand.ord[(i*blocksize*wbreps + (j-1)*blocksize + 1) : 
                          (i*blocksize*wbreps + j*blocksize)] <- 
                               sample((i%%nblocks*blocksize+1):(i%%nblocks*blocksize+blocksize))
                               
              if (repeat.only & randomize)
                 for (i in 0:(nblocks*bbreps-1))
                    rand.ord[(i*blocksize*wbreps + 1) : 
                          ((i+1)*blocksize*wbreps)] <- rep(sample((((i%%nblocks)*blocksize)+1):
                                        ((i%%nblocks+1)*blocksize)),each=wbreps)
              }
          else {
          #### implement randomization and replication for split-plots
          ####   standard approach; replications replicate complete plots
          ####   repeat.only generate repeated measurements within plots
                rand.ord <- rep(1:nruns,replications)
                if (replications > 1 & repeat.only) 
                                  rand.ord <- rep(1:nruns,each=replications)
                if ((!repeat.only) & randomize){
                    ## whole plots are replicated
                    ## might be more wise to use larger design

                    ## randomization on two levels
                    ## first: randomization within each whole plot
                    for (i in 1:(WPs*replications)) 
                        rand.ord[((i-1)*plotsize+1):(i*plotsize)] <- 
                            sample(rand.ord[((i-1)*plotsize+1):(i*plotsize)])
                    ## second: randomization of whole plots (in blocks per replication)
                    for (i in 1:replications){
                    ## only if not hard to change
                    if (is.null(hard)){
                        WPsamp <- sample(WPs) 
                        WPsamp <- (rep(WPsamp,each=plotsize)-1)*plotsize + rep(1:plotsize,WPs)
                        rand.ord[((i-1)*plotsize*WPs+1):(i*plotsize*WPs)] <- 
                            rand.ord[(i-1)*plotsize*WPs + WPsamp]
                        }
                    }
                    }
                if (repeat.only & randomize){
                    ## repeated measurements within whole plots
                    ## repetition of complete whole plots is not covered, 
                    ##     as I don't think that this makes sense

                    ## first: randomization within each whole plot
                    for (i in 1:WPs) 
                        rand.ord[((i-1)*plotsize*replications+1):(i*plotsize*replications)] <- 
                            rand.ord[(i-1)*plotsize*replications + 
                               rep(replications*(sample(plotsize)-1),each=replications) + 
                               rep(1:2,each=plotsize)]
                    ## second: randomization of whole plots
                    ## only if not hard to change
                    if (is.null(hard)){
                        WPsamp <- sample(WPs) 
                        WPsamp <- (rep(WPsamp,each=plotsize*replications)-1)*plotsize*replications + 
                             rep(1:(plotsize*replications),WPs)
                        rand.ord <- rand.ord[WPsamp]
                    }
                    }
          }
      }          
    orig.no <- rownames(desmat)
    orig.no <- orig.no[rand.ord]
    ## row added 27 01 2011 (for proper ordering of factor levels)
    orig.no.levord <- sort(as.numeric(orig.no),index=TRUE)$ix

    rownames(desmat) <- NULL
    desmat <- desmat[rand.ord,]
        
    if (is.list(blocks)) {
              Blocks <- Blocks[rand.ord]
              block.no <- block.no[rand.ord]
              }
    if (WPs > 1) wp.no <- wp.no[rand.ord]
      colnames(desmat) <- names(factor.names)
      ## adapt original number to replications
      if (is.list(blocks)) orig.no <- paste(orig.no,block.no,sep=".")
      if (WPs > 1) orig.no <- paste(orig.no,wp.no,sep=".")
      orig.no.rp <- orig.no
      if (bbreps * wbreps > 1){
           if (bbreps > 1) {
                ## !repeat.only covers all blocked cases and the repeat.only standard cases
                ## since bbreps stands in for replications
                if (repeat.only & !is.list(blocks))
                orig.no.rp <- paste(orig.no.rp, rep(1:bbreps,nruns),sep=".")
                else
                orig.no.rp <- paste(orig.no.rp, rep(1:bbreps,each=nruns*wbreps),sep=".")
           }
           if (wbreps > 1){
                ## blocked with within block replications
                if (repeat.only) 
                     orig.no.rp <- paste(orig.no.rp, rep(1:wbreps,nruns*bbreps),sep=".")
                else orig.no.rp <- paste(orig.no.rp, rep(1:wbreps,each=blocksize,nblocks*bbreps),sep=".")
           }
    }

    desdf <- data.frame(desmat)
    quant <- rep(FALSE,nfactors)
    for (i in 1:nfactors) {
        ## make all variables into factors
        desdf[,i] <- des.recode(desdf[,i],"-1=factor.names[[i]][1];1=factor.names[[i]][2]") 
        quant[i] <- is.numeric(desdf[,i])
        desdf[,i] <- factor(desdf[,i],levels=factor.names[[i]]) 
        contrasts(desdf[,i]) <- contr.FrF2(2)
        }

    if (is.list(blocks)) {
        desdf <- cbind(Blocks, desdf)
        ## reassign levels for Blocks to concatenation of manual build factors
        ## if they are built from only single factors
        ## !!! it is important to maintain the original order of numbered blocks factor
        ##     for more than four blocks (because e.g. grouping 1,2,3,7 vs. 4,5,6,8 not
        ##     orthogonal to everything else !
        hilf <- blocks
        if (all(sapply(hilf,length)==1) & !block.auto) {
            hilf <- as.numeric(hilf) + 1
            levels(desdf$Blocks) <- unique(apply(desdf[,hilf,drop=FALSE],1,paste,collapse=""))
            }
        colnames(desdf)[1] <- block.name
        ## delete any single block generator columns
        hilf <- blocks
        hilf <- as.numeric(hilf[which(sapply(hilf,length)==1)])
        if (length(hilf)>0) {
            desdf <- desdf[,-(hilf+1)]
            desmat <- desmat[,-hilf]
            factor.names <- factor.names[-hilf]
        }
        ## prepend block model matrix to desmat (not only generators but all block main effects)  
        desmat <- cbind(model.matrix(~desdf[,1])[,-1],desmat)
        colnames(desmat)[1:(2^k.block-1)] <- paste(block.name,1:(2^k.block-1),sep="")
        if (alias.info==3)
        hilf <- aliases(lm((1:nrow(desmat))~(.)^3,data=data.frame(desmat)))
        else 
        hilf <- aliases(lm((1:nrow(desmat))~(.)^2,data=data.frame(desmat)))
        ## check against blockpick.out here, if possible?
        ## blockpick.out$alias.2fis.block
        
        ## delete all aliases of Block factors themselves
        ## with or without "-"
        ## as these are not interesting
        if (length(hilf$aliases) > 0)
        for (i in 1:length(hilf$aliases)) {
             txt <- hilf$aliases[[i]]
             if (length(grep(paste("^-?",block.name,sep=""),txt)) > 0)
                  txt <- txt[-grep(paste("^-?",block.name,sep=""),txt)] 
                  if (length(txt)==length(grep("^-",txt))) txt <- sub("-","",txt)
                  hilf$aliases[[i]] <- txt
             }
        ## determine treatment effects that are aliased with blocks
        aliased.with.blocks <- hilf$aliases[1:(2^k.block-1)]
           aliased.with.blocks <- unlist(aliased.with.blocks)
        ## interim result for updating alias information
        if (nfactors<=50) leg <- paste(Letters[1:ntreat],names(factor.names),sep="=")
        else leg <- paste(paste("F",1:ntreat,sep=""),names(factor.names),sep="=")
        if (length(aliased.with.blocks)==0) aliased.with.blocks <- "none"
           else {
             aliased.with.blocks <- recalc.alias.block(aliased.with.blocks, leg)
             aliased.with.blocks <- aliased.with.blocks[ord(data.frame(nchar(aliased.with.blocks),aliased.with.blocks))]
             }
        ## determine treatment effects that are aliased with each other
        aliased <- hilf$aliases[-(1:(2^k.block-1))]
        aliased <- aliased[which(sapply(aliased,length)>1)]
        ## update: same format like aliased element of unblocked designs
        if (length(aliased)>0) aliased <- struc.aliased(recalc.alias.block(aliased, leg), nfactors, alias.info)

           
        ## prepare design info for blocked designs
        ntreat <- ncol(desdf) - 1
        if (block.auto) factor.names <- factor.names[1:ntreat]
 #       if (block.auto) factor.names <- factor.names[setdiff(1:nfactors,blocks)]
 
 ## up to version 0.96-1 nfactors was ntreat+1
 #         design.info <- list(type="FrF2.blocked", block.name=block.name, 
 #           nruns=nruns, nfactors=ntreat+1, nblocks=nblocks, blocksize=blocksize, 
 #           ntreat=ntreat,factor.names=factor.names,
 #           aliased.with.blocks=aliased.with.blocks, aliased=aliased,
 #           bbreps=bbreps, wbreps=wbreps)
          design.info <- list(type="FrF2.blocked", block.name=block.name, 
            nruns=nruns, nfactors=ntreat, nblocks=nblocks, block.gen=block.gen, blocksize=blocksize, 
            ntreat=ntreat,factor.names=factor.names,
            aliased.with.blocks=aliased.with.blocks, aliased=aliased,
            bbreps=bbreps, wbreps=wbreps, 
            FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
        if (!is.null(generators)) {
           if (g>0) 
           design.info <- c(design.info, 
                list(base.design=paste("generator columns:", paste(cand[[1]]$gen, collapse=", "))))
           else 
           design.info <- c(design.info, 
                list(base.design="full factorial"))
                }
           else design.info <- c(design.info, list(catlg.name = catlg.name, base.design=names(cand[1])))
           design.info <- c(design.info, list(map=map))
        
        if (bbreps>1) {
            hilflev <- paste(rep(levels(desdf[,1]), each=bbreps), rep(1:bbreps, nblocks), sep=".")
            desdf[,1] <- factor(paste(desdf[,1], rep(1:bbreps, each=nruns*wbreps),sep="."), levels=hilflev)
        }
            ## make block names reflect the between block replication
        }   ## end of blocked designs

        if (WPs > 1){
            
            ## output for split-plot designs
            if (alias.info==3)
            aliased <- aliases(lm((1:nrow(desmat))~(.)^3,data=data.frame(desmat)))$aliases
            else
            aliased <- aliases(lm((1:nrow(desmat))~(.)^2,data=data.frame(desmat)))$aliases
            aliased <- aliased[which(sapply(aliased,length)>1)]
            ## update: same format with aliased entry of other FrF2 designs
            if (length(aliased) > 0){ 
                if (nfactors<=50) leg <- paste(Letters[1:nfactors],names(factor.names),sep="=")
                else leg <- paste(paste("F",1:nfactors,sep=""),names(factor.names),sep="=")
                aliased <- struc.aliased(recalc.alias.block(aliased, leg), nfactors, alias.info)
                }

            design.info <- list(type="FrF2.splitplot", 
                nruns=nruns, nfactors=nfactors, nfac.WP=nfac.WP, nfac.SP=nfactors-nfac.WP, 
                      factor.names=factor.names,
                nWPs=WPs, plotsize=nruns/WPs, 
                res.WP=res.WP, aliased=aliased, FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
            ## below: changed entry in base.design to original generators, combined with map
            ## instead of cand[[1]]$gen from before
            if (!is.null(generators)) 
                design.info <- c(design.info, 
                     list(base.design=paste("generator columns:", 
                     paste(which(names(Yates)[1:(nruns-1)] %in% names(generators)), collapse=", ")), map=map, 
                     orig.fac.order = c(orignew, setdiff(1:nfactors,orignew))))
                else design.info <- c(design.info, list(catlg.name = catlg.name, base.design=names(cand[1]), map=map, 
                     orig.fac.order = c(orignew, setdiff(1:nfactors,orignew))))
                }

    if (is.null(estimable) & is.null(generators) & !(is.list(blocks) | WPs > 1))
        design.info <- list(type="FrF2", nruns=nruns, nfactors=nfactors, factor.names=factor.names, 
            catlg.name = catlg.name,
            catlg.entry=cand[1], aliased = alias3fi(k,cand[1][[1]]$gen,order=alias.info), 
            FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
        ## incorporate reasonable further info (blocks, WPs etc.)
    if ((!is.null(generators)) & !is.list(blocks) & !WPs > 1) {
         if (nfactors <= 50)
         names(generators) <- Letters[(k+1):nfactors]
         else names(generators) <- paste("F",(k+1):nfactors, sep="")
         gen.display <- paste(names(generators),sapply(generators,function(obj){
               if (nfactors <= 50)
               paste(if (obj[1]<0) "-" else "", paste(Letters[abs(obj)],collapse=""),sep="")
               else 
               paste(if (obj[1]<0) "-" else "", paste(paste("F",abs(obj),sep=""),collapse=":"),sep="")}), sep="=")
         design.info <- list(type="FrF2.generators", nruns=nruns, nfactors=nfactors, factor.names=factor.names, generators=gen.display, 
               aliased = alias3fi(k,generators,order=alias.info), 
               FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
         }
    aus <- desdf
      rownames(aus) <- rownames(desmat) <- 1:nrow(aus)
    class(aus) <- c("design","data.frame")
    attr(aus,"desnum") <- desmat
      ## change 27 Jan 2011: leave orig.no as a factor, but with better-ordered levels
      orig.no <- factor(orig.no, levels=unique(orig.no[orig.no.levord]))
      ## added 14 Feb 2013
      orig.no.rp <- factor(orig.no.rp, levels=unique(orig.no.rp[orig.no.levord]))
    if (!(is.list(blocks) | WPs > 1)) 
       attr(aus,"run.order") <- data.frame("run.no.in.std.order"=orig.no,"run.no"=1:nrow(desmat),"run.no.std.rp"=orig.no.rp)
    else attr(aus,"run.order") <- data.frame("run.no.in.std.order"=orig.no,"run.no"=1:nrow(desmat),"run.no.std.rp"=orig.no.rp)
    ## make repeat.only reflect the calculated status instead of the status requested by the user 
    ## (which can be seen in the creator element)
    if (design.info$type=="FrF2.blocked") {
        if (design.info$wbreps==1) repeat.only <- FALSE
        nfactors <- ntreat
    }
    if (nfactors<=50) design.info$aliased <- c(list(legend=paste(Letters[1:nfactors],names(factor.names),sep="=")),design.info$aliased)
       else design.info$aliased <- c(list(legend=paste(paste("F",1:nfactors,sep=""),names(factor.names),sep="=")),design.info$aliased)
    attr(aus,"design.info") <- c(design.info, list(replications=replications, repeat.only=repeat.only,
      randomize=randomize, seed=seed, creator=creator))
    ## add center points, if requested
    if (ncenter>0) aus <- add.center(aus, ncenter, distribute=center.distribute)
    aus
} 

find.gen <- function(design,...){
    if (!"design" %in% class(design))
        stop("find.gen works on class design objects only")
    di <- design.info(design)
    if ((length(grep("full factorial",di$type))>0 & all(di$nlevels==2)))
         return("full factorial")
    if (!(substr(di$type,1,4)=="FrF2"))
        stop("this is not a regular (fractional) factorial 2-level design")
  #  if (length(grep("blocked", di$type))>0 | length(grep("splitplot", di$type))>0)
  #      stop("find.gen does not work for blocked or splitplot designs")
    if (length(grep("splitplot", di$type))>0)
        stop("find.gen does not work for splitplot designs")
   gen <- NULL

   ## extract and remove old generator information
   ## also in form catlg.entry (or base.design)
   if (!is.null(di$catlg.entry)){
      gen <- di$catlg.entry[[1]]$gen
      if (is.null(di$base.design) & !is.null(di$catlg.entry)) di$base.design <- names(di$catlg.entry)
      di$catlg.entry <- NULL
   }
   else if (!is.null(di$generators))
        gen <- sapply(strsplit(di$generators,"="),
        function(obj) if (substr(obj[2],1,1)=="-") -1*which(names(Yates)==substring(obj[2],2))
                      else which(names(Yates) == obj[2]))
   if (is.null(gen) & !is.null(di$base.design)){ 
   if (substring(di$base.design,1,18)=="generator columns:")
      gen <- eval(parse(text=paste("c(",gsub("generator columns:", "",di$base.design),")")))
   else if (di$base.design %in% names(catlg)) gen <- catlg[di$base.design][[1]]$gen
   }
  ## now gen is a vector of signed columns, if it has been present before or NULL otherwise
  gen
}

generators.from.design <- function(design, ...){
## function to find generators from a design 
## that need not necessarily refer to the standard base factors
    if (!"design" %in% class(design))
        stop("generators.from.design works on class design objects only")
    di <- design.info(design)
    gen <- find.gen(design)
    
    ## relies on the fact that the first k factors are base factors
    ### must take care of map in estimable designs!!!
    ### try with FrF2(32,9, estimable=c("AC","BC","AB")) for reshuffled,
    ###          FrF2(32,9, estimable="CJ") for basic
    if (is.null(gen)){
           if (di$nfactors > 50) stop("generators.from.design does not work for more than 50 factors (and will presumably break down much earlier)")
           if (!is.null(di$ncube)) k <- round(log2(di$ncube)) else k <- round(log2(di$nruns))
           fn <- names(di$factor.names)
           g <- di$nfactors - k
           if (is.null(di$ncube))
              linmod <- lm(formula(paste("I(1:",(di$nruns*di$replications),")~(.)^",k+1,sep="")), design[,names(di$factor.names)])
           else
              linmod <- lm(formula(paste("I(1:",(di$ncube*di$replications),")~(.)^",k+1,sep="")), design[iscube(design),names(di$factor.names)])
           al <- aliases(linmod)
           sel <- al[[2]][1:di$nfactors]   ## aliases of base and generated factors

             sel <- sel[sapply(sel, function(obj) !obj[1] %in% fn[di$map[[1]][1:k]])]
                 ## generated factors
             hilf <- lapply(sel, function(obj){
                    for (i in 1:k){
                        ## complete
                        obj <- gsub(paste("^",fn[di$map[[1]]][i],"$",sep=""), "", obj)
                        ## first
                        obj <- gsub(paste("^",fn[di$map[[1]]][i],":",sep=""), ":", obj)
                        ## middle
                        obj <- gsub(paste(":",fn[di$map[[1]]][i],":",sep=""), ":", obj)
                        ## last
                        obj <- gsub(paste(":",fn[di$map[[1]]][i],"$",sep=""), ":", obj)
                      }
                      obj <- gsub(":","",obj)
                    })
           hilf <- sapply(hilf, function(obj) which(obj==""))  ## the one that contains base factors only
           gen <- sapply(sel, function(obj) paste(Letters[which(fn[di$map[[1]]] == obj[1])],"~",sep="")) ## beginning of generator equation
           if (!length(gen)==g) stop("a problem in determining the generator!")
           for (i in 1:g){ 
                gen[i] <- paste(gen[i],
                                    paste(Letters[which(fn[di$map[[1]][1:k]] %in% unlist(strsplit(sel[[i]][hilf[i]],":",fixed=TRUE)))],collapse="*"),
                                    sep="")
                                    }
    }
    else {
           if (!is.null(di$ncube)) k <- round(log2(di$ncube)) else k <- round(log2(di$nruns))
           gen <- paste(Letters[(k+1):di$nfactors],"~",sapply(Yates[gen], function(obj) paste(Letters[obj],collapse="*")),sep="")
    }
    gen <- lapply(gen, function(obj){ 
        for (i in 1:length(di$factor.names)) obj <- gsub(Letters[i], paste("x",i,sep=""), obj)
        as.formula(obj)
        })
    gen
}

FF_from_X <- function(X, randomize=TRUE, seed=NULL, alias.info=2){
  ## function to obtain a blocked full fractorial from a hand-crafted X matrix
  ## all information about dimensions is derived from X
  ## the function returns a data frame with first column Block 
  ##    and further columns numeric with -1/+1 notation of factors
  creator=sys.call()
  if (!all(X %in% c(0,1))) stop("all entries of X must be 0 or 1")
  if (!is.logical(randomize)) stop("randomize must be logical")
  n <- ncol(X)
  q <- nrow(X)
  nruns <- 2^n
  fn <- rep(list(c(-1,1)),n)
  names(fn) <- Letters[1:n] 
  desmat <- as.matrix(expand.grid(fn))
  
  Xlogic <- matrix(as.logical(X),q,n)
  allaliasedb <- blockgengroup(X)
  allaliasedb <- allaliasedb[nchar(allaliasedb)<=alias.info]
  block.gen <- strsplit(bg <- blockgencreate(X),"",fixed=TRUE)
  hilf <- 
  cbind(data.frame(Blocks=factor(as.numeric(factor(apply(sapply(block.gen, function(obj) apply(desmat[,obj, drop=FALSE], 1, prod)), 1, 
        function(obj) paste(obj, collapse=""))))), stringsAsFactors=TRUE), desmat, stringsAsFactors=TRUE)
  hilf <- hilf[ord(hilf),]
  run.no.in.std.order <- rownames(hilf)
  rownames(hilf) <- 1:nruns
  aliased <- list(legend=paste(Letters[1:n],Letters[1:n],sep="="),
                 main=character(0),
                 fi2=character(0))
  if (alias.info>2)
    aliased <- c(aliased, list(fi3=character(0)))
  di <- list(type="FrF2.blocked",
             block.name="Blocks",
             nruns=nruns, nfactors=n, 
             factor.names=fn, 
             nblocks = 2^(n-q),
             block.gen=bg,
             blocksize=2^q,
             ntreat=n,
             aliased.with.blocks=allaliasedb,
             aliased = aliased,
             bbreps=1,
             wbreps=1,
             FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version,
             base.design=paste("generator columns: ", numeric(0)),
             replications=1,
             repeat.only=FALSE,
             randomize=randomize,
             seed=seed,
             creator=creator
             )
  ro <- data.frame(run.no.in.std.order=paste(run.no.in.std.order, 
                                        rep(1:di$nblocks, each=di$blocksize), 
                                        rep(1:di$blocksize, di$nblocks), sep="."),
              run.no= 1:nruns, 
              run.no.std.rp=paste(run.no.in.std.order, 
                                  rep(1:di$nblocks, each=di$blocksize), 
                                  rep(1:di$blocksize, di$nblocks), sep="."), 
                                  stringsAsFactors = TRUE)
  desnum <- model.matrix(~., hilf)
  class(hilf) <- c("design", "data.frame")
  design.info(hilf) <- di
  run.order(hilf) <- ro
  desnum(hilf) <- desnum
  ## randomization automatically uses the correct block factor
  if (randomize) hilf <- rerandomize.design(hilf, seed=seed)
  hilf
  }

X_from_profile <- function(n, q, profile=NULL){
  if (!length(n)==1) stop("n must be a single number")
  if (!length(q)==1) stop("q must be a single number")
  if (!is.numeric(n)) stop("n must be a number")
  if (!is.numeric(q)) stop("q must be a number")
  if (!floor(n)==n) stop("n must be integer")
  if (!floor(q)==q) stop("q must be integer")
  if (!is.numeric(profile)) stop("profile must be numeric")
  if (!all(floor(profile)==profile)) stop("profile must be integer")
  if (!(q<n && n>0 && q>0)) stop("unreasonable request")
  if (!sum(profile)==n) stop("n and profile do not match")
  div <- 2^q-1
  if (!length(profile)<=div) stop("q and profile do not match")
  Xcands <- lapply(1:div, function(obj) digitsBase(obj, 2, ndigits=q))
  profile <- sort(profile, decreasing = TRUE)
  lp <- length(profile)
  do.call(cbind, rep(Xcands[1:lp], times=profile))
}

X_from_parts <- function(n, q, parts){
  if (!length(n)==1) stop("n must be a single number")
  if (!length(q)==1) stop("q must be a single number")
  if (!is.numeric(n)) stop("n must be a number")
  if (!is.numeric(q)) stop("q must be a number")
  if (!floor(n)==n) stop("n must be integer")
  if (!floor(q)==q) stop("q must be integer")
  if (!(q<n && n>0 && q>0)) stop("unreasonable request")
  if (!is.list(parts)) stop("parts must be a list")
  elmts <- sort(unlist(parts))
  if (!length(elmts)==n) stop("n and parts do not match")
  if (!(all(elmts == 1:n) || all(elmts==Letters[1:n]))) 
    stop("parts has unsuitable elements")
  if (is.character(elmts))
    parts <- lapply(parts, function (obj) which(Letters %in% obj))
  div <- 2^q-1
  if (!length(parts)<=div) stop("q and parts do not match")
  profile <- lengths(parts)
  Xcands <- lapply(1:div, function(obj) digitsBase(obj, 2, ndigits=q))
  X <- matrix(NA, q, n)
  for (i in 1:length(parts))
    X[,parts[[i]]] <- Xcands[[i]]
  X
}

clear2fis_from_profile <- function(n, q, profile=NULL){
  if (!is.numeric(n) && is.numeric(q)) stop("n and q must be integer numbers")
  if (is.null(profile)) {
      ### default: most balanced
      profile <- rep(n%/%(2^q-1), 2^q-1)
      hilf <- n%%(2^q-1)
      if (hilf > 0) for (a in 1:hilf) profile[a] <- profile[a]+1
  }
  if (!is.numeric(profile)) stop("if given, profile must be a numeric vector")
  if (!sum(profile)==n) stop("entries of profile must sum to n")
  #  if (length(profile < 2^q-1) profile <- c(profile, rep(0, 2^q-1 - length(profile)))
  if (length(profile) > 2^q-1) stop("profile can have at most 2^q-1 entries")
  profile <- sort(profile, decreasing=TRUE)
  profile <- profile[profile>0]
  lims <- cumsum(profile)
  parts <- mapply(":", c(1,lims[-length(lims)]+1),lims, SIMPLIFY=FALSE)
  clear2fis <- matrix(NA,2,0)
  paare <- DoE.base:::nchoosek(length(parts),2)
  for (i in 1:ncol(paare)){
      clear2fis <- cbind(clear2fis, t(as.matrix(expand.grid(parts[[paare[1,i]]], parts[[paare[2,i]]]))))
  }
  clear2fis
}

print.aliases <- function(x, ...){
   ### function for printing output objects from function aliases in nicer format
   if (!"aliases" %in% class(x)) stop("applicable for objectso of class aliases only")
   if ("aliases" %in% names(x)){
     als <- which(sapply(x$aliases,"length")>1)
     if (length(als)>0){
        alprint <- list(legend=x$legend, aliases=x$aliases[als])
        alprint$aliases <- t(t(sapply(alprint$aliases, "paste", collapse=" = ")))
        colnames(alprint$aliases) <- ""
        rownames(alprint$aliases) <- rep("",nrow(alprint$aliases))
        if(!is.null(alprint$legend)) names(alprint$legend)<-rep("", length(alprint$legend))
        }
     else alprint <- list(legend=NULL, aliases="no aliasing in the model")
     if (is.null(alprint$legend)) print(alprint$aliases,quote=FALSE, ...) else
     print(alprint,quote=FALSE, ...)
   }
   else print.default(x, quote=FALSE, ...)
}

mapcalc.block <- function (estimable, nfac, 
                           clearcur, method="VF2", sort = "natural") 
{
  ## used in colpickV
  ## clearcur is the set of clear 2fis after removing
  ## aliases with the block factor
  go2 <- make_empty_graph(n = nfac, directed = FALSE)
  go2 <- add_edges(go2, estimable)
  deg2 <- degree(go2)
  if (sort %in% c("high", "low")) {
    if (sort == "low") 
      ord2 <- order(deg2)
    else ord2 <- order(deg2, decreasing = TRUE)
    go2 <- permute(go2, invperm(ord2))
  }
  degree2 <- rev(cumsum(rev(table(deg2))))   ## make it faster to reject non-isomorphic cases
  degs2 <- as.numeric(names(degree2))        ## required minimum degrees
  indep2 <- ivs_size(go2)         ## required maximum independence number
  clique2 <- clique_num(go2)              ## required minimum clique size
  map <- NULL
  go1 <- make_empty_graph(n = nfac, directed = FALSE)
  go1 <- add_edges(go1, clearcur)
  ## optionally sort vertices by degree, 20 Jul 2012
  deg1 <- degree(go1)
  if (sort %in% c("high", "low")) {
    if (sort == "low") 
      ord1 <- order(deg1)
    else ord1 <- order(deg1, decreasing = TRUE)
    go1 <- permute(go1, invperm(ord1))
  }
  degree1 <- rev(cumsum(rev(table(deg1))))
  degs1 <- as.numeric(names(degree1))
  if (max(degs2) <= max(degs1)) {
    ## if max(degs2)>max(degs1), subgraph isomorphism is impossible
    comp <- sapply(degs2, function(obj) degree1[min(which(degs1 >= 
                                                            obj))])
    comp[is.na(comp)] <- 0
    if (any(comp < degree2)) return(NULL)
    ## added further pre-filtering criteria 9 July 2012
    if (ivs_size(go1) > indep2) return(NULL)
    if (clique_num(go1) < clique2) return(NULL)
    if (method=="LAD") 
      erg <- graph.subisomorphic.lad(go2, go1)
    else erg <- graph.subisomorphic.vf2(go1, go2)
    if (erg$iso) {
      if (method=="LAD") map <- list(erg$map)
      else map <- list(erg$map21)
      if (sort %in% c("high", "low")) 
        map <- list(ord1[map[[1]]][invperm(ord2)])
      return(map)
    }
  }
  return(NULL)
}

cubedraw <- function(min = 0, max = 1, cex = 4, cex.lab=2, cex.axlab=2, center=FALSE,star=FALSE,a=1,pch=20,
     pchextra=pch, main=NULL, sub=NULL, ticks=FALSE, xlab="A", ylab="B", zlab="C",
     xends=c(-1,1),yends=c(-1,1),zends=c(-1,1), 
     x.ticklabs=c(xends[1],NA,xends[2]), y.ticklabs=c(yends[1],NA,yends[2]),
     z.ticklabs=c(zends[1],NA,zends[2]), zadj=c(0.8,0.9), nint=c(2,2,2),...)
 {
 ## Purpose:
 ## Draw cube with corners and potentially center and or stars
 ## flexible regarding plot symbols
 ## and colors
    oldpar <- par()
    par(mar=c(5, 3, 4, 3) + 0.1, xpd=TRUE)
    cub <- expand.grid(A=c(min,max),B= c(min,max),C= c(min,max))
    res3d<-myscatterplot3d(cub,cex.symbols=cex,pch=pch,grid=FALSE,box=FALSE,
         main=main, sub=sub, tick.marks=ticks, xlab=xlab,ylab=ylab,zlab=zlab,
         cex.lab=cex.lab,cex.axis=cex.axlab, xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1),
         x.ticklabs=x.ticklabs, y.ticklabs=y.ticklabs,
         z.ticklabs=z.ticklabs, zadj=zadj, lab=nint,...)
    mid <- (max+min)/2
    ## hidden corner + lines
    res3d$points3d(cub[c(1,3,4,3,7), ], cex = cex, type = 'b', lty = 1,lwd=3,col=grey(0.4),pch=pch)
    ## visible corners + lines:
    res3d$points3d(cub[c(2,1,5,6,2,4,8,6,5,7,8) ,], cex = cex, type = 'b', lty = 1,lwd=3,pch=pch)
    res3d$points3d(cub,cex=cex,pch=pch,lwd=3) ## points in top layer
    if(center){
        xnullvis<-rbind(c(mid,min,min),c(mid,min,max),c(mid,max,max))
        xnullinvis <- rbind(c(mid,max,max),c(mid,max,min),c(mid,min,min))
        ynullvis<-rbind(c(max,mid,min),c(max,mid,max),c(min,mid,max))
        ynullinvis <- rbind(c(min,mid,max),c(min,mid,min),c(max,mid,min))
        znullvis<-rbind(c(min,min,mid),c(max,min,mid),c(max,max,mid))
        znullinvis <- rbind(c(max,max,mid),c(min,max,mid),c(min,min,mid))
        res3d$points3d(rbind(c(mid,mid,mid)), cex = cex, pch=pchextra, type = 'p')
        res3d$points3d(xnullvis, cex = cex, type = 'l', lty = max,lwd=max)
        res3d$points3d(ynullvis, cex = cex, type = 'l', lty = max,lwd=max)
        res3d$points3d(znullvis, cex = cex, type = 'l', lty = max,lwd=max)
        res3d$points3d(xnullinvis, cex = cex, type = 'l', lty = 3,lwd=max)
        res3d$points3d(ynullinvis, cex = cex, type = 'l', lty = 3,lwd=max)
        res3d$points3d(znullinvis, cex = cex, type = 'l', lty = 3,lwd=max)
        xc<-rbind(c(min,mid,mid),c(mid,mid,mid),c(max,mid,mid))
        yc<-rbind(c(mid,min,mid),c(mid,mid,mid),c(mid,max,mid))
        zc<-rbind(c(mid,mid,min),c(mid,mid,mid),c(mid,mid,max))
        res3d$points3d(xc, cex = cex, type = 'l', lty = 3,lwd=max)
        res3d$points3d(yc, cex = cex, type = 'l', lty = 3,lwd=max)
        res3d$points3d(zc, cex = cex, type = 'l', lty = 3,lwd=max)
    }
    if (star)
    {
        xc<-rbind(c(-a,mid,mid),c(mid,mid,mid),c(a,mid,mid))
        yc<-rbind(c(mid,-a,mid),c(mid,mid,mid),c(mid,a,mid))
        zc<-rbind(c(mid,mid,-a),c(mid,mid,mid),c(mid,mid,a))
        res3d$points3d(xc, cex = cex, pch=pchextra, type = 'b', lty = 3,lwd=3)
        res3d$points3d(yc, cex = cex, pch=pchextra, type = 'b', lty = 3,lwd=3)
        res3d$points3d(zc, cex = cex, pch=pchextra, type = 'b', lty = 3,lwd=3)
    }
    par <- oldpar
    list(res3d=res3d, cub=cub)
 }

FrF2 <- function(nruns=NULL, nfactors=NULL, 
                 factor.names = if(!is.null(nfactors)) {if(nfactors<=50) Letters[1:nfactors] 
                                else paste("F",1:nfactors,sep="")} else NULL, 
                 default.levels = c(-1,1), ncenter=0, center.distribute=NULL,
                 generators=NULL, design=NULL, resolution=NULL, select.catlg=catlg, 
                 estimable=NULL, clear=TRUE, method="VF2", sort="natural", 
                 ignore.dom=!isTRUE(all.equal(blocks,1)),
                 useV = TRUE, firsthit = FALSE, res3=FALSE, max.time=60, 
                 perm.start=NULL, perms=NULL, MaxC2=FALSE, 
                 replications=1, repeat.only=FALSE, 
                 randomize=TRUE, seed=NULL, alias.info=2, 
                 blocks=1, block.name="Blocks", block.old=FALSE, force.godolphin=alias.block.2fis,
                 bbreps=replications, wbreps=1, alias.block.2fis = FALSE,
                 hard=NULL, check.hard=10, WPs=1, nfac.WP=0, WPfacs=NULL, check.WPs=10, ...){
## Version 1.7: added the LAD method and made it the default for clear designs
## method option allows to change it to VF2
creator <- sys.call()
if (!is.logical(block.old)) stop("block.old must be logical")

if (block.old && ! identical(blocks, 1)){
  ## revert the call to the old FrF2 version
  cc <- as.list(creator)
  ## remove arguments that did not exist earlier
  cc$block.old <- NULL
  cc[[1]] <- FrF2old
  cc <- as.call(cc)
  erg <- eval(cc)
  di <- design.info(erg)
  di$creator <- creator
  di$block.old <- block.old
  design.info(erg) <- di
  return(erg)
}
catlg.name <- deparse(substitute(select.catlg))
    nichtda <- "try-error" %in% class(try(eval(parse(text=paste(catlg.name,"[1]",sep=""))),silent=TRUE))
    if (nichtda){
      ## provide valid catalogue names from FrF2.catlg128 (added for version 1.6-6
      
      catlgs128 <- c("catlg128.8to15","catlg128.26to33",paste("catlg128",16:25,sep="."))
      
      if (catlg.name %in% catlgs128){
             if (!requireNamespace("FrF2.catlg128", quietly=TRUE, character.only=TRUE)) 
                  stop("Package FrF2.catlg128 is not available")
             if (packageVersion("FrF2.catlg128") < numeric_version("1.2")){ 
                  if (catlg.name %in% catlgs128[c(1,3:11)])
                  stop("For this version of package FrF2.catlg128,\n",
                       "load ", catlg.name, " with the command data(", catlg.name,")\n",
                       "and then rerun the FrF2 command.\n",
                       "Alternatively, install the latest version of package FrF2.catlg128.")
                  else stop("You need to get the latest version of package FrF2.catlg128 for using ", catlg.name)
                       }
       }
     else stop(catlg.name, " not available")
     }
    if (!"catlg" %in% class(select.catlg)) stop("invalid choice for select.catlg")
## check validity of center point options
    if (!is.numeric(ncenter)) stop("ncenter must be a number")
    if (!length(ncenter)==1) stop("ncenter must be a number")
    if (!ncenter==floor(ncenter)) stop("ncenter must be an integer number")
    if (is.null(center.distribute)){
      if (!randomize) center.distribute <- min(ncenter, 1)
         else center.distribute <- min(ncenter, 3)}
    if (!is.numeric(center.distribute)) stop("center.distribute must be a number")
    if (!center.distribute==floor(center.distribute)) stop("center.distribute must be an integer number")
    if (center.distribute > ncenter)
       stop("center.distribute can be at most ncenter")
    if (randomize & center.distribute==1) warning("running all center point runs together is usually not a good idea.")
    block.name <- make.names(block.name) ## make block.name a valid R name

## check that no incompatible options are used together
## used || or && instead of | or & whereever appropriate (July 2019)
if (ncenter>0 && !identical(WPs,1)) stop("center points for split plot designs are not supported")
if (!(is.null(generators) || (identical(WPs,1) || !is.null(WPfacs)))) 
    stop("generators can only be used with split-plot designs, if WPfacs are specified.")
if (!is.null(nruns)) if (ncenter>0) if (center.distribute > nruns + 1) 
    stop("center.distribute must not be larger than nruns+1")
if (!(is.null(generators) || is.null(design))) 
  stop("generators and design must not be specified together.")
if (is.null(nruns) & !(is.null(generators))) 
  stop("If generators is specified, nruns must be given.")
if (!(is.null(generators) || is.null(estimable))) 
  stop("generators and estimable must not be specified together.")
if (!(identical(blocks,1) || is.null(estimable))){
  if (!(is.numeric(blocks) && length(blocks)==1)) 
  stop("estimable can only be combined with automatic blocking.")
  if (!clear) message("clear=FALSE will be ignored, ", 
                      "as estimability for blocked designs is only implemented for clear 2fis.")
}
igdom <- ignore.dom ## ignore.dom argument for estimable
if (!(is.null(design) || is.null(estimable))){ 
  ## design and estimable together, 12 August 2019
  ## validity of design was not yet checked
  message("estimability is only attempted for the specified design")
  igdom <- TRUE   ## always ignore dominating entry of one specific design
}
if (!is.null(useV)) {
  if (!is.logical(useV)) stop("useV must be logical")
}
if (!is.logical(firsthit)) stop("firsthit must be logical")
### NULL case will be decided when resolution of ingoing design is known
    
    ## 1.7 it might be desirable to alleviate this constraint, and it might also be possible!
    ## 1.8 alleviated this constraint
if (!(identical(WPs,1) || is.null(estimable))) stop("WPs and estimable must not be specified together.")
if (!(is.null(hard) | is.null(estimable))) stop("hard and estimable must not be specified together.")
if (!(identical(blocks,1) || identical(WPs,1))) stop("blocks and WPs must not be specified together.")
if (!(identical(blocks,1) || is.null(hard))) stop("blocks and hard must not be specified together.")
if (!(identical(WPs,1) || is.null(hard))) stop("WPs and hard must not be specified together.")
if (identical(blocks,1) & !identical(wbreps,1)) stop("wbreps must not differ from 1, if blocks = 1.")
if (!(is.null(WPfacs) || identical(WPs,1)) && is.null(design) && is.null(generators))
          stop("WPfacs requires explicit definition of a design via design or generators.")
if (identical(nfac.WP,0) && is.null(WPfacs) && !identical(WPs,1)) 
          stop("WPs whole plots require specification of whole plot factors", 
               "through nfac.WP or WPfacs!")
if ((nfac.WP > 0 || !is.null(WPfacs)) && identical(WPs, 1)) stop("WPs must be specified for creating a split-plot design")
    ## thus, wbreps = 1 can always be assumed for non-split-plot situations,
    ## which reduces necessary case distinctions
#if ((!identical(WPs,1)) && is.null(nruns)) stop("WPs only works if nruns is specified.")
    ### ??? or is that needed for the automatic version only ???
if (!(is.null(resolution) || is.null(estimable))) stop("You can only specify resolution OR estimable.")
if (!(is.null(resolution) || is.null(nruns))) warning("resolution is ignored, if nruns is given.")

## simple checks for individual option entries
    if (default.levels[1]==default.levels[2]) stop("Both default levels are identical.")
    if (!(is.logical(clear) & is.logical(res3) & is.logical(MaxC2) & is.logical(repeat.only) 
         & is.logical(randomize) & is.logical(alias.block.2fis) & is.logical(force.godolphin)))
         stop("clear, res3, MaxC2, repeat.only, randomize, alias.block.2fis and force.godolphin must be logicals (TRUE or FALSE).")
    if (!is.numeric(max.time)) 
         stop("max.time must be a positive maximum run time for searching a design with estimable given and clear=FALSE.")
    if (!is.numeric(check.hard)) stop("check.hard must be an integer number.")
    if (!is.numeric(check.WPs)) stop("check.WPs must be an integer number.")
    check.hard <- floor(check.hard)  ## avoid stupid errors
    check.WPs <- floor(check.WPs)  ## avoid stupid errors
    if (!is.numeric(bbreps)) stop("bbreps must be an integer number.")
    if (!is.numeric(wbreps)) stop("wbreps must be an integer number.")
    if (!is.numeric(replications)) stop("replications must be an integer number.")
    if (bbreps > 1  & identical(blocks,1) & !replications > 1) 
        stop("Use replications, not bbreps, for specifying replications for unblocked designs.")

    if (!alias.info %in% c(2,3)) 
         stop("alias.info can be 2 or 3 only.")
    if (!(is.numeric(default.levels) | is.character(default.levels))) 
         stop("default.levels must be a numeric or character vector of length 2")
    if (!length(default.levels) ==2) 
         stop("default.levels must be a numeric or character vector of length 2")
    if (!(is.null(hard) | is.numeric(hard))) 
         stop("hard must be numeric.")
    if (!(is.null(resolution) | is.numeric(resolution))) 
         stop("resolution must be numeric.")
    if (is.numeric(resolution)) if(!(resolution == floor(resolution) & resolution>=3)) 
         stop("resolution must be an integer number (at least 3), if specified.")
    res.WP <- NULL   ## for simple way of checking whether split-plot design later on

## treatment of one specifically-chosen design entry
    if (!is.null(design)){ 
        ## design is looked for in the catalogue specified in select.catlg
        if (!is.character(design)) stop("design must be a character string.")
        if (!length(design)==1) stop("design must be one character string.")
           if (design %in% names(select.catlg)){ 
                cand <- select.catlg[design]
                if (!is.null(nruns)) {if (!nruns==cand[[1]]$nruns) 
                          stop("selected design does not have the desired number of runs.")}
                       else nruns <- cand[[1]]$nruns
                if (!is.null(factor.names)) {if (!length(factor.names)==cand[[1]]$nfac) 
                          stop("selected design does not have the number of factors specified in factor.names.")}
                if (!is.null(nfactors)) {if (!nfactors==cand[[1]]$nfac) 
                          stop("selected design does not have the number of factors specified in nfactors.")}
                       else nfactors <- cand[[1]]$nfac
                }
                else stop("invalid entry for design")
    }
    
    ## should occur after special treatment for specific design
    if (!is.null(nruns)){
       k <- round(log2(nruns))
       if (!2^k==nruns) stop("nruns must be a power of 2.")
       if (nruns < 4 | nruns > 4096) stop("less than 4 or more than 4096 runs are not covered by function FrF2.")
       }

    ## check factor specifications
    if (is.null(factor.names) && is.null(nfactors) && (is.null(nruns) || is.null(generators)) && is.null(estimable)) 
         stop("The number of factors must be specified via nfactors, via factor.names, via estimable, through selecting 
         one specific catalogued design or via nruns together with generators.")
    if (!is.null(factor.names) && !(is.character(factor.names) || is.list(factor.names)) ) 
         stop("factor.names must be a character vector or a list.")
    if (is.null(nfactors)) {if (!is.null(factor.names)) nfactors <- length(factor.names)
                          else if (!is.null(generators)) nfactors <- length(generators)+k 
                          }
    if (!is.null(estimable)) {
              if (!is.character(sort)) stop("option sort must be a character string")
              if (!is.character(method)) stop("option method must be a character string")
              if (!sort %in% c("natural","high","low")) stop("invalid choice for option sort")
              if (clear && !method %in% c("LAD","VF2")) stop("invalid choice for option method")
              estimable <- estimable.check(estimable, nfactors, factor.names)
              if (is.null(nfactors)) nfactors <- estimable$nfac
              estimable <- estimable$estimable
              if (is.null(nruns)) {
                 ## make even
                 nruns <- nfactors+ncol(estimable)+1 + (nfactors+ncol(estimable)+1)%%2
                 if (!isTRUE(all.equal(log2(nruns) %% 1,0))) nruns <- 2^(floor(log2(nruns))+1)
                 k <- round(log2(nruns))
                 if (k<3) stop("Please specify nruns and/or nfactors. Calculated values are unreasonable.")
           }
                if (is.null(perm.start)) perm.start <- 1:nfactors
                    else if (!is.numeric(perm.start)) 
                         stop ("perm.start must be NULL or a numeric permutation vector of length nfactors.")
                if (!all(sort(perm.start)==1:nfactors)) 
                         stop ("perm.start must be NULL or a numeric permutation vector of length nfactors.")
                if (!is.null(perms)) {
                   if (!is.matrix(perms) | !is.numeric(perms)) stop("perms must be a numeric matrix.")
                   if (!ncol(perms)==nfactors) stop ("matrix perms must have nfactors columns.")
                   if (any(apply(perms,1,function(obj) any(!sort(obj)==1:nfactors)))) 
                        stop("Each row of perms must be a permutation of 1:nfactors.")
                }
        if (is.null(design)) cand <- select.catlg   ## otherwise, cand has already been set
    }
          ## from here on, estimable is a numeric matrix with two rows
          ## and nfactors is known
    if (!nfactors==floor(nfactors)) 
         stop("nfactors must be an integer number.")
    if (!is.null(factor.names) && !length(factor.names)==nfactors) 
         stop("There must be nfactors factor names, if any.")
    if (is.null(factor.names)) 
         if(nfactors<=50) factor.names <- Letters[1:nfactors] else factor.names <- paste("F",1:nfactors,sep="")
    
    ## check factor level specifications
    if (!((is.character(default.levels) | is.numeric(default.levels)) & length(default.levels)==2) ) 
                 stop("default.levels must be a vector of 2 levels.")
    if (is.list(factor.names)){ 
        if (is.null(names(factor.names))){
            if (nfactors<=50) names(factor.names) <- Letters[1:nfactors]
            else names(factor.names) <- paste("F", 1:nfactors, sep="")
        } 
        if (any(factor.names=="")) factor.names[which(factor.names=="")] <- list(default.levels)}
        else {hilf <- vector("list",nfactors)
              names(hilf) <- factor.names
              hilf[1:nfactors]<-list(default.levels)
              factor.names <- hilf}
    ## from here on, factor.names is a named list
    ## make all names valid R names
      names(factor.names) <- make.names(names(factor.names), unique=TRUE)
    
    if (ncenter > 0) if(any(is.na(sapply(factor.names,"is.numeric"))))
       stop("Center points are implemented for experiments with all factors quantitative only.")

    ### prepare generators case
    genspec <- FALSE  ## for later checks, April 2020
    if (!is.null(generators)){
              genspec <- TRUE
               ## nruns and k are always given for this situation
              generators <- gen.check(k, generators)
                 g <- nfactors - k
                 if (!length(generators)== g) 
                 stop("This design in ", nruns, " runs with ", nfactors," factors requires ", g, " generators.")
              
               res <- NA; nclear.2fis<-NA; clear.2fis<-NULL;all.2fis.clear<-NA
               if (g<10) wl <- words.all(k, generators,max.length=6)
               else if (g<15) wl <- words.all(k, generators,max.length=5)
               else if (g<20) wl <- words.all(k, generators,max.length=4)
               else if (g>=20) wl <- alias3fi(k, generators, order=2)  ## not a word list
               WLP <- NULL
               
               ### add meaningful clear.2fis July 2019
               clear.2fis <- combn(nfactors, 2)
               hilf <- rep(TRUE, choose(nfactors, 2))
               wl4 <- wl[[2]]
               wl4 <- wl4[lengths(wl4)==4]
               if (length(wl4) > 0)
               for (i in 1:choose(nfactors, 2))
                   if (any(sapply(wl4, function(obj) all(clear.2fis[,i] %in% obj)))) hilf[i] <- FALSE
               clear.2fis <- clear.2fis[,hilf]
               nclear.2fis <- ncol(clear.2fis)
               all.2fis.clear <- (nclear.2fis==choose(nfactors, 2))
                     
               if (g < 20){
               WLP <- wl$WLP
                 ## added if for avoiding warning (April 2020)
                 if (all(WLP==0)) res <- Inf else
                 res <- min(as.numeric(names(WLP)[which(WLP>0)]))
                 if (res==Inf) {if (g<10) res="7+"
                   else if (g<15) res="6+"
                   else if (g<20) res="5+" }
                 }
               else{
                   if (!is.list(wl)) res="5+"
                   else{
                   if (length(wl$"main")>0) res="3"
                      else if (length(wl$"fi2")>0) res="4"
                           else res="5+"}
               }

               gen <- sapply(generators,function(obj) which(sapply(Yates[1:(nruns-1)],
                             function(obj2) isTRUE(all.equal(sort(abs(obj)),obj2)))))
               gen <- gen*sapply(generators, function(obj) sign(obj[1]))

               cand <- list(custom=list(res=res, nfac=nfactors, nruns=nruns, 
                    gen=unlist(gen), 
                    WLP=WLP, nclear.2fis=nclear.2fis, clear.2fis=clear.2fis, all.2fis.clear=all.2fis.clear))
               ## needs to be list of list, because access later is always via cand[1]
               class(cand) <- c("catlg","list")
    }
    ## prepare blocks
    ## 1.8: if blocks and estimable go together, does this preparation need to accomodate estimable ?
    ## 1.8: so far, estimable is a two-row matrix, and nfactor and nruns are known (lines 135 to 170)
    ## 1.8: provisionally, it looks as though no adjustments are needed
    ## 1.8: estimable can only be non.null for a specified number of blocks
    if (!identical(blocks,1)) {
             blocks <- block.check(k, blocks, nfactors, factor.names)  
             if (is.list(blocks)) k.block <- length(blocks)
             block.auto=FALSE   ## default for block.auto, set to TRUE below, if TRUE
             map <- NULL   ## initialize, will be filled for use with estimable
             }
             ## blocks is now list of generators or number of blocks
    if (!is.list(blocks)){
    if (blocks>1){
       block.auto=TRUE
       if (is.null(nruns)) 
            stop("blocks>1 only works if nruns is specified.")
       k.block <- round(log2(blocks))
       if (blocks > nruns/2) 
            stop("There cannot be more blocks than half the run size.")
       if (nfactors+blocks-1>=nruns) 
            stop(paste(nfactors, "factors cannot be accomodated in", nruns, "runs with", blocks, "blocks."))
       ntreat <- nfactors
#       nfactors <- nfactors + k.block     #### omitted for new version, that alone is probably not yet the solution
    }
    }
    ### treat hard before WPs, so that it can be handled by split-plot approach
    if (!is.null(hard)){
      if (is.null(generators)){ 
           ## otherwise, cand has already been specified
           if (is.null(nruns)){
              ## resolution requested, k not yet defined
                 cand <- select.catlg[which(res.catlg(select.catlg)>=resolution & nfac.catlg(select.catlg)==nfactors)]
                 if (length(cand)==0) {
                    message("full factorial design needed for achieving requested resolution")
                    k <- nfactors
                    nruns <- 2^k
                    cand <- list(list(gen=numeric(0)))
                    }
                 else {
                     nruns <- min(nruns.catlg(cand))
                     k <- round(log2(nruns))
                     cand <- cand[which(nruns.catlg(cand)==nruns)]
                    }
           }
           else {
               ## nruns specified
               if (nfactors > k)
               cand <- select.catlg[which(nfac.catlg(select.catlg)==nfactors & 
                                            nruns.catlg(select.catlg)==nruns)]
               else cand <- list(list(gen=numeric(0)))
           }
      }
      if (hard == nfactors) stop("It does not make sense to choose hard equal to nfactors.")
      if (hard >= nruns/2) 
          warning ("Do you really need to declare so many factors as hard-to-change ?")
      nfac.WP <- hard
         ## determine WPs (implying k.WP)
         if (hard < nruns/2){
         WPs <- NA
         ## for full factorials, WPs <- 2^hard, 
         ## otherwise achievable WPs is determined using leftadjust 
         if (length(cand[[1]]$gen) > 0)
         for (i in 1:min(length(cand),check.hard)){
             leftadjust.out <- leftadjust(k,cand[[i]]$gen, early=hard, show=1)
             if (is.na(WPs) | WPs > 2^leftadjust.out$k.early) 
                 WPs <- 2^leftadjust.out$k.early
             }
             else WPs <- 2^hard
         }
      if (hard>=nruns/2 | WPs==nruns) {
           warning("There are so many hard-to-change factors that no good special design could be found.
                Randomization has been switched off.")
           randomize <- FALSE
           WPs <- 1
           ## desmat will be the special slow-change matrix
           leftadjust.out <- leftadjust(k,cand[[1]]$gen,early=hard,show=1)
           generators <- leftadjust.out$gen
           ## with this generator and the hard-to-change factors in the earliest columns
      }
    }
    ## prepare WPs
    if (!identical(WPs,1)) {
             if (is.null(nruns)) stop("WPs>1 only works if nruns is specified.")
             if (WPs > nruns/2) stop("There cannot be more whole plots (WPs) than half the run size.")
             k.WP <- round(log2(WPs))
             if (!WPs == 2^k.WP) stop("WPs must be a power of 2.")
             if (!is.null(WPfacs) & nfac.WP==0){
                    nfac.WP <- length(WPfacs)
                    if (nfac.WP < k.WP) stop("WPfacs must specify at least log2(WPs) whole plot factors.")
                 }
             if (nfac.WP==0) stop("If WPs > 1, a positive nfac.WP or WPfacs must also be given.")
             if (nfac.WP < k.WP) {
                 add <- k.WP - nfac.WP
                 names.add <- rep(list(default.levels),add)
                 names(names.add) <- paste("WP",(nfac.WP+1):(nfac.WP+add),sep="")
                 nfactors <- nfactors + add
                 factor.names <- c(factor.names[1:nfac.WP],names.add,factor.names[-(1:nfac.WP)])
                 nfac.WP <- k.WP
                 warning("There are fewer factors than needed for a full factorial whole plot design. ", add, 
                   " dummy splitting factor(s) have been introduced.")
              }
             if (!is.null(WPfacs)) {
                WPfacs <- WP.check(k, WPfacs, nfac.WP, nfactors, factor.names)
                ## WPfacs is now character vector of the form Fno or single number of whole plots
                WPsnum <- as.numeric(chartr("F", " ", WPfacs))
                ## the factor positions within the factor names list from which the design is generated
                WPsorig <- WPsnum 
             }
             else {WPsorig <- WPsnum <- 1:nfac.WP }  ## first nfac.WP factors are whole plot factors
             
             ## here, the original WPsorig has been documented
             ## this must not be overwritten later!
    }
    
    if (!is.null(nruns)){
       ## find FrF2 for given number of runs 
       ## or one specific design (because nruns has already been calculated if necessary)
       ## or estimable given (because nruns has already been calculated if necessary)
       if (nfactors<=k && identical(blocks,1) && identical(WPs,1)) {
                    ### full factorial without blocks or WPs finally treated here
                    ### result immediately returned, i.e. function ends here in this case
                    if (nfactors==k) aus <- fac.design(2, k, factor.names=factor.names, 
                             replications=replications, repeat.only=repeat.only, 
                             randomize=randomize, seed=seed)
                    else aus <- fac.design(2, nfactors, factor.names=factor.names, 
                             replications=replications*2^(k-nfactors), repeat.only=repeat.only, 
                             randomize=randomize, seed=seed)
                    aus <- qua.design(aus, quantitative="none", contrasts=rep("contr.FrF2",nfactors))
                    ## add center points, if requested
                    if (ncenter>0) aus <- add.center(aus, ncenter, distribute=center.distribute)
                    di <- design.info(aus)
                    di$creator <- creator
                    di <- c(di, list(FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version))
                    design.info(aus) <- di
                    return(aus)
              }
       else {
       ## block or WPs not possible for nfactors < k
       if (nfactors < k) stop("A full factorial for nfactors factors requires fewer than nruns runs. 
          Please reduce the number of runs and introduce replications instead.")
       ## artificial generators and cand for full factorial with blocks or WPs
       full <- FALSE
       if (nfactors == k) {
              full <- TRUE
              genspec <- TRUE   ## base.design entry of design.info handled in generators section
              generators <- as.list(numeric(0))
               cand <- list(custom=list(res=Inf, nfac=nfactors, nruns=nruns, 
                    gen=numeric(0), 
                    WLP=c(0,0,0,0), nclear.2fis=choose(k,2), 
                    clear.2fis=combn(k,2), all.2fis.clear="all"))
                   ## needs to be list of list, because access later is always via cand[1]
               class(cand) <- c("catlg","list")
           }
       ### fractional factorial
       if (nfactors > nruns - 1) 
             stop("You can accomodate at most ", nruns-1,
                  " factors in a FrF2 design with ", nruns, " runs." )
         g <- nfactors - k  ## number of generators needed
         if (!is.null(estimable)) {
            ## determine design that satisfies estimability requests
              desmat <- estimable(estimable, nfactors, nruns, 
                clear=clear, res3=res3, max.time=max.time, select.catlg=cand, 
                     method=method,sort=sort,
                     perm.start=perm.start, perms=perms, order=alias.info,
                     ignore.dom=igdom)              ## added ignore.dom, August 2019
              map <- desmat$map                  ## for use in blocking, if needed (July 2019)
              cand <- cand[names(map)]           ## for use in blocking, if needed (July 2019)
              design.info <- list(type="FrF2.estimable", 
                     nruns=nruns, nfactors=nfactors, factor.names=factor.names, 
                     catlg.name = catlg.name,
                     map=desmat$map, aliased=desmat$aliased, clear=clear, res3=res3, 
                     FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
              desmat <- desmat$design
              desmat <- as.matrix(sapply(desmat,function(obj) as.numeric(as.character(obj))))
              rownames(desmat) <- 1:nrow(desmat)
         }
         else if (is.null(generators) && is.null(design)) 
                 cand <- select.catlg[nruns.catlg(select.catlg)==nruns & 
                                        nfac.catlg(select.catlg)==nfactors]
               ## candidate designs for further steps
               ## resolution not here, because ignored for given nruns

            ## treating blocked designs
            ## 1.7 here is where the main changes are needed for allowing to block 
            ## 1.7    designs with estimable 2fis;
            ## 1.7    probably restrict choose(nruns - 1 - nfactors - length(estimable), k.block)
            ## 1.7    also generally probably choose(nruns - 1 - nfactors - ncol2fis, k.block)
            ## 1.7     if alias.block.2fis = FALSE; but do I get ncol2fis?
            block.gen <- NULL    ## for later checks
            if (!is.list(blocks)){
                if (blocks > 1) {
                    ### small case, without estimability requirements
                    if ((g==0 || choose(nruns - 1 - nfactors, k.block) < 100000)  && 
                    is.null(estimable) && !force.godolphin ){
                    for (i in 1:length(cand)){
                      ### loop through possible generator designs from best to worse overall
                      ### break stops the loop as soon as design has been found
                      if (g==0) {blockpick.out <- try(blockpick(k, gen=0, 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
#print(blockpick.out)  ## correct
                            }
                      else {
                      if (is.null(generators))
                      blockpick.out <- try(blockpick(k, design=names(cand[i]), 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
                      else  blockpick.out <- try(blockpick(k, gen=cand[[i]]$gen, 
                            k.block=k.block, show=1, alias.block.2fis = alias.block.2fis),TRUE)
                      }
                      if (!"try-error" %in% class(blockpick.out)) {
                         blocks <- blockpick.out$blockcols   ## column numbers in Yates matrix
                         block.gen <- unlist(c(blocks)) ## for design.info; ## cannot be a list
                         cand <- cand[i]
                         cand[[1]]$gen <- c(cand[[1]]$gen, block.gen)
                         blocks <- nfactors + (1:k.block) ## now in terms of factors
                      #### nfactors changed, will be reduced again later
                         nfactors <- nfactors + k.block
                         g <- g + k.block
                      ### adjust factor.names
                         hilf <- factor.names
                         factor.names <- vector("list", nfactors)
                         factor.names[-blocks] <- hilf
                         factor.names[blocks] <- list(default.levels)
                         names(factor.names) <- c(names(hilf),paste("b",1:k.block,sep=""))
                         blocks <- as.list(blocks)   ### rest is treated with the manual routine
                         break
                      }
                    }## else treats big case
                    }
                else{## blockpick.big up to version 1.7.x
                     ## since 1.8: Godolphin approach
                     ##    if estimable, cand contains a single design 
                     ##            that has been preselected with the estimable function
                     ##    !!! and map has to be observed !!! April 2020
                  for (i in 1:length(cand)){
                    if (is.null(useV)) useV <- cand[[1]]$res > 4 && !is.null(estimable)
  
                    if (useV)
                      X <- colpick(cand[i], k - k.block, estimable=estimable, quiet=TRUE, 
                                    method=method, sort=sort,
                                   select.catlg = cand[i], res3=res3, firsthit=firsthit)
                    else
                      X <- colpickIV(cand[i], k - k.block, estimable=estimable, quiet=TRUE, 
                                   method=method, sort=sort,
                                 select.catlg = cand[i], res3=res3, firsthit=firsthit)
                        if (!is.null(X)) {
                          ## found a successful block structure
                          ## in cand[i]
                          cand <- cand[i]
                          ## all possible generating vectors (character version with letters)
                          block.gen <- blockgencreate(X$X, p=g)
                          ## Yates column numbers
                          block.gen <- sapply(block.gen, function(obj) 
                                        which(names(Yates) %in% obj))  
                          if (any(block.gen %in% cand[[1]]$gen)) 
                                   warning("block generator coincides with main effect")
                          cand[[1]]$gen <- c(cand[[1]]$gen, block.gen)
                          blocks <- nfactors + 1:k.block  ## column numbers for blocks
                          #### nfactors changed, will be reduced again later
                          nfactors <- nfactors + k.block
                          cand[[1]]$nfactors <- nfactors
                          g <- g + k.block
                          ### adjust factor.names
                          if (!is.null(estimable)) map <- X$map
                          hilf <- factor.names # [map]
                          factor.names <- vector("list", nfactors)
                          factor.names[-blocks] <- hilf
                          factor.names[blocks] <- list(default.levels)
                          names(factor.names) <- c(names(hilf),
                                                   paste("b",1:k.block, sep=""))
                          blocks <- as.list(blocks)   
                                  ### rest is treated with the manual routine
                                  ## for a list of added factor columns
                          break
                        }
                    }
                    } ## end else (for big case)
                if (!is.list(blocks)) {
                    stopmsg <- "no adequate blocked design found"
                    if (nruns >= 128 && !full) stopmsg <- c(stopmsg, 
                              " in catalogue ", deparse(substitute(select.catlg)))
                    if (!alias.block.2fis) 
                      stopmsg <- c(stopmsg, " with 2fis unconfounded with blocks")
                    if (!is.null(design))
                      stopmsg <- c(stopmsg, paste(" in design", design))
                    else {
                      if (!is.null(estimable) && !full){
                        stopmsg <- c(stopmsg, paste(" in design", names(cand)))
                      }
                    if (!is.null(estimable) && !full){
                        if (ignore.dom) 
                          stopmsg <- c(stopmsg, " which is the first suitable element of select.catlg")
                        else 
                          stopmsg <- c(stopmsg, " which is the first suitable dominating element of select.catlg")
                        if (!useV && !nfactors==log2(nruns)) 
                          stopmsg <- c(stopmsg, 
                                       " without trying to reshuffle map from unblocked fraction", 
                                       " (useV=TRUE might be worth a try)")
                        }
                    }
                    stop(stopmsg)
                }
                }
                }
            if (is.list(blocks)) {
                # can be a pre-treated automatic situation or a manual specification
                hilf.gen <- c(2^(0:(k-1)), cand[[1]]$gen)    ### cand[[1]] exists 
                ## obtain Yates column numbers for block generators
                hilf.block.gen <- sapply(blocks, function(obj) 
                       as.intBase(paste(rowSums(do.call(cbind,
                          lapply(obj, function(obj2) 
                            digitsBase(hilf.gen[obj2],2,k))))%%2, collapse=""))
                       )
                     ## hilf.block.gen holds Yates matrix column numbers
                ## number of original factors that are manually declared block factors
                k.block.add <- length(intersect(hilf.block.gen, hilf.gen))
                if (is.null(block.gen)) {
                   ## manual allocation 
                   ntreat <- nfactors - k.block.add
                   block.gen <- hilf.block.gen  
                   ## hilf.gen still contains generators for block factors
                   ## these must be kept for the automatic case
                   }
                ## c makes matrix into vector (list remains list)
                ## blockfull works with any of blocks or block.gen or hilf.block.gen
                bf <- blockfull(blocks, k, hilf.gen)  ### obtain all block contrasts
                                                         ### as Yates column numbers
                ### check that manual choices for block entries contain independent entries only
                if (k.block > 1) {if (length(unique(bf)) < 2^k.block - 1)
                    stop("specified blocks is invalid (dependencies)")}

                ## was done with the code below before July 26 2019
                #{hilf <- hilf.block.gen
                #for (i in 2:k.block){
                #       sel <- combn(k.block,i)
                #       for (j in 1:ncol(sel)){
                #          neu <- as.intBase(paste(rowSums(do.call(cbind,
                #                lapply(sel[,j], function(obj) digitsBase(hilf.block.gen[obj],2,k))))%%2,
                #                collapse=""))
                #          if (neu %in% hilf) stop("specified blocks is invalid (dependencies)")
                #          else hilf <- c(hilf, neu)
                #          }
                #   }
                #rm(hilf)
                #}

                ### check for implied confounding of main effects (26 July 2019)
                if (length(intersect(bf, hilf.gen)) > k.block.add){
                  if (alias.block.2fis && length(bf) > 4) 
                    warning(paste("Main effects of factors", 
                             paste(names(factor.names)[hilf.gen %in% bf], collapse=", "), 
                             "are confounded with blocks.", 
                            "The design is a split-plot design",
                            "and has to be analyzed as such",
                            "(only recommended in case of many blocks).", 
                            "Preferrably, construct it with arguments WPs and WPfacs.", 
                            "You may also want to try using function FF_from_X", 
                            "for obtaining a better block structure."))
                  else stop("main effects confounded with blocks")
                }
                ### prepare alias information
                ### remove extra columns for blocks from hilf.gen
                hilf.gen <- setdiff(hilf.gen, block.gen) 
                sel <- combn(ntreat, 2)
                nam2fis <- sapply(1:ncol(sel), 
                                  function(obj) ifelse(ntreat<=50, paste0(Letters[sel[,obj]], collapse=":"), 
                                                                            paste0("F",sel[,obj], collapse=":")))
                twoficols <- sapply(1:ncol(sel), 
                                    function(obj) 
                                      as.intBase(paste(rowSums(do.call(cbind,
                                                                       lapply(sel[,obj], function(obj2) 
                                                                         digitsBase(hilf.gen[obj2],2,k))))%%2, collapse=""))
                )
                names(twoficols) <- nam2fis
                ## check for implied confounding with 2fis, if forbidden (July 26 2019)
                if (!alias.block.2fis)
                  if (length(intersect(bf, twoficols)) > 0){
                    if (force.godolphin) stop("blocks aliased with 2-factor interactions, ", 
                                              "although alias.block.2fis=FALSE; ",
                                              "force.godolphin=TRUE does not attempt ", 
                                              "to avoid confounding of non-clear 2fis")
                    else
                    stop("blocks aliased with 2-factor interactions, ", "although alias.block.2fis=FALSE")
                    }
            if (alias.info == 3){
                sel <- combn(ntreat, 3)
                nam3fis <- sapply(1:ncol(sel), 
                                  function(obj) ifelse(ntreat<=50, paste0(Letters[sel[,obj]], collapse=":"), 
                                                                            paste0("F",sel[,obj], collapse=":")))
                threeficols <- sapply(1:ncol(sel), 
                                      function(obj) 
                                      as.intBase(paste(rowSums(do.call(cbind,
                                                                       lapply(sel[,obj], function(obj2) 
                                                                         digitsBase(hilf.gen[obj2],2,k))))%%2, collapse=""))
                )
                names(threeficols) <- nam3fis
            }
            if (is.null(generators)) generators <- gen.check(k, hilf.gen[-(1:k)])  ## April 4 2020; extract of Yates (list)
            } ## end of is.list(blocks) case
            ## automatic treatment of split-plot designs
            if (WPs > 1){ 
               WP.auto <- FALSE
               map <- 1:k
               orignew <- WPsorig  ## orignew will be changed according to map later
                                   ## for cases without WPfacs
                                   ## this is for orig.fac.order element of design.info, 
                                   ## not for re-arranging columns in design
               if (is.null(WPfacs)){
                    WP.auto <- TRUE
                    ## max.res.5 is the max number of factors for resolution 5, 
                    ##    if k (or k.WP is of size ...
                    max.res.5 <- c(1,2,3, 5, 6, 8, 11, 17)
                    for (i in 1:length(cand)){
                      ## prevent wasting search time on impossible setups
                      if (is.null(generators)){
                        if (cand[[i]]$res>=5 & nfac.WP > max.res.5[k.WP]) next
                        if (cand[[i]]$res>=4 & nfac.WP > WPs/2) next
                      }
                      if (nfac.WP > WPs/2 || nfac.WP <= k.WP)
                      splitpick.out <- try(splitpick(k, cand[[i]]$gen, k.WP=k.WP, nfac.WP=nfac.WP, show=1),TRUE)
                      else  splitpick.out <- try(splitpick(k, cand[[i]]$gen, k.WP=k.WP, nfac.WP=nfac.WP, 
                             show=check.WPs),TRUE)
                      if (!"try-error" %in% class(splitpick.out)) {
                         WPfacs <- 1:k.WP    
                         if (nfac.WP > k.WP) WPfacs <- c(WPfacs, (k + 1):(k+nfac.WP-k.WP))
                         cand <- cand[i]
                         cand[[1]]$gen <- splitpick.out$gen[1,]
                         res.WP <- splitpick.out$res.WP[1]
                         map <- splitpick.out$perms[1,]
                         break
                      }
                    }
                if (is.null(res.WP)){
 #                     if (nruns >= 2^nfactors) stop("currently, splitplot designs for full factorials are not covered.")
                      if (nruns >= 2^nfactors) {
                          res.WP <- Inf
                          WP.auto <- TRUE
                          WPfacs <- 1:k.WP
                          if (!k.WP == nfac.WP) stop(nfac.WP, " whole plot factors cannot be accomodated in ", (2^k.WP), 
                              " whole plots for a full factorial. Please request smaller design with replication instead.")
                          cand <- list(list(gen=numeric(0)))  ## in order to be treatable below
                      }
                      else stop("no adequate splitplot design found")
                }
  #             WPsorig <- WPsnum <- WPfacs    ## this was a mistake, leading to other than the first nfac.WP factors being WP factors
                orignew <- WPsnum <- WPfacs
                WPfacs <- paste("F",WPfacs,sep="")  ## treat with manual below
                }
              }
              ## next closing brace for !(nfactors<=k & identical(blocks,1) & identical(WPs,1))
              }
              ### next closing brace for !is.null(nruns)
              } 
else {
       ## nruns not given and not already assigned for estimable
       ## find smallest FrF2 that fulfills the requirements regarding resolution/estimability
       if (is.null(resolution) && is.null(estimable)) 
                 stop("At least one of nruns or resolution or estimable must be given.")
       if (!is.null(resolution)){ 
                 cand <- select.catlg[which(res.catlg(select.catlg)>=resolution & 
                                              nfac.catlg(select.catlg)==nfactors)]
                 if (length(cand)==0) {
                    message("full factorial design needed")
                    aus <- fac.design(2, nfactors, factor.names=factor.names, 
                             replications=replications, repeat.only=repeat.only, 
                             randomize=randomize, seed=seed)
                    for (i in 1:nfactors) 
                           if (is.factor(aus[[i]])) contrasts(aus[[i]]) <- contr.FrF2(2)
                    ## add center points, if requested
                    if (ncenter>0) aus <- add.center(aus, ncenter, distribute=center.distribute)
                    return(aus)
                    }
            }
       else{ cand <- select.catlg[which(nfac.catlg(select.catlg)==nfactors)]   
           ## no prior restriction by resolution
         if (length(cand)==0) 
             stop("No design listed in catalogue ", deparse(substitute(select.catlg)), 
                  " fulfills all requirements.")
           }
    }
    ## standard FrF2 situations
    ## maximize clear 2fis among maximum resolution designs (res3=FALSE)
    ##     or among all designs (res3=TRUE)
    ##   changed 28 01 2011 from always maximum resolution
    if (MaxC2 && is.null(estimable) && is.null(generators) ) {
           if (!res3)
           cand <- cand[which.max(sapply(cand[which(sapply(cand, 
                   function(obj) obj$res)==max(sapply(cand, function(obj) obj$res)))], 
                   function(obj) obj$nclear.2fis))]
           else
           cand <- cand[which.max(sapply(cand, 
                   function(obj) obj$nclear.2fis))]
            }
    ### creation of actual design 
    if (is.null(nruns)) {nruns <- cand[[1]]$nruns 
                         k <- round(log2(nruns))
                         g <- nfactors - k}
    if (is.null(estimable) || is.list(blocks)){
       ### that is, blocks with or without estimable, 
       ### and anything else without estimable (e.g. WP) 
       ### for estimable with blocks, map has already been processed 
       destxt <- "expand.grid(c(-1,1)"
       for (i in 2:k) destxt <- paste(destxt,",c(-1,1)",sep="")
       destxt <- paste("as.matrix(",destxt,"))",sep="")
       desmat <- eval(parse(text=destxt))
       if (is.character(WPfacs) | is.list(blocks)) {
          desmat <- desmat[,k:1]  ## slow first rather than fast first
          ##rownames(desmat) <- slowfast(k)  ## 17 Feb 2013; this would make the 
          ##   run numbers refer to the Yates matrix row order with A changing fastest
          ##   but would also change row order of the resulting design
          ##   therefore not done; 
          }
       if (!is.null(hard)) {
             desmat <- rep(c(-1,1),each=nruns/2)
             for (i in 2:k) desmat <- cbind(desmat,rep(c(1,-1,-1,1),times=(2^i)/4,each=nruns/(2^i)))
             }
       if (g>0)
       for (i in 1:g) 
       desmat <- cbind(desmat, sign(cand[[1]]$gen[i][1])*apply(desmat[,unlist(Yates[abs(cand[[1]]$gen[i])])],1,prod))
       if (WPs > 1) {
          if (!WP.auto) {
            hilf <- apply(desmat[,WPsorig,drop=FALSE],1,paste,collapse="")
            if (!length(table(hilf))==WPs) 
            stop("The specified design creates ",
                 length(table(hilf)), " and not ", WPs, " whole plots.")
                 for (j in setdiff(1:nfactors,WPsorig))
                     if (!length(table(paste(hilf,desmat[,j],sep="")))>WPs) 
                         stop("Factor ", names(factor.names)[j], " is also a whole plot factor.")
            ## added 19 Feb 2013
            if (nfac.WP<3) res.WP <- Inf
            else res.WP <- GR((3-desmat[,WPsorig,drop=FALSE])%/%2)$GR%/%1           
          }
          }
       
       ## slow changing order, if required
      # if ((!is.character(WPfacs)) & !is.null(hard) ) 
      #     desmat <- desmat[,order(c(2^(0:(k-1)),abs(cand[[1]]$gen)))]

      ## make sure matrix desmat has rownames (14 Feb 2013)
      if (is.null(rownames(desmat))) rownames(desmat) <- 1:nruns
      if (is.list(blocks)) {
           ## manually blocked designs and continuation of automatically blocked designs
           ## including the estimable case (July 2019)
            if (is.null(block.gen)) block.gen <- blocks
            hilf <- blocks
            for (i in 1:k.block) hilf[[i]] <- apply(desmat[,hilf[[i]],drop=FALSE],1,prod)
            Blocks <- factor(as.numeric(factor(apply(matrix(as.character(unlist(hilf)),
                                          ncol=k.block), 1, paste, collapse=""))))
            hilf <- order(Blocks, as.numeric(rownames(desmat)))   ## rownames(desmat) added 17 Feb 2013
            desmat <- desmat[hilf,]
            Blocks <- Blocks[hilf]
            contrasts(Blocks) <- contr.FrF2(levels(Blocks))
            nblocks <- 2^length(blocks)
            blocksize <- nruns / nblocks
            block.no <- paste(Blocks,rep(1:blocksize,nblocks),sep=".")
            }
       if (is.character(WPfacs)) {
            ## make WP factor columns the first ones
            ## make factor.names match this order 
            #### for automatic selection the first WPs factors are WP factors !!!
            #if (nfac.WP > k.WP){
            
            ## are WPsnum is the current factor numbering 
            ## WPsorig is the original numbering before column permutations 
               desmat <- desmat[,c(WPsnum,setdiff(1:nfactors,WPsnum))]
               factor.names <- factor.names[c(WPsorig,setdiff(1:nfactors,WPsorig))]

               plotsize <- round(nruns/WPs)
               
               if (is.null(hard)){
                 hilf <- ord(cbind(desmat[,1:nfac.WP],as.numeric(rownames(desmat))))  ## as.numeric(rownames) added 17 Feb 2013
                 desmat <- desmat[hilf,]
               }
               wp.no <- paste(rep(1:WPs,each=plotsize),rep(1:plotsize,WPs),sep=".")
               
            }
    }

    ## until Feb 14 2013, rownames(desmat) were defined here, 
    ##    which did not make sense, especially since they should be available 
    ##    in already resorted form above for block and wp
    ## 14 Feb 2013: auskommentiert
    ## rownames(desmat) <- 1:nruns

    ## handle randomization and replication
    if (randomize & !is.null(seed)) set.seed(seed)
    if (!(is.list(blocks) || WPs > 1)){
      ## simple randomization situations
      rand.ord <- rep(1:nruns,replications)
      if (replications > 1 & repeat.only) rand.ord <- rep(1:nruns,each=replications)
      if (randomize & !repeat.only) for (i in 1:replications) 
                  rand.ord[((i-1)*nruns+1):(i*nruns)] <- sample(nruns)
      if (randomize & repeat.only) rand.ord <- rep(sample(1:nruns), each=replications)
      }
      else {   
          if (is.list(blocks)){
          #### implement randomization and replication for blocks
          ## bbreps=replications, wbreps=1
          ## if ((!repeat.only) & !randomize)
              rand.ord <- rep(1:nruns, bbreps * wbreps)
              if ((!repeat.only) & !randomize)
                 for (i in 0:(nblocks-1))
                    for (j in 1:wbreps)
                    rand.ord[(i*blocksize*wbreps+(j-1)*blocksize+1):((i+1)*blocksize*wbreps+j*blocksize)] <- 
                         (i*blocksize+1):((i+1)*blocksize)
                    rand.ord <- rep(rand.ord[1:(nruns*wbreps)],bbreps)
              if (repeat.only & !randomize)
                    for (j in 1:wbreps)
                    rand.ord[(i*blocksize*wbreps + (j-1)*blocksize + 1) : 
                          (i*blocksize*wbreps + j*blocksize)] <- 
                               sample((i%%nblocks*blocksize+1):(i%%nblocks*blocksize+blocksize))
              if (wbreps > 1 & repeat.only) rand.ord <- rep(1:nruns,bbreps, each=wbreps)
              if ((!repeat.only) & randomize)
                 for (i in 0:(nblocks*bbreps-1))
                    for (j in 1:wbreps)
                    rand.ord[(i*blocksize*wbreps + (j-1)*blocksize + 1) : 
                          (i*blocksize*wbreps + j*blocksize)] <- 
                               sample((i%%nblocks*blocksize+1):(i%%nblocks*blocksize+blocksize))
                               
              if (repeat.only & randomize)
                 for (i in 0:(nblocks*bbreps-1))
                    rand.ord[(i*blocksize*wbreps + 1) : 
                          ((i+1)*blocksize*wbreps)] <- rep(sample((((i%%nblocks)*blocksize)+1):
                                        ((i%%nblocks+1)*blocksize)),each=wbreps)
              }
          else {
          #### implement randomization and replication for split-plots
          ####   standard approach; replications replicate complete plots
          ####   repeat.only generate repeated measurements within plots
                rand.ord <- rep(1:nruns,replications)
                if (replications > 1 & repeat.only) 
                                  rand.ord <- rep(1:nruns,each=replications)
                if ((!repeat.only) & randomize){
                    ## whole plots are replicated
                    ## might be more wise to use larger design

                    ## randomization on two levels
                    ## first: randomization within each whole plot
                    for (i in 1:(WPs*replications)) 
                        rand.ord[((i-1)*plotsize+1):(i*plotsize)] <- 
                            sample(rand.ord[((i-1)*plotsize+1):(i*plotsize)])
                    ## second: randomization of whole plots (in blocks per replication)
                    for (i in 1:replications){
                    ## only if not hard to change
                    if (is.null(hard)){
                        WPsamp <- sample(WPs) 
                        WPsamp <- (rep(WPsamp,each=plotsize)-1)*plotsize + rep(1:plotsize,WPs)
                        rand.ord[((i-1)*plotsize*WPs+1):(i*plotsize*WPs)] <- 
                            rand.ord[(i-1)*plotsize*WPs + WPsamp]
                        }
                    }
                    }
                if (repeat.only & randomize){
                    ## repeated measurements within whole plots
                    ## repetition of complete whole plots is not covered, 
                    ##     as I don't think that this makes sense

                    ## first: randomization within each whole plot
                    for (i in 1:WPs) 
                        rand.ord[((i-1)*plotsize*replications+1):(i*plotsize*replications)] <- 
                            rand.ord[(i-1)*plotsize*replications + 
                               rep(replications*(sample(plotsize)-1),each=replications) + 
                               rep(1:2,each=plotsize)]
                    ## second: randomization of whole plots
                    ## only if not hard to change
                    if (is.null(hard)){
                        WPsamp <- sample(WPs) 
                        WPsamp <- (rep(WPsamp,each=plotsize*replications)-1)*plotsize*replications + 
                             rep(1:(plotsize*replications),WPs)
                        rand.ord <- rand.ord[WPsamp]
                    }
                    }
          }
      }          
    orig.no <- rownames(desmat)
    orig.no <- orig.no[rand.ord]
    ## row added 27 01 2011 (for proper ordering of factor levels)
    orig.no.levord <- sort(as.numeric(orig.no),index=TRUE)$ix

    rownames(desmat) <- NULL
    desmat <- desmat[rand.ord,]
        
    if (is.list(blocks)) {
              Blocks <- Blocks[rand.ord]
              block.no <- block.no[rand.ord]
              }
    if (WPs > 1) wp.no <- wp.no[rand.ord]
      colnames(desmat) <- names(factor.names)
      ## adapt original number to replications
      if (is.list(blocks)) orig.no <- paste(orig.no,block.no,sep=".")
      if (WPs > 1) orig.no <- paste(orig.no,wp.no,sep=".")
      orig.no.rp <- orig.no
      if (bbreps * wbreps > 1){
           if (bbreps > 1) {
                ## !repeat.only covers all blocked cases and the repeat.only standard cases
                ## since bbreps stands in for replications
                if (repeat.only & !is.list(blocks))
                orig.no.rp <- paste(orig.no.rp, rep(1:bbreps,nruns),sep=".")
                else
                orig.no.rp <- paste(orig.no.rp, rep(1:bbreps,each=nruns*wbreps),sep=".")
           }
           if (wbreps > 1){
                ## blocked with within block replications
                if (repeat.only) 
                     orig.no.rp <- paste(orig.no.rp, rep(1:wbreps,nruns*bbreps),sep=".")
                else orig.no.rp <- paste(orig.no.rp, rep(1:wbreps,each=blocksize,nblocks*bbreps),sep=".")
           }
    }

    desdf <- data.frame(desmat)
    quant <- rep(FALSE,nfactors)
    for (i in 1:nfactors) {
        ## make all variables into factors
        desdf[,i] <- des.recode(desdf[,i],"-1=factor.names[[i]][1];1=factor.names[[i]][2]") 
        quant[i] <- is.numeric(desdf[,i])
        desdf[,i] <- factor(desdf[,i],levels=factor.names[[i]]) 
        contrasts(desdf[,i]) <- contr.FrF2(2)
        }

    if (is.list(blocks)) {
        desdf <- cbind(Blocks, desdf, stringsAsFactors=TRUE)
        ## make sure that experimental factors are assigned to those 
        ## columns that reflect their estimability needs

        ## blocking makes approach analogous to function map2design more complicated
        ## reassign levels for Blocks to concatenation of manual build factors
        ## if they are built from only single factors
        ## !!! it is important to maintain the original order of numbered blocks factor
        ##     for more than four blocks (because e.g. grouping 1,2,3,7 vs. 4,5,6,8 not
        ##     orthogonal to everything else !
        hilf <- blocks
        if (all(sapply(hilf,length)==1) && !block.auto) {
            hilf <- as.numeric(hilf) + 1
            levels(desdf$Blocks) <- unique(apply(desdf[,hilf,drop=FALSE],1,paste,collapse=""))
            }
        colnames(desdf)[1] <- block.name
        ## delete any single block generator columns
        hilf <- blocks
        hilf <- as.numeric(hilf[which(sapply(hilf,length)==1)])
        if (length(hilf)>0) {
            desdf <- desdf[,-(hilf+1)]
            desmat <- desmat[,-hilf]
            factor.names <- factor.names[-hilf]
        }

        if (!is.null(estimable)) {
          ## April 2020
          #factor.names <- factor.names[map]
          factor.names <- factor.names[invperm(map)]
          names(desdf) <- c(block.name, names(factor.names))
        }
        
        ## prepend block model matrix to desmat (not only generators but all block main effects)  
        desmat <- cbind(model.matrix(~desdf[,1])[,-1],desmat)
        colnames(desmat)[1:(2^k.block-1)] <- paste(block.name,1:(2^k.block-1),sep="")
        ## new alias approach, July 26 2019
        MEcols <- hilf.gen
                ## remove single factor block generators ? via -hilf
                ## but what to do with twoficols in that case ?
                ## also remove related twoficols ?
                         
        if (ntreat<=50) names(MEcols) <- Letters[1:ntreat] else names(MEcols) <- paste0("F",1:ntreat)
        effs <- c(MEcols, twoficols)
        if (alias.info==3) effs <- c(effs, threeficols)
        aliased.with.blocks <- names(effs[effs %in% bf])
        effs <- effs[!effs %in% bf]   ## only those effects that are not aliased with blocks
        aliased <- split(names(effs), effs) ### list of effects on the same non-block Yates matrix column
        aliased <- aliased[lengths(aliased) > 1]    ## columns with more than one effect
      ### code before July 26 2019: this part was really slow for large designs or many blocks
      # if (alias.info==3)
      #  hilf <- aliases(lm((1:nrow(desmat))~(.)^3,data=data.frame(desmat)))
      #  else 
      #  hilf <- aliases(lm((1:nrow(desmat))~(.)^2,data=data.frame(desmat)))
        ## check against blockpick.out here, if possible?
        ## blockpick.out$alias.2fis.block
        
        ## delete all aliases of Block factors themselves
        ## with or without "-"
        ## as these are not interesting
      #  if (length(hilf$aliases) > 0)
      #  for (i in 1:length(hilf$aliases)) {
      #       txt <- hilf$aliases[[i]]
      #       if (length(grep(paste("^-?",block.name,sep=""),txt)) > 0)
      #            txt <- txt[-grep(paste("^-?",block.name,sep=""),txt)] 
      #            if (length(txt)==length(grep("^-",txt))) txt <- sub("-","",txt)
      #            hilf$aliases[[i]] <- txt
      #       }
      #  ## determine treatment effects that are aliased with blocks
      #  aliased.with.blocks <- hilf$aliases[1:(2^k.block-1)]
      #     aliased.with.blocks <- unlist(aliased.with.blocks)
      #  ## interim result for updating alias information
        if (nfactors<=50) leg <- paste(Letters[1:ntreat],names(factor.names),sep="=")
        else leg <- paste(paste("F",1:ntreat,sep=""),names(factor.names),sep="=")
        if (length(aliased.with.blocks)==0) aliased.with.blocks <- "none"
           else {
## recalc.alias.blocked destroys the structure, 
## if factor.names deviates from factor letters
             aliased.with.blocks <- recalc.alias.block.new(aliased.with.blocks, leg)
             #aliased.with.blocks <- recalc.alias.block(aliased.with.blocks, leg)
             aliased.with.blocks <- aliased.with.blocks[ord(data.frame(nchar(aliased.with.blocks),aliased.with.blocks))]
             if (nfactors <= 50) aliased.with.blocks <- gsub(":","",aliased.with.blocks)
           }
      ## determine treatment effects that are aliased with each other
      #  aliased <- hilf$aliases[-(1:(2^k.block-1))]
      #  aliased <- aliased[which(sapply(aliased,length)>1)]
        ## update: same format like aliased element of unblocked designs
        ## July 2019: adapt to new blocking strategy
#        if (length(aliased)>0) aliased <- struc.aliased.new(recalc.alias.block(aliased, leg),
        if (length(aliased) > 0) aliased <- struc.aliased.new(struc=recalc.alias.block.new(aliased, leg), 
                                                                     nk=nfactors, order=alias.info)
        ## prepare design info for blocked designs
        ntreat <- ncol(desdf) - 1
        if (block.auto) factor.names <- factor.names[1:ntreat]
 #       if (block.auto) factor.names <- factor.names[setdiff(1:nfactors,blocks)]
 
 ## up to version 0.96-1 nfactors was ntreat+1
 #         design.info <- list(type="FrF2.blocked", block.name=block.name, 
 #           nruns=nruns, nfactors=ntreat+1, nblocks=nblocks, blocksize=blocksize, 
 #           ntreat=ntreat,factor.names=factor.names,
 #           aliased.with.blocks=aliased.with.blocks, aliased=aliased,
 #           bbreps=bbreps, wbreps=wbreps)
        
        ## create new type FrF2.blocked.estimable ??? decided against this
          design.info <- list(type="FrF2.blocked", block.name=block.name, 
            nruns=nruns, nfactors=ntreat, nblocks=nblocks, block.gen=block.gen, blocksize=blocksize, 
            ntreat=ntreat,factor.names=factor.names,
            aliased.with.blocks=aliased.with.blocks, aliased=aliased,
            bbreps=bbreps, wbreps=wbreps, 
            FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
        if (!is.null(generators) && genspec) {## April 2020 added genspec
           if (g>0) 
           design.info <- c(design.info, 
                list(base.design=paste("generator columns:", paste(setdiff(cand[[1]]$gen, block.gen), collapse=", "))))
           else 
           design.info <- c(design.info, 
                list(base.design="full factorial"))
                }
           else design.info <- c(design.info, list(catlg.name = catlg.name, base.design=names(cand[1])))
           if (!is.null(estimable)) design.info <- c(design.info, list(map=map))
        
        if (bbreps>1) {
            hilflev <- paste(rep(levels(desdf[,1]), each=bbreps), rep(1:bbreps, nblocks), sep=".")
            desdf[,1] <- factor(paste(desdf[,1], rep(1:bbreps, each=nruns*wbreps),sep="."), levels=hilflev)
        }
            ## make block names reflect the between block replication
        }   ## end of blocked designs

        if (WPs > 1){
            
            ## output for split-plot designs
            if (alias.info==3)
            aliased <- aliases(lm((1:nrow(desmat))~(.)^3,data=data.frame(desmat)))$aliases
            else
            aliased <- aliases(lm((1:nrow(desmat))~(.)^2,data=data.frame(desmat)))$aliases
            aliased <- aliased[which(sapply(aliased,length)>1)]
            ## update: same format with aliased entry of other FrF2 designs
            if (length(aliased) > 0){ 
                if (nfactors<=50) leg <- paste(Letters[1:nfactors],names(factor.names),sep="=")
                else leg <- paste(paste("F",1:nfactors,sep=""),names(factor.names),sep="=")
                aliased <- struc.aliased(recalc.alias.block(aliased, leg), nfactors, alias.info)
                }

            design.info <- list(type="FrF2.splitplot", 
                nruns=nruns, nfactors=nfactors, nfac.WP=nfac.WP, nfac.SP=nfactors-nfac.WP, 
                      factor.names=factor.names,
                nWPs=WPs, plotsize=nruns/WPs, 
                res.WP=res.WP, aliased=aliased, FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
            ## below: changed entry in base.design to original generators, combined with map
            ## instead of cand[[1]]$gen from before
            ##    bug fix July 2019: 
            ##       provide the columns for character generators 
            ##       and provide them in original order instead of resorted for all cases
            ##       (was wrong use of which)
            if (!is.null(generators) && genspec){ 
                gennam <- names(generators)   ## generator names can only refer to basic factors
                if (is.null(gennam))
                  if (is.list(generators)) 
                    gennam <- sapply(generators, function(obj) paste0(Letters[sort(obj)], collapse=""))
                  else if(is.character(generators)) gennam <- generators  
                gencols <- sapply(gennam, function(obj) which(names(Yates)[1:(nruns-1)] == obj))
                design.info <- c(design.info, 
                     list(base.design=paste("generator columns:", paste(gencols, collapse=", ")), 
                     map=map, 
                     orig.fac.order = c(orignew, setdiff(1:nfactors,orignew))))
                }
                else design.info <- c(design.info, list(catlg.name = catlg.name, 
                                                        base.design=names(cand[1]), 
                     map=map, 
                     orig.fac.order = c(orignew, setdiff(1:nfactors,orignew))))
      } ## end of WPs > 1

## should this be changed to omit the condition on estimable ? July 30 2019
    #    if (is.null(generators) && !(is.list(blocks) || WPs > 1))
    if (is.null(estimable) && is.null(generators) && !(is.list(blocks) || WPs > 1))
         design.info <- list(type="FrF2", nruns=nruns, nfactors=nfactors, factor.names=factor.names, 
            catlg.name = catlg.name,
            catlg.entry=cand[1], aliased = alias3fi(k,cand[1][[1]]$gen,order=alias.info), 
            FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
        ## incorporate reasonable further info (blocks, WPs etc.)
    if ((!is.null(generators)) && !is.list(blocks) && !WPs > 1) {
         if (nfactors <= 50)
         names(generators) <- Letters[(k+1):nfactors]
         else names(generators) <- paste("F",(k+1):nfactors, sep="")
         gen.display <- paste(names(generators),sapply(generators,function(obj){
               if (nfactors <= 50)
               paste(if (obj[1]<0) "-" else "", paste(Letters[abs(obj)],collapse=""),sep="")
               else 
               paste(if (obj[1]<0) "-" else "", paste(paste("F",abs(obj),sep=""),collapse=":"),sep="")}), sep="=")
         design.info <- list(type="FrF2.generators", nruns=nruns, nfactors=nfactors, factor.names=factor.names, generators=gen.display, 
               aliased = alias3fi(k,generators,order=alias.info), 
               FrF2.version = sessionInfo(package="FrF2")$otherPkgs$FrF2$Version)
         }
    aus <- desdf
      rownames(aus) <- rownames(desmat) <- 1:nrow(aus)
    class(aus) <- c("design","data.frame")
    attr(aus,"desnum") <- desmat
      ## change 27 Jan 2011: leave orig.no as a factor, but with better-ordered levels
      orig.no <- factor(orig.no, levels=unique(orig.no[orig.no.levord]))
      ## added 14 Feb 2013
      orig.no.rp <- factor(orig.no.rp, levels=unique(orig.no.rp[orig.no.levord]))
    if (!(is.list(blocks) || WPs > 1)) 
       attr(aus,"run.order") <- data.frame("run.no.in.std.order"=orig.no,"run.no"=1:nrow(desmat),
           "run.no.std.rp"=orig.no.rp, stringsAsFactors=TRUE)
    else attr(aus,"run.order") <- data.frame("run.no.in.std.order"=orig.no,"run.no"=1:nrow(desmat),
           "run.no.std.rp"=orig.no.rp, stringsAsFactors=TRUE)
    ## make repeat.only reflect the calculated status instead of the status requested by the user 
    ## (which can be seen in the creator element)
    if (design.info$type=="FrF2.blocked") {
        if (design.info$wbreps==1) repeat.only <- FALSE
        design.info$block.old <- block.old   ## April 4 2020
        nfactors <- ntreat
    }
    if (nfactors<=50) design.info$aliased <- c(list(legend=paste(Letters[1:nfactors],names(factor.names),sep="=")),
                     design.info$aliased)
       else design.info$aliased <- c(list(legend=paste(paste("F",1:nfactors,sep=""),names(factor.names),sep="=")),
                     design.info$aliased)
    attr(aus,"design.info") <- c(design.info, list(replications=replications, repeat.only=repeat.only,
      randomize=randomize, seed=seed, creator=creator))
    ## add center points, if requested
    if (ncenter>0) aus <- add.center(aus, ncenter, distribute=center.distribute)
    aus
} 

library(FrF2)
### test programs for FrF2
### FrF2 functionality
### combination of everything with everything is not included, 
###    but presumably also not needed
FrF2(8,4,randomize=FALSE)
set.seed(98776)
test <- FrF2(8,4,replications=2, repeat.only=FALSE)
set.seed(98776)
test <- FrF2(8,4,replications=2, repeat.only=TRUE)
run.order(test)
test
test <- FrF2(8,4,replications=2, repeat.only=TRUE, randomize=FALSE)
test <- FrF2(8,4,replications=2, repeat.only=FALSE)
run.order(test)
test

strOptions(drop.deparse.attr = FALSE)
str.wover <- function(obj){
    di <- attr(obj, "design.info") 
    if (!is.null(di$orig.design.info)) di$orig.design.info <- di$orig.design.info[-which(names(di$orig.design.info)=="FrF2.version")]
    attr(obj, "design.info") <- di[-which(names(di)=="FrF2.version")]
    str(obj)
}

str.wover(FrF2(16,6,randomize=FALSE, factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs="")))
str.wover(FrF2(16,randomize=FALSE, factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs="")))
str.wover(FrF2(16,design="6-2.2",randomize=FALSE))
str.wover(FrF2(16,generators=catlg[["6-2.2"]]$gen,randomize=FALSE,factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs="")))
## alias structure for three generators that differ only by sign
design.info(FrF2(16,generators=c(7,13,15),randomize=FALSE))$aliased
design.info(FrF2(16,generators=c(7,-13,15),randomize=FALSE))$aliased
design.info(FrF2(16,generators=c(-7,-13,-15),randomize=FALSE))$aliased

str.wover(FrF2(16,design="6-2.2",randomize=FALSE,factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs="")))
str.wover(FrF2(16,design="6-2.2",nfactors=6,randomize=FALSE))
str.wover(FrF2(16,design="6-2.2",randomize=FALSE,factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs="")))

## estimable in examples
## replications in examples
## randomize in examples
## MaxC2 in examples

## blocks
  ## automatic selection
  FrF2(32,14,blocks=8,randomize=FALSE,alias.block.2fis=TRUE)
  ## automatic selection without aliasing of 2fis with blocks
  FrF2(32,6,blocks=4,randomize=FALSE)
  ## automatic selection with factor names
  FrF2(32,6,blocks=4,randomize=FALSE, factor.names=Letters[20:25])
  ## manual selection
  ##    new with version 2: checks for forbidden behavior
  FrF2(32,6,blocks="A",randomize=FALSE)
  FrF2(32,6,blocks=c("ABE","ACD"),randomize=FALSE, alias.block.2fis=TRUE)
  FrF2(32,8,blocks=list(c(1,2,3),c(4,5)),randomize=FALSE, alias.block.2fis=TRUE)
  FrF2(32,8,blocks=list(c(1,2,3,4),c(7,8)),randomize=FALSE, alias.block.2fis=TRUE)
  FrF2(32,8,blocks=list(2,5,7),randomize=FALSE, alias.block.2fis=TRUE)
  ## mixed single and generated
  str.wover(FrF2(32,8,blocks=list(2,5,c(6,7)),randomize=FALSE, alias.block.2fis=TRUE))
  ## with factor names
  str.wover(FrF2(32,blocks=c("ABE","ACD"),
       factor.names=list(first=c("-","+"),second=c("old","new"),
                         third=c(1000,2000),fourth="",fifth="",sixth=""),
       randomize=FALSE, alias.block.2fis=TRUE))
  str.wover(dat <- FrF2(32,blocks=c(7,20,25),
       factor.names=list(first=c("-","+"),second=c("old","new"),
                         third=c(1000,2000),fourth="",fifth="",sixth=""),
       randomize=FALSE,alias.info=3, alias.block.2fis=TRUE))
  str.wover(FrF2(32,blocks=list(1,2),
       factor.names=list(first=c("-","+"),second=c("old","new"),
                         third=c(1000,2000),fourth="",fifth="",sixth="",seven="",eight="",nine=""),
       randomize=FALSE, alias.block.2fis=TRUE))
  ## using blockpick.big
  str.wover(FrF2(64,7,blocks=32,alias.block.2fis=TRUE,randomize=FALSE, block.old=TRUE))
  ## using godolphin
  str.wover(FrF2(64,7,blocks=32,alias.block.2fis=TRUE,randomize=FALSE))

  ## godolphin with estimable
  str.wover(FrF2(64,9,blocks=16,factor.names=Letters[15:23], 
       estimable=compromise(9,3)$requirement,
       alias.block.2fis=TRUE,randomize=FALSE))
  
  ## godolphin with estimable and full factorial
  str.wover(FrF2(64,6,blocks=16,factor.names=Letters[15:20], 
                 estimable=compromise(6,3)$requirement,
                 alias.block.2fis=TRUE,randomize=FALSE, firsthit=FALSE))
  
  ## godolphin with estimable and the need that ignore.dom takes the value TRUE
  ## (which it should automatically do due to its default)
  str.wover(FrF2(64, 9, blocks=16, estimable=compromise(9,1:2)$requirement, 
       alias.block.2fis = TRUE, select.catlg = catlg[-1378]))
  summary(FrF2(256, 13, blocks=64, alias.block.2fis=TRUE, 
               factor.names = list(C1="",C2="",C3="",C4="",C5="",C6="",C7="",C8="",
                                   C9="",C10="",C11="",C12="",
                                   N1=c("low","high")), 
               default.levels = c("current","new"),
               estimable=compromise(13, 1)$requirement))

## split plot
  ## automatic selection
  FrF2(32,14,WPs=8,nfac.WP=4,randomize=FALSE)
  FrF2(32,14,WPs=8,nfac.WP=2,randomize=FALSE)
  test <- FrF2(32,14,WPs=8,nfac.WP=6,randomize=FALSE)
  str.wover(test[32:1,])
  str.wover(test[14:1,])
  str.wover(test[,14:1])
  str.wover(test[14:1,,drop.attr=FALSE])

## replications in combination with blocks
## (with randomization in examples)
run.order(FrF2(8,3,blocks=2,wbreps=2,randomize=FALSE))
run.order(FrF2(8,3,blocks=2,wbreps=2,repeat.only=TRUE,randomize=FALSE))
run.order(FrF2(8,3,blocks=2,bbreps=2,randomize=FALSE))
run.order(FrF2(8,3,blocks=2,bbreps=2,wbreps=2,randomize=FALSE))
FrF2(8,3,blocks=2,bbreps=2,wbreps=2,repeat.only=TRUE,randomize=FALSE)
set.seed(68702)
run.order(FrF2(8,3,blocks=2,bbreps=2,wbreps=2,randomize=TRUE))
set.seed(68702)
run.order(FrF2(8,3,blocks=2,bbreps=2,wbreps=2,repeat.only=TRUE,randomize=TRUE))

## replications in combination with split plot 
  FrF2(32,14,WPs=8,nfac.WP=2,randomize=FALSE, replications=3)
  FrF2(32,14,WPs=8,nfac.WP=6,randomize=FALSE, replications=3)
  set.seed(123451)
  FrF2(32,14,WPs=8,nfac.WP=6,replications=3)

## generators in combination!!!

## blocks with more than 50 factors
str.wover(FrF2(128, 55, blocks=64, alias.block.2fis = TRUE))

library(FrF2)
### test programs for FrF2
### pb functionality
### combination of everything with everything is not included, 
###    but presumably also not needed
pb(8,randomize=FALSE)
pb(8,nfactors=4,randomize=FALSE)
## n12 with and without Taguchi
pb(12,n12.taguchi=FALSE,randomize=FALSE, default.levels=c("-","+"))
   ## prevent error that both default.levels are identical ?
   try(pb(12,n12.taguchi=FALSE,randomize=FALSE, default.levels=c("-","-")))
pb(12,n12.taguchi=FALSE,randomize=FALSE, replications=2, repeat.only=TRUE)
pb(12,n12.taguchi=FALSE,randomize=FALSE, replications=2, repeat.only=FALSE)
set.seed(98776)
pb(12,n12.taguchi=FALSE,replications=2, repeat.only=FALSE)
pb(12,n12.taguchi=FALSE,randomize=FALSE, factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs=""))
pb(12,n12.taguchi=TRUE,randomize=FALSE)
## box-tyssedal
pb(16,nfactors=12,randomize=FALSE)
pb(16,nfactors=13,randomize=FALSE)
pb(16,nfactors=15,randomize=FALSE)
## doubling
pb(32, randomize=FALSE)
## special cases
pb(28, randomize=FALSE)
pb(52, randomize=FALSE)
pb(76, randomize=FALSE)
pb(92, randomize=FALSE)

blockpick <- function(k, gen, k.block, design=NULL, show=10, alias.block.2fis=FALSE, select.catlg=catlg){
  ## function that picks design with right number of blocks (power of 2, 2^k.block) 

  ## k.block is the number of independent block factors needed for block construction
  ##   (log2 of number of blocks)
  ## usual assumption: their interactions with treatment factors can be neglected

  ## 2^k.block-1 columns are used exclusively for blocks

  ## if requested (default), alias.block.2fis=FALSE stops confounding of treatment 2fis with blocks
  ## if this is not requested, no attempt is made to reduce the number of aliased 2fis, 
  ##       but they are reported

  if (!is.logical(alias.block.2fis)) stop("alias.block.2fis must be a logical.")
  if (!is.null(design)){ 
     if (!is.character(design)) stop("If specified, design must be a character string.")
     if (!design %in% names(select.catlg)) stop("If specified, design must be the name of a design in select.catlg.")
     if (!missing(gen)) warning("If design is specified, gen is ignored.")
     }
  if (is.null(design) & missing(gen)) stop("One of gen or design must be given.")
  if (!"catlg" %in% class(select.catlg)) stop("select.catlg must be a catalogue of designs of class catlg.")

  if (is.character(design)) {
       if (!design %in% names(select.catlg)) stop ("Design must be a valid design name.")
       if (select.catlg[[design]]$res == 3 & !alias.block.2fis)
           stop("Resolution III design. Package FrF2 does not allow to request all 2fis to be clear of blocks.")
       if (!select.catlg[[design]]$nruns == 2^k) stop("mismatch between k and design")
       gen <- select.catlg[[design]]$gen
       res <- select.catlg[[design]]$res
  }
  bcols <- NULL
  if (identical(gen,0) || length(gen)==0) {
      ## treat full factorials by catalogue (block.catlg)
      ## or by Godolphin (since April 2020)
      ##    before: error for untreated in block.catlg
      ##        or blocked design based on MA catlg entry
      gen <- numeric(0)
      if (k <= max(block.catlg$k) && k.block <= ncol(block.catlg)-2){
        if (length(which(block.catlg$k==k & block.catlg$k.block==k.block)) > 0)
        bcols <- block.catlg[which(block.catlg$k==k & block.catlg$k.block==k.block)[1],
                         paste("b",1:k.block,sep="")]
        # else stop("This full factorial is not in the catalogue.")
      }
      # else bcols <- catlg[[paste(k+k.block,"-",k.block,".",1,sep="")]]$gen
      ## if bcols has been found, full factorial will not be treated by 
      ## Godolphin method
      ## return result now
      if (is.null(bcols)) bcols <- which(names(Yates) %in% blockgencreate(colpick(k, k - k.block)$X))
      if (!is.null(bcols)) return(list(gen=gen, 
           basics = c(nruns=2^k, nblocks=2^k.block, ntreat=k, res.base=Inf), 
           blockcols=bcols))
      }
  g <- length(gen)
  minus <- 1
  if (g>0){
     minus <- which(gen<0)
     gen <- abs(gen)
  }

  hilf <- c(k,gen,k.block,show)
  if (!is.numeric(hilf))
      stop ("k, gen, k.block and show must be numeric.")
  if (!all(hilf == floor(hilf) & hilf > 0))
      stop ("k, gen, k.block and show must contain integer numbers.")
  if (!k >= 3) stop ("blockpick requires k>=3.")
  if (!k.block < k) stop ("blockpick requires k.block < k.")
  if (any(gen %in% 2^(0:(k-1)))) 
        stop ("gen must not contain column numbers of base factors.")
  if (any(!gen %in% 3:(2^k-1))) 
        stop ("Column numbers in gen must be smaller than ", 2^k,".")
        
  ## assignment of k.block factors to the remaining columns
  ## 
  hilf <- cols(k, gen) ## elements main, fi2s and freecols

  fi2s <- hilf$fi2s
        names(fi2s) <- apply(combn(k+g,2),2,function(obj) paste(Letters[obj],collapse=""))
        minus2fis <- which(apply(combn(k+g,2),2,function(obj) length(obj %in% minus)==1))
        names(fi2s)[minus2fis] <- paste("-",names(fi2s)[minus2fis],sep="")
  nfi2s <- table(fi2s)   
  fi2cols <- as.numeric(names(nfi2s))
  if (is.null(design)) {
      if (any(2^(0:(k-1)) %in% fi2cols)) res <- 3
      else if (any(nfi2s>1)) res <- 4
      else res="5+"
  }
  if (alias.block.2fis) {banned <- hilf$main; eligible <- c(fi2cols, hilf$freecols)}
  else {banned <- c(hilf$main,fi2cols); eligible <- hilf$freecols}
  if (length(eligible)<2^k.block-1 & !alias.block.2fis) 
          stop("no adequate block design found with 2fis unconfounded with blocks")
  if (length(eligible) < 2^k.block-1) stop("no adequate block design found")
  if (g > 0)
  perm <- t(combn(length(eligible),k.block))  ## rows are possible selections of k.block
  else {
      if (g==0 && !alias.block.2fis && !is.null(bcols)) if (!all(bcols %in% eligible))
             stop("no adequate block design found with 2fis unconfounded with blocks")
      perm <- matrix(which(eligible %in% bcols),nrow=1)
  }

  hilf <- digitsBase(eligible,ndigits=k) ## always k rows
         ## potential block columns as binary numbers
  ergeb <- matrix(0,nrow=nrow(perm),ncol=2^k.block-1)
         ## will contain generating and main effects columns for blocks
  banned.block <- 99
         ## will contain number of block main effects aliased with banned columns
  dependent.block <- TRUE
         ## will indicate presence of aliasing among block main effect columns
  n2fis.block <- NA
         ## number of 2fis aliased with block main effects
  n2fis.clear <- NA
         ## number of clear 2fis
  hilfc <- Yates[1:(2^k.block-1)]
         ## combination patterns of block factors into all block main effects
  count <- 0 ## number of possibilities found
  i <- 0 ## current position in vectors
  stopp <- FALSE
  last <- numeric(0)
  
  ## loop through selections of block columns
  while (!stopp){
      perm <- perm[setdiff(1:nrow(perm),last),,drop=FALSE]
      i <- i+1
      bg <- eligible[perm[1,]]  ## block generators
     bcols <- sapply(lapply(hilfc, function(obj) mult.gen(Yates[bg[obj]])), 
                 function(obj) {h2 <- which(sapply(Yates[1:(2^k-1)],function(obj2){
                                     hh <- FALSE 
                                     if (length(obj2)==length(obj)) 
                                          if (all(obj2==obj)) hh <- TRUE
                                     hh}))
                              if (length(h2)==0) 0 else h2
                             }
               )

      last <- which(apply(perm,1,function(obj) all(eligible[obj] %in% bcols)))
           ### remove redundant rows from perm
      ergeb[i,] <- bcols  ## 2^k.block-1 potential block main effect columns
      
      if (any(bcols==0)) next
           ## dependent.block is TRUE per default and remains so for bcols with 0
      banned.block[i] <- sum(bcols %in% banned) ## generated block main effects confounded with banned columns
      dependent.block[i] <- length(table(bcols)) < length(bcols) ## k.block columns not independent
      n2fis.block[i] <- sum(nfi2s[which(as.numeric(names(nfi2s)) %in% bcols)])
      n2fis.clear[i] <- length(nfi2s[which(nfi2s==1 & !as.numeric(names(nfi2s)) %in% c(2^(0:(k-1)),gen,bcols))])
      if (banned.block[i] == 0 & !dependent.block[i]){
                 count <- count + 1
                 if (count>=show) break
             }
      if (nrow(perm) <= length(last)) stopp <- TRUE
  }  ## end of loop over permutations
  pick <- which(banned.block==0 & !dependent.block)

  if (length(pick) < show) show <- length(pick) 
  if (show==0 && !alias.block.2fis) stop("no adequate block design found with 2fis unconfounded with blocks")
  if (show==0) stop("no adequate block design found") else {
    ntreat <- k+length(gen)
    blocks <- 2^k.block
    blockcols <- ergeb[pick[1:show],,drop=FALSE]
    if (alias.block.2fis){
        alias.2fis.block <- vector("list",show)
        for (i in 1:show) 
                 alias.2fis.block[[i]] <- names(fi2s)[which(abs(fi2s) %in% blockcols[i,])]
    }
    else alias.2fis.block <- "none"
    nclear.2fis <- n2fis.clear[pick[1:show]]
    nblock.2fis <- n2fis.block[pick[1:show]]
    if (sum(nclear.2fis)>0) {
        clear.2fis <- vector("list",show)
        for (i in 1:show) 
        clear.2fis[[i]] <- names(fi2s)[which(fi2s %in% 
             setdiff(as.numeric(names(nfi2s))[which(nfi2s==1)], c(2^(0:(k-1)),gen,blockcols[i,])))]
        }
    else clear.2fis <- character(0)
    gen[minus] <- -gen[minus]
    list(gen=gen, 
           basics = c(nruns=2^k,nblocks=blocks, ntreat=ntreat, res.base=res), 
           blockcols=blockcols[,2^(0:(k.block-1))],
           alias.2fis.block=alias.2fis.block,
           nblock.2fis=nblock.2fis,
           nclear.2fis=nclear.2fis,
           clear.2fis=clear.2fis)
  }
}

## functions for supporting generation of large resolution V designs
## some functions are also useful for dealing with generators in general
## it would make sense to tidy up utilcat together with some of these


revdigits <- function(obj, ndigits = 1 + floor(log2(max(obj)))){
    aus <- digitsBase(obj, ndigits=ndigits)[rev(1:ndigits),]
    if (!is.matrix(aus)) aus <- matrix(aus, ncol=1)
    aus
}

indexcalc <- function(obj, k = 1 + floor(log2(max(obj)))){
    ## obj is numeric vector with positive column numbers of Yates matrix
    digmat <- revdigits(obj, ndigits=k)
    ## Brief Yates column names, because large designs otherwise not possible
    namen <- apply(digmat, 2, function(obj) paste(Letters[which(obj==1)],collapse=""))
    zahlen <- lapply(as.data.frame(digmat), function(obj) which(obj==1))
    names(zahlen) <- namen
    zahlen
    }

gencalc <- function(gen){
    ## function to return Yates column numbers (= Walsh indices)
    ## from all types of admissible generators

    ## obj can be
    ##       a numeric vector of column numbers (remains unchanged then),
    ##  OR   a character vector with factor letter combinations (from Letters;
    ##       works for up to 50 base factors only, which does not currently seem a restriction)
    ##  OR   a list with numerical vectors of the base factor indices
    ##       in the generators must be used

    if (is.numeric(gen)) aus <- gen
    else{
    ## k is the minimum number needed for current generator
    if (is.list(gen)) k <- max(unlist(gen)) ## list entry for combinations
         else k <- max(which(Letters %in% unlist(strsplit(gen,""))))
    if (is.character(gen)) {hilf <- strsplit(gen,"")
    hilf <- lapply(hilf, function(obj1) which(Letters %in% unlist(obj1)))}
    else hilf <- gen
    aus <- sapply(hilf, function(obj2) sum(2^((1:k)-1)*(1:k %in% obj2)))
    }
    aus
}

YatesFly <- function(walshindex, k=NULL){
    ## the walshindex numbers include the numbers for the base columns
    ## in line with what Sanchez and Sanchez gave in their article

    ## it is possible but usually not wise to increase the dimension
    ##    which will then loose the first k-n.base.cols non-base columns as generators

    ## function for implementing Sanchez Sanchez and other non-catalogued designs
    ## dimension if not given
    if (is.null(k)) k <- 1 + floor(log2(max(walshindex)))
    ## base columns
    base.facs <- which(log2(walshindex)%%1==0)
    if (length(base.facs) > k) stop("k must be at least ", length(base.facs))
    if (length(base.facs) < k) {
       warning("A smaller design would have been possible.\nThe first ", k-length(base.facs), " non-base generators have been omitted for completing the base set to k=", k)
       walshindex <- c(2^((1:k)-1), walshindex[-base.facs][-(1:(k-length(base.facs)))])
       ## recalculate base.facs for later use
       base.facs <- which(log2(walshindex)%%1==0)
    }

    nfactors <- length(walshindex)
    ## check dimension vs. base
    ## removed this check, allowing larger matrix for fewer factors
    ## not usually useful, but ...
        ##if (!length(base.facs)==k) stop("invalid walshindex vector")
    ## index positions
    bf <- 1:k; names(bf) <- Letters[1:k]
    zahlen <- c(as.list(bf),indexcalc(walshindex[-base.facs]))
    mat <- matrix(0, nrow=2^k, ncol=nfactors)
    for (i in 1:k) mat[,i] <- rep(c(-1,1), each=2^(i-1), times=2^(k-i))
    for (i in (k+1):nfactors) mat[,i] <- apply(mat[,zahlen[[i]]],1,prod)
    colnames(mat) <- names(zahlen)
    return(list(mat=mat, Yates.brief=zahlen, k=k, nfactors=nfactors, nruns=2^k,
           gen=walshindex[-base.facs]))
}

MEPlot <- function(obj, ...){
    UseMethod("MEPlot")
}
MEPlot.design <- function(obj, ..., response=NULL){
    if (!"design" %in% class(obj)) 
        stop("MEPlot.design works for obj from class design only.")
    di <- design.info(obj)
    if (is.null(di$response.names)) 
        stop("The design obj must have at least one response.")
    if (is.null(response)) response <- di$response.names[1] else{
      if (!response %in% di$response.names)
        stop("Requested response is not a response variable in obj.")
        }
    if (!(length(grep("FrF2",di$type))>0 | 
           length(grep("pb",di$type))>0)){ 
           if (!(di$type=="full factorial" & all(di$nlevels==2)))
           stop("The design obj must be of a type containing FrF2 or pb.")
    }
    if (!length(grep("blocked",di$type))>0)
        MEPlot(lm(obj, degree=1, response=response), ...)
    else
        MEPlot(lm(as.formula(paste(response, "~ ", 
                      paste(names(di$factor.names), collapse="+"))),
               data=obj), ...)
}


MEPlot.default <-
function(obj, main=paste("Main effects plot for", respnam), pch=15, 
         cex.xax = par("cex.axis"), cex.yax = cex.xax, mgp.ylab = 4, 
         cex.title=1.5, cex.main=par("cex.main"), lwd=par("lwd"), las=par("las"), 
         abbrev=3, select=NULL, ...){
   # main      overall title
   # pch       plot character
   # lwd       line width
   # cex.xax   cex for levels on x-axes 
   # cex.yax   cex for label and levels on y-axis
   # mgp.ylab  mgp entry y-axis label 
   #           (relative to invisible axis of left-most plot)
   # cex.title multiplier for cex.main for the overall title given in option main
   # lwd
   # las
   # abbrev  maximum number of characters used for levels on x-axes
   if (! ("lm" %in% class(obj) | "aov" %in% class(obj))) 
      stop("obj must be a linear model object (lm or aov), or a design of class design")
   obj <- remodel(obj)
   labs <- lapply(obj$labs,function(sp) substr(sp,1,abbrev))
   obj <- obj$model
   if (!check(obj)) 
     stop("This routine is applicable for 2-level factorial designs without partial aliasing only.")
   mod <- obj$model
   term.ord <- attr(terms(obj),"order")
   nmain <- length(which(term.ord==1))
   intcol <- attr(attr(mod,"terms"),"intercept")
   respnam <- colnames(mod)[attr(attr(mod,"terms"),"response")]
   ymean <- mean(mod[,respnam])
   mm <- model.matrix(obj)
   if (intcol > 0) mm <- mm[,-intcol]
   terms1 <- colnames(mm)[which(term.ord==1)]
   if (is.null(select)) select <- 1:nmain
   else {
       if (!is.numeric(select)) stop("select must be numeric")
       if (!all(floor(select)==select)) stop("select must contain integer numbers")
       if (any(select<1 | select>nmain)) stop("select must contain numbers betweeen 1 and ", nmain, " only")
   }
   predmat <- matrix(rep(0,2*nmain),2,nmain)
   colnames(predmat) <- terms1
   predmat <- predmat[,select]
   nmain <- length(select)
   terms1 <- terms1[select]
   labs <- labs[select]
   #addnam <- setdiff(colnames(obj$model), terms1)
   #names <- c(terms1,addnam)
   for (i in 1:nmain)
      predmat[,i] <- ymean+c(-1,1)*coef(obj)[terms1[i]]
   omfrow <- par("mfrow")   
   omar <- par("mar")
   ooma <- par("oma")
   ax <- pretty(c(min(predmat),max(predmat)))
   par(mfrow=c(1,nmain),mar=c(2, 1, 2, 1) + 0.1, oma=c(3,5,4,0.1))
   for (i in 1:nmain){
          ## plot effect without axis drawn (but y label shown for i==1)
          ## plot effect without axis drawn and without axis labels for i>1
          if (i==1)
          plot(c(-1,1),predmat[,i],main=terms1[i],xlab="",xpd=NA, ylab=respnam,type="b",
              xlim=c(-1.3,1.3),ylim=c(min(ax),max(ax)), axes=FALSE, cex=2, 
              cex.lab=cex.yax, cex.axis=1.5, pch=pch, mgp=c(mgp.ylab,1,0), 
              cex.main=cex.main, lwd=lwd)
          else plot(c(-1,1),predmat[,i],main=terms1[i],xlab="",ylab="",type="b",
              xlim=c(-1.3,1.3),ylim=c(min(ax),max(ax)), axes=FALSE, cex=2, 
              cex.lab=1.2, cex.axis=1.5, pch=pch, cex.main=cex.main, lwd=lwd)
              
          box(which="figure")
          abline(h=ymean,xpd=TRUE)   ## line for mean in all plots
          axis(1, at = c(-1,1), labels = labs[[i]], 
               cex.axis=cex.xax, xpd=NA,lwd=lwd, las=las) ## draw bottom axes; 
                                         ## annotation may extend
                                         ## into outer area
          if (i==1)
          axis(2, at = ax, labels = ax, cex.axis=cex.yax, outer=TRUE,lwd=lwd, las=las)
              ## draw left-hand-side axis into the outer area
              ## (axis label comes from first actual plot
              ## placement controlled by mgp.ylab
     }
   title(main, line=1.5, outer=TRUE, cex.main=cex.title*cex.main)
   par(mfrow=omfrow,mar=omar,oma=ooma)
   rownames(predmat) <- c("-","+")
   invisible(predmat)
}

leftadjust <- function(k, gen, early=NULL, show=10){
  ## function that leftadjusts as long as the current factors remain generators
  ## it is however possible to find a better changing design with modified generator columns
  ##  ?? how can these be chosen ??
  ## for practical purposes, this will presumably be sufficient most of the time
  
  hilf <- c(k,abs(gen),show)
  if (!is.null(early)) hilf <- c(hilf,early)
  if (!is.numeric(hilf))
      stop ("All inputs to leftadjust must be numeric.")
  if (!all(hilf == floor(hilf) & hilf > 0))
      stop ("All inputs to leftadjust must contain integer numbers.")
  if (!k >= 3) stop ("leftadjust requires k>=3.")
  g <- length(gen)
  minus <- sign(gen)
  gen <- abs(gen)
  if (any(gen %in% 2^(0:(k-1)))) 
        stop ("gen must not contain column numbers of base factors.")
  if (any(!gen %in% 3:(2^k-1))) 
        stop ("Column numbers in gen must be smaller than ", 2^k,".")
  
  if (!is.null(early))
   if(early > k + g) stop ("early must not be larger than the total number of factors.")
  
  perm <- permutations(k)
  hilf <- digitsBase(gen,ndigits=k)
  ergeb <- matrix(0,nrow=nrow(perm),ncol=length(gen))
  if (!is.null(early)) {
         maxpos <- rep(NA,nrow(perm)) 
         k.early <- rep(NA,nrow(perm)) 
      }
      else {
         maxpos <- NULL
         k.early <- NULL
      }
  for (i in 1:factorial(k)){
    ergeb[i,] <- sort(as.integer(reord(hilf,perm[i,])))
    if (!is.null(early)) {
        for (j in ceiling(log2(early+1)):k){
           if (early==j) zwischen <- 2^(j-1) else
           zwischen <- max(ergeb[i,early-j],2^(j-1))
           if (is.na(maxpos[i]) | zwischen < maxpos[i]){
                  maxpos[i] <- zwischen
                  k.early[i] <- j
               }
           }
        }
  }
  if (is.null(early)) reorder <- ord(ergeb)
  else reorder <- ord(cbind(k.early,maxpos,ergeb))
  
  list(orig=gen*minus,basics=c(nruns=2^k, nfactors=k+g, early=early),
         perms=perm[reorder[1:show],,drop=FALSE],
         maxpos=maxpos[reorder[1:show]],k.early=k.early[reorder[1:show]],
         gen=ergeb[reorder[1:show],,drop=FALSE])
}

### global variables for avoiding R CMD check notes
### is needed because of file estimable.R
## if (getRversion() >= '2.15.1') globalVariables(".FrF2.currentlychecked")

## create package environment
FrF2Env <- new.env()
getFrF2 <- function(nam) get(nam, envir=FrF2Env) 
putFrF2 <- function(nam, obj) assign(nam, obj, envir=FrF2Env) 

`aliases` <-
function(fit, code=FALSE, condense=FALSE){
  if (is.null(stats::alias(fit)$Complete)){
  ## there is no complete aliasing whatsoever
  ## in the linear model
      aus <- list(legend=NULL, aliases=NULL)
      fit <- remodel(fit)$model
      ## there is partial aliasing 
      ## (even with -1 and 1 effects coding as done by remodel)
      if (!is.null(stats::alias(fit, partial=TRUE)$Partial))
         stop(paste("This model contains partially aliased effects.", "\n", 
            "You can look at the alias structure with alias(fit, partial=TRUE)."))
      }
  else {
      ## prevents non-admitted designs
      if (!check(fit)) 
        stop(paste("This routine is applicable for 2-level factorial designs", 
             "with not or fully aliased effects only.")) 
      
      ## create list of confounding structures
      ## using built-in function alias
      fit <- remodel(fit)$model  ## recoded factors to -1 and 1
      mm <- model.matrix(fit)
      al <- round(stats::alias(fit)$Complete,5)
          ## columns contain the "master" effects 
          ## that are not confounded with each other
      if (colnames(al)[1]=="(Intercept)") 
           al <- al[,2:ncol(al)]
      confounded <- as.list(colnames(al))
      for (i in 1:length(confounded))
         if (length(which(abs(al[,i])==1)) > 0) {
              ## in case of proper 2-level fractional factorials,
              ##    at most one entry of each row is non-zero;
              ## rows of al can thus be uniquely assigned 
              ##    to one list element of confounded
              posi <- which(abs(al[,i])==1)
              addnam <- rownames(al)[posi]
              addsign <- gsub("1","",as.character(sign(al[posi,i])))
              addnam <- paste(addsign,addnam,sep="")
              confounded[[i]] <- c(confounded[[i]],addnam)
             }
      
      ## replace variable names with codes (I and i not admitted) for brevity, 
      ##        if requested
      ## and add legend to output object
      legend <- NULL
      if (code){
        terms1 <- names(coef(fit))
        if (terms1[1]=="(Intercept)") terms1 <- terms1[-1]
        terms1 <- terms1[attr(terms(fit),"order")==1]
        faclet <- c(LETTERS[-9],letters[-9])
        codes <- faclet[1:length(terms1)]
        legend <- paste(codes,terms1,sep="=")
        for (i in 1:length(terms1)){
          for (j in 1:length(confounded)){
           for (k in 1:length(confounded[[j]])){
              if (confounded[[j]][k] == terms1[i]) 
                confounded[[j]][k] <- codes[i]
              else if (length( grep(paste(terms1[i],":",sep=""), 
                         confounded[[j]][k] )) > 0)
                confounded[[j]][k] <- gsub(paste(terms1[i],":",sep=""), 
                     paste(codes[i],":",sep=""), confounded[[j]][k] )
              else if (length( grep(paste(":",terms1[i],sep=""), 
                         confounded[[j]][k] )) > 0)
                confounded[[j]][k] <- gsub(paste(":",terms1[i],sep=""), 
                         paste(":",codes[i],sep=""), confounded[[j]][k] )
           }
          }
        }
      }  ## end of recoding
      if (condense){
      lang <- sapply(confounded,"length")
      lang <- which(lang>1)
      grad <- lapply(confounded, function(obj) unlist(strsplit(obj[1],":")))
      grad <- sapply(grad, "length")
      aliased <- list(main=NULL, fi2=NULL, fi3=NULL)
      m <- intersect(which(grad==1), lang)
      if (length(m) > 0)
         aliased$main <- sapply(confounded[m],"paste",collapse=" = ")
      m <- intersect(which(grad==2), lang)
      if (length(m) > 0)
         aliased$fi2 <- sapply(confounded[m],"paste",collapse=" = ")
      m <- intersect(which(grad==3), lang)
      if (length(m) > 0)
         aliased$fi3 <- sapply(confounded[m],"paste",collapse=" = ")
      
      aus <- c(legend=list(legend),aliased)
      }
      else aus <- list(legend = legend, aliases = confounded)
      }  ## end of long else
class(aus) <- c("aliases", class(aus))
aus
}

phimax <- function(n, q, profile=NULL) {
  div <- 2^q-1
  if (is.null(profile)){
    v <- floor(n/div)
    w <- n - v*div
    return(choose(n,2) - v*w - div*choose(v, 2))
  }
  else{
    if (!length(profile)<=div)
      stop("for q = ", q ,", ", "profile must have length <= ", div)
    if (any(profile < 0)) 
      stop("entries of profile must be non-negative")
    if (any(!profile==floor(profile)) ) 
      stop("profile entries must be integers")
    if (!sum(profile)==n) 
      stop("profile entries must sum to ", n)
    return((sum(outer(profile,profile))-sum(profile^2))/2)
  }
}

Xcalc <- function(XI, gen){
  ## function to calculate Godolphin X matrix
  ## from a k-column XI matrix
  ## and generators for p additional factors
  ## generators can be 
  ##      a catalogue (e.g. catlg["7-2.1"] or 
  ##         catlg[nruns(catlg)==32 & nfac(catlg)==7])
  ##      a vector of Yates column numbers (e.g. c(7, 27))
  ##      a vector of defining contrasts (e.g. c("ABC","ABDE"))
  ##      a list of vectors of base column numbers
  ##               e.g. list(1:3, c(1,2,4,5))
  ## k is determined from the number of columns in XI,
  ## p from gen; the X matrix is augmented by 
  ##             p additional columns
  if (!is.matrix(XI)) stop("XI must be a binary matrix")
  if (!is.numeric(XI)) stop("XI must be a binary matrix")
  if (any(!XI %in% c(0,1))) stop("XI must be a binary matrix")
  k <- ncol(XI)
  loop <- FALSE
  if ("catlg" %in% class(gen)){
    if (length(gen) > 1) loop <- TRUE
    nfac <- nfac(gen)
    generator <- lapply(gen, function(obj) obj$gen)
    ps <- lengths(generator)
    if (!all(k + ps==nfac)) stop("input dimensions do not match")
    gen <- generator
    if (!loop) gen <- gen[[1]]
  } 
  else{
    #gen <- gen.check(k, gen)  ## list version
    gen <- gencalc(gen)       ## calculates Yates matrix column numbers
  }
  if (!loop){
    ## return matrix
    Z <- t(sapply(gen, function(obj) rev(digitsBase(obj, 2, k))))
    return(cbind(XI, XI%*%t(Z))%%2)
  }
  else{
    ## return list of matrices
    return(lapply(gen, function(obj){
      Z <- t(sapply(obj, function(obj2) rev(digitsBase(obj2, 2, k))))
      cbind(XI, XI%*%t(Z))%%2
    }))
  }
}


blockgengroup <- function(X, p=0, num=FALSE){
  ### used in blockgencreate
  ### this function returns the base effects that coincide with blocks
  ### in a design with p treatment effect generators
  ### where the q linearly independent rows of the qxn matrix X 
  ###    generate the principal block of the n factor design
  ### The function uses the first k=n-p columns of X only
  ###    it is assumed (but not checked!!) 
  ###    that the last p columns comply with the design structure
  ###    (is guaranteed if X was created with function colpick)
  k <- ncol(X) - p
  ## the potential linear combinations of X columns
  ff <- sfsmisc::digitsBase(1:(2^k-1))[k:1,]
  nam <- Letters[1:k]
  colnames(ff) <- 
    sapply(1:ncol(ff), function(obj) 
      paste0(nam[as.logical(ff[,obj])], collapse=""))
  XIIcalc <- (X[,1:k]%*%ff)%%2
  ## return names of effects whose columns are constant zeroes
  ## (effects confounded with blocks)
  if (!num) return(colnames(XIIcalc)[colSums(XIIcalc)==0])
  else return(which(colSums(XIIcalc)==0))
}

blockgencreate <- function(X, p=0){
  ## X is the matrix that defines the principal block
  ##    with block size 2^(nrow(X))
  ## p is the number of generators for treatment factors 
  ##    in the base design
  ## the last p columns of X must comply with the structure 
  ##    of the design
  
  ## the function does currently not exploit the direct knowledge from 
  ##    matrix X
  if (!is.matrix(X)) stop("XI must be a binary matrix")
  if (!is.numeric(X)) stop("XI must be a binary matrix")
  if (any(!X %in% c(0,1))) stop("XI must be a binary matrix")
  
  k <- ncol(X) - p        ## 2^k is the run size
  q <- nrow(X)
  k.block <- k - q        ## number of block generators needed
  block.gen <- blockgengroup(X, p, num=TRUE)
  ## 2^k block generators as Yates column numbers
#  if (k.block == 1) return(names(block.gen))       ## done
#  picked <- block.gen[1:2]
#  index <- 3
#  already_in <- c(picked, as.intBase(paste(rowSums(cbind(digitsBase(picked[1],2,k), digitsBase(picked[2],2,k)))%%2, collapse="")))
#  while (length(picked) < k.block){
#    cur <- block.gen[index]
#    if (cur %in% already_in) {
#      index <- index + 1
#      next
#    }
#    possibly_in <- sapply(already_in, function(obj) 
#      as.intBase(paste(rowSums(cbind(digitsBase(obj, 2, k), digitsBase(cur, 2, k)))%%2, collapse="")))
#    if (length(intersect(already_in, possibly_in))==0){ 
#      picked <- c(picked, cur)
#      already_in <- c(already_in, cur, possibly_in)
#      index <- index + 1
#    }
#  }
#  names(picked)
  names(block.gen[2^(0:(k.block-1))])
}

lazyExpandGrid <- function(...) {
  ## function for looping through a large grid that is too large to expand
  # from https://stackoverflow.com/questions/36143323/
  # pythons-xrange-alternative-for-r-or-how-to-loop-over-large-dataset-lazilly/36144255#36144255
  # r2evans based on contributions by alexis_laz
  dots <- list(...)
  argnames <- names(dots)
  if (is.null(argnames)) argnames <- paste0('Var', seq_along(dots))
  sizes <- lengths(dots)
  indices <- cumprod(c(1L, sizes))
  maxcount <- indices[ length(indices) ]
  i <- 0
  function(index) {
    i <<- if (missing(index)) (i + 1L) else index
    if (length(i) > 1L) return(do.call(rbind.data.frame, lapply(i, sys.function(0))))
    if (i > maxcount || i < 1L) return(FALSE)
    setNames(Map(`[[`, dots, (i - 1L) %% indices[-1L] %/% indices[-length(indices)] + 1L  ),
             argnames)
  }
}

`remodel` <-
function(obj){   
     ## obj is a linear model object
     ## check whether as many coefficients as terms plus intercept
     ## i.e. two-level factors only
     if (length(coef(obj)) > length(attr(terms(obj),"term.labels")) + 
         sign(attr(terms(obj),"intercept"))) 
         stop("Only 2-level-designs are covered by this routine.")
     mod <- obj$model
     term.ord <- attr(terms(obj),"order")
     nmain <- length(which(term.ord==1))
     intcol <- attr(attr(mod,"terms"),"intercept")
     respcol <- attr(attr(mod,"terms"),"response")
     respnam <- colnames(mod)[respcol]
     labs <- lapply(vector("list",nmain),function(sp){c("-","+")})
     xmod <- mod[,-c(intcol,respcol)] #[,rowSums(attr(attr(mod,"terms"),"factors"))>0]
       # numeric -1 1 columns instead of factors 
       # because otherwise prediction difficult for intermediate (0) level
  #   if (any(sapply(xmod,"is.factor")) || "aov" %in% class(obj)) {
       hilf <- 0
       for (i in 1:length(xmod)) {
           ## character variables are already factors because of lm object
           if (is.factor(xmod[[i]])) 
              labs[[i]] <- levels(xmod[[i]])
              else labs[[i]] <- range(xmod[[i]])
              xmod[[i]] <- as.numeric(xmod[[i]])
              mu <- mean(xmod[[i]])
              r <- range(xmod[[i]])
              r <- r[2]-r[1]
              xmod[[i]] <- 2*(xmod[[i]]-mu)/r
  #         }
           }
       mod[,-c(intcol,respcol)] <- xmod
       obj <- lm(terms(obj),data=data.frame(mod))
  #     }
       ## return list with linear model for further calculations
       ## and labels for annotating plots
       list(model=obj,labs=labs)
}

## function for generating screening designs
## according to Plackett and Burman, Taguchi (L12, same as PB, but different order of rows and columns),
## and Box and Tyssedal for 16 runs
### references Plackett and Burman 1946, Box and Tyssedal 2001 and 1996, Samset and Tyssedal 1999 (with warning against mistakes,
##                if checked)
###     Hedayat and Stufken 1999, Williamson 1944

## Plackett Burman generating vectors and matrices, according to 1946 article
##    c(-1,-1,-1,-1,1,-1,1,-1,1,1,1,-1,1,1,-1,-1,-1,1,1,1,1,1,-1,-1,1,1,-1,1,-1,-1,1)
##    for n=16, Box-Tyssedal constructions are used per default
##        !!! apparently, Samset and Tyssedal 1999 are wrong about projectivity 4 of first 9 columns in large screen ?
##    for n=32 replaced by non-geometric one, which is a cyclic design
##        with projectivity 3
##        with generator obtained from Samset and Tyssedal (1999)
##    for n=92, Williamson type design has been added (normalized to (omitted) +1 first column and -1 last row)
##              Williamson generating vectors from Hedayat and Stufken (1999), p. 160
##    for doubled designs, doubling column is moved to last column, since it is worst for projectivity of design with many factors
##              (without it, it's projectivity 3, otherwise not)
##              If only half of the factors are used: first n/2 - 1 together with last column are of projectivity 4!!!
##    for designs for 52, 76 and 100 runs, first column moved to end, other odd columns in front,
##         because
##             design with only odd columns is almost of projectivity 4
##         and first column causes very strong partial aliasing (almost complete)
##    for the design for 92 runs, 69th column moved to end, because that one produces very strong partial aliasing



## Plackett Burman generating vectors and matrices, according to 1946 article
##    c(-1,-1,-1,-1,1,-1,1,-1,1,1,1,-1,1,1,-1,-1,-1,1,1,1,1,1,-1,-1,1,1,-1,1,-1,-1,1)
##    for n=16, Box-Tyssedal constructions are used per default
##        !!! apparently, Samset and Tyssedal 1999 are wrong about projectivity 4 of first 9 columns in large screen ?
##    for n=32 replaced by non-geometric one, which is a cyclic design
##        with projectivity 3
##        with generator obtained from Samset and Tyssedal (1999)
##    for n=92, Williamson type design has been added (normalized to (omitted) +1 first column and -1 last row)
##              Williamson generating vectors from Hedayat and Stufken (1999), p. 160
##    for doubled designs, doubling column is moved to last column, since it is worst for projectivity of design with many factors
##              (without it, it's projectivity 3, otherwise not)
##              If only half of the factors are used: last n/2 columns are of projectivity 4!!!


## function for generating screening designs
## according to Plackett and Burman, Taguchi (L12, same as PB, but different order of rows and columns), and Box and Tyssedal for 16 runs
pb <- function (nruns, nfactors = nruns - 1, factor.names = if (nfactors <=
    50) Letters[1:nfactors] else paste("F", 1:nfactors, sep = ""),
    default.levels = c(-1, 1), ncenter = 0, center.distribute = NULL,
    boxtyssedal = TRUE, n12.taguchi = FALSE, replications = 1,
    repeat.only = FALSE, randomize = TRUE, seed = NULL, oldver=FALSE, ...)
{
    creator <- sys.call()
    ## check validity of center point options
    if (!is.numeric(ncenter))
        stop("ncenter must be a number")
    if (!length(ncenter) == 1)
        stop("ncenter must be a number")
    if (!ncenter == floor(ncenter))
        stop("ncenter must be an integer number")
    if (is.null(center.distribute)) {
        if (!randomize)
            center.distribute <- min(ncenter, 1)
        else center.distribute <- min(ncenter, 3)
    }
    if (!is.numeric(center.distribute))
        stop("center.distribute must be a number")
    if (!center.distribute == floor(center.distribute))
        stop("center.distribute must be an integer number")
    if (center.distribute > min(ncenter, nruns + 1))
        stop("center.distribute can be at most min(ncenter, nruns+1)")
    if (randomize & center.distribute == 1)
        warning("running all center point runs together is usually not a good idea.")
  ## error checks
    if (default.levels[1] == default.levels[2])
        stop("Both default levels are identical.")
    if (nruns == 8) {
        if (nfactors > 4)
            warning("Plackett-Burman designs in 8 runs coincide with regular fractional factorials. \n          For screening more than four factors, you may want to consider increasing the number of runs to 12. \n          Make sure to take the alias structure into account for interpretation!")
        else {
            aus <- FrF2(nruns = 8, nfactors = nfactors, factor.names = factor.names,
                default.levels = default.levels, replications = replications,
                repeat.only = repeat.only, randomize = randomize,
                seed = seed, ...)
            warning("The design has been constructed with function FrF2 in order to ensure \n                resolution IV and show alias information.")
        }
    }
    if (!isTRUE(all.equal((nruns/4)%%1, 0)))
        stop("Plackett-Burman designs require that nruns is a multiple of 4.")
    if (is.null(nfactors))
        stop("nfactors must be given.")
    if (!nfactors == floor(nfactors))
        stop("nfactors must be an integer number.")
    if (!((is.character(default.levels) | is.numeric(default.levels)) &
        length(default.levels) == 2))
        stop("default.levels must be a vector of 2 levels.")
    if (!is.null(factor.names) & !(is.character(factor.names) |
        is.list(factor.names)))
        stop("factor.names must be a character vector or a list.")
    if (length(factor.names) > nfactors)
        stop("You have specified more than nfactors factors in factor.names.")
    hilf <- character(0)
    n.error <- 0
    if (length(factor.names) < nfactors) {
        n.error <- nfactors - length(factor.names)
        hilf <- rep("", n.error)
        names(hilf) <- paste("e", 1:n.error, sep = "")
    }
    if (is.list(factor.names)) {
        if (is.null(names(factor.names))) {
            if (length(factor.names) <= 50)
                names(factor.names) <- Letters[1:length(factor.names)]
            else names(factor.names) <- paste("F", 1:length(factor.names),
                sep = "")
        }
        factor.names <- c(factor.names, as.list(hilf))
        if (any(factor.names == ""))
            factor.names[which(factor.names == "")] <- list(default.levels)
    }
    else {
        factor.names <- c(factor.names, names(hilf))
        hilf <- vector("list", nfactors)
        names(hilf) <- factor.names
        hilf[1:nfactors] <- list(default.levels)
        factor.names <- hilf
    }
    names(factor.names) <- make.names(names(factor.names), unique = TRUE)
  ## from now on, factor.names is a list with all elements vectors
    if (ncenter > 0)
        if (any(is.na(sapply(factor.names, "is.numeric"))))
            stop("Center points are implemented for experiments with all factors quantitative only.")
    gen <- pb.list[[2]][[which(pb.list[[1]] == nruns)]]
    ## adapt column order so that complete aliasing or very severe aliasing is avoided
    ## as much as possible
    ## relevant (and new) for 40, 52, 56, 64, 76, 88, 92 and 96 runs
    ## for 52, 76 and 96 runs, columns 1, 1, and 69 can generally be moved to the end
    ## for the other arrays, column 1 can be moved to the end in case of nfactors-n.error > nruns/2
    ##     (for fewer factors, the first columns are resolution IV)
    if (nruns == 12 & n12.taguchi)
        sel <- taguchi(12)[, 1:nfactors]
    else {
        if ((nruns == 16) & boxtyssedal)
            sel <- boxtyssedal(nruns, nfactors)
        else {
            sel <- gen
            if (!length(gen) == nruns - 1) {
                if (nruns == 28)
                  sel <- rbind(sel, gen[, c(19:27, 1:18)], gen[,
                    c(10:27, 1:9)], rep(-1, 27))
                else if (nruns == 52)
                  sel <- rbind(sel, cbind(rep(c(1, -1), 5), gen[2:11,
                    c(42:51, 2:41)]), cbind(rep(c(1, -1), 5),
                    gen[2:11, c(32:51, 2:31)]), cbind(rep(c(1,
                    -1), 5), gen[2:11, c(22:51, 2:21)]), cbind(rep(c(1,
                    -1), 5), gen[2:11, c(12:51, 2:11)]), rep(-1,
                    51))
                else if (nruns == 76) {
                  sel <- matrix(0, 76, 75)
                  sel[1:3, ] <- gen
                  for (i in 1:36) sel[(2 + i * 2):(3 + i * 2),
                    ] <- cbind(c(1, -1), gen[2:3, c((75 - 2 *
                    i + 1):75, 2:(75 - 2 * i))])
                  sel[76, ] <- rep(-1, 75)
                }
                else if (nruns == 100) {
                  sel <- matrix(0, 100, 99)
                  sel[1:15, ] <- gen
                  sel[100,] <- rep(-1,99)
                  for (i in 1:6) sel[(2 + i * 14):(15 + i * 14),
                    ] <- cbind(rep(c(1, -1),7), gen[2:15, c((99 - 14 *
                    i + 1):99, 2:(99 - 14 * i))])
                }
                else if (nruns == 92) {
                  a <- c(1, 1, 1, -1, 1, 1, 1, -1, 1, -1, -1,
                    -1, -1, -1, -1, 1, -1, 1, 1, 1, -1, 1, 1)
                  b <- c(1, 1, 1, -1, -1, -1, 1, 1, -1, 1, -1,
                    1, 1, -1, 1, -1, 1, 1, -1, -1, -1, 1, 1)
                  c <- c(1, -1, 1, 1, -1, 1, 1, -1, -1, 1, 1,
                    1, 1, 1, 1, -1, -1, 1, 1, -1, 1, 1, -1)
                  d <- c(1, 1, -1, -1, -1, 1, -1, -1, -1, 1,
                    -1, 1, 1, -1, 1, -1, -1, -1, 1, -1, -1, -1,
                    1)
                  sel <- normalize.col.first(williamson(circ.mat(a),
                    circ.mat(b), circ.mat(c), circ.mat(d)))
                  sel <- normalize.row.last(sel)
                }
                else if (nruns %in% c(40, 56, 64, 88, 96)) {
                  sel <- normalize.row.last(double.des(desnum(pb(nruns/2,
                    randomize = FALSE))))
                }
                else stop(paste("Design for", nruns, "runs not yet implemented.",
                  "\n"))
            }
            else sel <- rbind(circ.mat(sel), rep(-1, nruns -
                1))
        }
    }
    ## except for boxtyssedal and taguchi 12 runs, so far no reduction of 
    ## column number to nfactors because of ordering influence
    
    ## take care of column ordering aspects (introduced in version 1.3)
    ## doubled designs
    if (nruns %in% c(40, 56, 64, 88, 96) & !oldver)
         sel <- sel[,c(2:(nruns-1),1)]
    ## designs from cycling blocks with extra first row/column
    if ((nruns %in% c(52, 76) & !oldver) | nruns==100)
         sel <- sel[,c((1:(nruns%/%2-1))*2+1,(1:(nruns%/%2-1))*2,1)]
    ## 92 run design; column 69 is the culprit for the most severe aliasing
    ## triple 23, 46, 69; 69 alone removes the five strongest aliases, 
    ## one of which is almost complete
    if (nruns ==92 & !oldver)
         sel <- sel[,c(1:68,70:91,69)]
    
    ## subset of nfactors columns now
    sel <- sel[,1:nfactors]
    
    rand.ord <- rep(1:nruns, replications)
    if (replications > 1 & repeat.only)
        rand.ord <- rep(1:nruns, each = replications)
    if (randomize & !is.null(seed))
        set.seed(seed)
    if (randomize & !repeat.only)
        for (i in 1:replications) rand.ord[((i - 1) * nruns +
            1):(i * nruns)] <- sample(nruns)
    if (randomize & repeat.only)
        rand.ord <- rep(sample(1:nruns), each = replications)
    colnames(sel) <- names(factor.names)
    rownames(sel) <- 1:nruns
    sel <- sel[rand.ord, ]
    orig.no <- orig.no.rp <- rownames(sel)
    orig.no.levord <- sort(as.numeric(orig.no), index = TRUE)$ix
    if (replications > 1) {
        if (repeat.only)
            orig.no.rp <- paste(orig.no.rp, rep(1:replications,
                nruns), sep = ".")
        else orig.no.rp <- paste(orig.no.rp, rep(1:replications,
            each = nruns), sep = ".")
    }
    rownames(sel) <- 1:(nruns * replications)
    quant <- rep(FALSE, nfactors)
    desdf <- data.frame(sel)
    for (i in 1:nfactors) {
        desdf[, i] <- des.recode(desdf[, i], "-1=factor.names[[i]][1];1=factor.names[[i]][2]")
        quant[i] <- is.numeric(desdf[, i])
        desdf[, i] <- factor(desdf[, i], levels = factor.names[[i]])
        contrasts(desdf[, i]) <- contr.FrF2(2)
    }
    if (nruns > 8 | nfactors > 4) {
          ## otherwise, aus has already been defined earlier
        aus <- desdf
        rownames(aus) <- rownames(sel) <- 1:nrow(aus)
        attr(aus, "desnum") <- sel
        orig.no <- factor(orig.no, levels = unique(orig.no[orig.no.levord]))
        attr(aus, "run.order") <- data.frame(run.no.in.std.order = orig.no,
            run.no = 1:nrow(sel), run.no.std.rp = orig.no.rp, stringsAsFactors=TRUE)
        attr(aus, "design.info") <- list(type = "pb", nruns = nruns,
            nfactors = nfactors, factor.names = factor.names, ndummies = n.error,
            replications = replications, repeat.only = repeat.only,
            randomize = randomize, seed = seed, creator = creator)
        class(aus) <- c("design", "data.frame")
    }
    if (ncenter > 0)
        aus <- add.center(aus, ncenter, distribute = center.distribute)
    if (nfactors == nruns-1 & nruns %in% c(40,56,64,88,96) & !oldver){
        if (n.error>0 & n.error < (nruns/2-1))
        warning("The dummy factor ", names(factor.names)[nfactors], " is completely aliased with ", nruns-1-nruns/2-n.error, " interactions.")
        if (n.error==0)
        warning("Factor ", names(factor.names)[nfactors], " is completely aliased with ", nruns/2-1, " interactions.")
        }
    if (nfactors == nruns-1 & nruns %in% c(52,76,100) & !oldver){
        if (n.error>0 & n.error < (nruns/2-1))
        warning("The dummy factor ", names(factor.names)[nfactors], " is heavily aliased with ", nruns-1-nruns/2-n.error, " interactions.")
        if (n.error==0)
        warning("Factor ", names(factor.names)[nfactors], " is heavily aliased with ", nruns/2-1, " interactions.")
        }
    if (nfactors-n.error > nruns%/%2 & nruns %in% c(40,56,64,88,96) & oldver)
        warning("Factor ", names(factor.names)[1], " is completely aliased with ", nfactors-n.error-nruns%/%2, " interactions.")
    if (nfactors-n.error >= 3 & nruns %in% c(52,76) & oldver )
        warning("Factor ", names(factor.names)[1], " is heavily aliased with ", (nfactors - n.error - 1)%/%2, " interactions.")
    aus
}

taguchi <- function(nruns){
  if (nruns==12) 
   sel <- L12 <- rbind(c(-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1),
      c(-1,-1,-1,-1,-1,+1,+1,+1,+1,+1,+1),
      c(-1,-1,+1,+1,+1,-1,-1,-1,+1,+1,+1),
      c(-1,+1,-1,+1,+1,-1,+1,+1,-1,-1,+1),
      c(-1,+1,+1,-1,+1,+1,-1,+1,-1,+1,-1),
      c(-1,+1,+1,+1,-1,+1,+1,-1,+1,-1,-1),
      c(+1,-1,+1,+1,-1,-1,+1,+1,-1,+1,-1),
      c(+1,-1,+1,-1,+1,+1,+1,-1,-1,-1,+1),
      c(+1,-1,-1,+1,+1,+1,-1,+1,+1,-1,-1),
      c(+1,+1,+1,-1,-1,-1,-1,+1,+1,-1,+1),
      c(+1,+1,-1,+1,-1,+1,-1,-1,-1,+1,+1),
      c(+1,+1,-1,-1,+1,-1,+1,-1,+1,+1,-1))
  sel
}

boxtyssedal <- function(nruns,k){
   ### some effects perfectly aliased, many partially aliased
   if (nruns==16 & k<=12) 
      sel <- rbind(c(-1,-1,-1,-1,-1,-1,-1,-1,+1,-1,-1,-1,+1,+1,+1),
                   c(+1,-1,-1,-1,+1,+1,+1,-1,-1,+1,-1,-1,-1,-1,+1),
                   c(-1,+1,-1,-1,+1,+1,-1,+1,-1,-1,+1,-1,-1,+1,-1),
                   c(+1,+1,-1,-1,-1,-1,+1,+1,-1,-1,-1,+1,+1,-1,-1), 
                   c(-1,-1,+1,-1,+1,-1,+1,+1,+1,+1,+1,-1,+1,-1,-1),
                   c(+1,-1,+1,-1,-1,+1,-1,+1,+1,+1,-1,+1,-1,+1,-1),
                   c(-1,+1,+1,-1,-1,+1,+1,-1,+1,-1,+1,+1,-1,-1,+1),
                   c(+1,+1,+1,-1,+1,-1,-1,-1,-1,+1,+1,+1,+1,+1,+1),
                   c(-1,-1,-1,+1,-1,+1,+1,+1,-1,+1,+1,+1,+1,+1,+1),
                   c(+1,-1,-1,+1,+1,-1,-1,+1,+1,-1,+1,+1,-1,-1,+1),
                   c(-1,+1,-1,+1,+1,-1,+1,-1,+1,+1,-1,+1,-1,+1,-1),
                   c(+1,+1,-1,+1,-1,+1,-1,-1,+1,+1,+1,-1,+1,-1,-1),
                   c(-1,-1,+1,+1,+1,+1,-1,-1,-1,-1,-1,+1,+1,-1,-1),
                   c(+1,-1,+1,+1,-1,-1,+1,-1,-1,-1,+1,-1,-1,+1,-1),
                   c(-1,+1,+1,+1,-1,-1,-1,+1,-1,+1,-1,-1,-1,-1,+1),
                   c(+1,+1,+1,+1,+1,+1,+1,+1,+1,-1,-1,-1,+1,+1,+1))
        ## N = AB = CE = DF
        ## O = AC = BE = DG
        ## P = AE = BC = DH
        ## ABCE
        ## ABDF
        ## ACDG
        ## BDEG
        ## ADEH
        ## BCDH
   if (nruns==16 & k>12) 
      sel <- rbind(c(-1,-1,-1,-1,-1,-1,+1,+1,+1,+1,+1,+1,-1,-1,+1),
                   c(+1,-1,-1,-1,+1,+1,+1,-1,+1,-1,+1,-1,+1,-1,-1),
                   c(-1,+1,-1,-1,+1,+1,-1,+1,+1,-1,-1,+1,-1,+1,-1),
                   c(+1,+1,-1,-1,-1,-1,-1,-1,-1,-1,+1,+1,+1,+1,+1),
                   c(-1,-1,+1,-1,+1,-1,+1,+1,-1,-1,-1,-1,+1,+1,+1),
                   c(+1,-1,+1,-1,-1,+1,-1,+1,-1,+1,-1,+1,+1,-1,-1),
                   c(-1,+1,+1,-1,-1,+1,+1,-1,-1,+1,+1,-1,-1,+1,-1),
                   c(+1,+1,+1,-1,+1,-1,-1,-1,+1,+1,-1,-1,-1,-1,+1),
                   c(-1,-1,-1,+1,-1,+1,-1,-1,+1,+1,-1,-1,+1,+1,+1),
                   c(+1,-1,-1,+1,+1,-1,+1,-1,-1,+1,-1,+1,-1,+1,-1),
                   c(-1,+1,-1,+1,+1,-1,-1,+1,-1,+1,+1,-1,+1,-1,-1),
                   c(+1,+1,-1,+1,-1,+1,+1,+1,-1,-1,-1,-1,-1,-1,+1),
                   c(-1,-1,+1,+1,+1,+1,-1,-1,-1,-1,+1,+1,-1,-1,+1),
                   c(+1,-1,+1,+1,-1,-1,-1,+1,+1,-1,+1,-1,-1,+1,-1),
                   c(-1,+1,+1,+1,-1,-1,+1,-1,+1,-1,-1,+1,+1,-1,-1),
                   c(+1,+1,+1,+1,+1,+1,+1,+1,+1,+1,+1,+1,+1,+1,+1))
        ## P = AB = CE = DF
        ## Box and Tyssedal (2001); Samset and Tyssedal: last column together with first 8 is of projectivity 4
                                    ## however, this seems to be wrong!
                                    ##if (k==9) sel <- sel[,c(1:8,15)]  ## projectivity 4
   if (k>14) cat("Screening 15 factors in 16 runs involves perfect aliasing of \n pairwise interactions of the first six factors with the last factor.\n")
       sel[,1:k]
}

aliasprint <- function(design,...){
   ##function to print but not return a design's alias information
   if(!"design" %in% class(design)) stop("design must be of class design")
   lgd <- design.info(design)$aliased[[1]]
   als <- design.info(design)$aliased[-1]
   if (all(names(als)%in% c("main","fi2"))) print(design.info(design)$aliased, quote=FALSE, ...)
   else{
   if (length(als)>1 | length(als[1])>1){
            alprint <- list(legend=lgd, aliases=t(t(sapply(als, "paste", collapse=" = "))))
            colnames(alprint$aliases) <- ""
            rownames(alprint$aliases) <- rep("",nrow(alprint$aliases))
       if (is.null(alprint$legend)) print(alprint$aliases) else{
            if(!is.null(alprint$legend)) names(alprint$legend)<-rep("", length(alprint$legend))
            print(alprint, quote=FALSE, ...)
       }}
   else print(design.info(design)$aliased, quote=FALSE, ...)
   }
}

print.catlg <- function(x, name="all", nruns="all", nfactors="all", 
                        res.min=3, MaxC2=FALSE, show=10, 
                        gen.letters=FALSE, show.alias=FALSE, ...){
     if (name[1]=="all" & nruns[1]=="all" & nfactors[1]=="all"){
       ## no filtering needed on x, apart from resolution filtering
       if (gen.letters) hilf <- names(Yates) else hilf <- 1:4095
       if (!is.numeric(res.min)) stop("res.min must be a number.")
       if (!(res.min==floor(res.min) & res.min>2)) stop("res.min must be an integer number larger than 2.")
       x <- x[res.catlg(x)>=res.min]
       if (MaxC2) x <- x[sort(nclear.2fis.catlg(x),index.return=TRUE, decreasing=TRUE)$ix]
       alias <- NULL
       for (i in 1:min(length(x),show)) {
           resp <- as.character(as.roman(x[[i]]$res))
           if (is.na(resp)) resp <- "full factorial"
           ## resp allows to extend catlg to include full factorials 
           ## with gen equal to numeric(0)
           ## warning for as.roman would have to be blocked
           ## further warnings in gen.check would have to be blocked
           ##     would there be further issues ???
           ## Sole purpose so far: also show full factorial possibilities with buttons in 
           ##     create design menu of RcmdrPlugin.DoE 
           if (show.alias) alias <- alias3fi(round(log2(x[[i]]$nruns)),x[[i]]$gen,order=2)
           cat("Design: ", names(x)[i], "\n  ",
                x[[i]]$nruns, " runs, ", 
                x[[i]]$nfac, " factors,  \n   Resolution ", resp, "\n") 
           if (!resp=="full factorial")
               cat("   Generating columns: ", hilf[x[[i]]$gen], "\n",
               "  WLP (3plus): ", x[[i]]$WLP[-(1:2)], ", ", x[[i]]$nclear.2fis, " clear 2fis")
               if (length(x[[i]]$all.2fis.clear)>0) if(!x[[i]]$all.2fis.clear[1]=="all"){
                    if (x[[i]]$nfac <= 50) cat("\n Factors with all 2fis clear: ", Letters[x[[i]]$all.2fis.clear])
                    else cat("\n Factors with all 2fis clear: ", paste("F",x[[i]]$all.2fis.clear,sep=""))}
               if (is.list(alias)){ 
                   cat("\n Alias structure: \n")
                   if (x[[i]]$res==3){ 
                         cat("   Main effects aliasing: \n ")
                         print(alias[[1]],quote=FALSE) 
                         }
                    cat("   2fi aliasing: \n ")
                   print(alias[[2]],quote=FALSE) 
               }
               cat("\n")
       }
       }
     else{
       ## filtering x, and recalling print function afterwards
       if (!name[1]=="all") {
          ## names take precedence over other specifications
          if (!is.character(name)) stop("name must be a vector of character strings naming catalogued designs.")
          if (!length(setdiff(name,names(x)))==0) stop("name must contain names of catalogued designs only.")
          x <- x[name]
       }
       else if (!(nruns[1]=="all" & nfactors[1]=="all")){
         if (!nruns[1]=="all") {
            if (!is.numeric(nruns)) stop("nruns must be a numeric vector.")
            if (!all(2^round(log2(nruns))==nruns)) 
                stop("nruns must be a vector of powers of 2.")
            x <- x[nruns.catlg(x) %in% nruns]
          }
         if (!nfactors[1]=="all"){
            if (!is.numeric(nfactors)) stop("nfactors must be a numeric vector.")
            x <- x[nfac.catlg(x) %in% nfactors]
          }
     }
     print(x, res.min=res.min, show=show, MaxC2=MaxC2, gen.letters=gen.letters, show.alias=show.alias, ...)
     }
}

#require(gtools)
## replace alias3fi by alias2fi ?
## incorporate map into output object of FrF2
## thus enable alias matrix calculation for all designs, even if columns have been swapped

## functions estimable determine design with some additional info
## function mapcalc determines a map - here, it should be possible to switch between really requesting "clear" or requesting "distinct"

#estimable <- function(estimable, ...){
#     UseMethod("estimable")
#}

mapcalc <- function (estimable, nfac, nruns, res3 = FALSE, select.catlg = catlg, 
    method="VF2", sort = "natural", ignore.dom = FALSE) 
{
    ## 12 August: added argument ignore.dom
    ##            with default FALSE
    ## TRUE can support the combination of estimable with blocking
    if (ncol(estimable) > nruns - nfac - 1) 
        stop("too many interactions requested for this number of runs and factors")
    if (nruns > 32 & res3) {
        warning("res3=TRUE has no effect for designs with more than 32 runs.")
        res3 <- FALSE
    }
    catlg <- select.catlg
    go2 <- make_empty_graph(n = nfac, directed = FALSE)
    go2 <- add_edges(go2, estimable)
    ## optionally sort vertices by degree, 20 Jul 2012
   deg2 <- degree(go2)
    if (sort %in% c("high", "low")) {
        if (sort == "low") 
            ord2 <- order(deg2)
        else ord2 <- order(deg2, decreasing = TRUE)
        go2 <- permute(go2, invperm(ord2))
    }
    degree2 <- rev(cumsum(rev(table(deg2))))   ## make it faster to reject non-isomorphic cases
                                      ## 7 Feb 2011
    degs2 <- as.numeric(names(degree2))        ## required minimum degrees
    ## added further pre-filtering criteria 9 July 2012
    indep2 <- ivs_size(go2)         ## required maximum independence number
    clique2 <- clique_num(go2)              ## required minimum clique size
    tobechecked <- catlg[which(nfac(catlg) == nfac & nruns(catlg) == nruns)]
    ## reduced attention to dominating designs, if applicable (29 June 2012)
    ## made reduction to dominating designs respond to ignore.dom argument (12 August 2019)
    if (!ignore.dom)
        tobechecked <- tobechecked[which(dominating(tobechecked))]
    if (length(tobechecked) > 0) 
        if (!res3) 
            tobechecked <- tobechecked[which(res(tobechecked) >= 
                4)]
    if (length(tobechecked) > 0) 
        tobechecked <- tobechecked[which(nclear.2fis(tobechecked) >= 
            ncol(estimable))]
    if (length(tobechecked) == 0) 
        if (res3) 
            stop("The required interactions cannot be accomodated clear of aliasing in ", 
                nruns, " runs.")
        else {
            if (nruns <= 64) 
                stop("The required interactions cannot be accomodated clear of aliasing in ", 
                  nruns, " runs with resolution IV or higher.")
            else stop("No resolution IV or higher design from the current catalogue can accomodate the required interactions clear of aliasing in ", 
                nruns, " runs.")
        }
    map <- NULL
    ## .FrF2.currentlychecked is stored in private environment
    ## for enabling user to access it after aborting without success
    for (i in 1:length(tobechecked)) {
        putFrF2(".FrF2.currentlychecked", names(tobechecked[i]))
        go1 <- make_empty_graph(n = nfac, directed = FALSE)
          ## previous version subtracted 1 from tobechecked[[i]]$clear.2fis 
          ## for previous igraph node definition; changed 29/06/2012
        go1 <- add_edges(go1, tobechecked[[i]]$clear.2fis)
       ## optionally sort vertices by degree, 20 Jul 2012
        deg1 <- degree(go1)
        if (sort %in% c("high", "low")) {
            if (sort == "low") 
                ord1 <- order(deg1)
            else ord1 <- order(deg1, decreasing = TRUE)
            go1 <- permute(go1, invperm(ord1))
        }
        degree1 <- rev(cumsum(rev(table(deg1))))
        degs1 <- as.numeric(names(degree1))
        if (max(degs2) <= max(degs1)) {
             ## check for no chance, 7.2.2011
             ## if max(degs2)>max(degs1), subgraph isomorphism is impossible
            comp <- sapply(degs2, function(obj) degree1[min(which(degs1 >= 
                obj))])
            comp[is.na(comp)] <- 0
            if (any(comp < degree2)) 
                next
          ## added further pre-filtering criteria 9 July 2012
            if (ivs_size(go1) > indep2) 
                next
            if (clique_num(go1) < clique2) 
                next
            if (method=="LAD") 
            erg <- graph.subisomorphic.lad(go2, go1)
            else erg <- graph.subisomorphic.vf2(go1, go2)
          ## +1 removed from map21 because of adapting to igraph (29 June 2012)
            if (erg$iso) {
                if (method=="LAD") map <- list(erg$map)
                else map <- list(erg$map21)
                if (sort %in% c("high", "low")) 
                  map <- list(ord1[map[[1]]][invperm(ord2)])
                names(map) <- getFrF2(".FrF2.currentlychecked")
                putFrF2(".FrF2.currentlychecked", names(tobechecked[i]))
                break
            }
        }
    }
    if (is.null(map)) 
        if (res3) 
            stop("The required interactions cannot be accomodated clear of aliasing in ", 
                nruns, " runs.")
        else {
            if (nruns <= 64) 
                stop("The required interactions cannot be accomodated clear of aliasing in ", 
                  nruns, " runs with resolution IV or higher.")
            else stop("No resolution IV or higher design from the current catalogue can accomodate the required interactions clear of aliasing in ", 
                nruns, " runs.")
        }
    map
}

##map <-FrF2:::mapcalc(matrix(c(1,2,1,3,1,4,2,3,3,5,4,8,3,7),2,7),9,64)
#print(FrF2:::mapcalc(matrix(c(1,2,1,3,1,4,1,5,3,5,4,5,5,7),2,7),9,32,sort="low"))
#print(FrF2:::mapcalc(matrix(c(1,2,1,3,1,4,1,5,3,5,4,5,5,7),2,7),9,32,sort="high"))
#print(FrF2:::mapcalc(matrix(c(1,2,1,3,1,4,1,5,3,5,4,5,5,7),2,7),9,32))

mapcalc.distinct <- function(estimable, nfac, nruns, res3=FALSE, max.time=60, select.catlg=catlg, perm.start=1:nfac, perms=NULL){
## words of length three not correctly represented yet
      ## called with checked inputs only --> no checks needed
      ## estimable is a matrix with two rows
      if (ncol(estimable)>nruns - nfac -1) stop("too many interactions requested for this number of runs and factors")

      ## implement further structural checks of impossibility (e.g. G1^2+G2^2, if union of G1 and G2 is everything)

      hilf1 <- combn(nfac,2)
      catlg <- select.catlg

      ## work through candidate designs
      if (res3) tobechecked <- catlg[which(nfac.catlg(catlg)==nfac & nruns.catlg(catlg)==nruns)]
      else tobechecked <- catlg[which(nfac.catlg(catlg)==nfac & nruns.catlg(catlg)==nruns & res.catlg(catlg)>=4)]
      if (length(tobechecked)==0){
              if (res3)
              stop("The required interactions cannot be accomodated on distinct columns in ", nruns, " runs.")
              else
              stop("The required interactions cannot be accomodated on distinct columns in ", nruns, " runs with resolution IV or higher.")
              }      
      map <- NULL

   begin_time <- Sys.time()
      ## start time to be handed to check.subisomorphism.special
      ## for stopping calculations in case of long run times
   for (i2 in 1:length(tobechecked)){
      ## determine words of length up to four
      hilf2 <- words.all(round(log2(nruns)),tobechecked[[i2]]$gen,max.length=4)
      n3 <- hilf2$WLP["3"]
      if (!is.na(n3)) hilf3 <- hilf2$words.up.to.length.4[which(sapply(hilf2$words.up.to.length.4,length)==3)]
           else hilf3 <- NULL
          ## list of vectors of length 3
      n4 <- hilf2$WLP["4"]
      if (!is.na(n4)) hilf2 <- hilf2$words.up.to.length.4[which(sapply(hilf2$words.up.to.length.4,length)==4)] 
           else hilf2 <- NULL
          ## list of vectors of length 4
      if (is.na(3) & is.na(n4)) stop("Resolution V+ design, use any allocation of experiment factors to design factors.")
      if (is.null(perms)) erg <- check.subisomorphic.special(estimable,nfac,hilf2,hilf3=hilf3,max.time=max.time,begin_time=begin_time,
               name=names(tobechecked[i2]),res3=res3,perm.start=perm.start)
      else erg <- check.subisomorphic.matrix(estimable,nfac,hilf2,hilf3=hilf3,max.time=max.time,begin_time=begin_time,
               name=names(tobechecked[i2]),res3=res3,perms=perms)
          if (erg$iso) {
                    ## "+1" removed from map21 because of adapting to igraph (29 June 2012)
                    map <- list(erg$map21[1:nfac])
                    names(map) <- names(tobechecked[i2])
                    break}
     }
     if (is.null(map)) {
              if (res3)
              stop("The required interactions cannot be accomodated on distinct columns in ", nruns, " runs.")
              else
              stop("The required interactions cannot be accomodated on distinct columns in ", nruns, " runs with resolution IV or higher.")            
            }
     map
}

check.subisomorphic.special <- function(estimable, nfac, hilf2, hilf3=NULL, res3=FALSE, max.time=60, perm.start=1:nfac, 
    begin_time=Sys.time(), name=NA){
    ## hilf2 contains the list of length 4 words, hilf3 length 3 words (if applicable)
    ## begin_time is current time, unless handed over from previously determined timing
    map <- perm.start

    iso <- TRUE ## default: unpermuted design does it

    if (is.null(hilf3)) res3.cur <- FALSE else res3.cur <- TRUE
    if (!is.null(hilf2)) colpairs <- matrix(NA,4,choose(ncol(estimable),2))

    if (res3.cur) {
       ## if any 2fi from requirement set in length three word --> not adequate
       if (any(apply(estimable,2,function(objcp) any(sapply(hilf3,function(obj) all(objcp %in% obj)))))) 
            iso <- FALSE    
    } 
    ## if not yet discarded, check four letter words
    if (ncol(estimable) > 1) hilf <- combn(ncol(estimable),2)
    if (iso & !is.null(hilf2) & ncol(estimable) > 1){
       ## check estimable for two columns being in same word
       ## words are sorted in ascending order
        for (i in 1:ncol(colpairs)) colpairs[,i] <- sort(c(estimable[,hilf[,i]]))
        if (any(apply(colpairs,2,function(objcp) any(sapply(hilf2,function(obj) all(obj==objcp))))))
            iso <- FALSE
    }
    if (!iso){
       ## if iso FALSE for the unpermuted design, loop through others

    ## loop through all permutations of map
    ## adjust estimable accordingly
    ## repeat above check
    for (i in 2:factorial(nfac)) {
      ## loop makes sure that getNext does not continue
      ## could be omitted if getNext would understand that it is finished
       if (difftime(Sys.time(),begin_time,units="sec") > max.time)               
              stop("A solution could not be found in the allocated max.time of ",max.time," seconds. \n",
                   if (!is.na(name)) paste("Final design:",name), ", Final permutation:", paste(map,collapse=",") )
      map <- getNext(map)
      iso <- TRUE ## start by assuming that current permutation does it

      ### adjust numbers in estimable to permutation
      estcur <- matrix(map[estimable],nrow=2)

    ## check here
    if (res3.cur) {
       ## if any 2fi from requirement set in length three word --> not adequate
       if (any(apply(estcur,2,function(objcp) any(sapply(hilf3,function(obj) all(objcp %in% obj)))))) 
            iso <- FALSE    
    } 
    ## bug fix with version 1.3: restricted to ncol(estimable) > 1
    if (iso & !is.null(hilf2) & ncol(estimable) > 1){
       for (i in 1:ncol(colpairs)) colpairs[,i] <- sort(c(estcur[,hilf[,i]]))
       if (any(apply(colpairs,2,function(objcp) any(sapply(hilf2,function(obj) all(obj==objcp)))))){
            iso <- FALSE
            }
    }
    if (iso) break
    }
    }
    ## previous version subtracted 1 from map for previous igraph node definition; changed 29/06/2012
    if (iso) aus <- list(iso=iso, map21 = map) 
        else aus <- list(iso=iso)
    aus
}

check.subisomorphic.matrix <- function(estimable, nfac, hilf2, hilf3=NULL, res3=FALSE, max.time=60, 
    begin_time=Sys.time(), name=NA, perms=perms){
    ## hilf2 contains the list of length 4 words, hilf3 length 3 words (if applicable)
    ## begin_time is current time, unless handed over from previously determined timing

    ## loop through perms
    ## adjust estimable accordingly
    ## repeat above check
    if (is.null(hilf3)) res3.cur <- FALSE else res3.cur <- TRUE
    if (ncol(estimable)>=2) hilf <- combn(ncol(estimable),2)
    else hilf <- matrix(1,1,1)
    if (!(is.null(hilf2) | ncol(estimable)<2)) {
        colpairs <- matrix(NA,4,choose(ncol(estimable),2))
        }

    for (i in 1:nrow(perms)) {
      ## loop makes sure that getNext does not continue
      ## could be omitted if getNext would understand that it is finished
       if (difftime(Sys.time(),begin_time,units="sec") > max.time)               
              stop("A solution could not be found in the allocated max.time of ",max.time," seconds. \n",
                   if (!is.na(name)) paste("Final design:",name), ", Final permutation:", paste(map,collapse=",") )
      map <- perms[i,]
      iso <- TRUE ## start by assuming that current permutation does it

      ### adjust numbers in estimable to permutation
      estcur <- matrix(map[estimable],nrow=2)

    ## check here
    if (res3.cur) {
       ## if any 2fi from requirement set in length three word --> not adequate
       if (any(apply(estcur,2,function(objcp) any(sapply(hilf3,function(obj) all(objcp %in% obj)))))) 
            iso <- FALSE    
    } 
    if (iso & !(is.null(hilf2) | ncol(estimable)<2)){
       ## create colpairs (=forbidden words of length four)
       for (i in 1:ncol(colpairs)) colpairs[,i] <- sort(c(estcur[,hilf[,i]]))
       if (any(apply(colpairs,2,function(objcp) any(sapply(hilf2,function(obj) all(obj==objcp)))))){
            iso <- FALSE
            }
    }
    if (iso) break
    }
    ## previous version subtracted 1 from map for previous igraph node definition; changed 29/06/2012
    if (iso) aus <- list(iso=iso, map21 = map) 
        else aus <- list(iso=iso)
    aus
}

map2design <- function(map, select.catlg=catlg){
          hilf <- select.catlg[[names(map)]]
          ## added for handling of full factorials August 2019
          gen <- hilf$gen
          if (length(gen)==0) gen <- NULL
          hilf <- FrF2(nruns=hilf$nruns,nfactors=hilf$nfac,
                       generators=gen,randomize=FALSE)[,map[[1]]]
          colnames(hilf) <- Letters[1:ncol(hilf)]
          hilf
    }
## test <- map2design(map)
## aliases(lm(rnorm(nrow(test))~.^2,test))

formula2matrix <- function(formula) {
          aus <- apply(attr(terms(formula(formula)),"factors")[,
                attr(terms(formula(formula)),"order")==2,drop=FALSE],2,function(obj) which(obj==1))
          if (!is.matrix(aus)) stop("The formula for option estimable must contain main effects for all factors that occur in interactions!")
          aus
    }

char2num <- function(char){
          char <- strsplit(char,"")
          matrix(sapply(char, function(obj) match(obj,Letters)),nrow=2)
    }

estimable.check <- function(estimable,nfac,factor.names){
    ## function to check validity of estimable entries
    ## returns estimable in matrix form
    ## in case of formula, factor.names is used for making sure that correct factors are picked
    if (is.character(factor.names)) fn <- factor.names else fn <- names(factor.names)
    if (!(is.matrix(estimable) | inherits(estimable,"formula") | is.character(estimable)))
         stop("estimable must be a numeric matrix with two rows or a formula or a character vector with length two entries")
    if (is.character(estimable)){
         if (any(!nchar(estimable)==2)) stop("All entries of character vector estimable must be of length 2.")
         if (is.null(nfac)) {if (!length(setdiff(hilf<-unique(unlist(strsplit(estimable,""))),Letters))==0)
             stop("Only factor letters from constant Letters are permitted in estimable.")
             nfac <- max(which(Letters %in% hilf))}
         else { 
         if (!length(setdiff(unique(unlist(strsplit(estimable,""))),Letters[1:nfac]))==0)
             stop("Only factor letters for nfac factors (Letters[1:nfac]) are permitted in estimable.")
             }
         estimable <- list(estimable=char2num(unique(estimable)),nfac=nfac)
    }
    if (inherits(estimable,"formula")){
          if (any(attr(terms(formula(estimable)),"order")>2)) stop("terms of order higher than 2 are not allowed in formula estimable")
          if (!is.null(nfac))
            if (nrow(attr(terms(formula(estimable)),"factors"))>nfac) stop("more than nfac main effects in formula estimable")
          if (all(!attr(terms(formula(estimable)),"order")==2)) {warning("no interaction requested in formula estimable, estimable is ignored.")
               estimable <- NULL}
          ### implement comparison to factor.names ###
          hilf <- row.names(attr(terms(formula(estimable)),"factors"))
          hilf2 <- 1:length(hilf)
          if (!length(setdiff(hilf,fn))==0){
                 if (!length(setdiff(hilf,Letters))==0)
                      stop("formula must refer to elements of Letters or to factor names.")
                 if (!is.null(nfac)) if (!length(setdiff(hilf,Letters[1:nfac]))==0)
                      stop("Factor letters are higher than permitted for number of factors.")
                 for (i in 1:length(hilf))
                     hilf2[i] <- which(Letters == hilf[i])
                 if (is.null(nfac)) nfac <- max(hilf2)
          }
          else { 
                 for (i in 1:length(hilf)){
                     hilf2[i] <- which(fn == hilf[i])
                     ## nfac is known from factor.names
               }
          }
         if (!is.null(estimable)) estimable <- list(estimable = formula2matrix(estimable), nfac=nfac)
         ## adjust factor numbers in case of factor names
         estimable[[1]] <- matrix(hilf2[estimable[[1]]],nrow=2)
    }
    if (is.matrix(estimable)){
         if (!is.numeric(estimable)) stop("matrix estimable must be numeric")
         if (!nrow(estimable)==2) stop("matrix estimable must have two rows")
         if (any(!round(estimable)==estimable)) stop("entries of matrix estimable must be integer numbers") 
         if (max(estimable)>nfac | min(estimable)<1) stop("entries of estimable must be between 1 and nfac")
         if (is.null(nfac)) nfac <- max(estimable)
         estimable <- list(estimable=estimable, nfac=nfac)
    }
    ## output the matrix of estimable interactions
    ## order as in factor.names or factor letters in case of formula
    estimable
}

estimable <- function(estimable, nfac, nruns, 
                      clear=FALSE, res3=FALSE, max.time=60, 
                      select.catlg=catlg, 
                      perm.start=1:nfac, perms=NULL,
                      order = 3, method="VF2", sort="natural", 
                      ignore.dom = FALSE){
        if (clear) {
            ## perhaps relabel estimable entries such that most frequently occurring ones in front
            estimable <- 
            map <- mapcalc(estimable,nfac,nruns,res3=res3, select.catlg=select.catlg, 
                           method=method, sort=sort, ignore.dom=ignore.dom)
        }
           else map <- mapcalc.distinct(estimable,nfac,nruns,res3=res3, max.time=max.time, 
                                select.catlg=select.catlg, perm.start=perm.start, perms=perms)
        test <- map2design(map, select.catlg=select.catlg)
        hilf <- select.catlg[names(map)][[1]]
        ## determine aliased here because of potential remap of factors
        hilf <- alias3fi(hilf$nfac-length(hilf$gen),hilf$gen, order=order)
        if (is.list(hilf)) aus <- list(map=map,design=test,
            aliased=list(main=sort(chartr(paste(Letters[map[[1]]],collapse=""),paste(Letters[1:nfac],collapse=""),hilf$main)),
                         fi2=sort(chartr(paste(Letters[map[[1]]],collapse=""),paste(Letters[1:nfac],collapse=""),hilf$fi2))))
        else aus <- list(map=map,design=test, aliased=hilf)
        aus
    }

#estimable.formula <- function(estimable, nfac, nruns,  clear=FALSE, res3=FALSE, max.time=60, select.catlg=catlg, perm.start=1:nfac, perms=NULL){
#        if (clear) map <- mapcalc(formula2matrix(estimable),nfac,nruns,res3=res3)
#           else map <- mapcalc.distinct(formula2matrix(estimable),nfac,nruns,res3=res3,max.time=max.time, select.catlg=select.catlg, 
#                  perm.start=perm.start, perms=perms)
#        test <- map2design(map)
#        hilf <- catlg[names(map)][[1]]
#        hilf <- alias3fi(hilf$nfac-length(hilf$gen),hilf$gen)      
#        list(map=map,design=test,aliased=list(main=sort(chartr(paste(Letters[map[[1]]],collapse=""),paste(Letters[1:nfac],collapse=""),hilf$main)),
#                                         fi2=sort(chartr(paste(Letters[map[[1]]],collapse=""),paste(Letters[1:nfac],collapse=""),hilf$fi2))))
#    }

#estimable.character <- function(estimable, nfac, nruns,  clear=FALSE, res3=FALSE, max.time=60, select.catlg=catlg, perm.start=1:nfac, perms=NULL){
#        if (clear) map <- mapcalc(char2num(estimable),nfac,nruns,res3=res3)
#           else map <- mapcalc.distinct(char2num(estimable),nfac,nruns,res3=res3,max.time=max.time, select.catlg=select.catlg, 
#                    perm.start=perm.start, perms=perms)
#        test <- map2design(map)
#        hilf <- catlg[names(map)][[1]]
#        hilf <- alias3fi(hilf$nfac-length(hilf$gen),hilf$gen)      
#        list(map=map,design=test,aliased=list(main=sort(chartr(paste(Letters[map[[1]]],collapse=""),paste(Letters[1:nfac],collapse=""),hilf$main)),
#                                         fi2=sort(chartr(paste(Letters[map[[1]]],collapse=""),paste(Letters[1:nfac],collapse=""),hilf$fi2))))
#    }


#estimable(formula2matrix(formula("~(A+B+C+D)^2+(E+F+G+H)+(A+B+C+D):(E+F+G+H)")), 10, 32)
#estimable(formula2matrix(formula("~(A+B+C+D)^2+(E+F+G+H)+(A+B+C+D):(E+F+G+H)")), 10, 64)
#estimable(formula2matrix(formula("~(A+B+C+D+E+F)+A:B+B:C+C:D+C:F+(D+E+F)^2")), 6, 16,res3=TRUE)

CIG <- function (design, select.catlg = catlg, nfac=NULL, static = FALSE, layout = layout.auto,
    label = "num", plot = TRUE, ...)
{

    ## updates needed for documentation:
    ##         add requirement set examples

    ## permit handover of vertex labels
    ll <- list(...)
    if ("vertex.label" %in% names(ll)) vertex.label <- ll$vertex.label

    ## create graph picture for design entry
    if ("catlg" %in% class(design)) {
        if (length(design) > 1)
            stop("design must not contain more than one catlg entry")
        design <- design[[1]]
    }
    else {
        if (is.matrix(design) || (is.character(design) & length(design)>1) || 
            is(design,"formula")){
        ## Formel
        if (is(design,"formula")){
            fn <- row.names(attr(terms(formula(design)), "factors"))
            design <- estimable.check(design, length(fn), fn)
            names(design) <- c("clear.2fis","nfac")
             if (!exists("vertex.label", inherits=FALSE))  vertex.label <- fn
             else if (!length(vertex.label)==length(fn)) warning("vertex.label has wrong length")
            design$res <- 4
        } 
        ## character or numeric two-row matrix, 
        ## or character vector with elements from Letters
        if (is.matrix(design)){
             if (!nrow(design)==2) stop("matrix design must have two rows.")
             if (any (design[1,]==design[2,])) 
               stop("entries in the same column of matrix design must be different")
             fn <- unique(design)
             if (!is.null(nfac) && all(fn %in% 1:nfac)) fn <- 1:nfac
             if (!exists("vertex.label", inherits=FALSE))  vertex.label <- fn
             else if (!length(vertex.label)==length(fn)) 
               warning("vertex.label has wrong length")
             if (is.character(design)) design <- list(clear.2fis=matrix(sapply(design, function(obj) 
               which(fn==obj)), nrow=2), nfac=length(fn),res=4)
             else design <- list(clear.2fis=design, nfac=length(fn), res=4)
        }
        if (is.character(design) && length(design)>1){
             if (!all(nchar(design)==2)) stop("character vector design must have length 2 entries only")
             if (is.null(nfac)) nfac <- 
                 max(which(Letters %in% unique(unlist(strsplit(design,"")))))
             design <- estimable.check(design,nfac,NULL)

             if (!exists("vertex.label", inherits=FALSE))  
               vertex.label <- Letters[1:design$nfac]
             else if (!length(vertex.label)==design$nfac) 
               warning("vertex.label has wrong length")
             names(design) <- c("clear.2fis","nfac")
             if (any (design$clear.2fis[1,]==design$clear.2fis[2,]))
               stop("characters in the same element of vector design must be different")
             design$res <- 4
             }
             ## now, design is a two-row numeric matrix with entries from 1 to nfac
        }
    else{
        if (!"catlg" %in% class(select.catlg))
            stop("select.catlg must be a catalogue")
        if (!(is.character(design) && length(design) == 1))
            stop("design must be a design name")
        if (!(design %in% names(select.catlg)))
            stop("design must be a design name that occurs in select.catlg")
        design <- select.catlg[[design]]
    }
    }
    if (!exists("vertex.label", inherits=FALSE)) {
        vertex.label <- 1:design$nfac
        if (!label == "num")
            vertex.label <- Letters[vertex.label]
    }
    go2 <- make_empty_graph(n = design$nfac, directed = FALSE)
    if (!length(design$clear.2fis) == 0)
        go2 <- add_edges(go2, design$clear.2fis)
    if (design$res < 4)
        warning("the design is of resolution less than IV")
    if (plot) {
        if (!static) {
            id <- tkplot(go2, vertex.label = vertex.label, ...)
            invisible(list(graph = go2, coords = tkplot.getcoords(id)))
        }
        else {
            invisible(go2)
            plot(go2, layout = layout, vertex.label = vertex.label,
                ...)
        }
    }
    else return(go2)
}

CIGstatic <- function(graph, id, label = "num", xlim = c(-1, 1), ylim = c(1, 
    -1), ...){
    ## get coordinates for static graph from dynamic one
    if ("list" %in% class(graph)) 
        if (names(graph)[1] == "graph") graph <- graph$graph
    if (!exists("vertex.label", inherits = FALSE)) {
        vertex.label <- 1:graph[[1]][[1]][2]
        ## was 1:graph[[1]], changed 18 Apr 2013
        if (!label == "num") 
            vertex.label <- Letters[vertex.label]
    }
    coords <- tkplot.getcoords(id)
    plot(graph, layout = coords, vertex.label = vertex.label, 
        xlim = xlim, ylim = ylim, ...)
}

WP.check <- function (k, WPfacs, nfac.WP, nfactors, factor.names=Letters[1:nfactors]) 
{
    if (!length(WPfacs)==nfac.WP) stop("WFfacs must be of length ", nfac.WP)
    if (!is.list(WPfacs)) {
        ## invalid blocks
        if (!(is.numeric(WPfacs) | is.character(WPfacs))) 
            stop("WPfacs must be a list of whole plot factor numbers, 
            or a character vector of whole plot factor names or factor letters.")

        ## factor names or character block generators (ABC etc.)
        if (is.character(WPfacs)) {
            ## factor names are made into numeric vector
            hilf <- factor.names
            if (is.list(hilf)) hilf <- names(hilf)
            if (all(WPfacs %in% hilf)) WPfacs <- which(hilf %in% WPfacs)
            else if (all(nchar(WPfacs)==1))
                   WPfacs <- which(Letters %in% WPfacs)   ## not foolproof
            if (is.character(WPfacs)) WPfacs <- as.numeric(chartr("F", " ", WPfacs))
            if (any(is.na(WPfacs))) stop("The character vector WPfacs must contain factor names, 
                        factor letters or factor numbers preceded by capital F.")
            }
    }
    ## generators based on factors
    ## or list of single factor numbers
    if (is.list(WPfacs)){
      if (any(!sapply(WPfacs, length)==1)) 
        stop("All entries in list WPfacs must be single factor numbers.")
      if (any(!sapply(WPfacs, is.numeric))) 
        stop("All entries in list WPfacs must be numbers.")
      if (any(sapply(WPfacs, function(obj) !(obj==floor(obj) & obj>=1 & obj < 2^k)))) 
        stop("All entries in list WPfacs must be integer numbers between 1 and 2^k-1.")
      WPfacs <- as.numeric(WPfacs)
    }
    paste("F",WPfacs,sep="")
}


#WP.check <- function (k, WPs, factor.names) 
#{
#    if (is.numeric(WPs) & length(WPs)==1) {
#            if (!(2^round(log2(WPs)))==WPs) stop ("The number WPs must be an integer power of 2.")
#            return(WPs)
#        }
#    if (!(is.numeric(WPs) | is.character(WPs))) 
#        stop ("WPs must be the number of whole plots or a vector of whole plot factors (position numbers, factor letters or factor names).")
#    if (is.character(WPs)) {
#            hilf <- factor.names
#            if (is.list(hilf)) hilf <- names(hilf)
#            if (all(WPs %in% hilf)) WPs <- which(hilf %in% WPs)
#            else {if (all(nchar(WPs)==1)) WPs <- which(Letters %in% WPs)
#                  else WPs <- as.numeric(chartr("F", " ", WPs))
#                  if (any(is.na(WPs))) stop("The character vector WPs must contain factor names, 
#                        factor letters or factor numbers preceded by capital F.")
#            }
#            }
#     if (is.numeric(WPs)){
#            if (any(!WPs == floor(WPs))) 
#                stop("All entries in WPs must be integer numbers.")
#            if (min(WPs) < 1 | max(WPs) > length(factor.names)) 
#                stop("Factor numbers in WPs must be in the range of 1 to ", length(factor.names), ".")
#         }
#     ## make WPs character
#     paste("F",WPs,sep="")
#}

library(FrF2)
### test programs for fold.design

##  function for independent checks
mult <- function(a1, a2, ...){ 
    l <- list(...)
    if (length(l))
        return(Recall(a1, Recall(a2, ...)))
    if (any(is.na(as.numeric(as.character(a1))))) a1 <- 2*as.numeric(as.factor(a1))-3
    if (any(is.na(as.numeric(as.character(a2))))) a2 <- 2*as.numeric(as.factor(a2))-3
    A <- as.numeric(as.character(a1))*as.numeric(as.character(a2))
    A
}

## check for replications and reptowide designs
plan1 <- FrF2(8,5,randomize=FALSE)
plan2 <- FrF2(8,5,replications=2, randomize=FALSE)
set.seed(98776)
plan3 <- FrF2(8,5,replications=2, repeat.only=TRUE)
fp1 <- fold.design(plan1)
fp1b <- fold.design(plan1, columns=5)
fp2 <- fold.design(plan2)
fp3 <- fold.design(reptowide(plan3))

plan4 <- FrF2(16,6,randomize=FALSE, factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs=""))
plan5 <- FrF2(16,randomize=FALSE, factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs=""))
fp4 <- fold.design(plan4)
fp5 <- fold.design(plan5)
plan6 <- FrF2(16,design="6-2.2",randomize=FALSE)
plan7 <- FrF2(16,generators=catlg[["6-2.2"]]$gen,randomize=FALSE,factor.names=list(eins=c(1,2),zwei=c(43,87),drei=c("alt","neu"),vier="",fuenf="",sechs=""))
fp6 <- fold.design(plan6)
fp7 <- fold.design(plan7)
fp6b <- fold.design(plan6, columns=c(4,6))
fp7b <- fold.design(plan7, columns=c(4,6))

## alias structure for three generators that differ only by sign
plan8 <- FrF2(16,generators=c(7,13,15),randomize=FALSE)
fp8 <- fold.design(plan8) ## seems to be correct, is at least compatible
fp8b <- fold.design(plan8,columns=1:7)
plan9 <- FrF2(16,generators=c(7,-13,15),randomize=FALSE)
fp9 <- fold.design(plan9, columns=5) 
fp9b <- fold.design(plan9, columns=c(5,6,7))
fp9c <- fold.design(plan9)
plan10 <- FrF2(16,generators=c(-7,-13,-15),randomize=FALSE) 
fp10 <- fold.design(plan10, columns=5)  ## wrong sign in design.info somewhere
fp10b <- fold.design(plan10, columns=c(5,6,7))  

## estimable muss noch her

set.seed(98776)
plan11 <- FrF2(estimable=formula("~one+two+three+four+two:three+two:four"), 
       factor.names=c("one","two","three","four"), res3=TRUE)
fp11 <- fold.design(plan11)
  ## clear=FALSE allows to allocate all effects on distinct columns in the 
  ##     8 run MA resolution IV design
set.seed(98776)
plan12 <- FrF2(estimable=formula("~one+two+three+four+two:three+two:four"), 
       factor.names=c("one","two","three","four"), clear=FALSE)
fp12 <- fold.design(plan12)

  ## 7 factors instead of 6, but no requirements for factor G
set.seed(98776)
plan13 <-   FrF2(16, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=FALSE)
fp13 <- fold.design(plan13)

plan14 <-  FrF2(32,14,WPs=8,nfac.WP=4,randomize=FALSE)
fp14 <- fold.design(plan14)
fp14b <- fold.design(plan14,columns=c(1,2))
fp14c <- fold.design(plan14,columns=5:14)

plan15 <- FrF2(32,14,WPs=8,nfac.WP=2,randomize=FALSE)
fp15 <- fold.design(plan15)
fp15b <- fold.design(plan15, columns=c(1,2))
fp15c <- fold.design(plan15, columns=c(4,2))

liste <- c("fp1","fp1b","fp2","fp3","fp4","fp5","fp6","fp6b","fp7","fp7b","fp8","fp8b",
           "fp9","fp9b","fp9c","fp10","fp10b","fp11","fp12","fp13","fp14","fp14b","fp14c","fp15","fp15b","fp15c")

for (cc in liste){
   print(paste("*********************",cc,"*******************************"))
   print(eval(parse(text=paste("design.info(",cc,")$creator"))))
   print(eval(parse(text=paste("design.info(",cc,")$generators"))))
   print(eval(parse(text=paste("design.info(",cc,")$aliased"))))
   print(eval(parse(text=paste("design.info(",cc,")$res.WP"))))
   print(eval(parse(text=paste("design.info(",cc,")$type"))))
   print(eval(parse(text=cc)))
   }

block.check <- function (k, blocks, nfactors, factor.names=Letters[1:nfactors]) 
{
    ## generators based on factors
    ## or list of single factor numbers
    if (is.list(blocks)){
      if (any(sapply(blocks, function(obj) any(obj < 1 | obj > nfactors | 
        !floor(obj) == obj)))) 
        stop(paste("All block generators must contain integer numbers from 1 to", 
            nfactors, "\n or letters from", Letters[1], "to", Letters[nfactors], 
            "only."))}
    else {
        ## number of blocks
        if (is.numeric(blocks) & length(blocks)==1) {
            if (!(2^round(log2(blocks)))==blocks) 
              stop ("The number of blocks must be an integer power of 2.")
            return(blocks)
        }
        ## invalid blocks
        if (!(is.numeric(blocks) | is.character(blocks))) 
            stop("blocks must be the number of blocks, a list of generator vectors, a character vector of block generators, 
                   a numeric vector of column numbers of the Yates matrix, or 
                   a character vector of factor names.")
        ## Yates column numbers
        if (is.numeric(blocks)) {
            if (any(!blocks == floor(blocks))) 
                stop("All entries in blocks must be integer numbers.")
            if (min(blocks) < 1 | max(blocks) > 2^k - 1) 
                stop("Column numbers in blocks must be in the range of 1 to 2^k-1.")
            blocks <- Yates[blocks]
            ## now a list
        }
        ## factor names or character block generators (ABC etc.)
        if (is.character(blocks)) {
            ## factor names are made into numeric vector
            ## block generators are made into list
            hilf <- factor.names
            if (is.list(hilf)) hilf <- names(hilf)
            if (all(blocks %in% hilf)) blocks <- as.list(which(hilf %in% blocks))
            else
            blocks <- lapply(strsplit(blocks, ""), function(obj) which(Letters %in% 
                obj))
            }
    }
    ## now blocks is a list (a number of blocks was returned above)
    ## must be checked for singularity outside of block.check
    blocks
}

blockfull <- function(block.gen, k, des.gen=NULL){
  ## function for obtaining the generating columns for 
  ## all block contrasts
  ## block.gen is a list of vectors of design column numbers
  ##    or a numeric vector of Yates column numbers
  ##    or a vector of character strings like "ABD", "CF" etc
  ## des.gen is a vector of Yates column numbers, 
  ##    including those for the base factors
  if (is.list(block.gen) && is.null(des.gen)) 
    stop("for list-valued block.gen, ", 
         "des.gen is needed (vector of Yates column numbers)")
  k.block <- length(block.gen)
  if (is.character(block.gen)) 
    block.gen <- sapply(block.gen, function(obj) which(names(Yates) == obj))
  if (is.list(block.gen))
    block.gen <- sapply(block.gen, function(obj) 
      as.intBase(paste(rowSums(do.call(cbind,
             lapply(obj, function(obj2) 
             digitsBase(des.gen[obj2],2,k))))%%2, collapse="")))

  ## now, blockgen is a vector of Yates column numbers
  aus <- block.gen
  if (k.block > 1)
    for (i in 2:k.block){
    sel <- DoE.base:::nchoosek(k.block, i)
    aus <- c(aus, 
             sapply(1:ncol(sel), 
                    function(obj) 
                      as.intBase(  ## convert to Yates column number
                        paste(     ## convert to character string
                          rowSums( ## obtain interaction pattern of combination
                            do.call(cbind,
                                    lapply(sel[,obj], function(obj2) 
                                      digitsBase(block.gen[obj2],2,k)  
                                      ## obtain binary for Yates column number
                                    ))
                          )%%2, 
                          collapse="")
                      )
             ))
  }
  aus
}

require(FrF2)
set.seed(3456)
planblock1 <- FrF2(32,7,blocks=c("AB","AF","CD"), alias.block.2fi=TRUE)
set.seed(3456)
planblock2 <- FrF2(32,7,blocks=8, alias.block.2fi=TRUE)
set.seed(3456)
planblockrepl <- FrF2(32,7,blocks=8, alias.block.2fi=TRUE, bbreps=2)
set.seed(3456)
planblockrepe <- FrF2(32,7,blocks=8, alias.block.2fi=TRUE, wbreps=2, repeat.only=TRUE)
set.seed(3456)
planblockreplwithin <- FrF2(32,7,blocks=8, alias.block.2fi=TRUE, wbreps=2)
set.seed(3456)
planblockreplboth <- FrF2(32,7,blocks=8, alias.block.2fi=TRUE, wbreps=2, bbreps=2)
set.seed(3456)
planblockreplrepe <- FrF2(32,7,blocks=8, alias.block.2fi=TRUE, wbreps=2, bbreps=2, repeat.only=TRUE)
set.seed(3456)
planest <- FrF2(16,6,estimable=c("AB","AC","AD","BC","CD"),clear=FALSE)

set.seed(3456)
plansimple <- FrF2(16,7)
set.seed(3456)
planrepl <-FrF2(16,7, repl=2)
set.seed(3456)
planrepe <-FrF2(16,7, repl=2, repe=TRUE)

test1 <- add.center(plansimple,3)  ## 3 center points distributed through the plan (3 positions)
test1b <- add.center(plansimple,3, distribute=1) ## 3 center points at the end
test1c <- add.center(plansimple,6, distribute=4)  ## 6 center points distributed through the plan (4 positions)

test2 <- add.center(planest, 3)
test2b <-add.center(planest, 5, distribute=1)
test2c <-add.center(planest, 10, distribute=7)

test4 <- add.center(planrepl,3)  ## 3 center points for proper replications
test4b <- add.center(planrepl,3, distribute=1)  ## 3 center points for proper replications

test5 <- add.center(planrepe,3)  ## 3 center points for repeated measurements
test5b <- add.center(planrepe,3, distribute=1)  ## 3 center points for repeated measurements

test6 <- add.center(planblock1, 3)  ## center points in blocked plan
test6b <- add.center(planblock1, 3, distribute=1)  ## center points in blocked plan

test7 <- add.center(planblock2, 3)  ## center points in blocked plan
test7b <- add.center(planblock2, 3, distribute=1)  ## center points in blocked plan

test8 <- add.center(planblockrepl, 3)  ## center points in replicated blocked plan
test8b <- add.center(planblockrepl, 3, distribute=1)  ## center points in replicated blocked plan

test9 <- add.center(planblockrepe, 3)  ## center points in replicated blocked plan
test9b <- add.center(planblockrepe, 3, distribute=1)  ## center points in replicated blocked plan

test10 <- add.center(planblockreplwithin, 3)  ## center points in replicated blocked plan
test10b <- add.center(planblockreplwithin, 3, distribute=1)  ## center points in replicated blocked plan

test11 <- add.center(planblockreplboth, 3)  ## center points in replicated blocked plan
test11b <- add.center(planblockreplboth, 3, distribute=1)  ## center points in replicated blocked plan

test12 <- add.center(planblockreplrepe, 3)  ## center points in replicated blocked plan
test12b <- add.center(planblockreplrepe, 3, distribute=1)  ## center points in replicated blocked plan

rownames(test1)
rownames(test1b)
rownames(test1c)
rownames(test2)
rownames(test2b)
rownames(test2c)

cbind(run.order(test1),test1)
cbind(run.order(test2),test2)
cbind(run.order(test4b),test4b)
cbind(run.order(test5),test5)
cbind(run.order(test6),test6)
cbind(run.order(test7),test7)
cbind(run.order(test8),test8)
cbind(run.order(test9),test9)
cbind(run.order(test10),test10)
cbind(run.order(test11),test11)
cbind(run.order(test12),test12)
getblock(planblockreplrepe)

rerandomize.design(test1)
rerandomize.design(test2)
rerandomize.design(test4)
rerandomize.design(test5b)
rerandomize.design(test6)
rerandomize.design(test7)
rerandomize.design(test8)
rerandomize.design(test9)
rerandomize.design(test10)
rerandomize.design(test11)
rerandomize.design(test12)

set.seed(3456)
planfull <- FrF2(32, 5)
planfullblock <- FrF2(32, 5, blocks=8, alias.block.2fis=TRUE)
planfullgodolphin <- FF_from_X(colpick(5,2)$X)
testfull <- add.center(planfull, 6) ## center points in full factorial
testfullblock <- add.center(planfullblock, 2) ## center points in blockedfull factorial
testfullgodolphin <- add.center(planfullgodolphin, 2) 
                 ## center points in blockedfull factorial, godolphin approach
cbind(run.order(testfullblock), testfullblock)
cbind(run.order(testfullgodolphin), testfullgodolphin)

cubePlot <- function(obj, eff1, eff2, eff3, main=paste("Cube plot for",respnam),
    cex.title=1.5, cex.lab=par("cex.lab"), cex.ax=par("cex.axis"), 
    cex.clab=1.2, size=0.3, round=NULL,
    abbrev=4,y.margin.add=-0.2, modeled=TRUE){
    # obj     a linear model
    # cex     plot character size
    # cex.lab size of variable labels in diagonal panels
    # cex.xax size of annotation for x-axis
    # cex.yax size of annotation for y-axis
    # abbrev  number of characters for factor levels in diagonal panel
    # show.alias  show effects with which this 3way-interaction is aliased 

   respnam <- gsub("$","",c(substitute(obj)),fixed=TRUE)
   sub <- paste("modeled =",modeled)
   
   if (!"lm" %in% class(obj)) {
   sub <- NULL
   hilf <- data.frame(obj, eff1, eff2, eff3)
   colnames(hilf) <- gsub("$","_",c(substitute(obj), substitute(eff1), substitute(eff2), substitute(eff3)),fixed=TRUE)
   respnam <- colnames(hilf[1])
   obj <- eval(parse(text=paste("lm(",respnam,"~.^3,hilf)")))
   #colnames(obj$model)[2:4] <- c(substitute(eff1),substitute(eff2),substitute(eff3))
   eff1<-colnames(obj$model)[2]
   eff2<-colnames(obj$model)[3]
   eff3<-colnames(obj$model)[4]
   }
   else respnam <- colnames(obj$model)[attr(obj$"terms","response")]
   xends <- yends <- zends <- c(-1,1)
   if (is.factor(eval(parse(text=paste("obj$model$",eff1,sep=""))))) 
       xends <- substr(levels(eval(parse(text=paste("obj$model$",eff1,sep="")))),1,abbrev)
   if (is.factor(eval(parse(text=paste("obj$model$",eff2,sep=""))))) 
       yends <- substr(levels(eval(parse(text=paste("obj$model$",eff2,sep="")))),1,abbrev)
   if (is.factor(eval(parse(text=paste("obj$model$",eff3,sep=""))))) 
       zends <- substr(levels(eval(parse(text=paste("obj$model$",eff3,sep="")))),1,abbrev)

   ### base everything on model recoded to -1 and 1 numerics
   obj <- remodel(obj)
   ## extract labels for factor levels to desired length (abbrev)
   labs <- lapply(obj$labs,function(sp) substr(sp,1,abbrev))
   ## reduce obj list to its linear model component
   obj <- obj$model
   mod <- obj$model
   respnam <- colnames(mod)[attr(attr(mod,"terms"),"response")]
   ## for some reason, this does not work when using obj instead of mod
   
   if (!check(obj))
     stop("This routine is applicable for 2-level factorial designs without partial aliasing only.")
   if (modeled) basis <- aggregate(predict(obj),list(eff1=eval(parse(text=paste("obj$model$",eff1,sep=""))),
        eff2=eval(parse(text=paste("obj$model$",eff2,sep=""))),
        eff3=eval(parse(text=paste("obj$model$",eff3,sep=""))) ),mean)
   else basis <- aggregate(eval(parse(text=paste("obj$model$",respnam,sep=""))),
        list(eff1=eval(parse(text=paste("obj$model$",eff1,sep=""))),
        eff2=eval(parse(text=paste("obj$model$",eff2,sep=""))),
        eff3=eval(parse(text=paste("obj$model$",eff3,sep=""))) ),mean)
   if (!is.null(round)) basis[,4] <- round(basis[,4],round)
 bild <- cubedraw(min=-1,max=1,ticks=TRUE,y.margin.add=y.margin.add,
             xlab=eff1,ylab=eff2,zlab=eff3,  
             xends=xends,yends=yends,zends=zends,
             cex.lab=cex.ax, cex.axlab=cex.lab, 
             cex.main=cex.title*par("cex.main"), main=main, sub=sub,
             mar=c(5,4,4,4)+0.1)
 cubecorners(bild,circles=1:8,size=size)
 cubelabel(bild,labs=basis[,4],pos=NULL,adj=c(0.4,0.4),cex=cex.clab)
}

"[.catlg" <- function(catlg,i){
       class(catlg) <- "list"
       aus <- catlg[i]
       if (!is.list(aus[1])) aus <- list(aus)
       class(aus) <- c("catlg","list")
       aus}
## same function for "[[.catlg" would destroy functionality of lapply

## define generics
res <- function(catlg) UseMethod("res")
nclear.2fis <- function(catlg) UseMethod("nclear.2fis")
clear.2fis <- function(catlg) UseMethod("clear.2fis")
#all.2fis.clear <- function(catlg) UseMethod("all.2fis.clear")
# not made generic because of conflict with generic all
nfac <- function(catlg) UseMethod("nfac")
nruns <- function(catlg) UseMethod("nruns")
WLP <- function(catlg) UseMethod("WLP")
dominating <- function(catlg) UseMethod("dominating")

## define defaults
#res.default <- function(catlg) stop(gettext("res works on class catlg objects only"))
#nclear.2fis.default <- function(catlg) stop(gettext("nclear.2fis works on class catlg objects only"))
#clear.2fis.default <- function(catlg) stop(gettext("clear.2fis works on class catlg objects only"))
#all.2fis.clear.default <- function(catlg) stop(gettext("all.2fis.clear works on class catlg objects only"))
#nfac.default <- function(catlg) stop(gettext("nfac works on class catlg objects only"))
#nruns.default <- function(catlg) stop(gettext("nruns works on class catlg objects only"))
#WLP.default <- function(catlg) stop(gettext("WLP works on class catlg objects only"))
#dominating.default <- function(catlg) stop(gettext("dominating works on class catlg objects only"))

## define character string methods
res.character <- function(catlg) res(FrF2::catlg[catlg])
nclear.2fis.character <- function(catlg) nclear.2fis(FrF2::catlg[catlg])
clear.2fis.character <- function(catlg) clear.2fis(FrF2::catlg[catlg])
#all.2fis.clear.character <- function(catlg) all.2fis.clear(FrF2::catlg[catlg])
# not made generic because of conflict with generic all
nfac.character <- function(catlg) nfac(FrF2::catlg[catlg])
nruns.character <- function(catlg) nruns(FrF2::catlg[catlg])
WLP.character <- function(catlg) WLP(FrF2::catlg[catlg])
dominating.character <- function(catlg) dominating(FrF2::catlg[catlg])

res.catlg <- function(catlg) sapply(catlg, function(obj) obj$res)
nclear.2fis.catlg <- function(catlg) sapply(catlg, function(obj) obj$nclear.2fis)
## takes care of designs with a single 2fi, for which this is a vector (e.g. in catlg128.21; April 2020)
clear.2fis.catlg <- function(catlg) lapply(catlg, function(obj) matrix(obj$clear.2fis, nrow=2))
all.2fis.clear.catlg <- function(catlg) lapply(catlg, function(obj) obj$all.2fis.clear)
nfac.catlg <- function(catlg) sapply(catlg, function(obj) obj$nfac)
nruns.catlg <- function(catlg) sapply(catlg, function(obj) obj$nruns)
WLP.catlg <- function(catlg) lapply(catlg, function(obj) obj$WLP)
dominating.catlg <- function(catlg) sapply(catlg, function(obj) {
      hilf <- obj$dominating
      if (is.null(hilf)) TRUE else hilf
      })

cubecorners <- function(cdobj, fg="black",bg="white",size=0.5,circles=NULL, squares=NULL){
    ### function to draw custom symbols into the corners of the cube
    ### e.g. empty symbols
    ### that can be filled e.g. with averages or sample sizes
    ### and can be circles (default: all 8 corners are circles) or squares
    ### indicated by row numbers in cube data frame

    ### size refers to the side length of a square - circles have the same area as that square
    if (is.null(circles) & is.null(squares))
    stop("You need to specify at least one of circles or squares in function cubecorners.")
    data <- as.data.frame(cdobj$res3d$xyz.convert(cdobj$cub))
    if (!is.null(circles))
       symbols(data[circles,], circles=rep(size/sqrt(pi),length(circles)),
           bg=bg,fg=fg,add=TRUE, lwd=2, inches=FALSE)
    if (!is.null(squares))
       symbols(data[squares,], squares=rep(size,length(squares)),
           bg=bg,fg=fg,add=TRUE, lwd=2, inches=FALSE)
}

compromise <- function(nfactors, G1, class=3, msg=TRUE){
    if (!class %in% c(1,2,3,4)) stop("class must be an integer from 1 to 4")
    if (!is.numeric(G1)) stop("G1 must be numeric")
    if (length(G1)==0) stop("At least one effect must be in group 1")
    if (length(G1)==nfactors) stop("Use a resolution V design")
    if (length(G1)==1 & class==1) stop("no estimable effects required!")
    if (!is.numeric(nfactors)) stop("nfactors must be numeric")
    if (!all(G1%%1==0)) stop("G1 must consist of integer values")
    if (!all(nfactors%%1==0)) stop("nfactors must be integer")
    if (!length(unique(G1))==length(G1)) stop("non-unique values in G1")
    if (!nfactors>=max(G1)) stop("G1 must not contain numbers larger than nfactors")
    if (!min(G1)>=1) stop("G1 must not contain numbers smaller than 1")
    G2 <- setdiff(1:nfactors,G1)
    if (length(G2)==0) stop("At least one effect must be in group 2")
    
    ## maximum number of factors for resolution V
    Mk <- c(2,3,5,6,8,11,17,23,32,47,65)
    k.cand <- 2:12
    names(Mk) <- k.cand   ## names are k (dimension of generating full factorial)
    
    perms <- combn(nfactors, length(G1))
    perms.full <- matrix(NA, ncol(perms), nfactors)
    for (i in 1:ncol(perms))
        perms.full[i,] <- c(perms[,i],setdiff(1:nfactors,perms[,i])) 
    perms.full <- perms.full[,invperm(c(G1,G2)),drop=FALSE]
    perms.full <- perms.full[ord(perms.full),,drop=FALSE]
    if (length(G1)>1)
    requirement <- apply(matrix(Letters[G1[combn(length(G1),2)]],nrow=2),2,"paste",collapse="")
    else requirement <- character(0)
    if (class==3)
    requirement <- c(requirement, outer(Letters[G1],Letters[G2],FUN=function(X,Y) paste(pmin(X,Y),pmax(X,Y),sep="")))
    if (class==2 & length(G2)>=2){
    requirement <- c(requirement, apply(matrix(Letters[G2[combn(length(G2),2)]],nrow=2),
                            2,"paste",collapse=""))
                            }
    if (class==4)
    requirement <- c(outer(Letters[G1],Letters[G2],FUN=function(X,Y) paste(pmin(X,Y),pmax(X,Y),sep="")))
    ## check existence rules for clear designs by Ke, Tang and Wu
    message <- NULL
    if (class==2) {
        minnrun.clear <- 2^k.cand[min(which(Mk>=nfactors))]
        message <- paste(gettext("a clear design requires at least"), 
              minnrun.clear,gettext("runs (resolution V)"))
          }
    else{
        check.0 <- 4*nfactors^2 + 4*nfactors - 2^(k.cand+3) + 9
        ## class 1
        if (class==1) {
          sel <- Mk-2 >= length(G1)
          if (any(check.0>=0))
               sel[check.0>=0] <- sel[check.0 >= 0] & 
                    nfactors - 1/2 - 1/2*sqrt(check.0[check.0>=0]) >= length(G1)
                    }
        ## class 3
        if (class==3){ 
             sel <- Mk-3 >= length(G1)
          if (any(check.0>=0))
               sel[check.0>=0] <- sel[check.0 >= 0] & 
                    nfactors - 3/2 - 1/2*sqrt(check.0[check.0>=0]) >= length(G1)
                    }
        ## class 4
        if (class==4){
             m1 <- min(length(G1), nfactors-length(G1))
             sel <- 2^k.cand >= m1
             check.0 <- nfactors^2 + 8*nfactors - 2^(k.cand+2) -4
             if (any(check.0>=0))
                 sel[check.0>=0] <- nfactors/2 - 1/2*sqrt(check.0[check.0>=0]) >= m1
        }
        pos <- min(c(which(sel), which(Mk >=nfactors)) )  ## Ke et al. rule or resolution V
        minnrun.clear <- 2^k.cand[pos]
        if (Mk[pos] >= nfactors) 
            message <- paste(gettext("a clear design requires at least"), minnrun.clear, gettext("runs (resolution V)"))
        else 
            message <- paste(gettext("a clear design requires at least"), minnrun.clear, gettext("runs"))
    }
    if (msg) message(message)
    list(perms.full=perms.full, requirement=requirement, class=class, minnrun.clear=minnrun.clear)
}

permutations <- function(n){
   ## from package e1071
    if (n == 1) 
        return(matrix(1))
    else if (n < 2) 
        stop("n must be a positive integer")
    z <- matrix(1)
    for (i in 2:n) {
        x <- cbind(z, i)
        a <- c(1:i, 1:(i - 1))
        z <- matrix(0, ncol = ncol(x), nrow = i * nrow(x))
        z[1:nrow(x), ] <- x
        for (j in 2:i - 1) {
            z[j * nrow(x) + 1:nrow(x), ] <- x[, a[1:i + j]]
        }
    }
    dimnames(z) <- NULL
    z
}

invperm <- function(perm){
        sort(perm,index.return=TRUE)$ix
    }

reord <- function(hilf, perm){
    ## reorder matrix of binary numbers
    ## generating factors are isomorphic with binary numbers
    ## A=1, B=10, C=100, D=1000, usw.
    ##     AB=A+B=11 usw.

    ## column numbers for reordered generating factors can be obtained by 
    ##     switching digit positions

    ## reord does this and prepares the resulting matrix for calculating the column numbers
    ## with package sfsmisc
    aus <- hilf[nrow(hilf):1,,drop=FALSE][perm,,drop=FALSE][nrow(hilf):1,,drop=FALSE]
    class(aus) <- "basedInt"
    attr(aus,"base") <- 2
    aus
  }


getNext <- function(perm){
   ## function for next permutation in lexicographical order
   ## adapted from http://www.cut-the-knot.org/do_you_know/AllPerm.shtml
   ##     provided by Alexander Bogomolny, based on Dijkstra 1997 p.71
    N <- length(perm)
    i <- N

    while (perm[i-1] >= perm[i] & i>2) i <- i-1
 
    j <- N+1

    while (perm[j-1] <= perm[i-1] & j>2) {
         j <- j-1
       }

   ## swap values at positions (i-1) and (j-1)
    perm[c(i-1, j-1)] <- perm[c(j-1, i-1)]

    i <- i+1; j <- N+1

    while (i < j)
    {
      perm[c(i-1, j-1)] <- perm[c(j-1, i-1)]
      i <- i+1; j <- j-1
    }
    perm
}

FrF2.currentlychecked <- function() print(getFrF2(".FrF2.currentlychecked"))

mult.gen <- function(liste){
   ## omitted error check, because slows down very much
   ## must be applied to absolute values of generators!!!
   if (is.list(liste)) hilf <- table(unlist(liste))%%2
   else hilf <- table(liste)%%2
   as.numeric(names(hilf[hilf==1]))
}

mult.gen.a <- function(string.vec,sep=""){
   ## omitted error check, because slows down very much
   ## must be applied to absolute values of generators!!!
   hilf <- table(unlist(strsplit(string.vec,sep)))%%2
   paste(names(hilf[hilf==1]),collapse=sep)
}


words.all <- function(k, gen, design=NULL, max.length=7, select.catlg=catlg){
   ## 2^k is the run number of the design
   ## gen is a list of vectors which generate additional factors from interactions among the first k ones 
   ##   column numbers can be incorporated by using Yates[coln], e.g. Yates[c(7,31)] for gen=list(c(1,2,3), c(1,2,3,4,5))
   ##   but now also directly (gen = vector of column numbers)
   ## max.length is the maximum length of words shown (calculations do become faster by reducing this)
   ##      can be set to NULL in order to use maximum possible word length (may be advisable for use in further calculations)
   if (!(k>0 & floor(k)==k)) stop("k must be a positive integer number.")
   
   if (is.character(design)) gen <- select.catlg[[design]]$gen
   ### check generator entry and transform to list of vector of numbers
   gen <- gen.check(k,gen)
   
   minus <- which(sapply(gen, function(obj) all(obj<0)))
   gen <- lapply(gen, "abs")

   g <- length(gen)
   words <- as.list(rep(0,2^min(g,max.length)-1))
   ## append the letter for factor that is generated by generator
   gen <- mapply(c,gen,as.list((k+1):(k+g)),SIMPLIFY=FALSE)
   if (is.null(max.length)) max.length <- k+g
   words[1:g] <- gen
   words[minus] <- lapply(words[minus],"-")
   if (g>1) {
      cur <- g
      if (max.length >= floor((g+1)/2))
           hilf <- combn(g,floor((g+1)/2))  
      else hilf <- combn(g,max.length)      ## for early breakdown in case of storage problems
      for (i in 2:min(g,max.length)){
            hilf <- combn(g,i)
            for (j in 1:ncol(hilf)){
               cur <- cur+1
               words[[cur]] <- mult.gen(gen[hilf[,j]])
               if (length(which(hilf[,j] %in% minus))%%2 == 1) words[[cur]] <- -words[[cur]]
               }
            }
   }
   if (g<=max.length) words[[2^g-1]] <- mult.gen(gen)
   if (length(minus)%%2==1) words[[2^g-1]] <-  - words[[2^g-1]] 
   WLP <- sapply(words, FUN=length)
      sellang <- WLP<=max.length
   WLP <- WLP[sellang]
   wl <- list(table(WLP),words[sellang][order(WLP)])
   wl[[2]] <- wl[[2]][1:sum(wl[[1]][as.numeric(names(wl[[1]]))<=max.length])]
   wl[[3]] <- max.length

   names(wl) <- c("WLP", paste("words.up.to.length",max.length,sep="."),"max.length")
   class(wl) <- c("list","wordlist")
   wl
}

##wl <- words.all(5,list(c(1,2),c(1,3,4),c(1,3,5),c(1,4,5),c(2,3,4,5)))

words.four <- function(wl, f4, max.length=wl$max.length){
### !!! not adapted to the new possibility of negative sign generators!!!
   if (!inherits(wl,"wordlist")) stop("wl must be a wordlist object.")
   if (!(is.vector(f4) | is.list(f4))) stop("f4 must be a vector of defining factors for 4-level factors or a list of such vectors.")
   if (!is.list(f4) & !length(f4) %in% c(2,3)) stop("The vector f4 must be of length 2 or 3. For more than one 4-level factor, use a list of vectors.")
   if (!is.list(f4)) f4 <- list(f4)
   if (as.numeric(names(wl[[1]])[1])<3) stop("This design confounds main effects with main effects. DO NOT USE IT!") 
   else if (!names(wl[[1]])[1]=="3") stop("Function words.four requires that the third df for the four level factor is also included as a main effect.")

   if (wl[[3]]<max.length) stop("Requested max.length not possible, because object wl has smaller max.length")

   wl[[2]]<-wl[[2]][1:sum(wl[[1]][as.numeric(names(wl[[1]]))<=max.length])]

   drei <- wl[[2]][1:wl[[1]][1]]  ## select three-letter words
   ## find three letter words that correspond to factors in f4
      hilf <- sapply(f4, function(obj) {pos <- which(sapply(drei, function(obj2) all(obj %in% obj2))) 
                                        if (length(pos)==0) pos <- 0 
                                        pos} )
      if (any(hilf==0)) stop(paste("f4 specifications in positions",which(hilf==0),"do not match three-letter words in the design."))
      f4 <- drei[hilf]            ## now all f4 elements have all three entries
   ## reduce word list in order to remove superfluous words
      for (i in 1:length(f4)){
             hilf <- sapply(wl[[2]], function(obj) f4[[i]] %in% obj)
             wl[[2]] <- wl[[2]][colSums(hilf)<2]   ## omit words that are already covered by presence of third factor
          }
     WLP4 <- sapply(wl[[2]],FUN=length)
      WLP4 <- table(WLP4)
      wl[[4]] <- f4
      wl[[5]] <- WLP4
      wl[[3]] <- max.length
      names(wl) <- c("WLP.orig", paste("words.reduced.up.to.length",max.length,sep="."), "max.length", "4.level.generators", "WLP4")
      class(wl) <- c("list","wordlist4")
      wl
}

#words.four(wl, c(1,2,6))

print.wordlist <- function(x, ...){
   if (!inherits(x,"wordlist")) stop("Function print.wordlist prints wordlist objects only.")
   print(x[[1]])
   cat("\n\nWord list (up to length ",x$max.length,"):\n",sep="")
   if (max(unlist(x[[2]]))>50) print(sapply(x[[2]],function(obj) 
                          paste(sign(obj[1]),"(",paste(abs(obj),collapse=","),")",sep=""),quote=FALSE))
   else print(sapply(x[[2]],function(obj) paste(if (obj[1]<0) "-" else "",paste(Letters[abs(obj)],collapse=""),sep="")),quote=FALSE)
}

print.wordlist4 <- function(x, ...){
### !!! not adapted to the new possibility of negative sign generators!!!
   if (!inherits(x,"wordlist4")) stop("Function print.wordlist4 prints wordlist4 objects only.")
   print(x[[1]])
   cat("\n\nWord list with 4-level factor internal words removed:\n")
   if (max(unlist(x[[2]]))>50) print(sapply(x[[2]],function(obj) paste("(",paste(obj,collapse=","),")",sep=""),quote=FALSE))
   else print(sapply(x[[2]],function(obj) paste(Letters[obj],collapse="")),quote=FALSE)
   cat("\n\nWord length pattern for reduced word list:\n")
   print(x[[5]])
   cat("\n\nContrast combinations for 4-level factors:\n")
   if (max(unlist(x[[4]]))>50) print(sapply(x[[4]],function(obj) paste("(",paste(obj,collapse=","),")",sep=""),quote=FALSE))
   else print(sapply(x[[4]],FUN=function(obj) paste(Letters[obj],collapse=""),simplify=FALSE),quote=FALSE)
}

alias3fi <- function(k, gen, order=3){
   ## k number of factors spanning basis full factorial
   ## gen list of vectors giving generators (in terms of position of full factorial factors, 
   ##       e.g. list element c(1,2) is AB
   if (!order %in% c(2,3)) stop("order must be 2 or 3")
   gen <- gen.check(k, gen)
   if (!(k>0 & floor(k)==k)) stop("k must be a positive integer number.")
   if (!is.list(gen)) stop("gen must be a list of generator vectors.")
   g <- length(gen)  ## number of generators
   struc <- NULL
   sep <- ""

   if (k + g > 50) {
      Letters <- paste("F",1:(k+g),sep="")
      sep <- ":"
      }

   minus <- c(rep("",k),sapply(gen, function(obj) if (all(obj<0)) "-" else ""))
   gen <- lapply(gen, "abs")
   
   ## vector of main effects in terms of generators in character form
   absall1 <- sapply(c(as.list(1:k),gen),function(obj) paste(Letters[obj],collapse=sep))
   all1 <- paste(minus,absall1,sep="")
   names(all1) <- Letters[1:(g+k)]

   ## vector of 2fis in terms of generators in character form
   all2 <- rep(0,choose(k+g,2))
   sel <- combn(k+g,2)
   n2 <- ncol(sel)
   for (i in 1:n2) all2[i] <- mult.gen.a(all1[sel[,i]],sep=sep)
   names(all2) <- apply(sel,2,function(obj) paste(Letters[obj],collapse=sep))

   all3 <- NULL
   if (order==3){
   ## vector of 3fis in terms of generators in character form
   all3 <- rep(0,choose(k+g,3))
   sel <- combn(k+g,3)
   n3 <- ncol(sel)
   for (i in 1:n3) all3[i] <- mult.gen.a(all1[sel[,i]],sep=sep)
       names(all3) <- apply(sel,2,function(obj) paste(Letters[obj],collapse=sep))}

   ## pattern contains number of occurrences of each effect among main to 3fis
   ## or 2fis only in case of order=2
   all3 <- c(all1,all2,all3)
   pattern <- table(absall3 <- sub("-","",all3))
       ## now all3 contains all lengths
   if (max(pattern)==1){
          if (order==2)
          aus <- "no aliasing among main effects and 2fis"
          else
          aus <- "no aliasing of main effects or 2fis with effects up to order 3"
   }
   else {
          struc <- as.list(rep(0,length(pattern)))
          for (i in 1:length(pattern)){ 
               ## make sure that alias information apears in systematic order
                  struc[[i]] <- c(names(all3)[all3==names(pattern)[i]], 
                        paste("-",names(all3)[which(all3==paste("-",names(pattern)[i],sep=""))],sep=""))
                  struc[[i]] <- struc[[i]][sort(gsub("-","",struc[[i]]),index.return=TRUE)$ix]
                  hilf <- gsub("-","",struc[[i]])
                  struc[[i]] <- struc[[i]][which(nchar(hilf)>0)]
                  hilf <- hilf[which(nchar(hilf)>0)]
                  hilf <- sort(nchar(hilf),index.return=TRUE)$ix
                  struc[[i]] <- struc[[i]][hilf]
                  }
          struc <- sapply(struc[which(sapply(struc,length)>1)], "paste",collapse="=")
          ## make every equation start positive
          struc <- sapply(struc, function(obj){
               if (length(grep("^-",obj))==1){
                   obj <- gsub("-","~",obj)
                   obj <- gsub("=","=-",obj)
                   obj <- gsub("=-~","=",obj)
                   obj <- gsub("~","",obj)
               } 
               obj
               })
          names(struc) <- NULL
   ## sort in ascending order
   ## with main effect
   if (k+g<=50){
   wme <- grep("^[[:alpha:]]=[[:alpha:][:punct:]]*",struc)
   ## with 2fi
   wme2 <- grep("^[[:alpha:]]{2}=[[:alpha:][:punct:]]*",struc)
   ## with 3fi
   if (order==3) wme3 <- grep("^[[:alpha:]]{3}=[[:alpha:][:punct:]]*",struc)
   ## sort order not yet good for more than 25 factors (a before A etc., and also unfortunately locale dependent)
   ## !!! improve by making sort depend on factor number rather than letter
   }
   else{
   wme <- grep("^F[[:digit:]]+=F[[:digit:][:punct:]]*",struc)
   ## with 2fi
   wme2 <- grep("^F[[:digit:]]+:F[[:digit:]]+=F[[:digit:][:punct:]]*",struc)
   ## with 3fi
   if (order==3) wme3 <- grep("^F[[:digit:]]+:F[[:digit:]]+:F[[:digit:]]+=F[[:digit:][:punct:]]*",struc)
   }
   if (order==2) aus <- list(main=sort(struc[wme]),fi2=sort(struc[wme2]))
   else aus <- list(main=sort(struc[wme]),fi2=sort(struc[wme2]),fi3=sort(struc[wme3]))
   }
   aus
   }

   gen.check <- function(k,gen){   if (!is.list(gen)) {
                 if (!(is.numeric(gen) | is.character(gen))) 
                      stop("gen must be a list of generator vectors, a vector of column numbers or a character vector of generators.")
             ## August 2019: handle full factorial
                 if (length(gen)==0) return(vector(mode="list"))
                 if (is.character(gen)){
                     absgen <- sub("-","",gen)
                     minus <- grep("^-",gen)
                     gen <- lapply(strsplit(absgen,""), function(obj) which(Letters %in% obj))
                 }
                 else {
                 ## numeric gen
                 absgen <- abs(gen)
                 minus <- which(gen<0)
                 if (any(!gen==floor(gen))) stop("All entries in gen must be integer numbers.")
                 if (any(2^(0:(k-1)) %in% absgen)) stop("This design is of resolution II and is not permitted in package FrF2.")
                 if (min(absgen)<1 | max(absgen)>2^k-1) stop("Column numbers in gen must be in the range of 3 to 2^k-1.")
                 ## gen <- Yates[absgen]}
                 ## usage of Yates replaced by indexcalc in order to work for larger designs
                 gen <- indexcalc(absgen)}
                 gen[minus] <- lapply(gen[minus],"-")
              }
              if (any(sapply(gen,function(obj) any(abs(obj)<1 | obj>k | !floor(obj)==obj) )))
                   stop(paste("All generators must contain integer numbers from 1 to", k, 
                     "\n or letters from",Letters[1],"to", Letters[k], "only."))
              gen
              }
 
   slowfast <- function(k){
   ## yields a permutation vector 
   ## for switching back and forth between slow and fast ordering
   ## of the base factors of a design
   ind <- 0:(2^k-1)
   ord(as.data.frame(lapply(1:k, function(obj) ind%%(2^obj))))
}

\name{blockpick}
\alias{blockpick}
\alias{blockpick.big}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Function to show potential block assignments }
\description{
  Functions to investigate potential assignments of blocks 
  and show alias information of resulting designs, meant for expert users
}
\usage{
blockpick(k, gen, k.block, design = NULL, show = 10, 
                       alias.block.2fis = FALSE, select.catlg = catlg)
blockpick.big(k, gen, k.block, design = NULL, show = 10, 
                       alias.block.2fis = FALSE, select.catlg = catlg)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{k}{ the number of base factors (designs have \code{2^k} runs) }
  \item{gen}{ vector of generating columns from Yates matrix; 
        for a full factorial, choose \code{gen = 0} or 
        \code{gen=numeric(0)} for 
        no generating columns; but note that there is always just the one 
        and only catalogued design returned for a full factorial.\cr
        For function \code{blockpick}, \code{gen} refers to the generators 
        of the base design only, and block columns are automatically added by 
        \code{blockpick}.\cr
        For function \code{blockpick.big}, \code{gen} refers to the generators 
        for treatment factors and block generators. In fact, \code{blockpick.big} 
        will always use the first \code{k.block} (base) factors for block generation. 
        Hence, for example for generating a design in 64 runs and 7 factors with 
        32 blocks, \code{gen} must have 6 entries in order to accomodate the 
        7 treatment factors together with the 5 block generators.
        }
  \item{k.block}{ number of base factors needed for constructing blocks; 
        there will be \code{2^k.block} blocks in the design }
  \item{design}{ design name (character string) of a specific design from the 
        catalogue given in select.catlg }
  \item{show}{ numeric integer indicating how many results are to be shown;
        the search for possible allocations stops, once \code{show} variants 
        have been found. Note that the best designs may not be found early in 
        the process, especially if a large number of eligible columns is available 
        and many blocks are needed (e.g. 
        full factorial in 64 runs with 16 blocks). In such cases, increasing \code{show} may 
        lead to finding a better design (but may also increase calculation 
        time from long to unbearable).}
  \item{alias.block.2fis}{ logical, indicates whether 2fis may be aliased with blocks }
  \item{select.catlg}{ design catalogue of class \code{catlg} }
}
\details{
  Function \code{blockpick} is used per default by function \code{\link{FrF2}} 
  for problems with \code{choose(nruns-1-nfactors,k.block) < 100000} and without 
  estimability requirements. \code{blockpick} will find a design, if it exists. 
  However, it may take a long time and/or much storage space in problems with 
  large numbers of runs and blocks.

  In \pkg{FrF2} versions before 2.0, function \code{blockpick.big} was used for 
  large use cases; this can still be requested using argument \code{block.old=TRUE}. 
  Since FrF2 version 2, the \code{\link[FrF2:godolphin]{Godolphin (2021) based approach}} 
  is used instead, both for 
  large cases and for cases where blocking is combined with estimability requirements 
  (\code{clear=TRUE} only); the big advantage is the ability of combining blocking with 
  estimability requirements, and a substantial speed gain if small blocks are needed. 
  
  All approaches investigate the potential assignment of blocks such that 
  main effects of treatment factors are not aliased with block main effects. 
  It is left to the user whether or not 2fis amoong treatment effects may be 
  aliased with block main effects (option \code{alias.block.2fis}). (For the Godolphin 
  approach to work, one will usually need to set \code{alias.block.2fis} to TRUE.)
  
  Following Sun, Wu and Chen (1997), there is no single best block assignment. 
  \code{blockpick} uses their catalogue for full factorials (implemented up to 256 runs). 
  For fractional factorials, it develops designs according to a 
  principle similar to that underlying the Sun Wu Chen catalogue that works also in 
  uncatalogued situations. 
  
  Function \code{blockpick.big} uses a strategy similar to \code{\link{splitpick}} 
  and \code{\link{leftadjust}}
  and often finds a solution quickly where \code{blockpick} does not work with the 
  given ressources. However, it is not guaranteed to find existing solutions 
  or a best solution. 
  }

\value{
  The function \code{blockpick} outputs a list of entries with information on at most \code{show} suitable 
  assignments. It ends with an error, if no suitable solution can be found.
  
  \item{gen}{generator column numbers of the base design (w.r.t. the Yates matrix)}
  \item{basics}{named vector with number of runs (\code{nruns}), number of blocks (\code{nblocks}), 
        number of treatment factors (\code{ntreat}) and resolution of base design (\code{res.base});
        the vector is numeric or character, depending on whether resolution is known exactly or 
        as \dQuote{5+} only}
  \item{blockcols}{matrix with at most show rows; each row contains the \code{k.block} 
        column numbers (w.r.t. the Yates matrix) of the block generators
        for the current assignment (the \code{2^k.block-1} columns for block main 
        effects can be obtained from these).}
  \item{alias.2fis.block}{list of character vectors, which contain the 2fis 
        aliased with block main effects for the respective rows of \code{blockcols}}
  \item{nblock.2fis}{vector with number of 2fis aliased with block main effects 
        for the respective rows of \code{blockcols}}
  \item{nclear.2fis}{vector with number of 2fis clear (of aliasing with block main effects 
        and treatment main effects or 2fis) 
        for the respective rows of \code{blockcols}}
  \item{clear.2fis}{list of character vectors, which contain the 2fis that are 
        counted in \code{nclear.2fis} 
        for the respective rows of \code{blockcols}}
  }

\references{ 
Chen, J., Sun, D.X. and Wu, C.F.J. (1993) 
A catalogue of 2-level and 3-level orthogonal arrays. 
\emph{International Statistical Review} \bold{61}, 131-145. 

Sun, D.X., Wu, C.F.J. and Chen, Y.Y. (1997). 
Optimal blocking schemes for \eqn{2^n} and \eqn{2^{n-p}}{2^(n-p)} designs. \emph{Technometrics} \bold{39}, 
298-307. 
}
\author{ Ulrike Groemping }

\seealso{ See Also \code{\link{FrF2}}  }
\examples{
## look at possibilities for running a 32 run design with 6 factors in 8 blocks
## running this without alias.block.2fis=TRUE throws an error: not possible
\dontrun{blockpick(k=5,design="6-1.1",k.block=3)}
## the 8th to 10th design have more clear 2fis than the earlier ones
blockpick(k=5,design="6-1.1",k.block=3,alias.block.2fis=TRUE)
## function FrF2 can be used to manually accomodate this 
des32.6fac.8blocks.MaxC2 <- FrF2(32,6,blocks=c(3,12,21),alias.block.2fis=TRUE)
summary(des32.6fac.8blocks.MaxC2)
## automatic block generation leads to more aliased 2fis
summary(FrF2(32,6,blocks=8,alias.block.2fis=TRUE))

## look at possibilities for blocking design 7-3.1 from Chen, Sun, Wu catalogue
blockpick(4,design="7-3.1",k.block=2,alias.block.2fis=TRUE)

## big design
## running this throws an error on many machines because of too little memory
\dontrun{blockpick(6,design="7-1.2",k.block=5,alias.block.2fis=TRUE)}
## for obtaining a design for this scenario with blockpick.big, 
## the number of factors must be increased to 7+k.block=12
## designs 12-6.1 and 12-6.2 dont do it, 12-6.3 does
bpb <- blockpick.big(6,design="12-6.3",k.block=5,alias.block.2fis=TRUE)
bpb
## based on the result of blockpick.big, a blocked design can be obtained as follows:
## (not run for saving check time on CRAN)
\dontrun{
des64.7fac.32blocks <- FrF2(64,gen=bpb$gen[1,], blocks = as.list(1:5), 
   alias.block.2fis=TRUE)
str(des64.7fac.32blocks)
## if the seven factors are to be named A,...,G:
des64.7fac.32blocks <- FrF2(64,gen=bpb$gen[1,], blocks = as.list(1:5), 
   alias.block.2fis=TRUE, factor.names=c(paste("b",1:5,sep=""),Letters[1:7]))
str(des64.7fac.32blocks)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{CatalogueAccessors}
\alias{[.catlg}
\alias{print.catlg}
\alias{res}
\alias{nruns}
\alias{nfac}
\alias{WLP}
\alias{nclear.2fis}
\alias{clear.2fis}
%\alias{all.2fis.clear}
\alias{dominating}
\alias{res.catlg}
\alias{nruns.catlg}
\alias{nfac.catlg}
\alias{WLP.catlg}
\alias{nclear.2fis.catlg}
\alias{clear.2fis.catlg}
\alias{all.2fis.clear.catlg}
\alias{dominating.catlg}
\alias{res.character}
\alias{nruns.character}
\alias{nfac.character}
\alias{WLP.character}
\alias{nclear.2fis.character}
\alias{clear.2fis.character}
\alias{all.2fis.clear.character}
\alias{dominating.character}
\alias{catlg}
\alias{block.catlg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Catalogue file and accessor functions }
\description{
  Functions to select elements or extract information from design catalogues of class catlg
}
\usage{
res(catlg)
\method{res}{catlg}(catlg)
\method{res}{character}(catlg)
nruns(catlg)
\method{nruns}{catlg}(catlg)
\method{nruns}{character}(catlg)
nfac(catlg)
\method{nfac}{catlg}(catlg)
\method{nfac}{character}(catlg)
WLP(catlg)
\method{WLP}{catlg}(catlg)
\method{WLP}{character}(catlg)
nclear.2fis(catlg)
\method{nclear.2fis}{catlg}(catlg)
\method{nclear.2fis}{character}(catlg)
clear.2fis(catlg)
\method{clear.2fis}{catlg}(catlg)
\method{clear.2fis}{character}(catlg)
all.2fis.clear.catlg(catlg)
%\method{all.2fis.clear}{catlg}(catlg)
dominating(catlg)
\method{dominating}{catlg}(catlg)
\method{dominating}{character}(catlg)
catlg
\method{[}{catlg}(catlg,i)
\method{print}{catlg}(x, name="all", nruns="all", nfactors="all", 
                        res.min=3, MaxC2=FALSE, show=10, 
                        gen.letters=FALSE, show.alias=FALSE, ...)
block.catlg
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{catlg}{ Catalogue of designs of class \code{catlg} (cf. details section), or character vector with name(s) of \code{catlg} element(s) in case of accessor functions}
  \item{i}{ vector of index positions or logical vector that can be used for indexing a \code{catlg} object }
  \item{x}{ an object of class \code{catlg} }
  \item{name}{ character vector of entry names from \code{x}; default \dQuote{all} means: no selection made }
  \item{nruns}{ numeric integer (vector), giving the run size(s) for entries of \code{x} to be shown; 
                default \dQuote{all} means: no selection made  }
  \item{nfactors}{ numeric integer (vector), giving the factor number(s) for entries of \code{x} to be shown; 
                default \dQuote{all} means: no selection made }
  \item{res.min}{ numeric integer giving the minimum resolution for entries of \code{x} to be shown }
  \item{MaxC2}{ logical indicating whether designs are ordered by minimum aberration (default, \code{MaxC2=FALSE}) 
                or by maximum number of clear 2fis (\code{MaxC2=TRUE})}
  \item{show}{ integer number indicating maximum number of designs to be shown; default is 10 }
  \item{gen.letters}{ logical indicating whether the generators should be shown as 
                column numbers (default, \code{gen.letters=FALSE}) or as generators with 
                factor letters (e.g. E=ABCD, \code{gen.letters=TRUE})}
  \item{show.alias}{ logical indicating whether the alias structure (up to 2fis) is to be printed }
  \item{\dots}{ further arguments to function \code{print} }
  \item{block.catlg}{ data frame with block generators for full factorial designs up to 256~runs, 
                taken from Sun, Wu and Chen (1997)}
}
\details{
  The class \code{catlg} is a named list of design entries. 
  Each design entry is again a list with the following items:
  \describe{
      \item{res}{resolution, numeric, i.e. 3 denotes resolution III and so forth }
      \item{nfac}{number of factors} 
      \item{nruns}{number of runs} 
      \item{gen}{column numbers of additional factors in Yates order} 
      \item{WLP}{word length pattern (starting with words of length 1, 
              i.e. the first two entries are 0 for all designs in \code{catlg})} 
      \item{nclear.2fis}{number of clear 2-factor interactions (i.e. free of aliasing with main effects or other 2-factor interactions)} 
      \item{clear.2fis}{2x\code{nclear.2fis} matrix of clear 2-factor interactions 
              (clear to be understood in the above sense); this matrix represents 
              each designs clear interaction graph, which can be used in automated searches for 
              designs that can accomodate (i.e. clearly) a certain requirement set 
              of 2-factor interactions; cf. also \code{\link{estimable.2fis}} } 
      \item{all.2fis.clear}{vector of factors with all 2-factor interactions clear in the above sense}
      \item{dominating}{logical that indicates whether the current design adds a 
              CIG structure that has not been seen for a design with less aberration 
              (cf. Wu, Mee and Tang 2012 p.196 for dominating designs);
              TRUE, if so; FALSE, if current CIG is isomorphic to previous one or 
              has no edges (IMPORTANT: the dominance assessment refers to the current 
              catalogue; for designs with more than 64 runs, it is possible that 
              a design marked dominating in catalogue \code{catlg} is not dominating 
              when considering ALL non-isomorphic designs). \cr
              This element is helpful in omitting non-promising 
              designs from a search for a clear design.\cr
              This element may be missing. In that case, all catalogue entries 
              are assumed dominating. }
      }
  Reference to factors in components \code{clear.2fis} and \code{all.2fis.clear} 
  is via their position number (element of (1:nfac)). 
   
  The \code{print} function for class \code{catlg} gives a concise overview of selected designs in any design catalogue of class \code{catlg}. 
  It is possible to restrict attention to designs with certain run sizes, numbers of factors, and/or to request a minimum resolutions. 
  Designs are ordered in decreasing quality, where the default is aberration order, but number of clear 2fis can be requested alternatively. 
  The best 10 designs are displayed per default; this number can be changed by the \code{show} option. 
  Options \code{gen.letters} and \code{show.alias} influence the style and amount of printed output.
   
  The catalogue \code{catlg}, which is included with package \code{FrF2}, 
  is of class \code{catlg} and is a living object, since it has to be updated with 
  recent research results for larger designs. In particular, new MA designs may be found, 
  or it may be proven that previous \dQuote{good} designs are in fact of minimum aberration.
  
  Currently, the catalogue contains
  \itemize{
  \item the Chen, Sun and Wu (1993) 2-level designs 
  (complete list of 2-level fractional factorials from 4 to 32~runs, 
  complete list of resolution IV 2-level fractional factorials with 64~runs). 
  Note that the Chen Sun Wu paper only shows a selection of the 64~run designs, the 
  complete catalogue has been obtained from Don Sun and is numbered according to 
  minimum aberration (lower number = better design); numbering in the paper is 
  not everywhere in line with this numbering. 
  
  \item minimum aberration (MA) resolution III designs for 33 to 63 factors in 
  64 runs. The first few of these have been obtained from Appendix G of 
  Mee 2009, the designs for 38 and more factors have been constructed by combining 
  a duplicated minimum aberration design in 32 runs and the required number of factors 
  with columns 32 to 63 of the Yates matrix for 64 run designs. Using complementary 
  design theory (cf. e.g. Chapter 6.2.2 in Mee 2009), it can be shown that the resulting 
  designs are minimum aberration (because they are complementary to basically the same 
  designs as the designs in 32 runs on which they are based). The author is grateful 
  to Robert Mee for pointing this out.
  
  \item the MA designs in 128 runs:
  \itemize{
  \item for up to 24 factors obtained from Xu (2009),
  \item for 25 to 64 factors taken from Block and Mee (2005, with corrigendum 2006),
  \item for 65 to 127 factors (resolution III): up to 69 factors coming from Appendix G 
  in Mee, whereas the designs for 70 or more factors have been  
  constructed according to the same principle mentioned for the 64 run designs.}
  
  \item various further \dQuote{good} resolution IV designs in 128 runs obtained by evaluating 
  designs from the complete catalogue by Xu (2009, catalogue on his website) 
  w.r.t. aberration and number of clear 2fis (including also all designs that yield 
  minimum aberration clear compromise designs according to Groemping 2010); all designs 
  with resolution at least IV for up to 11 factors have been added with version 2.2.
  
  \item the MA even designs in 128 runs, in support of blocking according to the 
  Godolphin approach have been added with version 2.2.

  Note that additional non-isomorphic resolution IV designs in 128 runs are 
  available in package (\pkg{FrF2.catlg128}); since the catalogues are quite large, 
  they are not forced upon users of this package who do not need them. 
  Since version 1.1 of that package, the catalogues are not complete but contain 
  high resolution fractions and even/odd fractions only (status: version 1.2-x); 
  re-inclusion of at least selected even fractions is intended, because these 
  may yield improved support of blocking in connection with estimable 2fis.
  
  \item the best (MA) resolution IV or higher designs in \cr
  256 runs for up to 36 factors (resolution V up to 17 factors),\cr 
  512 runs for up to 29 factors (resolution V for up to 23 factors).\cr 
  These have been taken from Xu (2009) with additions by Ryan and Bulutoglu 
  (2010). 
  
  \item Further \dQuote{good} resolution IV designs with up to 80 factors in 
  256 runs and up to 160 factors in 512 runs have also been implemented from Xu (2009).
  
  \item the best (MA) resolution V or higher design for each number of factors or 
  a \dQuote{good} such design (if it is not known which one is best) in \cr
  1024 runs (up to 33 factors, MA up to 28 factors, resolution VI up to 24 factors), \cr
  2048 runs (up to 47 factors, MA up to 32 factors, resolution VI up to 34 factors),\cr
  and 4096 runs (up to 65 factors, MA up to 26 factors, resolution VI up to 48 factors).\cr 

  Most of the large designs in \code{catlg} have been taken from Xu (2009), 
  where complete catalogues of some scenarios are provided 
  (cf. also his website) as well as \dQuote{good} (not necessarily MA) designs for a larger 
  set of situations. Some of the good designs by Xu (2009) have later been shown 
  to be MA by Ryan and Bulutoglu (2010), who also found some additional larger MA designs, 
  which are also included in \code{catlg}. Non-MA designs that 
  were already available before Bulutoglu (2010) are still in the catalogue with their old name.
  (Note that designs that are not MA and cannot be placed in the ranking do not 
  have a running number in the design name; for example, the MA 2048 runs 
  design in 28 factors is named 28-17.1, the older previous design 
  that was not MA is named 28-17 (without \dQuote{.1} or another placement, 
  because the designs position in the ranking of all designs is not known.)) 
  }
  
  There are also some non-regular 2-level fractional factorial designs of resolution V 
  which may be interesting, as it is possible to increase the number of factors for which 
  resolution V is possible (cf. Mee 2009, Chapter 8). 
  These are part of package \pkg{DoE.base}, which is automatically 
  loaded with this package. With versions higher than 0.9-14 of that package, 
  the following arrays are available: \cr
  \code{L128.2.15.8.1}, which allows 4 additional factors and blocking into up to 8 blocks\cr
  \code{L256.2.19}, which allows just 2 additonal factors\cr
  \code{L2048.2.63}, which allows 16 additional factors.
  These non-regular arrays should be fine for most purposes; the difference to the arrays generated 
  by function \code{FrF2} lies in the fact that there is partial aliasing, e.g. between 3-factor interactions 
  and 2-factor interactions. This means that an affected 3-factor interaction is 
  partially aliased with several different 
  2-factor interactions rather than being aliased either fully or not at all.
  }

\value{
  \code{[} selects a subset of designs based on \code{i}, which is again a list of class \code{catlg}, even if a single element is selected.
  \code{res}, \code{nruns}, \code{nfac}, \code{nclear.2fis} 
  and \code{dominating} return a named vector, 
  the \code{print} method does not return anything (i.e. it returns \code{NULL}), and 
  the remaining functions return a list.
}
\references{ 
Block, R. and Mee, R. (2005) Resolution IV Designs with 128 Runs \emph{Journal of Quality Technology} \bold{37}, 282-293.

Block, R. and Mee, R. (2006) Corrigenda \emph{Journal of Quality Technology} \bold{38}, 196.

Chen, J., Sun, D.X. and Wu, C.F.J. (1993) A catalogue of 2-level and 3-level orthogonal arrays. \emph{Int. Statistical Review} \bold{61}, 131-145.

Groemping, U. (2012). Creating clear designs: a graph-based algorithm and a 
catalog of clear compromise plans. \emph{IIE Transactions} \bold{44}, 988-1001. 
\doi{10.1080/0740817X.2012.654848}. Early preprint at 
\url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-005.pdf}.

Mee, R. (2009). \emph{A Comprehensive Guide to Factorial Two-Level Experimentation}. 
New York: Springer.
    

Ryan, K.J. and Bulutoglu, D.A. (2010). Minimum Aberration Fractional Factorial Designs With Large N.
     \emph{Technometrics} \bold{52}, 250-255.

Sun, D.X., Wu, C.F.J. and Chen, Y.Y. (1997). 
Optimal blocking schemes for \eqn{2^p} and \eqn{2^{n-p}}{(n-p)} designs. \emph{Technometrics} \bold{39}, 
298-307. 

Wu, H., Mee, R. and Tang, B. (2012). Fractional Factorial Designs With Admissible 
         Sets of Clear Two-Factor Interactions. \emph{Technometrics} \bold{54}, 191-197.

Xu, H. (2009) Algorithmic Construction of Efficient Fractional Factorial Designs 
         With Large Run Sizes. \emph{Technometrics} \bold{51}, 262-277.
}
\author{ Ulrike Groemping }

\seealso{ See also \code{\link{FrF2}}, \code{\link[DoE.base]{oa.design}}  }
\examples{
c8 <- catlg[nruns(catlg)==8]
nclear.2fis(c8)
clear.2fis(c8)
all.2fis.clear.catlg(c8)

## inspecting a specific catalogue element
clear.2fis("9-4.2")

## usage of print function for inspecting catalogued designs
## the first 10 resolution V+ designs in catalogue catlg
print(catlg, res.min=5)
## the 10 resolution V+ designs in catalogue catlg with the most factors
## (for more than one possible value of nfactors, MaxC2 does usually not make sense)
print(catlg, res.min=5, MaxC2=TRUE)

## designs with 12 factors in 64 runs (minimum resolution IV because 
## no resolution III designs of this size are in the catalogue)
## best 10 aberration designs
print(catlg, nfactors=12, nruns=64)
## best 10 clear 2fi designs
print(catlg, nfactors=12, nruns=64, MaxC2=TRUE)
## show alias structure
print(catlg, nfactors=12, nruns=64, MaxC2=TRUE, show.alias=TRUE)
## show best 20 designs
print(catlg, nfactors=12, nruns=64, MaxC2=TRUE, show=20)

## use vector-valued nruns 
print(catlg, nfactors=7, nruns=c(16,32))
## all designs (as show=100 is larger than available number of designs)
##    with 7 or 8 factors in 16 runs
print(catlg, nfactors=c(7,8), nruns=16, show=100)

## the irregular resolution V arrays from package DoE.base (from version 0.9-17)
## designs can be created from them using function oa.design
\dontrun{
## not run in case older version of DoE.base does not have these
length3(L128.2.15.8.1)
length4(L128.2.15.8.1)  ## aliasing of 2fis with block factor
length4(L128.2.15.8.1[,-16])

length3(L256.2.19)
length4(L256.2.19)

##length3(L2048.2.63) 
##length4(L2048.2.63) do not work resource wise
## but the array is also resolution V (but irregular)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{cubePlot}
\alias{cubePlot}
\alias{cubedraw}
\alias{cubecorners}
\alias{cubelabel}
\alias{myscatterplot3d}
\title{ Cube plot for three-factor-effects }
\description{
  A cube plot for the combined effect of three factors is produced (function cubePlot). 
  Functions cubedraw, cubecorners, cubelabel and myscatterplot3d are not intended for users. 
}
\usage{
cubePlot(obj, eff1, eff2, eff3, main=paste("Cube plot for",respnam),
    cex.title=1.5,cex.lab=par("cex.lab"), cex.ax=par("cex.axis"),  
    cex.clab=1.2, size=0.3, round=NULL,
    abbrev=4,y.margin.add=-0.2, modeled=TRUE)
}

\arguments{
  \item{obj}{ a vector of response values to be analyzed
  
                     OR
  
              a linear model object with 2-level factors or numerical 
              2-level variables (CAUTION: numerical x-variable have to be 
              coded as -1 and +1 only!); 
              the structure must be such that effects are either fully 
              aliased or orthogonal, like in a fractional factorial 2-level 
              design}
  \item{eff1}{ cf. \code{eff3} }
  \item{eff2}{ cf. \code{eff3} }
  \item{eff3}{ effects to be included in the cube plot (x-, y-, z-direction),
               EITHER vectors of equal length (two-level factors or numerical 
               with the two values -1 and 1)
               OR variable names of main effects within the \code{obj} linear
               model object (character strings) }
  \item{main}{ title for the plot, \code{respnam} is the name of the 
               response variable as determined from the call }
  \item{cex.title}{multiplier for size of overall title 
              (\code{cex.main} is multiplied with this factor) } 
  \item{cex.ax}{ size of axis tick marks, defaults to \code{cex.axis}-parameter }
  \item{cex.lab}{ size of axis labels }
  \item{cex.clab}{ size of corner labels }
  \item{size}{ size of cube corners }
  \item{round}{ optional rounding of corner labels (digits argument for function 
               \code{round}, e.g. round=0 for integers, round=-1 for multiples
               of 10, round=1 for 1 decimal place}
  \item{abbrev}{ number of characters shown for factor levels }
  \item{y.margin.add}{ adjustment parameter for placement of y-axis labeling }
  \item{modeled}{ TRUE (default: show modeled means; FALSE: show averages 
  
               NOTE: Even when showing modeled means, there also appears to 
               be a three-factor-interaction, if the model contains an effect
               that is aliased with this interaction!}
}

\details{
  \code{cubePlot} produces a cube plot of the modeled means or averages of 
              all combinations for three factors. The other functions are internal
              and are called by cubePlot.
  \code{myscatterplot3d} is a modified version of scatterplot3d, 
              made more suitable for this situation.
}
\value{
  \code{cubePlot} is used for its side effects only. 
}

\references{ 
    Box G. E. P, Hunter, W. C. and Hunter, J. S. (2005)
    \emph{Statistics for Experimenters, 2nd edition}.
    New York: Wiley.
 }
\author{ Ulrike Groemping }

\seealso{ \code{\link{FrF2-package}} for examples }
\keyword{ design }

\name{FrF2Large}
\alias{FrF2Large}
\alias{nrunsV}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Function to provide large (at least 8192 runs) regular Fractional Factorial designs
that are not necessarily optimal, especially large resolution V designs.}
\description{
  Large regular fractional factorial 2-level designs in 8192 or more runs are provided:
  Resolution V designs in 8096 to 32768 runs with up to 120 factors according to the suggestion by Sanchez and Sanchez 2005
  are automatically created (these are not necessarily optimal). Furthermore, manual generation
  of large regular fractional factorial designs via specification of generators is possible.
}
\usage{
FrF2Large(nruns, nfactors = NULL, factor.names = if (!is.null(nfactors)){
    if (nfactors <= 50)
        Letters[1:nfactors]
    else paste("F", 1:nfactors, sep = "")
    } else NULL,
    default.levels = c(-1, 1), ncenter = 0, center.distribute = NULL,
    generators = NULL,
    replications = 1, repeat.only = FALSE,
    randomize = TRUE, seed = NULL, alias.info = 2, ...)
nrunsV(nfactors)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nruns}{ Number of runs, must be a power of 2 (8192 to 32768).

      The number of runs must match the number of factors. Function \code{nrunsV}
      can be used for determining the number of runs needed for a resolution V design,
      and for advice on the function to be used.\cr
      For more detail on specification of the number of runs, see the Details section.
      }
  \item{nfactors}{ is the number of 2-level factors to be investigated.
      It can be omitted, if it is obvious from options \code{factor.names} or
      \code{generators}.\cr
      The number of factors must match the length of \code{generators}.
      }
  \item{factor.names}{ a character vector of \code{nfactors} factor names
      or a list with \code{nfactors} elements;\cr
      if the list is named, list names represent factor names, otherwise default factor names are used;\cr
      the elements of the list are \cr
      EITHER vectors of length 2 with factor levels for the respective factor\cr
      OR empty strings.
      For each factor with an empty string in factor.names, the levels given in
      \code{default.levels} are used;\cr
       Default factor names are the first elements of the character vector \code{\link[DoE.base:utilities]{Letters}},
       or the factors position numbers preceded by capital F in case of more than 50 factors. }
  \item{default.levels}{ default levels (vector of length 2) for all factors for
      which no specific levels are given }
  \item{ncenter}{ number of center points per block; \code{ncenter > 0}
        is permitted, if all factors are quantitative and the design is not a split-plot design }
  \item{center.distribute}{ the number of positions over which the center points
        are to be distributed for each block; if NULL (default), center points are
        distributed over end, beginning, and middle (in that order, if there are fewer than three center points)
        for randomized designs, and appended to the end for non-randomized designs.
        for more detail, see function \code{\link{add.center}}, which does the work.
        }
  \item{generators}{ There are \code{log2(nruns)} base factors the full factorial
        of which spans the design (e.g. 10 for 1024 runs).
        The generators specify how the remaining factors
        are to be allocated to interactions of these. \cr
        WARNING: Of course, with manual
        specification of generators, the structure of the design is in the users
        responsibility; the function only prevents confounding of two main effects with each
        other.

        \code{generators} can be

        a list of vectors with position numbers of base factors (e.g. c(1,3,4)
        stands for the interaction between first, third and fourth base factor)

        a vector of character representations of these interactions,
        e.g. \dQuote{ACD} stands for the same interaction as above

        a vector of columns numbers in Yates order (e.g. 13 stands for ACD).
        Note that the columns 1, 2, 4, 8, etc., i.e. all powers of 2, are reserved
        for the base factors and cannot be used for assigning additional factors,
        because the design would become a resolution II design. For looking up
        which column number stands for which interaction, type e.g.
        \code{names(Yates)[1:15]} for a 16 run design.

        WARNING: Contrary to function \code{\link{FrF2}}, it is not possible to
        precede generator entries with a minus sign for reversing column levels;
        instead, the levels must be swapped.
        }
  \item{replications}{ positive integer number. Default 1 (i.e. each row just once).
       If larger, each design run is executed replication times.
       If \code{repeat.only}, repeated measurements
       are carried out directly in sequence, i.e. no true replication takes place,
       and all the repeat runs are conducted together. It is likely that the error
       variation generated by such a procedure will be too small, so that average values
       should be analyzed for an unreplicated design.

       Otherwise (default), the full experiment is first carried out once, then
       for the second replication and so forth. In case of randomization,
       each such blocks is randomized separately. In this case, replication variance is
       more likely suitable for usage as error variance
       (unless e.g. the same parts are used for replication runs although build
       variation is important).}
  \item{repeat.only}{ logical, relevant only if replications > 1. If TRUE,
        replications of each run are grouped together
       (repeated measurement rather than true replication). The default is
       \code{repeat.only=FALSE}, i.e. the complete experiment
       is conducted in \code{replications} blocks, and each run occurs in each block.  }
  \item{randomize}{ logical. If TRUE, the design is randomized. This is the default.
       In case of replications, the nature of randomization depends on the setting of
       option \code{repeat.only}.}
  \item{seed}{ optional seed for the randomization process \cr
        In R version 3.6.0 and later, the default behavior of function \code{\link{sample}}
        has changed. If you work in a new (i.e., >= 3.6.-0) R version and want to reproduce
        a randomized design from an earlier R version (before 3.6.0),
        you have to change the RNGkind setting by\cr
        \code{RNGkind(sample.kind="Rounding")}\cr
        before running function \code{FrF2Large}.\cr
        It is recommended to change the setting back to the new recommended way afterwards:\cr
        \code{RNGkind(sample.kind="default")}\cr
        For an example, see the documentation of the example data set \code{\link[DoE.base]{VSGFS}}.
        }
  \item{alias.info}{ can be 2 or 3, gives the order of interaction effects for which
       alias information is to be included in the \code{aliased} component of the
       \code{design.info} element of the output object. }
  \item{\dots}{ currently not used }
}
\details{
      If generators are not explicitly specified, function \code{FrF2Large} creates a
      resolution V design according to the
      rules by Sanchez and Sanchez (2005) for the specified number of factors in
      the specified number of runs. The Sanchez and Sanchez article offers designs with \cr
      at least 1024 runs for 25 to 29 factors (1024 up to 33 factors with \code{\link{FrF2}}),\cr
      at least 2048 runs for 30 to 38 factors (2048 up to 47 factors with \code{\link{FrF2}}),\cr
      at least 4096 runs for 39 to 52 factors (4096 up to 65 factors with \code{\link{FrF2}}),\cr
      at least 8192 runs for 53 to 69 factors (up to 65 factors in half the run size with \code{\link{FrF2}}),\cr
      at least 16384 runs for 70 to 92 factors, (\cr
      at least 32768 runs for 93 to 120 factors.\cr
      For designs with up to 4096 runs, function \code{\link{FrF2}}
      creates better automatic designs.
      Therefore, function \code{FrF2Large} is restricted to usage for larger designs.

      Users can explicitly specify a design through specifying
      generators via the \code{generators} option. For up to 4096 runs, this is also possible
      with function \code{\link{FrF2}}, even with more flexibility. Therefore, manual design generation
      with function \code{FrF2Large} is also restricted to designs of at least 8192 runs.

      Manual generation of large designs with the option \code{generators} is limited by
      computer memory only. \code{nruns} must be at least large enough to accomodate the
      rightmost generator column; for example, if generators contains an element \code{ABEP},
      \code{P} is the 15th base factor (15th letter in \code{\link[DoE.base:utilities]{Letters}}),
      i.e. \code{nruns} must be at least \code{2^15}=32768;
      if the largest generator column number in Yates column notation is 4201,
      \code{nruns} must be at least \code{2^ceiling(log2(4201))}=8192.
}
\value{
  Function \code{nrunsV} invisibly returns the number of runs requested and
  prints a message with the number of runs and the appropriate function.

  Function \code{FrF2Large} returns a data frame of S3 class
  \code{\link[DoE.base:class-design]{design}}
  and has attached attributes that can be accessed
  by functions \code{\link[DoE.base:class-design-methods]{desnum}},
  \code{\link[DoE.base:class-design-methods]{run.order}} and
  \code{\link[DoE.base:class-design-methods]{design.info}}.

  The data frame itself contains the design with levels coded as requested.
  If no center points have been requested, the design columns are factors with
  contrasts \code{-1} and \code{+1} (cf. also \code{\link[DoE.base]{contr.FrF2}}); in case
  of center points, the design columns are numeric.

  The following attributes are attached to it:
  \item{desnum}{Design matrix in -1/1 coding}
  \item{run.order}{three column data frame, first column contains the run number in
       standard order, second column the run number as randomized, third column the
       run number with replication number as postfix;
       useful for switching back and forth between actual and standard run
       number}
  \item{design.info}{list with the entries
  \describe{
    \item{type}{ character string \dQuote{FrF2.large}}
    \item{nruns}{ number of runs (replications are not counted) }
    \item{nfactors}{ number of factors}
    \item{factor.names }{ list named with (treatment) factor names and containing as
         entries vectors of length two each with coded factor levels }
    \item{generators }{ for designs of type \code{FrF2.generators} only; \cr
         character vector of generators in the form D=ABC etc.}
    \item{aliased }{ alias structure of main effects, 2fis and possibly 3fis,
         depending on the choice of \code{alias.info};
         For non-blocked and non-split-plot designs, \code{aliased} is
         itself a list of the two or three components main, fi2, and optionally fi3, given
         in terms of factor letters from \code{Letters} (up to 50~factors)
         or \code{F1}, \code{F2}, and so forth (more than 50~factors).
         For blocked and split-plot designs, \code{aliased} is a single list
         with an entry for each column of the Yates matrix that accomodates
         aliased low-order effects, and entries are in terms of factor names.)}
    \item{replications }{ option setting in call to \code{FrF2} }
    \item{repeat.only }{ option setting in call to \code{FrF2} }
    \item{randomize }{ option setting in call to \code{FrF2} }
    \item{seed }{ option setting in call to \code{FrF2} }
    \item{creator }{ call to function FrF2Large;
         (in future, may also contain stored menu settings from R commander plugin
         \pkg{RcmdrPlugin.DoE}, once the function has been implemented in that package)
         }
    \item{FrF2.version }{ version number of package FrF2,
         supporting correct usage of \code{FrF2}-specific functionality in
         functions \code{summary} and \code{generators}
         methods for class \code{design}}
    \item{ncube }{ number of cube points per block, in case center points have been requested }
    \item{ncenter }{ number of center points per block, in case center points have been requested }
         }
       }
}
\references{
  Mee, R. (2009). \emph{A Comprehensive Guide to Factorial Two-Level Experimentation}.
  New York: Springer.

    Sanchez, S.M. and Sanchez, P.J. (2005). Very Large Fractional Factorial
     and Central Composite Designs.
     \emph{ACM Transactions on Modeling and Computer Simulation}
     \bold{15}, 362-377.
}
\section{Warning}{
   Since R version 3.6.0, the behavior of function \code{\link{sample}} has changed
   (correction of a biased previous behavior that might be relevant for the randomization
   of very large designs).
   For reproducing a randomized design that was produced with an earlier R version,
   please follow the steps described with the argument \code{seed}.
}
\author{ Ulrike Groemping }

\seealso{ See also \code{\link{FrF2}} for smaller regular fractional factorials and
\code{\link[DoE.base]{oacat}} for two non-regular resolution V fractional factorials (reported e.g. by
Mee 2009) for up to 19 factors in 256 runs or up to 63 factors in 2048 runs }
\examples{
## numbers of runs needed for resolution V designs in different numbers of factors
nrunsV(8)
nrunsV(18)
needed <- nrunsV(27)
needed
nrunsV(65)
nrunsV(71)

\dontrun{
plan <- FrF2Large(nrunsV(75),75)
summary(plan)
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{fold.design}
\Rdversion{1.1}
\alias{fold.design}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Function to create a foldover for 2-level fractional factorials
}
\description{
This function creates a foldover design for a 2-level fractional 
factorial. The purpose is to dealias (some) effects. 
Per default, all factors are folded upon, which makes the resulting 
design at least resolution IV. Different foldover versions can be requested.
}
\usage{
fold.design(design, columns = "full", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{design}{
  a data frame of class design that contains a 2-level fractional factorial; 
  currently, \code{design} must neither be blocked nor a long version parameter design
}
  \item{columns}{
  indicates which columns to fold on; the default \dQuote{full} folds on all columns, 
  i.e. swaps levels for all columns. \cr  
  A specific fold on certain columns can be requested giving a character vector 
  of factor names or a numeric vector of factor positions.\cr
  See the details section for some statistical comments.
   }
  \item{\dots}{
   currently not used}
}
\details{
  Foldover is a method to dealias effects in relatively small 
  2-level fractional factorial designs. The folded design has twice the number 
  of runs from the original design, and an additional column \dQuote{fold} that 
  distinguishes the original runs from the mirror runs. This column should be used 
  in analyses, since it captures a block effect on time (often the mirror runs 
  are conducted substantially later than the original experiment).
  
  Like most other software, this function conducts a full foldover per default, 
  i.e. the mirror portion reverses the levels of all factors. In terms of the 
  convenient -1/1 notation for factor levels, this can be written as 
  a multiplication with \dQuote{-1} for the mirror portion of all factors. 
  Thus, all confounding relations involving 
  an odd number of factors (e.g. A=BC) are resolved, because the odd side of the 
  equation involves a minus for the mirror runs, and the even side does not 
  (since the minuses cancel each other). (These 
  confounding relations are replaced by even ones 
  for which the odd side of the equation is multiplied with minus the new mirror factor \code{fold}.) 
  
  There are many situations, for which the default full foldover is 
  not the best possible foldover fraction, cf. e.g. Li and Mee (2002). 
  It is therefore possible to choose an arbitrary foldover fraction. For example, 
  folding on one particular factor alone dealiases all confounding relations 
  for that factor, folding on two particular factors dealiases all confounding relations 
  of these two with others but not of these two together with others and so on.
  
  Folding Plackett-Burman designs also removes the (partial) aliasing with 2-factor 
  interactions for all main effects that are mirrored.
  
}
\value{
  A data frame of class design with twice as many rows as \code{design} and 
  the additional factor \code{fold} (added as the last factor for folded \code{pb} 
  designs, as the first factor for splitplot designs, 
  and as the last \emph{base} factor for other folded regular fractional 
  factorial designs).
  
  Existing response values are of course preserved, and response values for the 
  new mirror runs are NA.
  
  The type in attribute \code{design.info} is suffixed with \dQuote{.folded}, and 
  \code{nruns} (and, if applicable, \code{nWPs}) is doubled, 
  \code{nfactors} (and, if applicable, \code{nfac.WP}) 
  is increased by one (for the factor fold, which 
  is a block factor and can also be treated as such, but will currently be treated as a fixed 
  (whole plot) factor by any automated analysis routine). The creator element receives a list entry for the fold columns.\cr
  For regular fractional factorials (design type starting with \code{FrF2}), the generator element is adjusted 
  (the generators for all generated fold factors now involve the folding factor), and an existing 
  catlg.entry element is replaced by a new generators element. The \code{aliased} element is 
  adapted to the new alias structure. Note that the fold factor enters as a new base factor and therefore 
  is added to the factor matrix after the first log2(nruns) factors. This implies that all factor 
  letters previously used for the generated factors are changed - for avoiding confusion it is always recommended to 
  work with factor names that are meaningful in a subject-matter sense.
  
  Furthermore, for the regular fractional factorial designs, 
  the column run.no.in.std.order in attribute \code{run.order} for the mirror portion of the design is 
  populated such that the base factors remain in the conventional order when ordered by 
  run.no.in.std.order (regardless whether or not they are included in the fold; 
  it is always possible to reorder runs such that the original base factors 
  together with the folding factor form the new base in standard order).
}
\references{

    Li, H. and Mee, R. (2002). Better foldover fractions for resolution III 2^(k-p) designs. 
    \emph{Technometrics} \bold{44}, 278--283. 
    New York: Springer.

    Mee, R. (2009). \emph{A Comprehensive Guide to Factorial Two-Level Experimentation}. 
    New York: Springer.

    Montgomery, D.C. (2001). \emph{Design and Analysis of Experiments (5th ed.)}. Wiley, New York.

}
\author{
Ulrike Groemping
}
\note{
    This function is still somewhat experimental.
}

\seealso{
See also as \code{\link{pb}}, \code{\link{FrF2}}
}
\examples{
  ## create resolution III design
  plan <- FrF2(8,5, factor.names=c("one","two","three","four","five"))
  ## add some resonse data
  y <- c(2+desnum(plan)\%*\%c(2,3,0,0,0) +
     1.5*apply(desnum(plan)[,c(1,2)],1,"prod") + rnorm(8))
     ## the "c()" makes y into a vector rather than a 1-column matrix
  plan <- add.response(plan, y)
  DanielPlot(lm(y~(.)^2,plan), alpha=0.2, half=TRUE)
  ## alias information
  design.info(plan)
  ## full foldover for dealiasing all main effects
  plan <- fold.design(plan)
  design.info(plan)
  ## further data, shifted by -2
  y <- c(y, desnum(plan)[9:16,1:5]\%*\%c(2,3,0,0,0) +
     1.5*apply(desnum(plan)[9:16,c(1,2)],1,"prod") + rnorm(8))
  plan <- add.response(plan, y, replace=TRUE)
  linmod <- lm(y~(.)^2,plan)
  DanielPlot(linmod, alpha=0.2, half=TRUE)
  MEPlot(linmod)
  IAPlot(linmod)
  
  ## fold on factor a only (also removes main effect aliasing here)
  plan <- FrF2(8,5, factor.names=c("one","two","three","four","five"))
  aliasprint(plan)
  plan <- fold.design(plan, columns=1)
  aliasprint(plan)
  
  ## fold a Plackett-Burman design with 11 factors
  plan <- pb(12)
  fold.design(plan)
   }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ design }
\keyword{ array }% __ONLY ONE__ keyword per line

\name{add.center}
\Rdversion{1.1}
\alias{add.center}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Function to add center points to a 2-level fractional factorial
}
\description{
This function adds center points to a 2-level fractional factorial design. 
All factors must be quantitative!
}
\usage{
add.center(design, ncenter, distribute=NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{design}{
  a data frame of class design that contains a 2-level fractional factorial 
  (regular or non-regular); 
  \code{design} must neither be a split-plot nor a long version parameter design.\cr
  For function \code{add.center}, the design must not contain center points yet, 
  while it has to contain center points for function \code{iscube}.
  
  For function \code{add.center}, blocked and replicated (or repeated measurement) designs 
  must be in the original run order (column run.no in run.order attribute 
  in ascending order), as the algorithm relies on the related runs being grouped as 
  expected. An error is thrown, if this condition is violated.
}
  \item{ncenter}{
  the number of center points to be added to each block
   }
  \item{distribute}{
  the number of positions over which to distribute the center points within each block; 
  note that the center points are not randomized but placed evenly throughout the 
  (hopefully randomdomized) design (but see also the details section);\cr
  if distribute is NULL, center points are all added at the end for non-randomized designs 
  and are distributed as evenly as possible to beginning, middle and end of the experiment 
  for randomized designs. 
  
  \code{distribute} must neither be larger than \code{ncenter} nor than 
  the number of runs of the design plus one. 
   }
  \item{\dots}{
   currently not used}
}
\details{
  Function \code{add.center} adds center points to 2-level fractional factorial 
  designs. Instead of using this function directly, center points should usually 
  be added directly with calls to functions \code{\link{FrF2}} or \code{\link{pb}}. 
  These make use of function \code{add.center} for this purpose.

  Center points are added to designs for three main reasons: 
  they provide a repeated benchmark run that can alert the experimenter to unplanned changes in experimental conditions, 
  they provide an independent estimate of experimental error, and finally they provide a possibility for 
  checking whether a first order model is sufficient. Especially for the first purpose, 
  package FrF2 follows the recommendation in Montgomery (2001, p.275). To distinguish them from the center points, 
  the original fractional factorial runs are called \dQuote{cube points}. 
  
  Addition of center points does not affect estimates for main effects and interactions. 
  The difference between the averages of cube points and center points gives an indication whether 
  quadratic terms might be needed in the model.
  
  For blocked designs and properly replicated designs, 
  \code{ncenter} center points are added to \emph{each} (replication) block. 
  In case of repeated measurements, center points are also measured repeatedly.
  
  Center points are distributed as evenly as possible over the \code{distribute} selected 
  positions throughout each block. \code{distribute=1} always adds all center points at the end of 
  each block. If \code{distribute > 1}, (each block of) the design starts and ends 
  with a (group of) center point(s), 
  and the \code{distribute} positions for placing center points are as evenly 
  placed throughout (each block of) the design as possible. \cr
  If \code{ncenter} is not a multiple of \code{distribute}, 
  some center point groups have one more center point than others. If \code{ncenter\%\%distribute} 
  is one or two only, the beginning and (for two) the end of (each block of) the design have one more center point, 
  otherwise the \code{ncenter\%\%distribute} extra center points are randomized over the center point positions.
  
  Function \code{\link[DoE.base]{iscube}} from package \pkg{DoE.base} 
  provides a logical vector that is TRUE for cube points 
  and FALSE for center points, which allows to use of simple functions 
  for \dQuote{clean} 2-level fractional factorials like \code{MEPlot}. 
}
\value{
  A data frame of class design with \code{ncenter} center point runs per block 
  (or per replication block) added to the \code{design} 
  (and its \code{desnum} and \code{run.order} attributes). 
  The run.no.in.std.order column of run.order is \dQuote{0} for the center points.
  
  Existing response values for cube runs are preserved, and response values for the 
  new center point runs are NA. Note, however, that center points should be added 
  BEFORE running the experiment in order to benefit from all their useful properties; 
  this should best be done within functions \code{\link{pb}} or \code{\link{FrF2}}.
  
  The design is identifiable as a design with center points by the 
  suffix \code{.center} to the \code{type} element of attribute \code{design.info},
  and the elements \code{ncube} and \code{ncenter} are added 
  (with the updated \code{nruns} being their sum). The element \code{coding} is 
  also added to the \code{design.info}, in order to support steepest ascent/descent 
  analysis from the center point.

}
\references{

    Montgomery, D.C. (2001). \emph{Design and Analysis of Experiments (5th ed.)}. Wiley, New York.

}
\author{
Ulrike Groemping
}
\note{
    This function is still somewhat experimental.
}

\seealso{
See also as \code{\link{pb}}, \code{\link{FrF2}}
}
\examples{
  ## purely technical example 
  plan <- FrF2(8,5, factor.names=c("one","two","three","four","five"))
  add.center(plan, 6)
  add.center(plan, 6, distribute=1)
  add.center(plan, 6, distribute=6)
  add.center(plan, 6, distribute=4)
  
  ## very artificial analysis example
    plan <- FrF2(8,4, factor.names=list(one=c(0,10),two=c(1,3),three=c(25,32),four=c(3.7,4.8)))
  ## add some response data
    y <- c(2+desnum(plan)\%*\%c(2,3,0,0) +
       1.5*apply(desnum(plan)[,c(1,2)],1,"prod") + rnorm(8))
       ## the "c()" makes y into a vector rather than a 1-column matrix
    plan <- add.response(plan, y)
  ## analysing this design provides an impression
    MEPlot(lm(y~(.)^2, plan))
    IAPlot(lm(y~(.)^2, plan))
    DanielPlot(lm(y~(.)^2,plan), half=TRUE, alpha=0.2)
  ## tentative conclusion: factors one and two do something
  ## wonder whether the model with one and two and their interaction is sufficient
  ## look at center points (!!! SHOULD HAVE BEEN INCLUDED FROM THE START,
  ##      but maybe better now than not at all)
  ## use distribute=1, because all center points are run at the end
    planc <- add.center(plan, 6, distribute=1)
      ## conduct additional runs for the center points
        y <- c(y, c(2+desnum(planc)[!iscube(planc),1:4]\%*\%c(2,3,0,0) +
             1.5*apply(desnum(planc)[!iscube(planc),][,c(1,2)],1,"prod") + rnorm(6)))
  ## add to the design
    planc <- add.response(planc, y, replace=TRUE)
  ## sanity check: repeat previous analyses for comparison, with the help of function iscube()
    MEPlot(lm(y~(.)^2, planc, subset=iscube(planc)))
    IAPlot(lm(y~(.)^2, planc, subset=iscube(planc)))
    DanielPlot(lm(y~(.)^2, planc, subset=iscube(planc)), half=TRUE, alpha=0.2)
  ## quick check whether there a quadratic effect is needed: is the cube indicator significant ?
    summary(lm(y~(.)^2+iscube(planc), planc))
        ## (in this unrealistic example, the quadratic effect is dominating everything else;
        ## with an effect that strong in practice, it is likely that
        ## one would either have expected a strong non-linearity before conducting the experiment, 
        ## OR that the effect is not real but the result of some stupid mistake
  ## alternatively, the check can be calculated per hand (cf. e.g. Montgomery, Chapter 11):
    (mean(planc$y[iscube(planc)])-mean(planc$y[!iscube(planc)]))^2*8*6/(8+6)/var(y[!iscube(planc)])
    ## must be compared to the F-quantile with 1 degree of freedom
    ## is the square of the t-value for the cube indicator in the linear model
   }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ design }
\keyword{ array }% __ONLY ONE__ keyword per line

\name{block}
\alias{block}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Statistical and algorithmic aspects of blocking in FrF2 }
\description{
  This help page documents the statistical and algorithmic details of blocking in FrF2 
}
\details{
  Blocking is done with the purpose to balance the design with respect to a factor 
  that is known or strongly suspected to have an influence but is not in itself of interest, 
  and it is usually assumed that block factors do not interact with experimental factors. 
  Examples are batches of material that are not large enough to accomodate the complete 
  experiment so that e.g. half the experiment is done on the first batch and the other half 
  on the second batch (two blocks). The block factor should be orthogonal to the experimental 
  factors, at least to their main effects. Per default, it is also requested that the block 
  factor is orthogonal to the 2-factor interactions. This can be changed by the user, if no 
  such design can be found.

  Blocking is currently implemented for regular fractional factorial designs only.\cr
  There are two principal ways to handle blocked designs, manual definition 
  (i.e. the user specifies exactly which columns are to be used for which purpose) and automatic 
  definition. Each situation has its specifics. These are detailed below. For users with 
  not so much mathematical/statistical background, it will often be best to use the automatic way, 
  specifying the treatment factors of interest via \code{nfactors} or \code{factor.names} 
  and a single number for \code{blocks} or \code{WPs}. 
  Users with more mathematical background may want to use the manual definitions, perhaps 
  in conjunction with published catalogues of good block designs, or 
  after inspecting possibilities with functions \code{\link{blockpick}}, \code{\link{blockpick.big}} (default before version 2 for large settings) or \code{\link{colpick}} (default since version 2 for large settings or settings with estimability requirements).
  
  \describe{
  \item{Manual definition of blocked designs for regular fractional factorials}{
  The user can start from a design with a number of factors and manually specify which factors or 
  interactions are to be used as block generators. If this route is chosen, \code{blocks} 
  can be a vector of factor names or factor letters, or of the same form as generators, except that 
  not only base factors but all factors can be used and single factors are permitted 
  (which would lead to resolution II designs if used in generators). For example, \cr
    \code{block = Letters[c(2,4,5)]} \cr
  or \cr
    \code{block = list(2,4,5)} \cr
  specify that the 2nd, 4th and 5th factor are to be used as block generators, while \cr
    \code{block = c("Day","Shift")}\cr
  indicates that the named factors \dQuote{Day} and \dQuote{Shift} specified in \code{factor.names} 
  are to be treated as blocking factors). In this case, the number of blocks is calculated, 
  and a new factor with the default name \dQuote{Blocks} (in general the name chosen in 
  option \code{block.name}) is generated, which would for example contain as levels 
  the Day/Shift combinations. It is also possible to choose interaction effects rather than factors themselves 
  as block generators, e.g. \cr
    \code{block = c("ABCD","EFGH") } \cr
  or\cr
    \code{block = list(c(1,2,3,4),c(5,6,7,8)) }. \cr
  Finally, it is also possible to specify choice of blocks using a vector of Yates column numbers, 
  in order to be able to use catalogued blocking structures of this form, e.g. from Sitter, Chen and Feder 
  (1997).\cr
  The block main effects are defined by the \code{k.block} specified effect 
  and all interactions between them. The specified block effects are required to be independent from each other, 
  which implies that they generate \code{2^k.block} blocks. \cr
  CAUTION: If the user manually generates a blocked design, it is his/her responsibility to ensure a 
  good choice of design (e.g. by using a catalogued design from Bisgaard 1994, 
  Sun, Wu and Chen 1997, Sitter, Chen and Feder (1997), or Cheng and Wu 2002).
  Since version 2 of package \pkg{FrF2}, manual blocking is also checked for confounding 
  of the block factor with main effects or two-factor interactions; 
  this implies that some earlier code will now require the additional specification 
  of argument \code{alias.block.2fis=TRUE} in order to avoid errors.
  }
    \item{Automatic definition of blocked designs for regular fractional factorials}{
  If the user only specifies the number of blocks required for the experiment, function \code{\link{FrF2}} 
  automatically generates the blocks. For full factorial designs, function \code{\link{FrF2}} uses function \code{\link{colpick}} 
  with subsequent \code{\link{blockgencreate}}, except where the Sun, Wu and Chen (1997) catalogue of blocked designs 
  contains suitable block generators for a design without estimability requirements 
  (implemented in function \code{\link{blockpick}}, which also calls \code{colpick}, if that 
  catalogue does not offer a solution). 
  Otherwise, depending on the situation, 
  function \code{\link{FrF2}} uses function \code{\link{blockpick}} or function \code{\link{colpick}} 
  with subsequent \code{\link{blockgencreate}}; 
  function \code{\link{blockpick}} treats smaller problems (\code{choose(nruns-1-nfactors,k.block) < 100000}) 
  without estimability requirements and with \code{force.godolphin=FALSE} (the latter is per default set to TRUE 
  whenever \code{alias.block.2fis=TRUE}), 
  other problems are treated by function \code{\link{colpick}}.
  
  Use of the earlier default function \code{\link{blockpick.big}} for large cases or the earlier behavior for full factorial designs 
  can be requested with the argument 
  \code{block.old=TRUE}; this should only be done for reproducing earlier results, as the new methodology is definitely superior. 
  
  The search for an appropriate blocked design starts with the overall best unblocked design 
  (in terms of aberration or MaxC2, if requested). 
  If this best design does not yield an adequate blocking possibility, the search continues with 
  the next best design and so forth (exception: with an estimability requirement, only a single design, prefiltered for the estimability requirement, is subjected to the blocking algorithm). \cr
  For the smaller problems, function \code{\link{blockpick}} 
  looks for \code{k.block} independent subsets among the eligible columns of the design. 
  (The eligible columns are all columns of the Yates matrix that are neither occupied 
  by treatment main effects nor by 2fis among treatments (if \code{alias.block.2fis=FALSE}, 
  which is the default), or all columns of the Yates matrix that are not occupied by treatment main effects 
  (if \code{alias.block.2fis=TRUE}). Note that no effort is made to avoid aliasing with 2-factor interactions, 
  if \code{alias.block.2fis=TRUE} is chosen. 
  
  For the larger problems, or blocking in combination with requiring some 2fis to be clear of aliasing, 
  or per default for blocking with permitting 2fis to be aliased with blocks, 
  function \code{\link{colpick}} creates a \eqn{q \times n}{q times \code{nfactors}} \bold{X} matrix for creating 
  blocks of size \eqn{2^q} based on the approach described by Godolphin (2021); 
  function \code{blockgencreate} creates block generators from this matrix. 
  This approach can be used in combination with argument \code{estimable}, 
  as long as \code{clear=TRUE}. The implementation of this approach is described in Groemping (2021).
  The argument \code{force.godolphin} of function \code{\link{FrF2}} can enforce the Godolphin approach instead 
  of the default approach for small blocked designs without \code{alias.block.2fis=TRUE}, 
  and the Godolphin approach can be switched off for \code{alias.block.2fis=TRUE} applications by explicitly requesting 
  \code{force.godolphin=FALSE}. Note that the Godolphin approach solely focuses on clear 2fis 
  of the blocked design and does not attempt to avoid confounding of the block factor with non-clear 2fis; it may 
  thus confound 2fis with the block factor even if this were avoidable, maintaining the same number of clear 2fis.
  
  For the larger problems, in versions before 2.0, which can be activated in current versions with \code{block.old=TRUE},
  function \code{\link{blockpick.big}} permutes the k~base factors of candidate designs with \code{nfactors + k.block} factors 
  in search of a design the first \code{k.block}~factors of which can be used for block construction. Any 
  specification of design (via options \code{design} or \code{generators}) is ignored. Note that function 
  \code{\link{blockpick.big}} is not guaranteed to find an existing blocked design.
  
  Sun, Wu and Chen (1997) provided a catalogue of blocked designs 
  with a few quality criteria, and they stated that there is no single best design, but that the choice 
  depends on the situation. \code{FrF2} always comes up with one specific solution design. 
  Comparisons to the catalogued designs in Sun, Wu and Chen (1997) have shown that 
  the designs found in \code{FrF2} are often but not always isomorphic to the catalogued ones. 
  Differences do occur, especially if the base designs are resolution III, or if \code{\link{blockpick.big}} 
  has to be used. Expert users who want to be certain to use a \dQuote{best} blocked design should manually 
  implement a specific catalogued design or inspect several solutions from functions \code{\link{blockpick}} or \code{\link{colpick}} (or, if desparate, \code{\link{blockpick.big}}). 
  }
  }
  
  Please contact me with any suggestions for improvements.
}
\references{
Bisgaard, S. (1994a). Blocking generators for small \eqn{2^{k-p}}{2^(k-p)} designs. 
\emph{J. Quality Technology} \bold{26}, 288-294.

Chen, J., Sun, D.X. and Wu, C.F.J. (1993) 
A catalogue of 2-level and 3-level orthogonal arrays. 
\emph{International Statistical Review} \bold{61}, 131-145. 

Cheng, C.-S. and Tsai, P.-W. (2009). 
Optimal two-level regular fractional factorial block and split-plot designs. 
\emph{Biometrika} \bold{96}, 83-93.

Cheng, S.W. and Wu, C.F.J. (2002). 
Choice of optimal blocking schemes in 2-level and 3-level designs. 
\emph{Technometrics} \bold{44}, 269-277. 

Godolphin, J. (2021). Construction of Blocked Factorial Designs to Estimate 
   Main Effects and Selected Two-Factor Interactions. \emph{J. Royal Statistical Society} \bold{B 83}, 5-29. 
   \doi{10.1111/rssb.12397}.

Groemping, U. (2019). An algorithm for blocking regular fractional 
  factorial 2-level designs with clear two-factor interactions. 
   \emph{Reports in Mathematics, Physics and Chemistry}, \href{http://www1.bht-berlin.de/FB_II/reports/Report-2019-003.pdf}{Report 3/2019}, 
   Department II, Beuth University of Applied Sciences Berlin.

Sitter, R.R., Chen, J. and Feder, M. (1997). 
Fractional Resolution and Minimum Aberration in Blocked 2n-k Designs.
\emph{Technometrics} \bold{39}, 382--390.

Sun, D.X., Wu, C.F.J. and Chen, Y.Y. (1997). 
Optimal blocking schemes for \eqn{2^p} and \eqn{2^{n-p}}{2^(n-p)} designs. \emph{Technometrics} \bold{39}, 
298-307. 

}
\author{ Ulrike Groemping }

\seealso{ See Also \code{\link{FrF2}} for regular fractional factorials, 
\code{\link{catlg}} for the Chen, Sun, Wu catalogue of designs 
and some accessor functions, 
and \code{\link{splitplot}} for the statistical aspects of split-plot designs.}
\examples{
########## automatic blocked designs ###################
## from a full factorial ##
FrF2(8,3,blocks=2)
## with replication
run.order(FrF2(8,3,blocks=2,wbreps=2))
run.order(FrF2(8,3,blocks=2,wbreps=2,repeat.only=TRUE))
run.order(FrF2(8,3,blocks=2,bbreps=2))
run.order(FrF2(8,3,blocks=2,bbreps=2,wbreps=2))

## automatic blocked design with fractions
FrF2(16,7,blocks=4,alias.block.2fis=TRUE)
## isomorphic non-catalogued design as basis
FrF2(16,gen=c(7,11,14),blocks=4,alias.block.2fis=TRUE)
## FrF2 uses blockpick.big and ignores the generator
FrF2(64,gen=c(7,11,14),blocks=16,alias.block.2fis=TRUE)

########## manual blocked design ####################
### example that shows why order of blocks is not randomized
### can of course be randomized by user, if appropriate
FrF2(32,9,blocks=c("Day","Shift"),alias.block.2fis=TRUE, 
    factor.names=list(Day=c("Wednesday","Thursday"), Shift=c("Morning","Afternoon"),
        F1="",F2="",F3="",F4="",F5="",F6="",F7=""), default.levels=c("current","new"))

########## blocked design with estimable 2fis ####################
### all interactions of last two factors to be estimable clearly
### in 64 run design with blocks of size 4
### not possible with catalogue entry 9-3.1
FrF2(design="9-3.2", blocks=16, alias.block.2fis=TRUE, 
    factor.names = list(C1="",C2="",C3="",C4="",C5="",C6="",C7="",
    N1=c("low","high"),N2=c("low","high")), 
    default.levels = c("current","new"),
    estimable=compromise(9, 8:9)$requirement)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{aliases}
\alias{aliases}
\alias{aliasprint}
\alias{print.aliases}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Alias structure for fractional factorial 2-level designs }
\description{
  Functions to examine the alias structure of a 
  fractional factorial 2-level design
}
\usage{
aliases(fit, code = FALSE, condense=FALSE)
aliasprint(design, ...)
\method{print}{aliases}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fit}{ a linear model object with only 2-level factors as explanatory variables;
              the function will return an error, if the model contains partially
              aliased effects (like interactions in a Plackett-Burman design 
              for most cases) }
  \item{code}{ if TRUE, requests that aliasing is given in code letters (A, B, C 
              etc.) instead of (potentially lengthy) variable names; 
              in this case, a legend is included in the output object.}
  \item{condense}{ if TRUE, reformats the alias information to 
              be comparable to the version calculated by internal function alias3fi;
              does not work with models with higher than 3-way interactions;
              for up to 3-way interactions, the output may be more easily readible}
  \item{design}{ a data frame of class \code{design} that should contain a fractional 
              factorial 2-level design; the function does not print anything if 
              the design is of different nature}
  \item{x}{   an object of class \code{aliases} that should be the output from 
              function \code{aliases} }
  \item{\dots}{further arguments to function \code{\link{print.default}};\cr
              the \code{quote} argument cannot be used}
}
\value{
  Function \code{aliasprint} returns NULL and is called for its side effects only.
  
  Per default, Function \code{aliases} returns a list with two elements:
  \item{legend }{links the codes to variable names, if \code{code=TRUE}.}
  \item{aliases }{is a list of vectors of aliased effects.}
  
  If option \code{condense} is TRUE, the function returns a list with elements legend, 
  main, fi2 and fi3; this may be preferrable for looking at the alias structure of larger designs. 
  
  The output object of function \code{aliases} has class \code{aliases}, 
  which is used for customized printing with the \code{print} method.
}
\references{ 
    Box G. E. P, Hunter, W. C. and Hunter, J. S. (2005) 
    \emph{Statistics for Experimenters, 2nd edition}.
    New York: Wiley.
    }
\author{ Ulrike Groemping }
\seealso{ \code{\link{FrF2-package}} for information on the package, 
     \code{\link{alias}} for the built-in R-function, 
     \code{\link{IAPlot}} for effects plots }
\keyword{ design }

\examples{
    ### Injection Molding Experiment. Box et al. 1978.
    ## data(BM93.e3.data, package="BsMD")  #from BsMD
    ## iMdat <- BM93.e3.data[1:16,2:10]  #only original experiment
    ## re-create here
    y=c(14, 16.8, 15, 15.4, 27.6, 24, 27.4, 22.6, 
     22.3, 17.1, 21.5, 17.5, 15.9, 21.9, 16.7, 20.3)
    iMdat <- FrF2(8,7,randomize=FALSE)
    iMdat <- desnum(iMdat)
    iMdat <- rbind(cbind(iMdat,H=1),cbind(-iMdat,H=-1))
    iMdat <- cbind(as.data.frame(iMdat), y=y)

    # make data more user-friendly
    colnames(iMdat) <- c("MoldTemp","Moisture","HoldPress","CavityThick",
             "BoostPress","CycleTime","GateSize","ScrewSpeed","y")
    # determine aliases with all 2-factor-interactions
    aliases(lm(y ~ (.)^2, data = iMdat))
    # coded version
    aliases(lm(y ~ (.)^2, data = iMdat), code=TRUE)
    # determine aliases with all 3-factor-interactions
    aliases(lm(y ~ (.)^3, data = iMdat), code=TRUE)
    # show condensed form
    aliases(lm(y ~ (.)^3, data = iMdat), code=TRUE, condense=TRUE)
    # determine aliases for unaliased model
    aliases(lm(y ~ ., data = iMdat))
}

\name{utilitiesV}
\alias{revdigits}
\alias{indexcalc}
\alias{gencalc}
\alias{YatesFly}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{ ~~ Internal utility functions for generating regular fractional factorials ~~ }
\description{
  ~~ Internal functions, mainly for generating designs without relying on a catalogue of designs ~~
}
\usage{
revdigits(obj, ndigits = 1 + floor(log2(max(obj))))

indexcalc(obj, k = 1 + floor(log2(max(obj))))

gencalc(gen)

YatesFly(walshindex, k=NULL)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{obj}{ non-negative integer (vector) with Yates matrix column numbers 
          (called Walsh indices by Sanchez and Sanchez 2005) }
  \item{ndigits}{ number of binary digits; default is the necessary number for the largest element of \code{obj} }
  \item{k}{ number of base factors; default is the minimum necessary number for the maximum of \code{obj} }
  \item{gen}{ generators in any admissible format (\code{gen.check} transforms any admissible format to the default needed for calculations)) }
  \item{walshindex}{ non-negative integer (vector) with Yates matrix column numbers 
       (called Walsh indices by Sanchez and Sanchez 2005; \cr
       must contain the base column numbers 1, 2, 4, 8 and so forth for the desired dimension;
       if this rule is violated, the first non-base column(s) are omitted in favor of supplementing 
       the set of base columns)  }
}
\details{
    These internal functions are used for generating regular 2-level fractional factorials. 
    The Sanchez and Sanchez (2005) way of providing large resolution V designs 
    is supported, as well as manual design generation of large designs with 
    functions \code{\link{FrF2}} and \code{\link{FrF2Large}}.
    }
\value{
    \code{revdigits} creates a matrix with 0-1 entries in an order appropriate for 
       creating character generators from Yates matrix column numbers,\cr 
    \code{indexcalc} calculates a named list of vectors of base column numbers, 
        named by the corresponding character generator, \cr
    \code{gencalc} calculates Yates column numbers from arbitrary types of generator 
        representations (except for negative numbers),\cr
    \code{YatesFly} produces an experimental plan from a vector of Yates column numbers 
        which have to include the base column numbers (contrary to the 
        \code{generators} option in \code{FrF2} or \code{FrF2Large}
}
\references{ 
    Sanchez, S.M. and Sanchez, P.J. (2005). Very Large Fractional Factorial 
     and Central Composite Designs. 
     \emph{ACM Transactions on Modeling and Computer Simulation} 
     \bold{15}, 362-377.
}
\author{ Ulrike Groemping }
\note{ This package is still under development, but does already provide useful and well-tested results.}
\seealso{ See also \code{\link{FrF2Large}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line
\keyword{ internal }

\name{DanielPlot}
\alias{DanielPlot}
\alias{DanielPlot.design}
\alias{DanielPlot.default}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Normal or Half-Normal Effects Plots }
\description{
  The function is modified from the same-name function in packge \pkg{BsMD}
  with the purpose of providing more usage comfort (correct effect sizes 
  in case of factors, automatic annotation, 
  automatic labelling of the most significant factors only).
}
\usage{
DanielPlot(fit, ...)
\method{DanielPlot}{design}(fit, ..., response = NULL)
\method{DanielPlot}{default}(fit, code = FALSE, autolab = TRUE, alpha = 0.05, faclab = NULL, 
       block = FALSE, datax = TRUE, half = FALSE, pch = "*", 
       cex.fac = par("cex.lab"), cex.lab = par("cex.lab"), 
       cex.pch = par("cex"), cex.legend = par("cex.lab"), 
       main = NULL, subtitle=NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fit}{an experimental design of class \code{\link[DoE.base:class-design]{design}} 
              with the \code{type} element of the \code{design.info} attribute 
              containing \dQuote{FrF2} or \dQuote{pb}\cr
              OR\cr
              object of class \code{lm}. Fitted model from \code{lm} or \code{aov}.}
  \item{\dots}{further arguments to be passed to the default function, 
      or graphical parameters to be passed to \code{plot};
      note that one should not use \code{pch} for split-plot designs.}
  \item{response}{ NULL or a character string that specifies response variable to be used, 
              must be an element of \code{response.names(obj)}; 
              if NULL, the first response from \code{response.names(obj)} is used}
  \item{code}{logical. If \code{TRUE} labels \dQuote{A},\dQuote{B}, etc. are used 
        instead of the names of the coefficients (factors). A legend linking 
        codes to names is provided.}
  \item{autolab}{ If TRUE, only the significant factors according to the Lenth method
                  (significance level given by \code{alpha}) are labelled. }
  \item{alpha}{ significanc level for the Lenth method }
  \item{faclab}{NULL or list. 
    If \code{NULL}, point labels are automatically determined according to the 
    setting of \code{code} (i.e. A,B,C etc. for \code{code=TRUE}, natural effect 
    names otherwise) and \code{autolab} (i.e. all effects are labelled if \code{autolab=FALSE},
    only significant effects are labelled if \code{autolab=TRUE}).  
    Otherwise, \code{faclab} can be used for manual labelling of certain effects 
    and should be a list with \code{idx} (integer vector referring to 
    position of effects to be labelled) and \code{lab} (character vector of labels)
    components. 
    }
  \item{block}{logical. If \code{TRUE}, the first factor is labelled as \dQuote{BK} (block).}
  \item{datax}{logical. If \code{TRUE}, the x-axis is used for the factor effects the
                the y-axis for the normal scores. The opposite otherwise.}
  \item{half}{logical. If \code{TRUE}, half-normal plot of effects is display.}
  \item{pch}{numeric or character. Points character.}
  \item{cex.fac}{numeric. Factor label character size.}
  \item{cex.lab}{numeric. Labels character size.}
  \item{cex.pch}{numeric. Points character size.}
  \item{cex.legend}{numeric. Legend size in case of codes.}
  \item{main}{NULL or character. Title of plot. If NULL, automatic title is generated.}
  \item{subtitle}{NULL or character. Sub title of plot. Should not be used for split-plot designs, 
     because automatic subtitle is generated for these.}
}

\details{
    The design underlying \code{fit} 
    has to be a (regular or non-regular) fractional factorial 2-level design. 
    Effects (except for the intercept) are displayed in a normal or half-normal 
    plot with the effects in the x-axis by default. 
    
    If \code{fit} is a design with at least one response variable 
    rather than a linear model fit, 
    the \code{lm}-method for class \code{design} is applied to it with 
    degree high enough that at least one effect is assigned to each column of the Yates matrix, 
    and the default method for \code{DanielPlot} is afterwards applied to the 
    resulting linear model.
    
    For split-plot designs, whole plot effects are shown as different plotting 
    characters, because they are potentially subject to larger variability, 
    and one should not be too impressed, if they look impressively large, 
    as this may well be indication of plot-to-plot variability rather than a 
    true effect.
}

\value{
    The function invisibly returns a data frame with columns: \code{x}, \code{y},
    \code{no}, \code{effect}, \code{coded} (if coded plot was requested) 
    and \code{pchs}, for the coordinates, the position numbers, 
    the effect names, the coded effect names, and the plotting characters 
    for plotted points.
    
    The plotting characters are particularly useful for split-plot designs and can be used 
    for subsequent separate plotting of whole-plot and split-plot effects, if necessary.
}

\references{
    Box G. E. P, Hunter, W. C. and Hunter, J. S. (2005)
    \emph{Statistics for Experimenters, 2nd edition}.
    New York: Wiley.

    Daniel, C. (1959) 
    Use of Half Normal Plots in Interpreting Two Level Experiments.
    \emph{Technometrics} \bold{1}, 311--340.
    
    Daniel, C. (1976) 
    \emph{Application of Statistics to Industrial Experimentation.}
    New York: Wiley.

    Lenth, R.V. (1989) Quick and easy analysis of unreplicated factorials.
    \emph{Technometrics} \bold{31}, 469--473.
    
    Lenth, R.V. (2006) Lenth s Method for the Analysis of Unreplicated Experiments.
    To appear in \emph{Encyclopedia of Statistics in Quality and Reliability}, Wiley, New York.
    Downloadable at \url{https://www.wiley.com/legacy/wileychi/eqr/docs/sample_1.pdf}.
}
\note{If you load package \pkg{BsMD} after package \pkg{FrF2}, 
    a mere call to function \code{DanielPlot} will use the function from package \pkg{BsMD} 
    rather than the one from package \pkg{FrF2}. You can explicitly request 
    usage of the \pkg{FrF2} function by \code{FrF2::DanielPlot}. }
\author{Ernesto Barrios, modified by Ulrike Groemping.}
\seealso{\code{\link{qqnorm}}, \code{\link[DoE.base]{halfnormal}}, 
   \code{\link[BsMD]{LenthPlot}}, \code{BsMD-package} }
\keyword{design}

\name{BsProb.design}
\alias{BsProb.design}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Bayesian posterior probabilities from Box and Meyer method }
\description{
  The function calculates Bayesian posterior probabilities according 
  to Box and Meyer (1993) for screening experiments with 2-level factors.
  The function is modified from function BsProb in packge \pkg{BsMD}
  with the purpose of providing usage comfort for 
  class design objects.
}
\usage{
BsProb.design(design, mFac = NULL, response=NULL, select=NULL, mInt = 2, p = 0.25, g = 2,
    ng = 1, nMod = 10)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{design}{an experimental design of class \code{\link[DoE.base:class-design]{design}} 
              with the \code{type} element of the \code{design.info} attribute 
              containing \dQuote{FrF2} or \dQuote{pb} and at least one response variable}
  \item{response}{ NULL or a character string that specifies response variable to be used, 
              must be an element of \code{response.names(obj)}; 
              if NULL, the first response from \code{response.names(obj)} is used}
  \item{mFac}{integer. Maximum number of factors included in the models.
              The default is the number of factors in the design.}
  \item{select}{ vector with position numbers of the factors to be included; \cr
              default: all factors. }
  \item{mInt}{integer <= 3. Maximum order of interactions considered in the models.
              This can strongly impact the result.}
  \item{p}{numeric. Prior probability assigned to active factors.
              This can strongly impact the result.}
  \item{g}{numeric vector. Variance inflation factor(s) gamma associated to active and interaction factors;
              see "Details" section}
  \item{ng}{integer <=20. Number of different variance inflation factors (g) used in calculations.}
  \item{nMod}{integer <=100. Number of models to keep with the highest posterior probability.}
}

\details{
Factor and model posterior probabilities are computed by the Box and Meyer (1993) Bayesian procedure. 
The design factors - or a selection of these given by column numbers in \code{select} - 
are considered together with the specified response or the first response of the design. 
The function has been adapted from function \code{\link[BsMD]{BsProb}} in package \pkg{BsMD}, 
and a vignette in that package (\url{../../BsMD/doc/BsMD.pdf}) explains 
details of the usage regarding the parameters.

If \code{g}, the variance inflation factor (VIF) gamma, is a vector of length 1, 
the same VIF is used for factor main effects and interactions. 
If the length of \code{g} is 2 and \code{ng} is 1, \code{g[1]} 
is used for factor main effects and \code{g[2]} for the interaction effects. 
If \code{ng} greater than 1, then \code{ng} values of VIFs between \code{g[1]} and 
\code{g[2]} are used for calculations with the same gamma value for main effects 
and interactions. The function calls the FORTRAN subroutine \code{bm} and captures 
summary results. The complete output of the FORTRAN code is save in the \code{BsPrint.out} 
file in the working directory. The output is a list of class \code{BsProb} 
for which print, plot and summary methods are available from package \pkg{BsMD}.
}

\value{
    cf. documentation of function \code{\link[BsMD]{BsProb}}
}

\note{
    This method relies on the availability of package \pkg{BsMD}.
}

\references{
Barrios, E. (2013). Using the BsMD Package for Bayesian Screening and Model
   Discrimination. Vignette. \url{../../BsMD/doc/BsMD.pdf}.

Box, G. E. P and R. D. Meyer (1986). An Analysis for Unreplicated Fractional 
      Factorials. \emph{Technometrics} \bold{28}, 11-18.

Box, G. E. P and R. D. Meyer (1993). 
   Finding the Active Factors in Fractionated Screening Experiments. 
   \emph{Journal of Quality Technology} \bold{25}, 94-105. }
\author{Daniel Meyer, ported to R by Ernesto Barrios, port adapted to designs by Ulrike Groemping.}
\examples{
   ### there are several success stories and recommendations for this method
   ### in the simulated example here (not fabricated, 
   ###         it was the first one that came to my mind), 
   ### the method goes wrong, at least when using mInt=2 (the default, because 
   ###         Daniel plots work quite well for pure main effects models):
   ### active factors are A to E (perhaps too many for the method to work),
   ### the method identifies F, J, and L with highest probability 
   ### (but is quite undecided)
   plan <- pb(12)
   dn <- desnum(plan)
   set.seed(8655)
   y <- dn\%*\%c(2,2,2,2,3,0,0,0,0,0,0) + dn[,1]*dn[,3]*2 - dn[,5]*dn[,4] + rnorm(12)/10
   plan.r <- add.response(plan, response=y)
   if (requireNamespace("BsMD", quiet=TRUE)){
   plot(bpmInt2 <- BsProb.design(plan.r), code=FALSE)
   plot(bpmInt1 <- BsProb.design(plan.r, mInt=1), code=FALSE) ## much better!
   summary(bpmInt2)
   summary(bpmInt1)
   }
   ### For comparison: A Daniel plot does not show any significant effects according 
   ### to Lenths method, but makes the right effects stick out
   DanielPlot(plan.r, half=TRUE, alpha=1)
}
\seealso{\code{\link[BsMD]{plot.BsProb}}, \code{\link[BsMD]{print.BsProb}}, 
\code{\link[BsMD]{summary.BsProb}}, \code{\link[BsMD]{BsMD}} }
\keyword{design}

\name{FrF2}
\alias{FrF2}
\alias{FrF2.currentlychecked}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Function to provide regular Fractional Factorial 2-level designs }
\description{
  Regular fractional factorial 2-level designs are provided. 
  Apart from obtaining the usual minimum aberration designs in a fixed number of runs, it is possible to 
  request highest number of free 2-factor interactions instead of minimum aberration or to 
  request the smallest design that fulfills certain requirements (e.g. resolution V with 8 factors).
}
\usage{
FrF2(nruns = NULL, nfactors = NULL, factor.names = if (!is.null(nfactors)) {
        if (nfactors <= 50) Letters[1:nfactors] else 
                             paste("F", 1:nfactors, sep = "")} else NULL, 
        default.levels = c(-1, 1), ncenter=0, center.distribute=NULL, 
        generators = NULL, design = NULL, 
        resolution = NULL, select.catlg=catlg, 
        estimable = NULL, clear = TRUE, method="VF2", sort="natural", 
        ignore.dom = !isTRUE(all.equal(blocks,1)), 
        useV = TRUE, firsthit=FALSE, res3 = FALSE, max.time = 60, 
        perm.start=NULL, perms = NULL, 
        MaxC2 = FALSE, replications = 1, repeat.only = FALSE, 
        randomize = TRUE, seed = NULL, alias.info = 2, 
        blocks = 1, block.name = "Blocks", block.old=FALSE, 
        force.godolphin=alias.block.2fis,
        bbreps=replications, wbreps=1, 
        alias.block.2fis = FALSE, 
        hard = NULL, check.hard=10, WPs=1,nfac.WP=0,
        WPfacs=NULL, check.WPs = 10, ...)
FrF2.currentlychecked()
}

%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nruns}{ Number of runs, must be a power of 2 (4 to 4096), if given. 
  
      The number of runs can also be omitted. In that case, 
      if \code{resolution} is specified, the function looks for the smallest 
      design of the requested resolution that accomodates \code{nfactors} factors. 
      If the smallest possible design is a full factorial or not catalogued, 
      the function stops with an error.
      
      If \code{generators} is specified, \code{nruns} is required.
      
      If estimable is specified and \code{nruns} omitted, 
      \code{nruns} becomes the size of the smallest design that 
      MIGHT accomodate the effects requested in \code{estimable}. 
      If this run size turns out to be too low, an error is thrown. 
      In that case, explicitly choose \code{nruns} as twice the run size 
      given in the error message and retry. }
  \item{nfactors}{ is the number of 2-level factors to be investigated. 
      It can be omitted, if it is obvious from \code{factor.names}, 
      a specific catalogued design given in \code{design}, 
      \code{nruns} together with \code{generators}, or 
      \code{estimable}. 
      
      If \code{estimable} is used for determining 
      the number of factors, it is assumed that the largest main effect position number 
      occurring in \code{estimable} coincides with \code{nfactors}. 
      
      For blocked designs, block generator columns are not included in 
      \code{nfactors}, except if 
      the user explicitly specifies 2-level block generation factors as 
      part of the fractional factorial design (e.g. a factor shift with levels 
      morning and afternoon). 
      
      For automatically-generated split-plot designs (cf. details section), 
      \code{nfactors} simply is the number of all factors (whole plot and split plot together). 
      If \code{nfac.WP < log2(WPs)}, the algorithm will add (an) artificial plot generation factor(s). 
      \cr
      For manually-specified split-plot designs (through options \code{generators} or 
      \code{design} together with \code{WPfacs}), the user must specify at least \code{log2(WPs)} 
      split plot factors, i.e. \code{nfac.WP >= log2(WPs)} is required (and must, if necessary, 
      be achieved by adding \code{log2(WPs) - nfac.WP} extra independent generator columns 
      for the whole plot structure, which have to be counted in \code{nfac.WP} and \code{nfactors}).}
  \item{factor.names}{ a character vector of \code{nfactors} factor names 
      or a list with \code{nfactors} elements;\cr 
      if the list is named, list names represent factor names, otherwise default factor names are used;\cr 
      the elements of the list are \cr
      EITHER vectors of length 2 with factor levels for the respective factor\cr
      OR empty strings. 
      For each factor with an empty string in factor.names, the levels given in 
      \code{default.levels} are used;\cr
       Default factor names are the first elements of the character vector \code{\link[DoE.base:utilities]{Letters}}, 
       or the factors position numbers preceded by capital F in case of more than 50 factors. }
  \item{default.levels}{ default levels (vector of length 2) for all factors for 
      which no specific levels are given }
  \item{ncenter}{ number of center points per block; \code{ncenter > 0} 
        is permitted, if all factors are quantitative and the design is not a split-plot design }
  \item{center.distribute}{ the number of positions over which the center points 
        are to be distributed for each block; if NULL (default), center points are 
        distributed over end, beginning, and middle (in that order, if there are fewer than three center points) 
        for randomized designs, and appended to the end for non-randomized designs. 
        for more detail, see function \code{\link{add.center}}, which does the work.
        }
  \item{generators}{ There are \code{log2(nruns)} base factors the full factorial 
        of which spans the design (e.g. 3 for 8 runs). 
        The generators specify how the remaining factors 
        are to be allocated to interactions of these.
        
        \code{generators} can be 
        
        a list of vectors with position numbers of base factors (e.g. c(1,3,4) 
        stands for the interaction between first, third and fourth base factor) 
        
        a vector of character representations of these interactions, 
        e.g. \dQuote{ACD} stands for the same interaction as above
        
        a vector of columns numbers in Yates order (e.g. 13 stands for ACD). 
        Note that the columns 1, 2, 4, 8, etc., i.e. all powers of 2, are reserved 
        for the base factors and cannot be used for assigning additional factors, 
        because the design would become a resolution II design. For looking up 
        which column number stands for which interaction, type e.g. 
        \code{names(Yates)[1:15]} for a 16 run design.
        
        In all cases, preceding the respective entry with a minus sign 
        (e.g. -c(1,3,4), \dQuote{-ACD}, -13) implies that the levels 
        of the respective column are reversed.\cr
        WARNING: Minus signs do not cause an error, but neither have an effect in case 
        of automatic assignment of split-plot designs or hard-to-change columns.
        }
  \item{design}{ is a character string specifying the name of a design listed 
        in the catalogue specified as \code{select.catlg}, which is usually 
        the catalogue \code{\link{catlg}}
        }
  \item{resolution}{ is the arabic numeral for the requested resolution of the design. 
        \code{FrF2} looks for a design with at least this resolution. 
        Option \code{resolution} does not work, if \code{estimable}, \code{blocks} 
        or \code{WPs} are specified, and neither if \code{nruns} is given.\cr
        A design with resolution III (resolution=3) confounds main effects 
        with 2-factor interactions, a design with resolution IV confounds main 
        effects with three-factor interactions or 2-factor interactions with each other, 
        and designs with resolution V or higher are usually regarded as very strong, 
        because all 2-factor interactions are unconfounded with each other and 
        with main effects.}
  \item{select.catlg}{ specifies a catalogue of class \code{catlg} from 
        which an adequate design is selected and adapted. \cr
        The specified catalogue is used for design construction, 
        unless \code{generators} explicitly constructs a 
        non-catalogued design. The default \code{\link{catlg}} is adequate for most 
        applications. 
        
        If a specific different catalogue of designs is available, 
        this can be specified here. \cr
        Specification of a smaller subset of designs is useful, if \code{estimable} 
        has been given and \code{clear=FALSE}, for restricting the search 
        to promising designs.
        
        \link[FrF2.catlg128:Catalogues]{Names of catalogues from package} \pkg{FrF2.catlg128} 
        can be given here without prior loading of that package; loading of the package 
        and the selected catalogue will then happen automatically, provided the 
        package is installed (for version >=1.2 of package \pkg{FrF2.catlg128}; 
        for earlier versions, the suitable catalogue has to be manually loaded 
        using the \code{data()} command).
        }                
  \item{estimable}{ indicates the 2-factor interactions (2fis) that are to be estimable in 
        the design. Consult the specific help file (\code{\link{estimable.2fis}}) 
        for details of two different approaches of requesting estimability, 
        as indicated by the status of the \code{clear} option. \code{estimable} cannot be 
        specified together with \code{splitplot}, \code{generators} or \code{design}.

        \code{estimable} can be\cr 
        a numeric matrix with two rows, each column of which indicates one interaction,
        e.g. column 1 3 for interaction of the first with the third factor\cr
        OR\cr
        a character vector containing strings of length 2 with capital letters from \code{Letters} 
        (cf. package \pkg{DoE.base})
        for the first 25 factors and small letters for the last 25 (e.g. \code{c("AB","BE")} \cr
        OR\cr
        a formula that contains an adequate model formula, e.g. \cr
        \code{formula("~A+B+C+D+E+(F+G+H+J+K+L)^2")} \cr
        for a model with (at least) eleven factors. \cr
        The names of the factors used in the formula can be the same letters usable in 
        the character vector (cf. above, A the first factor, B the second etc.), 
        or they can correspond to the factor names from 
        \code{factor.names}. 
        }
  \item{clear}{ logical, indicating how estimable is to be used. See \code{\link{estimable.2fis}}. }
  \item{method}{ character string indicating which subgraph isomorphism search routine 
        of package \pkg{igraph} is used (\code{graph.subisomorphic.vf2} or 
        \code{graph.subisomorphic.lad}). 
        The default \code{"VF2"} uses VF2 algorithm by Cordella et al. (2001), 
        which was the only available algorithm before version 1.7 of package FrF2. 
        The alternative \code{"LAD"} uses the LAD algorithm by Solnon (2010), which was 
        reported to be substantially faster than VF2 especially for some notoriously 
        difficult VF2 cases (see Gr?mping 2014b).         
        This option is relevant for \code{estimable} with \code{clear=TRUE} only.\cr
        NOTE: The resulting design may be different for different settings of this option!}
  \item{sort}{ character string indicating how the estimability requirement and the 
        candidate design clear 2fis are handed to 
        the subgraph isomorphism search routine of package \pkg{igraph}. 
        The default \code{"natural"} leaves them in unchanged order (like in 
        \pkg{FrF2} versions up to 1.6). \code{sort="high"} and \code{sort="low"} sort 
        both requirement set and candidate design graph according to vertex degrees (high 
        first or low first). 
        
        This option is relevant for \code{estimable} with \code{clear=TRUE} only.\cr
        It has been added, because pre-sorting of vertices sometimes speeds up the 
        search by several orders of magnitude especially for the \code{VF2} method.\cr
        NOTE: The resulting design may be different for different settings of this option!}
  \item{ignore.dom}{ logical, default FALSE for unblocked designs, TRUE for blocked designs; 
        if TRUE, \code{estimable} ignores 
        the dominating attribute of the catalogue entries; can be useful for 
        searching a blocked design with estimable 2fis from a reduced catalogue}
  \item{useV}{ NULL or logical; relevant for designs with \code{blocks} and \code{estimable} only; 
        if TRUE, function \code{link{colpick}} is used (default), otherwise function \code{link{colpickIV}}; 
        if set to NULL, \code{colpick} is used for fractions with resolution at least V and function 
        \code{link{colpickIV}} for resolution IV fractions;\cr
        \code{useV=FALSE} does the subgraph isomorphism check once only, 
        at the expense of missing out opportunities; it may be worth trying \code{useV=NULL} 
        or \code{useV=FALSE} for resolution IV situations, 
        for which the search takes very long with the default}
  \item{firsthit}{ logical; relevant for designs with \code{blocks} and \code{estimable} only; 
        if FALSE, the function tries to find a design with as many as possible clear 2fis, 
        otherwise the function stops at the first possible blocking; in case of resource problems, 
        setting \code{firsthit} to TRUE may be helpful}
  \item{res3}{ logical; if TRUE, \code{estimable} includes resolution III designs 
        into the search for adequate designs; otherwise resolution IV and higher designs 
        are included only. }
  \item{max.time}{ maximum time for design search as requested by \code{estimable}, 
        in seconds (default 60); introduced for \code{clear=FALSE} situations because
        the search can take a long time in complicated or unlucky situations;
        set max.time to Inf if you want to force a search over an extended period of 
        time; however, be aware that it may still take longer than feasible 
        (cf. also \code{\link{estimable.2fis}})}
  \item{perm.start}{ used with \code{estimable} specified, and \code{clear=FALSE}.\cr
        Provides a start permutation for permuting experiment factors (numeric vector). 
        This is useful for the case that a previous search was not (yet) successful 
        because of a time limit, since the algorithm notifies the user about the 
        permutation at which it had to stop.}
  \item{perms}{ used with \code{estimable} specified, and \code{clear=FALSE}.\cr
        Provides the matrix of permutations of experiment factors to be tried; each 
        row is a permutation. For example, for an 11-factor design with the 
        first six factors and their 2fis 
        estimable, it is only relevant, which of the eleven factors are to be allocated 
        to the first six experiment factors, and these as well as the other five factors can be 
        in arbitrary order. This reduces the number of required permutations from 
        about 40 Mio to 462. 
        It is recommended to use \code{perms} whenever possible, if \code{clear=FALSE}, 
        since this dramatically improves performance of the algorithm.
        
        It is planned to automatically generate perms for certain structures like 
        compromise designs in the (not so near) future.}
  \item{MaxC2}{ is a logical and defaults to FALSE. If TRUE, 
        maximizing the number of clear 2-factor interactions takes precedence 
        over minimizing aberration. Resolution is always considered first. 
        \code{MaxC2} is ignored when using the \code{estimable} option.
        \code{MaxC2=TRUE} is not a recommended choice. }
  \item{replications}{ positive integer number. Default 1 (i.e. each row just once). 
       If larger, each design run is executed replication times. 
       If \code{repeat.only}, repeated measurements 
       are carried out directly in sequence, i.e. no true replication takes place, 
       and all the repeat runs are conducted together. It is likely that the error 
       variation generated by such a procedure will be too small, so that average values 
       should be analyzed for an unreplicated design. 
       
       Otherwise (default), the full experiment is first carried out once, then 
       for the second replication and so forth. In case of randomization, 
       each such blocks is randomized separately. In this case, replication variance is 
       more likely suitable for usage as error variance 
       (unless e.g. the same parts are used for replication runs although build 
       variation is important).}
  \item{repeat.only}{ logical, relevant only if replications > 1. If TRUE, 
        replications of each run are grouped together 
       (repeated measurement rather than true replication). The default is 
       \code{repeat.only=FALSE}, i.e. the complete experiment 
       is conducted in \code{replications} blocks, and each run occurs in each block.  }
  \item{randomize}{ logical. If TRUE, the design is randomized. This is the default. 
       In case of replications, the nature of randomization depends on the setting of 
       option \code{repeat.only}.}
  \item{seed}{ optional seed for the randomization process \cr
        In R version 3.6.0 and later, the default behavior of function \code{\link{sample}}
        has changed. If you work in a new (i.e., >= 3.6.-0) R version and want to reproduce 
        a randomized design from an earlier R version (before 3.6.0), 
        you have to change the RNGkind setting by\cr
        \code{RNGkind(sample.kind="Rounding")}\cr
        before running function \code{FrF2}.\cr
        It is recommended to change the setting back to the new recommended way afterwards:\cr
        \code{RNGkind(sample.kind="default")}\cr
        For an example, see the documentation of the example data set \code{\link[DoE.base]{VSGFS}}.
        }
  \item{alias.info}{ can be 2 or 3, gives the order of interaction effects for which 
       alias information is to be included in the \code{aliased} component of the 
       \code{design.info} element of the output object. }
  \item{blocks}{ is EITHER \cr 
       the number of blocks into which the experiment is subdivided \cr
       OR a character vector of names of independent factors that are used as block constructors \cr
       OR a vector of Yates column numbers
       OR a list of generators similar to list entries for \code{generators}. \cr
       In the latter case, 
       the differences to \code{generators} are \cr 
       \itemize{
       \item that numbers/letters refer to the factors of the experiment and not to 
       column numbers of the Yates matrix 
       \item that numbers/letters can refer to *all* \code{nfactors} factors rather than the log2(nruns) base factors only, 
       \item that one single number is always interpreted as the number of blocks rather than a column reference, 
       \item that individual numbers are allowed in a list (i.e. individual factors specified in the experiment can be used as block factors) and
       \item that no negative signs are allowed. }
       
       If \code{blocks} is a single number, it must be a power of 2. 
       A blocked design can have at most \code{nruns-blocks-1} treatment 
       factors, but should usually have fewer than that. \cr
       
       If the experiment is randomized, randomization happens within blocks. 
       In case of many blocks, units should also be randomized to blocks wherever possible! \cr
       
       For the statistical and algorithmic background of blocked designs, see \code{\link{block}}.
       }
  \item{block.name}{ name of the block factor, default \dQuote{Blocks}}
  \item{block.old}{logical; if TRUE, blocking behavior of FrF2 version 1.7.2 is activated}
  \item{force.godolphin}{logical; if TRUE, blocking is forced to be done with the 
       Godolphin method (using function \code{colpick}, see \code{\link{block}}), 
       even if the default would have been to use function \code{blockpick}. \cr
       The Godolphin method was introduced with package version 2.0, and since package version 2.3, the default for \code{alias.block.2fis=TRUE} is to force the use of the Godolphin method.}
  \item{bbreps}{ between block replications; these are always taken as genuine replications, 
       not repeat runs; default: equal to \code{replications};
       CAUTION: you should not modify \code{bbreps} if you do not work with blocks, 
       because the program code uses it instead of \code{replications} in some places }
  \item{wbreps}{ within block replications; whether or not these are taken as genuine replications 
       depends on the setting of \code{repeat.only}}
  \item{alias.block.2fis}{ logical indicating whether blocks may be aliased 
       with 2fis (default: \code{FALSE}); \cr
       it will often be necessary to modify this option, because there is otherwise no solution.}
  \item{hard}{ gives the number of hard to change factors. These must be the 
       first factors in \code{factor.names}. Implementation is via a non-randomized split-plot 
       design with as few as possible whole plots (number of possible whole plots is 
       determined via left-adjustment) and as few as possible non-hard factors 
       within the whole plot structure (by applying split-plot after left-adjustment).\cr
       Observations within whole plots are randomized, whole plots themselves are not 
       randomized (contrary to split plot designs).\cr
       From the statistical point of view, randomisation of whole plots is strongly 
       preferrable (cf. \code{\link{splitplot}} for a brief discussion of the difference).
       If this appears feasible, you may want to explicitly handle the situation 
       by treating the hard to change factors as whole-plot factors via \code{WPs} and \code{nfac.WP}.
       }
  \item{check.hard}{ is the number of candidate designs from the catalogue specified 
       in \code{select.catlg} that are checked for making hard-to-change factors 
       change as little as possible.\cr
       The default is 10 - if too many changes are needed, a larger choice might 
       help find a design with fewer level changes (but will also take longer run time
       and will find a worse design in terms of resolution / aberration). \cr
       If you want to use the best design and do not want to compromise the confounding 
       structure for ease-of-change reasons, set check.hard to 1.}  
  \item{WPs}{ is the number of whole plots and must be a power of 2.\cr
       If \code{WPs > 1}, at least one of \code{nfac.WP} or \code{WPfacs} must be given.\cr
       If \code{WPs = 1}, all settings for split-plot related options are ignored.\cr
       
       For statistical and algorithmic information on treatment of split-plot designs 
       see the separate help file \code{\link{splitplot}}.
       }
  \item{nfac.WP}{ is the number of whole plot factors and must be smaller than \code{WPs}.
       
       The \code{nfac.WP} whole plot factors are counted within \code{nfactors}. 
       Per default, the first \code{nfac.WP} factors are the whole plot factors.
       
       If a design is provided and whole plot factors are manually provided 
       (design or generators option together with \code{WPfacs}), 
       \code{nfac.WP} can be omitted (i.e. remains 0). If given, it must coincide with 
       the length of \code{WPfacs}.

       If \code{nfac.WP} is given without \code{WPfacs}only, \code{generators} must not be given.

       If \code{nfac.WP} is omitted (i.e. remains 0) and \code{WPs} > 1, an error is thrown, 
       because the situation is a block rather than a split-plot situation 
       (and either it was forgotten to specify 
       the number of whole plot factors, or blocks should be specified).
        }
  \item{WPfacs}{ is per default NULL. In this case, the first \code{nfac.WP} factors are 
       considered whole plot factors (and are, if necessary, automatically supplemented by 
       additional whole plot constructor factors). \cr
       If \code{WPfacs} is given, it must specify at least \code{log2(WPs)} whole plot factors. 
       A custom design must be specified 
       with options \code{design} or \code{generators}, and the requested factors in 
       \code{WPfacs} must indeed create a split-plot structure with \code{WPs} whole plots. 
       If the number of whole plots created by the whole plot factors differs from 
       \code{WPs} or factors other than the specified factors from \code{WPfacs} would 
       in fact become whole plot factors as well, an error is thrown.
       
       \code{WPfacs} can be any of\cr 
       a vector or list of factor position numbers\cr
       OR\cr
       a character vector of factor position letters from \code{Letters}\cr
       OR\cr
       a character vector with entries \dQuote{F} followed by factor position number\cr
       OR\cr
       a character vector of factor names (this takes precedence over factor letters, 
       i.e. if the factor names were B, A, C, D, and E, factor letter entries in the \code{nfac.WP} 
       are interpreted as factor names, not position letters).\cr
       It is not possible to specify additional whole plot generators 
       from interaction effects manually through \code{WPfacs}. 
       Rather, all whole plot factors - even artificial ones needed only to increase the number 
       of plots - need to be included in the design factors. }
  \item{check.WPs}{ is the number of potential split-plot designs that are 
       compared by function \code{splitpick} w.r.t. resolution of the whole plot
       portion of the design. This option is effective, if \code{nfac.WP>k.WP} 
       (i.e. bad resolution possible) and nfac.WP not larger than half 
       the number of plots (i.e. resolution better than III is possible). 
       The default is 10 - if not satisfied with the structure of the whole plot 
       factors, a larger choice might help find a better design (but also take 
       longer run time). }
  \item{\dots}{ currently not used }
}
\details{
  Per default, the function picks the best design from the 
  default design catalogue \code{\link{catlg}} (a list object of class \code{catlg}). 
  
  Alternatively, the user can explicitly specify a design through accessing 
  a specific catalogued design using the \code{design} option or specifying non-catalogued 
  generators via the \code{generators} option.

  Apart from generation of simple fractional factorial designs based on catalogued 
  or non-catalogued generators, function \code{FrF2} allows specification of blocked designs 
  and split-plot designs, as well as specification of a set of 2fis that are required to be estimable. 
  The implementation of these possibilities is explained in the separate help files \code{\link{block}}, 
  \code{\link{splitplot}} and \code{\link{estimable.2fis}}. If you consider to use 
  option \code{hard}, it may also be worth while to look at the \code{\link{splitplot}} 
  help file.
  
  Function \code{FrF2} is still under development, although most features are 
  now included, and the principle structure of inputs and outputs should not change 
  much any more. Please contact me with any suggestions for improvements.
  
  Function \code{FrF2.currentlychecked} is meant as a diagnostic tool, 
  when searching for designs with option \code{estimable}
  and \code{clear=TRUE}. If the search takes very long, it can be interrupted 
  (CAUTION: in some igraph versions, interrupting the search may crash R). 
  After a successful interruption, and \code{FrF2.currentlychecked()} 
  returns a character string with the name of the design that was checked at the 
  time of interruption. 
}
\value{
  Function \code{FrF2} returns a data frame of S3 class \code{\link[DoE.base:class-design]{design}} 
  that has attached attributes that can be accessed 
  by functions \code{\link[DoE.base:class-design-methods]{desnum}}, 
  \code{\link[DoE.base:class-design-methods]{run.order}} and 
  \code{\link[DoE.base:class-design-methods]{design.info}}. 
  
  The data frame itself contains the design with levels coded as requested. 
  If no center points have been requested, the design columns are factors with 
  contrasts \code{-1} and \code{+1} (cf. also \code{\link[DoE.base]{contr.FrF2}}); in case 
  of center points, the design columns are numeric.
  
  The following attributes are attached to it: 
  \item{desnum}{Design matrix in -1/1 coding}
  \item{run.order}{a three column data frame;\cr 
       the first column (\code{run.no.in.std.order})
       contains the run number in standard order, possibly including a block number 
       and in that case also a number for the original position within the block,\cr 
       the second column (\code{run.no}) contains the actual run number as randomized, \cr
       the third column contains (\code{run.no.std.rp}) contains the content 
       of the first column accumulated with a replication identifier, if applicable.\cr
       A few remarks on the run number in standard order are needed here: \cr
       In blocked and split plot designs, the run number in standard 
       order refers to a row ordering with the first base factor changing \emph{slowest}, 
       different from the usual order with the first base factor changing \emph{fastest}. 
       Also note that the run number in standard order may not refer to 
       the base columns one would naturally expect for designs created with blocking,  
       estimable 2fis or split plot designs; the elements \code{map} and/or  
       \code{orig.fac.order} of the attribute \code{design.info} help identify 
       which base factors drive the run number in standard order.
       (In case of option \code{hard=TRUE}, the remark on split plot designs applies, 
       and the numbering refers to the special slow change matrix from 
       Cheng et al. 1998.)\cr
       Before version 2 of package FrF2, blocking for large situations was internally 
       done with function \code{blockpick.big}. This behavior can be reproduced 
       using the argument \code{block.old=TRUE}; if blocking for a large design is 
       not successful otherwise, \code{block.old=TRUE} might be worth a try. 
       For blocked designs created internally with 
       function \code{blockpick.big}, 
       the run number in standard order is not easily related to the final 
       design.}
  \item{design.info}{list with the entries 
  \describe{
    \item{type}{ character string \dQuote{full factorial}, \dQuote{FrF2}, 
         \dQuote{FrF2.estimable}, \dQuote{FrF2.generators}, \dQuote{FrF2.blocked} 
         or \dQuote{FrF2.splitplot} depending on the 
         type of design}
    \item{nruns}{ number of runs (replications are not counted) }
    \item{nfactors}{ number of factors; since version 0.97, 
         this is also true for designs of type \code{FrF2.blocked} 
         (\code{nfactors} is now equal to \code{ntreat}) and for designs of type 
         \code{FrF2.splitplot}, where \code{nfactors} is now the sum of \code{nfac.WP} and \code{nfac.SP}.}
    \item{ntreat}{ for designs of type \code{FrF2.blocked} only; \cr
         number of treatment factors }
    \item{nfac.WP}{ for designs of type \code{FrF2.splitplot} only; \cr
         number of whole plot factors (including extra factors that may have been 
         added for whole plot construction); these are the first factors in the 
         design data frame }
    \item{nfac.SP}{ for designs of type \code{FrF2.splitplot} only; \cr
         number of split-plot factors }
    \item{nlevels}{for designs of type \code{full factorial} only; \cr
         vector with number of levels for each factor 
         (of course, all the \code{nfactors} entries are \dQuote{2} for \code{FrF2})}
    \item{factor.names }{ list named with (treatment) factor names and containing as 
         entries vectors of length two each with coded factor levels }
    \item{FrF2.version }{ version number of package FrF2, 
         supporting correct usage of \code{FrF2}-specific functionality in 
         functions \code{summary} and \code{generators} 
         methods for class \code{design}}
    \item{nblocks}{for designs of type \code{FrF2.blocked} only; \cr
         number of blocks}
    \item{block.gen}{vector of columns of the Yates matrix for generating 
         block factors in case of automatic block generation \cr 
         OR list of (vectors of) factor numbers in case the \code{blocks} 
         argument specifies certain factor combinations for blocking; ; \cr
         for designs of type \code{FrF2.blocked} only}
    \item{blocksize}{for designs of type \code{FrF2.blocked} only; \cr
         size of each block (without consideration of \code{wbreps})}
    \item{nWPs}{for designs of type \code{FrF2.splitplot} only; \cr
         number of whole plots}
    \item{plotsize}{for designs of type \code{FrF2.splitplot} only; \cr
         size of each plot (without consideration of repeat.only replications if applicable)}
    \item{orig.fac.order }{designs of type \code{FrF2.splitplot} only; \cr
         factor order of the design before reshuffling of factors in order 
         to accomodate the split plot request; 
         the run number in standard order can only be interpreted together 
         with this information}
    \item{catlg.entry }{ for designs of type \code{FrF2} only; \cr
         list with one element, which is the entry of \code{catlg} 
         on which the design is based}
    \item{generators }{ for designs of type \code{FrF2.generators} only; \cr
         character vector of generators in the form D=ABC etc.}
    \item{base.design }{ for designs of type \code{FrF2.blocked} or \code{FrF2.splitplot} only; \cr
         gives a character string that contains the name of the base design 
         in the catalogue or the column numbers of generating columns in Yates matrix; 
         in case of automatic block generation, the exclusion or inclusion of \code{k.block} 
         in the number of design factors / generators 
         indicates whether the design was generated using function 
         \code{\link{blockpick}} or \code{\link{blockpick.big}}.}
    \item{aliased.with.blocks}{ for designs of type \code{FrF2.blocked} only; \cr
         treatment effects that are aliased with block main effects,
         up to 2fis or 3fis, depending on the choice of \code{alias.info}}
    \item{aliased }{ alias structure of main effects, 2fis and possibly 3fis,
         depending on the choice of \code{alias.info}; 
         For non-blocked and non-split-plot designs, \code{aliased} is 
         itself a list of the two or three components main, fi2, and optionally fi3, given 
         in terms of factor letters from \code{Letters} (up to 50~factors) 
         or \code{F1}, \code{F2}, and so forth (more than 50~factors). 
         For blocked and split-plot designs, \code{aliased} is a single list 
         with an entry for each column of the Yates matrix that accomodates 
         aliased low-order effects, and entries are in terms of factor names.)}
    \item{replication }{ option setting in call to \code{FrF2} }
    \item{repeat.only }{ option setting in call to \code{FrF2} }
    \item{bbreps}{for designs of type \code{FrF2.blocked} only; 
         number of between block replications}
    \item{wbreps}{for designs of type \code{FrF2.blocked} only; 
         number of within block replications;\cr
         \code{repeat.only} indicates whether these are replications or repetitions only}
    \item{map }{ the mapping relation between factors in the base design and 
         experimental factors, after using option \code{estimable} and for split-plot designs}
    \item{clear }{ option setting in call to \code{FrF2}, in case of \code{estimable} }
    \item{res3}{ option setting in call to \code{FrF2}, in case of \code{estimable} }
    \item{randomize }{ option setting in call to \code{FrF2} }
    \item{seed }{ option setting in call to \code{FrF2} }
    \item{creator }{ call to function FrF2 
         (or stored menu settings, if the function has been called via the R commander plugin 
         \pkg{RcmdrPlugin.DoE}) }
    \item{ncube }{ number of cube points per block, in case center points have been requested }
    \item{ncenter }{ number of center points per block, in case center points have been requested }
         }
       }
}
\references{
Bingham, D.R., Schoen, E.D. and Sitter, R.R. (2004). 
Designing Fractional Factorial Split-Plot Experiments with Few Whole-Plot Factors. 
\emph{Applied Statistics} \bold{53}, 325-339.

Bingham, D. and Sitter, R.R. (2003). Fractional Factorial Split-Plot Designs
for Robust Parameter Experiments. \emph{Technometrics} \bold{45}, 80-89.

Bisgaard, S. (1994a). Blocking generators for small \eqn{2^{k-p}}{2^(k-p)} designs. 
\emph{J. Quality Technology} \bold{26}, 288-294.

Chen, J., Sun, D.X. and Wu, C.F.J. (1993) 
A catalogue of 2-level and 3-level orthogonal arrays. 
\emph{International Statistical Review} \bold{61}, 131-145. 

Cheng, C.-S., Martin, R.J., and Tang, B. (1998). 
Two-level factorial designs with extreme numbers of level changes. \emph{Annals of Statistics} 
\bold{26}, 1522-1539.

Cheng, C.-S. and Tsai, P.-W. (2009). 
Optimal two-level regular fractional factorial block and split-plot designs. 
\emph{Biometrika} \bold{96}, 83-93.

Cheng, S.W. and Wu, C.F.J. (2002). 
Choice of optimal blocking schemes in 2-level and 3-level designs. 
\emph{Technometrics} \bold{44}, 269-277. 

Cordella, L.P., Foggia, P., Sansone, C. and Vento, M. (2001). 
An improved algorithm for matching large graphs. \emph{Proc. of the 3rd 
IAPR TC-15 Workshop on Graphbased Representations in Pattern Recognition},
149--159.

Godolphin, J. (2021). Construction of Blocked Factorial Designs to Estimate 
   Main Effects and Selected Two-Factor Interactions. \emph{J. Royal Statistical Society} \bold{B 83}, 5-29. 
   \doi{10.1111/rssb.12397}.

Groemping, U. (2012). Creating clear designs: a graph-based algorithm and a 
catalogue of clear compromise plans. \emph{IIE Transactions} \bold{44}, 988--1001.
Early preprint available at \url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-005.pdf}. 

Groemping, U. (2014a). {R} Package {FrF2} for Creating and Analyzing Fractional 
Factorial 2-Level Designs. \emph{Journal of Statistical Software}, \bold{56}, 
Issue 1, 1-56. \url{https://www.jstatsoft.org/v56/i01/}.
    
Groemping, U. (2014b). A Note on Dominating Fractional Factorial 
Two-Level Designs With Clear Two-Factor Interactions. \emph{Technometrics} 
\bold{56}, 42--45.
    
Groemping, U. (2021). An algorithm for blocking regular fractional factorial 
2-level designs with clear two-factor interactions. \emph{Computational Statistics and Data Analysis} 
\bold{153}, 1-18. \doi{10.1016/j.csda.2020.107059}. Preprint at 
\href{http://www1.bht-berlin.de/FB_II/reports/Report-2019-003.pdf}{Report 3/2019}.

Huang, P., Chen, D. and Voelkel, J.O. (1998). Minimum-Aberration Two-Level Split-Plot Designs. 
\emph{Technometrics} \bold{40}, 314-326. 

Mee, R. (2009). \emph{A Comprehensive Guide to Factorial Two-Level Experimentation}. 
New York: Springer.

Solnon, C. (2010). AllDifferent-based Filtering for Subgraph Isomorphism.
\emph{Artificial Intelligence} \bold{174}, 850--864.

Sun, D.X., Wu, C.F.J. and Chen, Y.Y. (1997). 
Optimal blocking schemes for \eqn{2^n} and \eqn{2^{n-p}}{2^(n-p)} designs. \emph{Technometrics} \bold{39}, 
298-307. 

Wu, C.F.J. and Chen, Y. (1992) 
A graph-aided method for planning two-level experiments when certain interactions 
are important. 
\emph{Technometrics} \bold{34}, 162-175. 
}

\section{Warning}{
   Since R version 3.6.0, the behavior of function \code{\link{sample}} has changed 
   (correction of a biased previous behavior that should not be relevant for the randomization of designs). 
   For reproducing a randomized design that was produced with an earlier R version, 
   please follow the steps described with the argument \code{seed}.
}
\author{ Ulrike Groemping }

\seealso{ See also \cr
\code{\link{FrF2Large}} for regular fractional factorial designs 
with more than 4096 runs (these are not supported by a design catalogue, except for 
a few resolution V designs which have not been checked for any optimality among the resolution V designs), \cr
\code{\link{pb}} for non-regular fractional factorials according 
to Plackett-Burman, \cr
\code{\link{catlg}} for the underlying design catalogue and some accessor functions,\cr
and \code{\link{block}}, \code{\link{splitplot}} or \code{\link{estimable.2fis}} 
for statistical and algorithmic information on the respective topic.}
\examples{
## maximum resolution minimum aberration design with 4 factors in 8 runs
FrF2(8,4)
## the design with changed default level codes
FrF2(8,4, default.level=c("current","new"))
## the design with number of factors specified via factor names 
      ## (standard level codes)
FrF2(8,factor.names=list(temp="",press="",material="",state=""))
## the design with changed factor names and factor-specific level codes
FrF2(8,4, factor.names=list(temp=c("min","max"),press=c("low","normal"),
     material=c("current","new"),state=c("new","aged")))
## a full factorial
FrF2(8,3, factor.names=list(temp=c("min","max"),press=c("low","normal"),
     material=c("current","new")))
## a replicated full factorial (implicit by low number of factors)
FrF2(16,3, factor.names=list(temp=c("min","max"),press=c("low","normal"),
     material=c("current","new")))
## three ways for custom specification of the same design
FrF2(8, generators = "ABC")
FrF2(8, generators = 7)
FrF2(8, generators = list(c(1,2,3)))
## more than one generator
FrF2(8, generators = c("ABC","BC"))
FrF2(8, generators = c(7,6))
FrF2(8, generators = list(c(1,2,3),c(2,3)))
## alias structure for three generators that differ only by sign
design.info(FrF2(16,generators=c(7,13,15),randomize=FALSE))$aliased
design.info(FrF2(16,generators=c(7,-13,15),randomize=FALSE))$aliased
design.info(FrF2(16,generators=c(-7,-13,-15),randomize=FALSE))$aliased
## finding smallest design with resolution 5 in 7 factors
FrF2(nfactors=7, resolution=5)
## same design, but with 12 center points in 6 positions
FrF2(nfactors=7, resolution=5, ncenter=12, center.distribute=6)


## maximum resolution minimum aberration design with 9 factors in 32 runs
## show design information instead of design itself
design.info(FrF2(32,9))
## maximum number of free 2-factor interactions instead of minimum aberration
## show design information instead of design itself
design.info(FrF2(32,9,MaxC2=TRUE))

## usage of replication
## shows run order instead of design itself
run.order(FrF2(8,4,replication=2,randomize=FALSE))
run.order(FrF2(8,4,replication=2,repeat.only=TRUE,randomize=FALSE))
run.order(FrF2(8,4,replication=2))
run.order(FrF2(8,4,replication=2,repeat.only=TRUE))


\dontrun{
## examples below do work, but are repeated in the 
## respective method's separate help file and are therefore prevented 
## from running twice

########## automatic blocked designs ###################
## from a full factorial ##
FrF2(8,3,blocks=2)
## with replication
run.order(FrF2(8,3,blocks=2,wbreps=2))
run.order(FrF2(8,3,blocks=2,wbreps=2,repeat.only=TRUE))
run.order(FrF2(8,3,blocks=2,bbreps=2))
run.order(FrF2(8,3,blocks=2,bbreps=2,wbreps=2))

## automatic blocked design with fractions
FrF2(16,7,blocks=4,alias.block.2fis=TRUE,factor.names=c("MotorSpeed", 
      "FeedMode","FeedSizing","MaterialType","Gain","ScreenAngle","ScreenVibLevel"))
## isomorphic non-catalogued design as basis, using Godolphin approach
FrF2(16,gen=c(7,11,14),blocks=4,alias.block.2fis=TRUE)
## isomorphic non-catalogued design as basis, not using Godolphin approach
## (different design of comparable quality in this case)
FrF2(16,gen=c(7,11,14),blocks=4,alias.block.2fis=TRUE, force.godolphin=FALSE)
## FrF2 uses blockpick.big and ignores the generator
FrF2(64,gen=c(7,11,14),blocks=16,alias.block.2fis=TRUE, block.old=TRUE)
## FrF2 uses Godolphin approach, regardless of force.godolphin argument
##      because the setting is large
FrF2(64,gen=c(7,11,14),blocks=16,alias.block.2fis=TRUE)

########## manual blocked design ####################
### example that shows why order of blocks is not randomized
### can of course be randomized by user, if appropriate
FrF2(32,9,blocks=c("Day","Shift"),alias.block.2fis=TRUE, 
    factor.names=list(Day=c("Wednesday","Thursday"), Shift=c("Morning","Afternoon"),
        F1="",F2="",F3="",F4="",F5="",F6="",F7=""), default.levels=c("current","new"))

########## blocked design with estimable 2fis ####################
### all interactions of last two factors to be estimable clearly
### in 64 run design with blocks of size 4
### not possible with catalogue entry 9-3.1
FrF2(64, 6, blocks=16, factor.names=Letters[15:20], 
                 estimable=compromise(6,3)$requirement,
                 alias.block.2fis=TRUE, randomize=FALSE)
FrF2(design="9-3.2", blocks=16, alias.block.2fis=TRUE, 
    factor.names = list(C1="",C2="",C3="",C4="",C5="",C6="",C7="",
    N1=c("low","high"),N2=c("low","high")), 
    default.levels = c("current","new"),
    estimable=compromise(9, 8:9)$requirement)
FrF2(256, 13, blocks=64, alias.block.2fis=TRUE, 
    factor.names = list(C1="",C2="",C3="",C4="",C5="",C6="",C7="",C8="",
    N1=c("low","high")), 
    default.levels = c("current","new"),
    estimable=compromise(13, 1)$requirement)

########## hard to change factors ####################
## example from Bingham and Sitter Technometrics 19999
## MotorSpeed, FeedMode,FeedSizing,MaterialType are hard to change
BS.ex <- FrF2(16,7,hard=4,
     factor.names=c("MotorSpeed", "FeedMode","FeedSizing","MaterialType",
                  "Gain","ScreenAngle","ScreenVibLevel"), 
     default.levels=c("-","+"),randomize=FALSE)
design.info(BS.ex)
BS.ex
## NOTE: the design has 8 whole plots.
## If randomize=FALSE is used like here, the first hard-to-change factors 
## do not always change between whole plots. 
## A conscious and honest decision is required whether this is 
##    acceptable for the situation at hand!
## randomize=TRUE would cause more changes in the first four factors.

########## automatic generation for split plot ##########
## 3 control factors, 5 noise factors, control factors are whole plot factors
## 8 plots desired in a total of 32 runs
## Bingham Sitter 2003
BS.ex2a <- FrF2(32, 8, WPs=8, nfac.WP=3, 
      factor.names=c(paste("C",1:3,sep=""), paste("N",1:5,sep="")),randomize=TRUE)

## manual generation of this same design
BS.ex2m <- FrF2(32, 8, generators=c("ABD","ACD","BCDE"),WPs=8, WPfacs=c("C1","C2","C3"), nfac.WP=3, 
      factor.names=c(paste("C",1:3,sep=""),paste("N",1:5,sep="")),randomize=TRUE)

## design with few whole plot factors
## 2 whole plot factors, 7 split plot factors
## 8 whole plots, i.e. one extra WP factor needed
BSS.cheese.exa <- FrF2(32, 9, WPs=8, nfac.WP=2, 
      factor.names=c("A","B","p","q","r","s","t","u","v"))
design.info(BSS.cheese.exa)
## manual generation of the design used by Bingham, Schoen and Sitter
## note that the generators include a generator for the 10th spplitting factor
    ## s= ABq, t = Apq, u = ABpr and v = Aqr, splitting factor rho=Apqr
BSS.cheese.exm <- FrF2(32, gen=list(c(1,2,4),c(1,3,4),c(1,2,3,5),c(1,4,5),c(1,3,4,5)), 
      WPs=8, nfac.WP=3, WPfacs=c(1,2,10),
      factor.names=c("A","B","p","q","r","s","t","u","v","rho"))
design.info(BSS.cheese.exm)

########## usage of estimable ###########################
  ## design with all 2fis of factor A estimable on distinct columns in 16 runs
  FrF2(16, nfactors=6, estimable = rbind(rep(1,5),2:6), clear=FALSE)
  FrF2(16, nfactors=6, estimable = c("AB","AC","AD","AE","AF"), clear=FALSE)
  FrF2(16, nfactors=6, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=FALSE)
            ## formula would also accept self-defined factor names
            ## from factor.names instead of letters A, B, C, ...
            
  ## estimable does not need any other input
  FrF2(estimable=formula("~(A+B+C)^2+D+E"))

  ## estimable with factor names 
  ## resolution three must be permitted, as FrF2 first determines that 8 runs 
  ##     would be sufficient degrees of freedom to estimate all effects 
  ##     and then tries to accomodate the 2fis from the model clear of aliasing in 8 runs
  FrF2(estimable=formula("~one+two+three+four+two:three+two:four"), 
       factor.names=c("one","two","three","four"), res3=TRUE)
  ## clear=FALSE allows to allocate all effects on distinct columns in the 
  ##     8 run MA resolution IV design
  FrF2(estimable=formula("~one+two+three+four+two:three+two:four"), 
       factor.names=c("one","two","three","four"), clear=FALSE)

  ## 7 factors instead of 6, but no requirements for factor G
  FrF2(16, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=FALSE)
  ## larger design for handling this with all required effects clear
  FrF2(32, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=TRUE)
  ## 16 run design for handling this with required 2fis clear, but main effects aliased
  ## (does not usually make sense)
  FrF2(16, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=TRUE, res3=TRUE)
}
## example for the sort option added with version 1.6-1
## and for usage of a catalogue from package FrF2.catlg128 (simplified with version 1.6-5)
  \dontrun{
  estim <- compromise(17,15:17)$requirement  ## all interactions of factors 15 to 17 (P,Q,R)
  ## VF2 algorithm without pre-sorting of vertices
  ### CAUTION: in some igraph versions, the following may crash R ###
  FrF2(128, 17, estimable=estim, select.catlg=catlg128.17) 
                        ## very slow, interrupt with ESC key after a short while
                        ## !!! save all important work before, in case R crashes
  FrF2.currentlychecked()   ## displays the design that was currently checked 
                            ## should be 17-10.2407, if the interrupt was successful
  ## VF2 algorithm with pre-sorting of vertices
  FrF2(128, 17, estimable=estim, sort="high", select.catlg=catlg128.17)  ## very fast
  FrF2(128, 17, estimable=estim, sort="low", select.catlg=catlg128.17)  ## very fast
  ## LAD algorithm
  FrF2(128, 17, estimable=estim, method="LAD", select.catlg=catlg128.17)  ## very fast
  ## guaranteed to be MA clear design 
  ## only works, if package FrF2.catlg128 is installed
  }

## example for necessity of perms, and uses of select.catlg and perm.start
## based on Wu and Chen Example 1
  \dontrun{
  ## runs per default about max.time=60 seconds, before throwing error with 
  ##        interim results
  ## results could be used in select.catlg and perm.start for restarting with 
  ##       calculation of further possibilities
  FrF2(32, nfactors=11, estimable = formula("~(A+B+C+D+E+F)^2"), clear=FALSE)
  ## would run for a long long time (I have not yet been patient enough)
  FrF2(32, nfactors=11, estimable = formula("~(A+B+C+D+E+F)^2"), clear=FALSE, 
       max.time=Inf)
  ## can be easily done with perms, 
  ## as only different subsets of six factors are non-isomorphic
  perms.6 <- combn(11,6)
  perms.full <- matrix(NA,ncol(perms.6),11)
  for (i in 1:ncol(perms.6))
     perms.full[i,] <- c(perms.6[,i],setdiff(1:11,perms.6[,i]))
  FrF2(32, nfactors=11, estimable = formula("~(A+B+C+D+E+F)^2"), clear=FALSE, 
      perms = perms.full )
  }
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{utilitiesCat}
\alias{slowfast}
\alias{mult.gen}
\alias{mult.gen.a}
\alias{words.all}
\alias{alias3fi}
\alias{gen.check}
\alias{estimable.check}
\alias{mapcalc}
\alias{mapcalc.distinct}
\alias{mapcalc.block}
\alias{map2design}
\alias{gen2clear.2fis}
\alias{formula2matrix}
\alias{char2num}
\alias{check.subisomorphic.special}
\alias{check.subisomorphic.matrix}
\alias{getNext}
\alias{estimable}
\alias{Yates}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ ~~ Internal utility functions and a user-visible constant for working with regular fractional factorials ~~ }
\description{
  ~~ Internal functions, mainly for deriving words, alias structures and so forth from catalogue information for regular 2-level fractional factorials. 
  They need not be accessed for normal use of the package - some of them may be made user-usable for future versions of the package. 
  
  The constant Yates is user-visible, as it can be useful for expert users. ~~
}
\usage{
slowfast(k) 
mult.gen(liste) 
mult.gen.a(string.vec,sep="")
words.all(k, gen, design = NULL, max.length=7, select.catlg = catlg)
alias3fi(k, gen, order = 3)
gen.check(k, gen)
estimable.check(estimable, nfac, factor.names)
estimable(estimable, nfac, nruns, clear=FALSE, res3=FALSE, max.time=60, 
            select.catlg=catlg, perm.start=1:nfac, perms=NULL, order = 3, 
            method="VF2", sort="natural", ignore.dom=FALSE)
mapcalc(estimable, nfac, nruns, res3=FALSE, select.catlg=catlg, 
            method="VF2", sort="natural", ignore.dom=FALSE)
mapcalc.block(estimable, nfac, clearcur, 
            method="VF2", sort="natural")
mapcalc.distinct(estimable, nfac, nruns, res3=FALSE, max.time=60, 
        select.catlg=catlg, perm.start=1:nfac, perms=NULL)
map2design(map, select.catlg=catlg)
gen2clear.2fis(k, gen)
formula2matrix(formula)
char2num(char)
check.subisomorphic.special(estimable, nfac, hilf2, hilf3=NULL, 
    res3=FALSE, max.time=60, 
    perm.start=1:nfac, begin_time=Sys.time(), name=NA)
check.subisomorphic.matrix(estimable, nfac, hilf2, hilf3=NULL, 
    res3=FALSE, max.time=60, 
    begin_time=Sys.time(), name=NA, perms=perms)
getNext(perm)
Yates
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{k}{ number of base factors spanning a full factorial with the desired number of runs (i.e. \code{log2(nruns)}) }
  \item{liste}{ list of words in terms of vectors of factor numbers to be combined into one new word }
  \item{string.vec}{ vector of words in terms of factor letter combinations (e.g. \dQuote{ABC}) to be combined into one new word }
  \item{sep}{ character by which interaction terms are separated or concatenated; default is empty, i.e. ABC are separated into A, B and C; 
          for more than 50~factors, a colon is needed so that F1:F2:F127 can be separated into F1, F2 and F127, for example.}
  \item{gen}{ generators in any admissible format (\code{gen.check} transforms any admissible format to the default needed for calculations)) }
  \item{design}{ character string that contains a name of a design from \code{select.catlg}, like in \code{\link{FrF2}} }
  \item{max.length}{ the maximum word length to be calculated for the words list; in large design, choosing this to be 4 or 5 may substantially speed things up! }
  \item{select.catlg}{ catalogue of fractional factorial 2-level designs of class \code{catlg} }
  \item{order}{ 2 or 3, order of interactions to be shown }
  \item{estimable}{ like in \code{\link{FrF2}} }
  \item{nfac}{ number of factors, like \code{nfactors} in \code{\link{FrF2}} }
  \item{factor.names}{ a named list with \code{nfac} elements, 
      for which the names represent factor names }
  \item{nruns}{ like in \code{\link{FrF2}} }
  \item{clear}{ like in \code{\link{FrF2}} }
  \item{res3}{ like in \code{\link{FrF2}} }
  \item{max.time}{ like in \code{\link{FrF2}} }
  \item{perm.start}{ like in \code{\link{FrF2}} }
  \item{perms}{ like in \code{\link{FrF2}} }
  \item{method}{ like in \code{\link{FrF2}} }
  \item{sort}{ like in \code{\link{FrF2}} }
  \item{ignore.dom}{per default (\code{ignore.dom=FALSE}), 
      non-dominating elements of \code{select.catlg} will be removed before searching; 
      set \code{ignore.dom} to TRUE to change that (needed for combination of blocking 
      with estimable requirement)}
  \item{clearcur}{is the set of clear 2fis after removing aliases with the block factor}
  \item{map}{ map of experiment factors to design factors, used in \code{estimable} }
  \item{formula}{ formula to be translated into matrix for \code{estimable} }
  \item{char}{ character vector to be translated into matrix for \code{estimable} }
  \item{hilf2}{ list of length four words }
  \item{hilf3}{ list of length three words, if there are any }
  \item{begin_time}{ timing option used for controlling overall run time, calculated }
  \item{name}{ name of design in catalogue, used in case of max.time end for 
               information where the stop occurred }
  \item{perm}{ current permutation, of which \code{getNext} determines the 
               next permutation in lexicographic order }
}
\details{
   \code{Yates} is a user-visible constant that is useful in design construction: 
   
       \code{Yates} is a list of design column generators in Yates order (for 4096 runs), e.g. \code{Yates[1:8]} is identical to 
       
       \code{list(1,2,c(1,2),3,c(1,3),c(2,3),c(1,2,3))}.
       
}
\value{
    \code{mult.gen} and \code{mult.gen.a} determine a new word as the product of the argument words (numeric vector or character string respectively), 
    \code{words.all} calculates a word list object with word length pattern, the word list itself and a max.length entry, 
    \code{alias3fi} calculates an alias pattern based on the structure of the design (less demanding than the word list approach.
    
    \code{gen.check} produces a list of vectors with integers from 1 to k for denoting the generators, 
    which may have come as a vector of factor letter combinations or a vector of column numbers in Yates order,
    \code{estimable.check} determines a list including elements \code{estimable}, 
    a 2-row-matrix indicating estimable 2fis from the permissible inputs, and \code{nfac}, 
       the number of factors from input parameter \code{estimable}, 
    function \code{estimable} calculates the appropriate design if certain effects have been requested to be estimable, and 
    all subsequent functions are auxiliary functions to function \code{estimable}.
    
   \code{Yates} is a user-visible constant (cf. details section). 
}
\references{ 
  Hedayat, A.S., Sloane, N.J.A. and Stufken, J. (1999) \emph{Orthogonal Arrays: Theory and Applications}, Springer, New York. 
}
\author{ Ulrike Groemping }
\note{ This package is still under development, but does already provide useful and well-tested results.}
\seealso{ See also \code{\link{FrF2}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line
\keyword{ internal }

\name{FrF2-package}
\alias{FrF2-package}
\docType{package}
\title{
Fractional Factorial designs with 2-level factors
}
\description{
creates regular and non-regular Fractional Factorial 2-level designs. 
Furthermore, analysis tools for Fractional Factorial designs with 2-level factors 
are offered (main effects and interaction plots for all factors simultaneously, 
cube plot for looking at the simultaneous effects of three factors, full or half normal plot, 
alias structure in a more readable format than with the built-in function alias). 

The package works together with packages DoE.base and DoE.wrapper.
}
\details{
The package is still subject to development; most key functionality is now included. 
Please contact me, if you have suggestions.

This package designs and analyses Fractional Factorial experiments with 2-level factors. 
Regular (function \code{\link{FrF2}}) and non-regular (function \code{\link{pb}}) 2-level fractional factorial 
designs can be generated. For regular fractional factorials, function \code{\link{FrF2}} 
permits the specification of effects of interest, whose estimation is requested clear of aliasing 
with other effects. The function can furthermore generate regular fractional factorials 
as blocked or split-plot designs, and hard-to-change 
factors can be specified in order to keep the number of level changes low.
Regular resolution V designs larger than those obtainable from function \code{FrF2} 
can be created by function \code{\link{FrF2Large}} (these are not guaranteed to be optimal). 
Analysis facilities work for completely aliased designs only, 
i.e. e.g. not for analysing Plackett-Burman designs with interactions. 

Functions \code{fac.design}, \code{fractionate} or \code{oa.design} from 
Chambers and Hastie (1993) have been used as role models e.g. for 
the option \code{factor.names} or for outputting a data frame with attributes. 
However, S compatibility has not been considered in devising this package. The original 
above-mentioned functions are not available in \code{R}; similar 
functions have been implemented in package \pkg{\link[DoE.base:DoE.base-package]{DoE.base}} 
together with other general functionality for experimental designs.

In terms of analysis, package \code{FrF2} works on linear models and enables convenient main effects and 
interaction plots (functions \code{MEPlot} and \code{IAPlot}) similar to those 
offered by Minitab software for all factors simultaneously, even though especially the 
interactions are often aliased, i.e. the model is typically singular. 
For the (less frequent) case of suspected three-factor-interactions, function 
\code{cubePlot} displays a cube with corners labeled with the (modeled) 
means of three factors simultaneously.
Furthermore, the function \code{DanielPlot} from package \pkg{BsMD} has been 
modified to automatically label effects significant according to the 
Lenth-criterion, to automatically distinguish between whole-plot and 
split-plot effects for split-plot designs, and to provide more usage comfort 
to the analyst.\cr
Finally, the function \code{aliases} determines the alias structure of a 
Fractional Factorial 2-level design in a format more suitable for human readers 
than the output from the built-in function \code{alias}. 

}
\author{
Ulrike Groemping

Maintainer: Ulrike Groemping <ulrike.groemping@bht-berlin.de>
}
\references{
    Box G. E. P, Hunter, W. C. and Hunter, J. S. (2005) 
    \emph{Statistics for Experimenters, 2nd edition}.
    New York: Wiley.
    
    Chambers, J.M. and Hastie, T.J. (1993). \emph{Statistical Models in S}, 
    Chapman and Hall, London.
     
    Chen, J., Sun, D.X. and Wu, C.F.J. (1993) 
    A catalogue of 2-level and 3-level orthogonal arrays. 
    \emph{International Statistical Review} \bold{61}, 131-145.

    Daniel, C. (1959) 
    Use of Half Normal Plots in Interpreting Two Level Experiments.
    \emph{Technometrics}, \bold{1}, 311-340.
    
    Groemping, U. (2014). {R} Package {FrF2} for Creating and Analyzing Fractional 
    Factorial 2-Level Designs. \emph{Journal of Statistical Software}, \bold{56}, 
    Issue 1, 1-56. \url{https://www.jstatsoft.org/v56/i01/}.
    
    Hedayat, A.S., Sloane, N.J.A. and Stufken, J. (1999) 
    \emph{Orthogonal Arrays: Theory and Applications}, Springer, New York. 
    
    Lenth, R.V. (1989) Quick and easy analysis of unreplicated factorials.
    \emph{Technometrics}, \bold{31}, 469-473.
  
    Mee, R. (2009). \emph{A Comprehensive Guide to Factorial Two-Level Experimentation}. 
    New York: Springer.
    
    Montgomery, D.C. (2001). Design and Analysis of Experiments (5th ed.). Wiley, New York.

    Plackett, R.L.; Burman, J.P. (1946) The design of optimum multifactorial 
     experiments. \emph{Biometrika} \bold{33}, 305-325.
     
    Ryan, K.J. and Bulutoglu, D.A. (2010). Minimum Aberration Fractional Factorial Designs With Large N.
      \emph{Technometrics} \bold{52}, 250-255.

    Sanchez, S.M. and Sanchez, P.J. (2005). Very Large Fractional Factorial 
     and Central Composite Designs. 
     \emph{ACM Transactions on Modeling and Computer Simulation} 
     \bold{15}, 362-377.
}
\keyword{ array }
\keyword{ design }
\seealso{
The key design generating functions: \code{\link{FrF2}}, \code{\link{pb}}, \code{\link{FrF2Large}}\cr
S3 class \code{\link[DoE.base:class-design]{design}}\cr 
Related packages: 
\code{\link[DoE.base:DoE.base-package]{DoE.base}}, 
\code{\link[DoE.wrapper:DoE.wrapper-package]{DoE.wrapper}}, 
\code{\link[BsMD:BsMD-package]{BsMD}};\cr 
Graphical analysis functions: \code{\link{MEPlot}}, \code{\link{IAPlot}}, \code{\link{cubePlot}}, 
\code{\link{DanielPlot}}\cr
Analysis of alias structure for linear models of \code{FrF2} designs: \code{\link{aliases}}\cr
}
\examples{
    ### for examples on design generation, cf. functions pb and FrF2
  
    ### Injection Molding Experiment. Box et al. 1978.
    ## data(BM93.e3.data, package="BsMD")  #from BsMD
    ## iMdat <- BM93.e3.data[1:16,2:10]  #only original experiment
    ## re-create here
    y=c(14, 16.8, 15, 15.4, 27.6, 24, 27.4, 22.6, 
     22.3, 17.1, 21.5, 17.5, 15.9, 21.9, 16.7, 20.3)
    iMdat <- FrF2(8,7,randomize=FALSE)
    iMdat <- desnum(iMdat)
    iMdat <- rbind(cbind(iMdat,H=1),cbind(-iMdat,H=-1))
    iMdat <- cbind(as.data.frame(iMdat), y=y)

    # make data more user-friendly
    colnames(iMdat) <- c("MoldTemp","Moisture","HoldPress","CavityThick","BoostPress",
             "CycleTime","GateSize","ScrewSpeed", "y")
    # linear model with all main effects and 2-factor interactions
    iM.lm <- lm(y ~ (.)^2, data = iMdat)
    # determine aliases
    aliases(iM.lm)
    # coded version
    aliases(iM.lm, code=TRUE)
    # normal plot of effects, default is autolabel with alpha=0.05
    DanielPlot(iM.lm)  
    DanielPlot(iM.lm,code=TRUE)
    DanielPlot(iM.lm,code=TRUE,alpha=0.5)
    # half normal plot of effects
    DanielPlot(iM.lm,code=TRUE,alpha=0.5,half=TRUE)
    # main effects plots
    MEPlot(iM.lm, las=1)
    # interaction plots
    IAPlot(iM.lm, las=1)
    # interaction plots with attention drawn to aliases
    aus <- IAPlot(iM.lm, show.alias=TRUE)
    # alias groups corresponding to interaction plots
    aliases(iM.lm)$aliases[9:15]
    # returned object
    aus
    # cube plot of three factors
    # (not very useful for this model, for demonstration only)
      ## per default, modeled means are shown
      ## this does not make a difference here, since the main effect of 
      ## ScrewSpeed is confounded with the MoldTemp:HoldPress:BoostPress
      ## interaction, so that the three-factor-interaction is indirectly included 
      ## in the modeled means
      cubePlot(iM.lm, "MoldTemp", "HoldPress", "BoostPress")
      ## modeled means without a three-factor interaction
      cubePlot(lm(y ~ (MoldTemp+HoldPress+BoostPress)^2, data = iMdat), 
         "MoldTemp", "HoldPress", "BoostPress")
      ## modeled=FALSE reverts to showing the apparent three-factor interaction
      cubePlot(lm(y ~ (MoldTemp+HoldPress+BoostPress)^2, data = iMdat), 
         "MoldTemp", "HoldPress", "BoostPress", modeled=FALSE)
      ## cubePlot also works on raw data
      cubePlot(iMdat$y, iMdat$MoldTemp, iMdat$HoldPress, iMdat$BoostPress)
    ## plotting functions also work directly on designs, 
    ## if these have been generated from functions FrF2 or pb:
      plan <- FrF2(16, 7)
      plan <- add.response(plan, rnorm(16))
      MEPlot(plan)
      IAPlot(plan)
      DanielPlot(plan)
      
}

\name{pb}
\alias{pb}
\alias{pb.list}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Function to generate non-regular fractional factorial screening designs }
\description{
  The function generates Plackett-Burman designs and in some cases other screening
  designs in run numbers that are a multiple of 4. These designs are particularly suitable for screening a
  large number of factors, since interactions are not fully aliased with one main effect each
  but partially aliased. (The design in 8 runs is an exception from this rule.)
}
\usage{
pb(nruns, nfactors = nruns - 1, factor.names = if (nfactors <= 50)
     Letters[1:nfactors] else paste("F", 1:nfactors, sep = ""),
     default.levels = c(-1, 1), ncenter=0, center.distribute=NULL,
     boxtyssedal = TRUE, n12.taguchi = FALSE,
     replications = 1, repeat.only = FALSE,
     randomize = TRUE, seed = NULL, oldver = FALSE, ...)

pb.list
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nruns}{ number of runs, must be a multiple of 4 }
  \item{nfactors}{ number of factors, default is nruns - 1,
   and it is recommended to retain this default.\cr
   It is possible to specify factor names for fewer factors,
   and the remaining columns will be named \code{e1}, \code{e2}, ...
   They are useful for representing error in effects plots
   (so-called dummy factors). }
  \item{factor.names}{ a character vector of factor names (length up to nfactors)
        or a list with \code{nfactors} elements;\cr
        if the list is named, list names represent factor names, otherwise default factor names are used;\cr
        the elements of the list are \cr
        EITHER vectors of length 2 with factor levels for the respective factor\cr
        OR empty strings.
        For each factor with an empty string in factor.names, the levels given in
        \code{default.levels} are used;\cr
       Default factor names are the first elements of the character vector \code{\link[DoE.base:utilities]{Letters}},
       or the factors position numbers preceded by capital F in case of more than 50 factors. }
  \item{default.levels}{ default levels (vector of length 2) for all factors for which no specific levels are given }
  \item{ncenter}{ number of center points; \code{ncenter > 0}
        is permitted, if all factors are quantitative }
  \item{center.distribute}{ the number of positions over which the center points
        are to be distributed ; if NULL (default), center points are
        distributed over end, beginning, and middle (in that order, if there are fewer than three center points)
        for randomized designs, and appended to the end for non-randomized designs.
        for more detail, see function \code{\link{add.center}}, which does the work.
        }
  \item{boxtyssedal}{ logical, relevant only for nruns=16.
               If FALSE, the geometric (=standard) 16 run plan is used.
               If TRUE, the proposal by Box and Tyssedal is used instead, which has the advantage (for screening) of aliasing each interaction with several main effects, like the other Plackett-Burman designs. }
  \item{n12.taguchi}{ logical, relevant only for nruns=12. If TRUE, the 12 run design is given in Taguchi order. }
  \item{replications}{ positive integer number. Default 1 (i.e. each row just once).
       If larger, each design run is executed replication times.
       If \code{repeat.only}, repeated measurements
       are carried out directly in sequence, i.e. no true replication takes place,
       and all the repeat runs are conducted together. It is likely that the error
       variation generated by such a procedure will be too small, so that average values
       should be analyzed for an unreplicated design.

       Otherwise (default), the full experiment is first carried out once, then
       for the second replication and so forth. In case of randomization,
       each such blocks is randomized separately. In this case, replication variance is
       more likely suitable for usage as error variance
       (unless e.g. the same parts are used for replication runs although build
       variation is important). }
  \item{repeat.only}{ logical, relevant only if replications > 1. If TRUE,
        replications of each run are grouped together
       (repeated measurement rather than true replication). The default is
       \code{repeat.only=FALSE}, i.e. the complete experiment
       is conducted in \code{replications} blocks, and each run occurs in each block. }
  \item{randomize}{ logical. If TRUE, the design is randomized. This is the default. }
  \item{seed}{ optional seed for the randomization process \cr
        In R version 3.6.0 and later, the default behavior of function \code{\link{sample}}
        has changed. If you work in a new (i.e., >= 3.6.-0) R version and want to reproduce
        a randomized design from an earlier R version (before 3.6.0),
        you have to change the RNGkind setting by\cr
        \code{RNGkind(sample.kind="Rounding")}\cr
        before running function \code{pb}.\cr
        It is recommended to change the setting back to the new recommended way afterwards:\cr
        \code{RNGkind(sample.kind="default")}\cr
        For an example, see the documentation of the example data set \code{\link[DoE.base]{VSGFS}}.
        }
  \item{oldver}{ logical. If \code{TRUE}, the column ordering from
       package versions 1.0-5 to 1.2.10 is used. This affects designs in
       40, 52, 56, 64, 76, 92, 96 and 100 runs.
       Usually, option \code{oldver} should not be set to is useful for reproducing an
       old design, or for making a design in 40, 56, 64, 88 or 96 runs with
       exactly half the number of factors resolution IV.
         }
  \item{\dots}{ currently not used }
}
\details{
  \code{pb} stands for Plackett-Burman. Plackett-Burman designs (Plackett and Burman 1946) are generally
  used for screening many variables in relatively few runs, when interest is in
  main effects only, at least initially. Different from the regular
  fractional factorial designs created by function \code{\link{FrF2}}, they
  do not perfectly confound interaction terms with main effects but distribute
  interaction effects over several main effects. The designs with number of runs
  a power of 2 are an exception to this rule: they are just the resolution III
  regular fractional factorial designs and are as such not very suitable for
  screening because of a high risk of very biased estimates for the main effects
  of the factors. Where possible, these are therefore replaced by different designs (cf. below).

  For most run numbers, function \code{pb} uses Plackett-Burman designs,
  and simply fills columns from left to right.
  The generating rows for these designs can be found in the list \code{pb.list}
  (a 0 entry indicates that the design is constructed by a different method, e.g. doubling).

  For 12 runs, the isomorphic design by Taguchi can be requested.
  For 16 runs, the default is to use the designs suggested by Box and Tyssedal (2001),
  which up to 14 factors do not suffer from perfect aliasing.
  For 32 runs, a cyclic design with generating row given in Samset and Tyssedal (1999) is used.
  For 64 runs, the 32 run design is doubled.
  For 92 runs, a design is constructed according to the Williamson construction with matrices A, B, C and D from Hedayat and Stufken (1999), p. 160.

  Designs up to 100~runs are covered.

  Usage of the 8 run design for more than 4 factors is discouraged,
  as it completely aliases main effects with individual two-factor interactions.
  It is recommended to use at least the 12 run design instead for screening more than 4 factors.
}
\value{
  Value is a data frame of S3 class \code{\link[DoE.base:class-design]{design}}
  and has attached attributes that can be accessed
  by functions \code{\link[DoE.base:class-design-methods]{desnum}},
  \code{\link[DoE.base:class-design-methods]{run.order}} and
  \code{\link[DoE.base:class-design-methods]{design.info}}.

  The data frame itself contains the design with levels coded as requested.
  If no center points have been requested, the design columns are factors with
  contrasts \code{-1} and \code{+1} (cf. also \code{\link[DoE.base]{contr.FrF2}}); in case
  of center points, the design columns are numeric.

  The following attributes are attached to it:
  \item{desnum }{Design matrix in -1/1 coding}
  \item{run.order }{three column data frame, first column contains the run number in
       standard order, second column the run number as randomized, third column the
       run number with replication number as postfix;
       useful for switching back and forth between actual and standard run
       number}
  \item{design.info }{list with entries
  \describe{
        \item{type }{ character string \dQuote{pb}, except for 8~runs with up to
           4~factors, for which a type \dQuote{FrF2} design is output}
        \item{nruns }{ number of runs (replications are not counted) }
        \item{nfactors }{ number of factors}
        \item{factor.names }{ list named with (treatment) factor names and containing as
           entries vectors of length two each with coded factor levels }
        \item{ndummies }{ number of dummy factors for error }
       \item{replication }{ option setting in call to \code{pb} }
       \item{repeat.only }{ option setting in call to \code{pb} }
       \item{randomize }{ option setting in call to \code{pb} }
       \item{seed }{ option setting in call to \code{pb} }
       \item{creator }{ call to function pb
          (or stored menu settings, if the function has been called via the R commander plugin
          \pkg{RcmdrPlugin.DoE}) }
       }
       }
}
\section{Warning}{With version 1.0-5 of package \pkg{FrF2},
   design generation for the designs based on doubling has changed (internal function
   \code{double.des}). This affected designs for 40,56,64,88,96 runs.
   With version 1.3 of package \pkg{FrF2}, this and further behaviors (52, 76) has changed
   again, in the interest of improving generalized resolution of desigs produced by function \code{pb}.

   For the affected run sizes, package versions from 1.0-5 onwards
   cannot exactly reproduce pb designs that have been created with a version before
   1.0-5. Package versions from 1.3 onwards reproduce the behavior of versions 1.0-5 to 1.2-10
   through option \code{oldver}.}

\references{
  Box, G.E.P. and Tyssedal, J. (2001) Sixteen Run Designs of High Projectivity for Factor Screening.
      \emph{Communications in Statistics - Simulation and Computation} \bold{30}, 217-228.

  Hedayat, A.S., Sloane, N.J.A. and Stufken, J. (1999) \emph{Orthogonal Arrays: Theory and Applications}, Springer, New York.

    Groemping, U. (2014). {R} Package {FrF2} for Creating and Analyzing Fractional
    Factorial 2-Level Designs. \emph{Journal of Statistical Software}, \bold{56},
    Issue 1, 1-56. \url{https://www.jstatsoft.org/v56/i01/}.

  Mee, R. (2009). \emph{A Comprehensive Guide to Factorial Two-Level Experimentation}.
  New York: Springer.

  Plackett, R.L.; Burman, J.P. (1946) The design of optimum multifactorial experiments. \emph{Biometrika} \bold{33}, 305-325.

  Samset, O.; Tyssedal, J. (1999) Two-level designs with good projection properties. \emph{Technical Report 12,
  Department of Mathematical Sciences, The Norwegian University of Science and Technology, Norway}.

  Williamson, J. (1946) Determinants whose elements are 0 and 1. \emph{American Mathematical Monthly} \bold{53}, 427-434.
}
\section{Warning}{
   Since R version 3.6.0, the behavior of function \code{\link{sample}} has changed
   (correction of a biased previous behavior that should not be relevant for the randomization of designs).
   For reproducing a randomized design that was produced with an earlier R version,
   please follow the steps described with the argument \code{seed}.
}
\author{ Ulrike Groemping }

\seealso{ See also \code{\link{FrF2}} for regular fractional factorial designs,
    \code{\link[DoE.base]{generalized.word.length}} for functions \code{\link[DoE.base:generalized.word.length]{length3}}
    and \code{\link[DoE.base:generalized.word.length]{length4}} used in examples}
\examples{
   pb(12,randomize=FALSE)
   pb(12,randomize=FALSE,n12.taguchi=TRUE)
   pb(20,seed=29869)
   pb(16,factor.names=list(A="",B="",C="",D=c("min","max"),
          E="",F="",G="",H="",J=c("new","old")))
   pb(8,default.levels=c("current","new"))
   test <- pb(40) ## design created by doubling the 20 run design
   pb(12, ncenter=6) ## 6 center points with default placement

   \dontrun{
   ## note: designs in 40, 56, 64, 88, and 96 runs are resolution IV,
   ## if the number of factors is up to nruns/2 - 1, e.g.:
   plan1 <- pb(40, 19)
   length3(plan1)  ## 0 generalized words of length 3
   length4(plan1)  ## 228 generalized words of length 4
   ## they can be made resolution IV by oldver=TRUE for
   ## nfactors=nruns/2, e.g.:
   plan2 <- pb(40, 20)
   plan3 <- pb(40, 20, oldver=TRUE)
   length3(plan2)  ## 9 generalized words of length 3
   length3(plan3)  ## 0 generalized words of length 3
   length4(plan3)  ## 285 generalized words of length 4

   ## note: designs in 52, 76, and 100 runs are almost resolution IV,
   ## if the number of factors is up to nruns/2 - 1, e.g.:
   plan4 <- pb(52, 25)
   GR(plan4)       ## generalized resolution 3.92

   ## note: versions >1.3 avoid complete and heavy aliasing of triples of factors
   ## for up to nruns-2 factors for 40, 52, 56, 64, 76, 88, 92 and 96 runs
   ## (the same for 100 runs, which were not implemented before version 1.3)
   plan5 <- pb(40, 38)
   plan6 <- pb(40, 38, oldver=TRUE)
   GR(plan5)       ## generalized resolution 3.4
   GR(plan6)       ## generalized resolution 3
   plan7 <- pb(52, 50)
   plan8 <- pb(52, 50, oldver=TRUE)
   GR(plan7)       ## generalized resolution 3.62
   GR(plan8)       ## generalized resolution 3.15
   }
   }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{CIG}
\alias{CIGstatic}
\alias{CIG}
\alias{gen2CIG}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Clear interactions graph from catlg entry }
\description{
  Function CIG creates a clear interactions graph (CIG) 
  from a catlg design (design name must be given). 
  Function CIGstatic allows to create a static graph from a dynamically-adjusted 
  one. 
}
\usage{
CIG(design, select.catlg = catlg, nfac = NULL, static = FALSE, 
   layout = layout.auto, label = "num", plot = TRUE, ...)
CIGstatic(graph, id, label = "num", xlim = c(-1,1), ylim = c(1,-1), ...)
gen2CIG(nruns, gen)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{design}{a character string that identifies a design in the catalogue specified 
       by option \code{select.catlg}, \cr
       OR a class \code{catlg} object with a single entry, \cr
       OR a formula with all main effects and the requested clear 2-factor 
       interactions, \cr
       OR a character vector of length more than one with two-letter combinations of 
       the clear 2-factor interactions,\cr
       OR a numeric two-row matrix with factor numbers of the clear 2-factor 
       interactions,\cr
       OR a character two-row matrix with factor names of the clear 2-factor 
       interactions.\cr
       The first two are for graphing design CIGs, the other ones for requirement 
       set CIGs.}
  \item{select.catlg}{name of catalogue (not in quotes); 
        only relevant, if \code{design} is a character string}
  \item{nfac}{number of factors; this is not needed for a class \code{catlg} object, 
       or if the graph is supposed to show only factors that are involved 
       in at least one interaction}
  \item{static}{logical. If \code{TRUE}, a static graphic is produced, 
        otherwise an interactive graphic is created that can be modified 
        by moving aroung nodes; only relevant for \code{plot=TRUE}}
  \item{layout}{ignored for \code{static=FALSE};\cr
        possible values are two-column matrices with number of rows 
        equal to the number of vertices of the graph, or layout parameters for 
        function \code{\link[igraph]{plot.igraph}} as described in 
        \code{\link[igraph]{plot.common}} 
        and \code{\link[igraph]{layout}};\cr
        default: \code{layout.auto} (changed with version 1.6, was \code{layout.circle} 
        before}
  \item{label}{in effect for \code{catlg} object only (character design name or 
        class \code{catlg} object);\cr
        a character string that decides for numeric labels or 
        character labels; any string other than the default will invoke the 
        factor letters as labels}
  \item{plot}{a logical that decides whether a plot is requested (default: \code{TRUE});\cr
        plotting can be suppressed, if graph creation is desired for calculating 
        graph characteristics with functions from package \code{igraph} 
        (e.g. \code{\link[igraph:cliques]{clique.number}},
        \code{\link[igraph:cliques]{largest.cliques}}, 
        \code{\link[igraph:independent.vertex.sets]{independence.number}},
        \code{\link[igraph:degree]{degree}}) }
  \item{\dots}{further arguments to be passed to function \code{\link[igraph]{tkplot}}, 
      or graphical parameters to be passed to \code{plot}.}
  \item{graph}{a graph object from package \code{igraph}, or a list whose first 
        element is such a graph object (like the output from function \code{CIG})}
  \item{id}{identification number of the interactive graph to be reproduced in 
        static form; this number can be found in the header line of the graphics window}
  \item{xlim}{horizontal axis limits}
  \item{ylim}{vertical axis limits (per default reversed in order to exactly 
        reproduce the interactive graph) }
  \item{nruns}{number of runs of the design to be graphed }
  \item{gen}{generator (vector of Yates matrix column numbers) }
}

\details{
    The design depicted in \code{CIG} has to be the name (character string) of a 
    regular fractional factorial 2-level design present in \code{select.catlg}. 
    
    Clear 2fis are depicted as edges in the graph. In the interactive graph, 
    users can change the layout manually or with the menus. For example, 
    the Reingold-Tilford layout can be chosen, with a root vertex specified; 
    this sometimes helps in identifying groups 
    of vertices that are not connected with each other. 
    
    Previous versions of package \code{igraph} used to internally number the 
    vertices from 0 to number of vertices -1, not from 1 to number of vertices. 
    This has been changed 
    in June 2012 (FrF2 adapted to this change with version 1.5).
    
    Function \code{CIGstatic} serves the purpose to statically create the current 
    interactively modified graph; 
    the usual annotation possibilities for plots are available.
    
    Function \code{gen2CIG} returns a graph object that can be plotted or otherwise 
    investigated with graph-related 
    functionality.
}

\value{
    For \code{plot=FALSE} or \code{plot=TRUE} with \code{static=TRUE}, 
    function \code{CIG} visibly (\code{plot=FALSE}) or invisibly (\code{plot=TRUE}) 
    returns a graph from package \code{igraph}.\cr
    For \code{plot=TRUE} with \code{static=FALSE}, 
    the function returns a list with the first element \code{graph} 
    the element \code{coords} with the coordinates of that graph. \cr
    Function \code{CIGstatic} works on the list produced by function \code{CIG} 
    by plotting the graph statically using the positioning from the 
    current interactive picture. 
    
    Function \code{gen2CIG} returns a clear interactions graph that can e.g. be 
    plotted with functions \code{plot} (\code{\link[igraph]{plot.igraph}}) or 
    \code{\link[igraph]{tkplot}}.
}

\references{
Groemping, U. (2012). Creating clear designs: a graph-based algorithm and a 
catalog of clear compromise plans. \emph{IIE Transactions} \bold{44}, 988-1001. 
\doi{10.1080/0740817X.2012.654848}. Early preprint at 
\url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-005.pdf}.

}
\examples{
\dontrun{
ex.CIG <- CIG("9-4.2", vertex.color="white", vertex.label.color="darkred")
## play around with the dynamic graph until it looks right
## look up its id number in the title bar of the graph window and use it for id
par(xpd=TRUE)
CIGstatic(ex.CIG, id=1)
}

graph1 <- CIG("9-4.2", plot=FALSE)   ### create graph object from design name
### calculate graph properties
require(igraph)
degree(graph1)
clique.number(graph1)
independence.number(graph1)
largest.cliques(graph1)

graph2 <- gen2CIG(32, c(7,11,14,29))   ### create graph object from generator columns
### check isomorphism to graph1
graph.isomorphic(graph1, graph2)

\dontrun{
## use a CIG for manual design search
## requirement set: 
estim <- compromise(9, 8:9)$requirement ## all interactions of factors 8 and 9 (H, J)
## graph the requirement set CIG
CIG(estim, vertex.color="white", vertex.label.color="darkred")
## a human can easily see that columns 1, 8 and 9 are worth a try for factors P, Q and R
CIG("9-4.1", vertex.color="white", vertex.label.color="darkred")
    ## obviously, 9-4.1 cannot accommodate the requirement set
CIG("9-4.2", vertex.color="white", vertex.label.color="darkred")
    ## 9-4.2 can, by assigning factors H and J to columns 5 and 9
## function FrF2 automatically does such matchings
}
}
\author{Ulrike Groemping}
\seealso{\code{\link[igraph]{plot.igraph}}, \code{\link[igraph]{tkplot}},
   \code{\link[igraph]{plot.common}} }
\keyword{design}

\name{StructurePickers}
\alias{splitpick}
\alias{leftadjust}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Functions to find split-plot or left-adjusted designs }
\description{
  Functions to restructure a fractional factorial by permuting the base factors 
  such that the leftmost base factors have a suitable alias structure for the 
  problem at hand; meant for expert users
}
\usage{
splitpick(k, gen, k.WP, nfac.WP, show=10)
leftadjust(k, gen, early=NULL, show=10)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{k}{ the number of base factors (designs have \code{2^k} runs) }
  \item{gen}{ vector of generating columns from Yates matrix }
  \item{k.WP}{ integer number of base factors used for whole plot generation; 
        there will be \code{2^k.WP} plots in the design}
  \item{nfac.WP}{ integer number of whole plot factors, must not be smaller than \code{k.WP};
        the other \code{k + length(gen)} factors are split-plot factors, i.e. 
        they change within plots }
  \item{show}{ numeric integer indicating how many results are to be shown;
        for function \code{splitpick}, this number also determines how many designs 
        are investigated;
        the other two functions investigate all designs and show the best results only }
  \item{early}{ number that indicates how many \dQuote{leftmost} factors are 
        needed in the design; used by \code{FrF2} for accomodating hard-to-change 
        factors }
}
\details{
  These functions exploit the fact that a factorial design can be arranged such 
  that the \code{2^k.WP-1} leftmost columns have exactly \code{2^k.WP} 
  different patterns. They can thus accomodate whole plot effects if \code{2^k.WP} 
  plots are available; also, with a specially rearranged version of the Yates matrix, 
  the leftmost columns can have particularly few or particularly many level changes, 
  cf. e.g. Cheng, Martin and Tang 1998.
  
  By permuting the \code{k} base factors , the functions try to find \code{2^k.WP} 
  ones that accomodate the current needs, if taken as the first base factors. They are 
  used by function \code{FrF2}, if a user requests an automatically-generated 
  split-plot design or a design with some factors declared hard-to-change.
  
  There may be a possibility to better accomodate the experimenters needs within 
  a given design by trying different sets of base factors. This is not done 
  in these functions. Also, custom user needs may be better fulfilled, if an expert 
  user directly uses one of these functions for inspecting the possibilities, rather 
  than relying on the automatic selection routine in function \code{FrF2}. 
  }

\value{
  Both functions output a list of entries with information on at most \code{show} suitable 
  permutations. \code{splitpick} ends with an error, if no suitable 
  solution can be found.
  
  \item{orig}{original generator column numbers}
  \item{basics}{named vector with the following entries: \cr
        for function \code{splitpick}, 
        entries are the number of runs, the number of plots, the
        number of whole plot factors and the number of split-plot factors \cr
        for function \code{leftadjust}, 
        entries are the number of runs, the number of factors, 
        and - if \code{early} is not NULL - the entry for \code{early} }
  \item{perms}{matrix with rows containing permutations of base factors}
  \item{res.WP}{for \code{splitpick} only; \cr 
        vector of resolutions for the respective rows of perms}
  \item{maxpos}{for \code{leftadjust} only; \cr 
        vector of maximum positions for the \code{early} leftmost factors for the respective rows of perms}
  \item{k.early}{for \code{leftadjust} only; \cr 
        vector of numbers of base factors needed for spanning the \code{early} leftmost factors for the respective rows of perms}
  \item{gen}{matrix the rows of which contain the generator columns 
        for the respective rows of perms}
}
\references{ 
Cheng, C.-S., Martin, R.J., and Tang, B. (1998). 
Two-level factorial designs with extreme numbers of level changes. \emph{Annals of Statistics} 
\bold{26}, 1522-1539.
}
\author{ Ulrike Groemping }

\seealso{ See Also \code{\link{FrF2}}  }
\examples{
## leftadjusting MA design from table 6.22 in BHH2, 9 factors, 32 runs
## NOTE: nevertheless not as well left-adjusted as the isomorphic design 9-4.1 from catlg
leftadjust(5,c(30,29,27,23))
## with option early=4 (i.e. 4 columns as early as possible are requested)
leftadjust(5,c(30,29,27,23),early=4)
leftadjust(5,catlg$'9-4.1'$gen,early=4)

## look for a split plot design in 32 runs with 7 factors, 
##       3 of which are whole plot factors, 
##       and 8 plots
splitpick(5,catlg$'7-2.1'$gen,nfac.WP=3,k.WP=3)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{compromise}
\alias{compromise}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Function to support estimability requests for compromise designs }
\description{
  Addelman (1962) and Ke and Wu (2005) discuss compromise plans of different types. 
  Their creation is supported by the function compromise.
}
\usage{
    compromise(nfactors, G1, class=3, msg=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nfactors}{ overall number of factors}
  \item{G1}{ vector with indices of factors in group G1 (cf. details)}
  \item{class}{ class of compromise designs that is to be generated; 1, 2, 3, or 4, cf. details below }
  \item{msg}{ logical stating whether the \code{minnruns.clear} element of the result
      should be reported in a message}
}
\details{
  For compromise plans, the factors are decomposed into a group G1 and a group G2. 
  The different classes of compromise plans require estimability of different subsets 
  of 2fis in addition to main effects:\cr\cr
  Class 1: all 2fis within group G1 are estimable\cr
  Class 2: all 2fis within group G1 are estimable, 
  as well as all 2fis within group G2\cr
  Class 3: all 2fis within group G1 are estimable, 
  as well as all 2fis between groups G1 and G2\cr
  Class 4: all 2fis between groups G1 and G2 are estimable\cr

  The function returns a list of four components (cf. section \dQuote{Value}). 
  They can be used as input for the function \code{\link{FrF2}}, if compromise 
  plans are to be created. Both distinct designs (Addelman 1962) and clear designs 
  (Ke, Tang and Wu 2005) can be constructed, 
  depending on the settings of option \code{clear} in function 
  \code{\link{FrF2}}. More explanations on specifying estimability requirements 
  for 2fis in general are provided under \code{\link{estimable.2fis}}.
}
\value{
  Value is a list of the four components \code{perms.full}, \code{requirement}, 
  \code{class}, and \code{minnrun.clear}. The last two components are purely imformative, 
  while the first two provide input parameters for function \code{\link{FrF2}}.\cr
  \code{requirement} can be used for specifying the required 2fis in the \code{estimable} option, 
  both with \code{clear=FALSE} and \code{clear=TRUE}. 
  For \code{clear=FALSE}, \code{perms.full} can be used in the \code{perms} option 
  for speeding up the search into a hopefully realistic time frame.\cr 
  \code{minnrun.clear} indicates the minimum number of runs needed for a clear design.
  
  Note that the catalogue \code{\link{catlg}} contains all designs needed for 
  accomodating existing clear compromise designs in up to 128 runs (even minimum aberration
  among all existing clear compromise designs; for a catalogue of these, cf. Gr\"omping 2010).
}
\references{
Addelman, S. (1962). Symmetrical and asymmetrical fractional factorial plans. 
\emph{Technometrics} \bold{4}, 47-58.

Groemping, U. (2012). Creating clear designs: a graph-based algorithm and a 
catalog of clear compromise plans. \emph{IIE Transactions} \bold{44}, 988-1001. 
\doi{10.1080/0740817X.2012.654848}. Early preprint at 
\url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-005.pdf}.

Ke, W., Tang, B. and Wu, H. (2005). Compromise plans with clear two-factor interactions. 
\emph{Statistica Sinica} \bold{15}, 709-715.
}
\author{ Ulrike Groemping }

\seealso{ See Also \code{\link{FrF2}} for creation of regular fractional factorial designs 
as well as \code{\link{estimable.2fis}} 
for statistical and algorithmic information on estimability of 2-factor interactions}
\examples{
## seven factors two of which are in group G1
C1 <- compromise(7, c(2,4), class=1)
C1$perms.full  ## the same for all classes
C1$requirement
C2 <- compromise(7, c(2,4), class=2)
C2$requirement
C3 <- compromise(7, c(2,4), class=3)
C3$requirement
C4 <- compromise(7, c(2,4), class=4)
C4$requirement

\dontrun{
########## usage of estimable ###########################
  ## design with with BD clear in 16 runs
  FrF2(16,7,estimable = C1$requirement)
  ## design with BD estimable on a distinct column in 16 runs (any design will do,
  ##    if resolution IV!!!
  FrF2(16,7,estimable = C1$requirement, clear=FALSE, perms=C1$perms.full)
  ## all four classes, mostly clear, for 32 runs
  FrF2(32,7,estimable = C1$requirement)
  FrF2(32,7,estimable = C2$requirement)   ## requires resolution V
         ## as clear class 2 compromise designs do not exist due to Ke et al. 2005
  FrF2(32,7,estimable = C2$requirement, clear=FALSE, perms=C2$perms.full)
  FrF2(32,7,estimable = C3$requirement)
  FrF2(32,7,estimable = C4$requirement)
  ## two additional factors H and J that do not show up in the requirement set
  FrF2(32,9,estimable = C3$requirement)
  ## two additional factors H and J that do not show up in the requirement set
  FrF2(32,9,estimable = C3$requirement, clear=FALSE)
  ## note that this is not possible for distinct designs in case perms is needed,
  ## because perms must have nfactors columns
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{utilities.blocksplitplot.aliases.shortrep}
\alias{recalc.alias.block}
\alias{recalc.alias.block.new}
\alias{struc.aliased}
\alias{struc.aliased.new}
\alias{blockfull}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Utility functions for short representation of block and splitplot generators }
\description{
  Utility functions for short representation of block and splitplot generators, 
  not intended for direct use
}
\usage{
recalc.alias.block(dia, leg)
recalc.alias.block.new(dia, leg)
struc.aliased(struc, nk, order)
struc.aliased.new(struc, nk, order) 
blockfull(block.gen, k, des.gen = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dia}{ a list of alias information from function aliases }
  \item{leg}{ a character vector of legend entries shaped as \code{A=name1} or \code{F1=name1}, 
        depending on whether there are up to 50 factors or more}
  \item{struc}{ an output from function \code{recalc.alias.block} }
  \item{nk}{ number of factors (<=50: Letters; >50: F1, F2, ...) }
  \item{order}{ 2 or 3 (aliasing order) }
  \item{block.gen}{ generators for \code{k.block} independent block columns;\cr
        a list of vectors of design column numbers\cr
        or a numeric vector of Yates column numbers\cr
        or a vector of character strings like "ABD", "CF" etc }
  \item{k}{ number of base factors (i.e. design has \eqn{2^k} runs) }
  \item{des.gen}{ generating columns for the design as Yates column numbers 
      (including base columns)}
}
\details{
   The functions are not intended for direct use. 
}
\value{The functions are not intended for direct use. 
Function \code{recalc.alias.block} outputs a character vector of alias relations, 
function \code{struc.aliased} structures these into a list with main effects, 
2fis and 3fis (the latter for \code{order=3} only). The functions with suffix \code{new} do the same thing for package version 2 in case of blocked designs (split plot functionality still uses the old ones).

\code{blockfull} supports reporting about aliasing with blocks.

These functions are used by function \code{FrF2}.
}
\author{ Ulrike Groemping }
\note{ This package is still under development regarding some of its aspects; 
especially internal functions cannot be relied upon to remain stable.}
\seealso{ See Also \code{\link[FrF2]{FrF2-package}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line
\keyword{internal}

\name{godolphin}
\alias{colpick}
\alias{FF_from_X}
\alias{X_from_parts}
\alias{X_from_profile}
\alias{clear2fis_from_profile}
\alias{blockgencreate}
\alias{phimax}
\alias{Xcalc}
\alias{blockgengroup}
\alias{colpickIV}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Functions in support of Godolphin's approach for blocking designs }
\description{
  Function colpick handles the creation of X matrices for blocking, 
  function FF_from_X blocks a full factorial, function X_from_profile creates 
  an X matrix from a profile, 
  function phimax calculations the maximum number of clear 2fis from Godolphin's approach.
  Further helper functions support the use of the method. 
  The functions are meant for expert users only.
}
\usage{
colpick(design, q, all = FALSE, select.catlg = catlg, 
      estimable = NULL, method = "VF2", sort = "natural",
      res3 = FALSE, all0 = FALSE, quiet = FALSE, 
      firsthit = is.numeric(design))
FF_from_X(X, randomize = TRUE, seed = NULL, alias.info=2)
X_from_profile(n, q, profile = NULL)
clear2fis_from_profile(n, q, profile = NULL)
X_from_parts(n, q, parts)
phimax(n, q, profile = NULL)
blockgencreate(X, p = 0)
Xcalc(XI, gen)
blockgengroup(X, p = 0, num = FALSE)
colpickIV(design, q, all = FALSE, select.catlg = catlg, 
      estimable = NULL, method = "VF2", sort = "natural",
      res3 = FALSE, all0 = FALSE, quiet = FALSE, 
      firsthit = is.numeric(design))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{design}{a character string that identifies a design in the cataloge specified 
       by option \code{select.catlg}, \cr
       OR a class \code{catlg} object with a single entry (of longer, only the first one is used), \cr
       OR an integer number of factors for which a full factorial is assumed.}
  \item{q}{the requested block size is \eqn{2^q}}
  \item{all}{if TRUE (default FALSE), all possible \bold{X} matrices are returned; 
       otherwise, \code{colpick} returns the first successful one (if \code{estimable} 
       is not NULL) or the best one (otherwise)}
  \item{select.catlg}{name of catalogue (not in quotes); 
        only relevant, if \code{design} is a character string}
  \item{estimable}{a specification of 2fis to be kept clear in the blocked design, 
       either as a character vector of pairs of factor letters (using the first 
       elements of `Letters`) or as a two-row matrix of pairs of factor numbers occurring 
       in 2fis)}
  \item{method}{character string identifying a subgraph isomorphism method
       (VF2 or LAD), see \code{\link{FrF2}}}
  \item{sort}{character string specifying a presort strategy for subgraph 
       isomorphism search, see \code{\link{FrF2}}}
  \item{res3}{relevant only if \code{estimable} is not NULL;\cr
       per default (\code{res3=FALSE}), \code{design} will yield no result, 
       if it has resolution III only; 
       set \code{res3} to TRUE for allowing a \code{design} of resolution III 
       (almost never useful)}
  \item{all0}{per default (\code{all0=FALSE}), \bold{X} matrices are requested to be 
       free of all-zero columns. Set \code{all0} to TRUE for permitting aliasing 
       between blocks and factor main effects, i.e. for finding a suitable split-plot
       constructor. }
  \item{quiet}{if TRUE, the message about failure is suppressed (for using the function 
       inside other functions, like \code{FrF2}) }
  \item{firsthit}{if TRUE, the function does not attempt to optimize the 
       number of clear 2fis but accepts the first acceptable blocking
       (relevant for non-null \code{estimable} only); \cr
       per default, optimizing is suppressed for full factorials only (where it 
       is not very reasonable to use function \code{colpick} instead of simply
       hand-crafting an \bold{X} matrix; function \code{FrF2} sets it to FALSE 
       for creation of full factorials with estimability requirements); \cr
       for large applications with estimability requirements, 
       specifying \code{firsthit} as TRUE may allow to inspect more candidates 
       and to then possibly deepdive some of them}
  \item{X}{a \eqn{q \times n}{q times n} \bold{X} matrix 
       with only 0/1 elements for block construction }
  \item{randomize}{ logical. If TRUE, the design is randomized. This is the default.
       Randomization is implemented using function 
       \code{\link[DoE.base]{rerandomize.design}} of package \pkg{DoE.base} 
       as the last step in design creation.
       }
  \item{seed}{ optional seed for the randomization process \cr
        In R version 3.6.0 and later, the default behavior of function \code{\link{sample}}
        has changed. If you work in a new (i.e., >= 3.6.-0) R version and want to reproduce 
        a randomized design from an earlier R version (before 3.6.0), 
        you have to change the RNGkind setting by\cr
        \code{RNGkind(sample.kind="Rounding")}\cr
        before running function \code{FrF2}.\cr
        It is recommended to change the setting back to the new recommended way afterwards:\cr
        \code{RNGkind(sample.kind="default")}\cr
        For an example, see the documentation of the example data set \code{\link[DoE.base]{VSGFS}}.
        }
  \item{alias.info}{degree of effects aliased with blocks to be included in the 
       \code{design.info} attribute}
  \item{profile}{profile to use for calculation 
       (NULL or integer vector of up to \eqn{2^q-1} elements that sum to \eqn{n}); 
       if NULL, the maximally balanced profile is used (which yields the overall maximum
       number of clear 2fis when blocking a full factorial)}
  \item{parts}{list that provides factor partitions; list entries 
        must either be all integers from 1 to \code{n} or the elements
        of \code{Letters[1:n]}}
  \item{n}{number of factors}
  \item{p}{the number of generated factors (among the n factors); 
       \eqn{2^{n-p}}{2^(n-p)} is the number of runs in the experiment)}
  \item{XI}{a \eqn{q \times k}{q times k} \bold{X}_I matrix 
       with only 0/1 elements, 
       to be extended into a \eqn{q \times n}{q times n} \bold{X} matrix for 
       block construction, given the generators in \code{gen} (\eqn{k=n-p})}
  \item{gen}{generators for extending \code{XI}: \code{gen} can be\cr 
        a class \code{catlg} object (e.g. \code{catlg["7-2.1"]} or 
        \code{catlg[nruns(catlg)==32 & nfac(catlg)==7])}; each \code{nfac} 
        entry must be the sum of \eqn{k} and the length of the \code{gen} element)\cr
        a vector of Yates column numbers (e.g. \code{c(7, 27)})\cr
        a vector of defining contrasts (e.g. \code{c("ABC","ABDE")})
        a list of vectors of base column numbers}
  \item{num}{if TRUE (default FALSE), Yates column numbers are returned 
        instead of their character representations}
}

\details{
    These are the functions for the Godolphin (2021) approach to blocking; 
    most of them are user-visible. 
    This approach and its implementation are described in Groemping (2021). 
    Direct use of this functions is intended for expert use only.

    Function \code{colpick} is
    the main workhorse function for blocking larger situations 
    in function \code{\link{FrF2}} 
    (since version 2 of the package, it replaces the earlier approach 
    with function \code{\link{blockpick.big}}); it makes use of function 
    \code{blockgencreate}, and of the internal function \code{blockgengroup}.
    
    Function \code{FF_from_X} creates a class \code{design} object. 
    Design size is limited by computer memory and run time. 
    The function can use an \bold{X} matrix that was produced by function 
    \code{colpick}; but note that it is quite easy to hand-craft 
    an \bold{X} matrix for a full factorial, even with estimability requirements.
    The light-weight function does not have arguments for customization; it can 
    be post-processed, however, e.g. using function 
    \code{\link[DoE.base]{factor.names<-}}.
    
    Function \code{X_from_profile} creates an X matrix that corresponds to 
    the specified profile.
    
    Function \code{phimax} returns the maximum number of 2fis that can 
    be kept clear when blocking a full factorial design in \eqn{n} factors into 
    blocks of size \eqn{2^q}, given the specified profile.
    
    Function \code{blockgencreate} creates block generators from an \bold{X} matrix 
    for blocking a design in \eqn{2^{n-p}=2^k}{2^(n-p)=2^k} runs into blocks of size \eqn{2^q}, 
    where \eqn{n} and \eqn{q} are derived from \code{X} as the number of columns and rows, 
    respectively. The generators are returned as a character vector that consists of 
    strings of base factor letters.
    
    Function \code{Xcalc} extends a \eqn{q\times k}{q times k} 
    matrix \bold{X}_I by \eqn{p=n-k} columns 
    (\bold{X}_II in Godolphin notatation) based 
    on the generators provided in \code{gen}.
    
    Function \code{blockgengroup} is internal only, 
    as are functions \code{colpickIV} and \code{clear2fis_from_profile}.
}

\value{
    Function \code{colpick} returns a list of at least two elements: \cr
    if \code{all} is FALSE, the list consists of the matrix \code{X}, 
    the character vector 
    \code{clear.2fis} and possibly the integer vector \code{map}, \cr
    otherwise of list-valued elements \code{X_matrices}, \code{clearlist} and 
    \code{profiles} and \code{maplist}.
    
    Function \code{FF_from_X} returns a class design object of type \code{FrF2.blocked}.
    
    Function \code{phimax} returns a real number.
    
    Function \code{blockgencreate} returns a character vector of generators
    in terms of \code{Letters} combinations of the first $n-p$ factors.
    
    Function \code{Xcalc} returns a \eqn{q\times n}{\code{q} times \code{n}} 
    matrix (in case of a single generator) or a list of such matrices 
    (if \code{gen} is a class \code{catlg} object with more than one element).
    
    The internal function \code{blockgengroup} returns a character vector of all effects 
    (denoted as base column letter combinations) aliased with the block main effect, 
    or corresponding Yates column numbers.
    
    The internal function \code{colpickIV} returns almost the same type of results as \code{colpick}. 
    The difference: \cr
    if \code{all} is TRUE, there is an integer vector \code{map} instead of the 
    \code{maplist} element, because the \code{map} does not depend on the choice of X-matrix 
    (separate subgraph isomorphism checking is skipped with this function).

}

\references{
Groemping, U. (2012). Creating clear designs: a graph-based algorithm and a 
catalogue of clear compromise plans. \emph{IIE Transactions} \bold{44}, 988--1001. 
Early preprint available at \url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-005.pdf}.

Godolphin, J. (2021). Construction of Blocked Factorial Designs to Estimate 
   Main Effects and Selected Two-Factor Interactions. \emph{J. Royal Statistical Society} \bold{B 83}, 5-29. 
   \doi{10.1111/rssb.12397}.

Groemping, U. (2021). An algorithm for blocking regular fractional factorial 
2-level designs with clear two-factor interactions. \emph{Computational Statistics and Data Analysis} 
\bold{153}, 1-18. \doi{10.1016/j.csda.2020.107059}. Preprint at 
\href{http://www1.bht-berlin.de/FB_II/reports/Report-2019-003.pdf}{Report 3/2019}.

}
\examples{
phimax(7, 2)  ## 16 2fis can be clear, if 128 run full factorial is blocked 
              ## into 32 blocks of size 2^2=4

## X matrices for blocking full factorials
## do not care about which factors have which role
X_from_profile(7, 2, c(3,2,2))
   # X_from_profile(7, 2, c(2,2,3)) returns same matrix
## ensure specific partition, i.e. specific requirement CIG to be accommodated
X <- X_from_parts(7, 2, parts=list(c("A","D","F"), c("B","G"), c("C","E")))

## blocked full factorial
summary(FF_from_X(X))

## using colpick
## estimable in standard letters
requ <- c("BA", "BC", "BD", "BE", "BF", "BG", "BH", "BJ")
## estimability requirement in factor names
fn <- Letters[15:23] ## P to X
requfn <- requ
requfn <- sapply(1:8, function(obj) gsub(Letters[obj], fn[obj], requfn[obj]))

## obtain X matrix for accommodating estimability requirement in 9-4.2
(aus <- colpick("9-4.2", 2, estimable=requ))
## obtain the same matrix manually with Xcalc
XI <- aus$X[,1:5]
## obtain the same matrix manually with Xcalc
all(Xcalc(XI, catlg["9-4.2"])==aus$X)
## inspect X matrices generated from XI
Xcalc(XI, catlg[nruns(catlg)==32 & nfac(catlg)==9 & res(catlg)>=4])

## factor permutation needed
aus$map
## calculate block generators
blockgencreate(aus$X, p=4)
## automatic creation from the design 9-4.2 uses these block generators
summary(FrF2(32, 9, blocks=8, estimable=requ, factor.names=fn, 
    alias.block.2fis = TRUE, select.catlg = catlg["9-4.2"]), 
    brief=TRUE)
## can also be reproduced manually (internal function invperm does the permuting)
summary(FrF2(design="9-4.2", blocks=blockgencreate(aus$X, p=4), 
    factor.names=fn[FrF2:::invperm(aus$map)], 
    alias.block.2fis = TRUE), 
    brief=TRUE)
}
\author{Ulrike Groemping}
\seealso{\code{\link[igraph]{plot.igraph}}, \code{\link[igraph]{tkplot}},
   \code{\link[igraph]{plot.common}} }
\keyword{design}

\name{utilitiesPB}
\alias{williamson}
\alias{double.des}
\alias{circ.mat}
\alias{normalize.col.first}
\alias{normalize.row.last}
\alias{des.recode}
\alias{taguchi}
\alias{boxtyssedal}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ ~~ Internal utility functions ~~ }
\description{
  Internal functions, mainly for calculation of Plackett-Burman and other non-regular screening designs. 
  They need not be accessed for normal use of the package. 
}
\usage{
circ.mat(gen) 
double.des(des)
normalize.col.first(mat)
normalize.row.last(mat)
williamson(A,B,C,D)
taguchi(nruns)
boxtyssedal(nruns,k)
des.recode(var, recodes, as.factor.result, char)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gen}{ ~~Generating vector for circular design matrix (used in Plackett-Burman construction)}
  \item{des}{ ~~numeric design matrix (elements -1 / +1 ) to be doubled}
  \item{mat}{ ~~numeric design matrix (elements -1 / +1 ) to be normalized }
  \item{A}{ ~~numeric square matrix (elements -1 / +1 ) to be combined with \code{B}, \code{C}, and \code{D} by Williamson construction. 
     Although the function combines any square matrices of identical dimension, 
     the matrices must observe a special structure to in order to yield a useful design matrix. }
  \item{B}{ ~~numeric square matrix (elements -1 / +1 ) with same dimensions as \code{A} }
  \item{C}{ ~~numeric square matrix (elements -1 / +1 ) with same dimensions as \code{A} }
  \item{D}{ ~~numeric square matrix (elements -1 / +1 ) with same dimensions as \code{A} }
  \item{nruns}{ number of runs; currently has to be 12 for \code{taguchi} and 16 for \code{boxtyssedal}. }
  \item{k}{ number of factors; for up to 12 factors, a different design is picked. For 15 factors, a warning against perfect aliasing (of the last columnn) is issued. }
  \item{var}{ variable }
  \item{recodes}{ character string specifying recodes, so far only used in the principal form \dQuote{-1=level[1];1=level[2]} }
  \item{as.factor.result}{ logical that specifies whether or not results are to be coerced to factor. 
            The default is that factors are coerced to factor, while numeric variables with numeric recodes are kept numeric.}
  \item{char}{ logical: should replacement be treated as character string (TRUE) or evaluated? default is FALSE}
}

\details{
Function \code{des.recode} has been adapted to the needs of this package from function \code{recode} of package \code{car}.
}
\value{
    All functions return except \code{des.recode} return a numeric matrix, \code{circ.mat} a square matrix of dimension \code{length(gen)}, 
   \code{double.des} a matrix with \code{2*nrow(des)} rows and \code{2*ncol(des)+1} columns (full foldover),
   the \code{normalize...} functions a matrix of the same dimenensions as \code{mat}, 
   and \code{williamson} a square matrix of dimension \code{4*nrow(A)}.
   \code{taguchi} returns the Plackett-Burman array in 12 runs in Taguchi order (it is isomorphic to the standard version), and \code{boxtyssedal} returns the suitable version of the 16 run Hadamard matrix described by Box and Tyssedal, which is more appropriate for screening than the standard 16 run factorial.
   
   \code{des.recode} returns a numeric vector or factor, depending on the type of var and the setting of \code{as.factor.result}. 
   This function is identical to the function in package \pkg{\link[DoE.base:DoE.base-package]{DoE.base}} and will eventually be deleted in package \pkg{FrF2}.
}
\references{ 
  Box, G.E.P. and Tyssedal, J. (2001) Sixteen Run Designs of High Projectivity for Factor Screening. 
      \emph{Communications in Statistics - Simulation and Computation} \bold{30}, 217-228. 
      
  Hedayat, A.S., Sloane, N.J.A. and Stufken, J. (1999) \emph{Orthogonal Arrays: Theory and Applications}, Springer, New York. 
  
  Plackett, R.L.; Burman, J.P. (1946) The design of optimum multifactorial experiments. \emph{Biometrika} \bold{33}, 305-325.
  
  Williamson, J. (1946) Determinants whose elements are 0 and 1. \emph{American Mathematical Monthly} \bold{53}, 427-434.
}
\author{ Ulrike Groemping }
\note{ This package is currently under intensive development. Substantial changes are to be expected in the near future.}
\seealso{ See also \code{\link{pb}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line
\keyword{ internal }% __ONLY ONE__ keyword per line

\name{splitplot}
\alias{splitplot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Statistical and algorithmic aspects of split-plot designs in FrF2 }
\description{
  This help page documents the statistical and algorithmic details of split-plot designs in FrF2
}
\details{
  A split-plot design is similar to a \code{\link{block}}ed design, with the difference that 
  there are also factors of interest that can be only changed on block level (so-called whole 
  plot factors). The blocks are called \dQuote{plots} in the context of split-plot designs. 
  The factors that can (and should!) be varied within a plot are called split-plot factors. 
  Note that the experiment provides more information on split-plot factors than on whole-plot factors.
  
  \emph{Warning:} In terms of analysis, split-plot designs would have to be treated by advanced random effects models, 
  but often are not. At the very least, the user must be aware that all whole-plot effects (i.e. effects 
  on columns that only change between plots) are (likely to be) more variable than split-plot effects so that 
  e.g. it does not necessarily mean anything if they stick out in a normal or half-normal effects plot. 
  
  Designs for hard-to-change factors are also treated by the split-plot approach in function \code{FrF2}, 
  although they are not quite split-plot designs: The are non-randomized split-plot designs arranged in an order 
  such that the first whole-plot factors have as few as possible changes. This gives very poor information on 
  these first whole-plot factors (which in the extreme are only changed once or twice), 
  if there is variability involved with setting the factor levels. \cr
  If hard-to-change factors can be implemented as true whole-plot factors with randomization, this is by far preferrable from a statistical 
  point of view (but may nevertheless be rejected from a feasibility point of view, as the necessary changes may seem unaffordable). 
  
  For design generation, there are two principal ways to handle split-plot designs, manual definition 
  (i.e. the user specifies exactly which columns are to be used for which purpose) and automatic 
  definition. Each situation has its specifics. These are detailed below. For users with 
  not so much mathematical/statistical background, it will often be best to use the automatic way, 
  specifying the treatement factors of interest via \code{nfactors} or \code{factor.names} 
  and a single number for \code{WPs}. 
  Users with more mathematical background may want to use the manual definitions, perhaps 
  in conjunction with published catalogues of good split-plot designs, or 
  after inspecting possibilities with function \code{\link{splitpick}}.
  
  \describe{
  \item{Manual definition of split-plot designs}{
  The user can specify a design with the \code{design} or the \code{generators} option 
  and specify manually with the \code{WPfacs} option, which factors are whole plot factors 
  (i.e. factors that do not change within a plot). 
  The other factors become split-plot factors (i.e. factors that do change within a plot). 
  If the user chooses this route, \code{WPfacs} must be character vectors of factor names, factor letters, 
  factor numbers preceded by capital F, or a vector or list of factor position numbers (NOT: Yates column numbers). 
  Caution: It is the users responsibility to ensure a 
  good choice of split-plot design (e.g. by using a catalogued design from Huang, Chen and Voelkel 1998, 
  Bingham and Sitter 2003, or Bingham Schoen and Sitter 2004).
  In case of a user-mistake such that the resulting design is not a split-plot design with 
  the alleged number of whole plots, an error is thrown.
  }
  \item{Automatic definition of split-plot designs}{
  As mentioned above, split-plot designs differ from block designs by the fact that the block main 
  effects are purely nuisance parameters which are assumed (based on prior knowledge) 
  to be relevant but are not of interest, while the plots are structured by 
  \code{nfac.WP} whole plot factors, which are of interest. 
  The user has to decide on a number of whole plots (\code{WPs}) as well as 
  the number of whole plot factors \code{nfac.WP}. 
  If \code{log2(WPs) <= nfac.WP <= WPs-1}, it is obviously in principle possible to accomodate the 
  desired number of whole plot factors in the desired number of whole plots. If \code{nfac.WP > WPs/2}, 
  the base design for the split-plot structure has to be of resolution III. Sometimes, 
  subject matter considerations limit whole plot sizes, and there are only few interesting 
  whole plot factors, i.e. \code{nfac.WP < log2(WPs)}. 
  In this case, it is of course nevertheless necessary to 
  have a total of \code{log2(WPs)} whole plot \emph{construction} factors; 
  the missing \code{log2(WPs) - nfac.WP} factors are added 
  to the design (names starting with \code{WP}), and \code{nfactors} is increased accordingly. 
    
  In all cases, the first \code{nfac.WPs} user-specified factors are treated as whole plot factors, the 
  remaining factors as split-plot factors. 
  
  From there, function \code{\link{FrF2}} proceeds like in the blocked situation by starting 
  with the best design and working its way down to worse designs, if the best design cannot 
  accomodate the desired split-plot structure. For each design, function \code{\link{FrF2}} 
  calls function \code{\link{splitpick}}, which permutes base factors until 
  the requested whole plot / split-plot structure is achieved, or until impossibility for 
  this design with these base factors has been ascertained. In the latter case, function \code{FrF2} 
  proceeds to the next best design and so forth. 
  
  If several competing split-plot designs based on the same base design are found, 
  the best possible resolution among the first \code{check.WPs} such designs is chosen. 
  No further criteria are automatically implemented, and no more than \code{check.WPs} 
  designs are checked. If not satisfied with the structure of the whole plot portion of the experiment, 
  increasing \code{check.WPs} vs. the default 10 may help. 
  Expert users may want to inspect possibilities, 
  using function \code{\link{splitpick}} directly.
  
  Note that the algorithm does not necessarily find an existing split-plot design. 
  It has been checked out which catalogued designs it can find: designs for all catalogued situations 
  from Bingham and Sitter (2003) have been found, as well as for most catalogued situations 
  from Huang, Chen and Voelkel (1998). Occasionally, a better design than catalogued has been found, e.g. 
  for 4 whole plot and 10 split plot factors in 32 runs with 16 whole plots, the design found by the algorithm 
  is resolution IV, while Huang, Chen and Voelkel propose a resolution III design. 
  The algorithm has the largest difficulties with extreme designs in the sense that a large number of 
  whole plots with a small number of whole plot factors are to be accomodated; thus it does not find designs for 
  the more extreme situations in Bingham, Schoen and Sitter (2004).
  }
  } 
  Please contact me with any suggestions for improvements.
  }

\references{
Bingham, D.R., Schoen, E.D. and Sitter, R.R. (2004). 
Designing Fractional Factorial Split-Plot Experiments with Few Whole-Plot Factors. 
\emph{Applied Statistics} \bold{53}, 325-339.

Bingham, D. and Sitter, R.R. (2003). Fractional Factorial Split-Plot Designs
for Robust Parameter Experiments. \emph{Technometrics} \bold{45}, 80-89.

Chen, J., Sun, D.X. and Wu, C.F.J. (1993) 
A catalogue of 2-level and 3-level orthogonal arrays. 
\emph{International Statistical Review} \bold{61}, 131-145. 

Cheng, C.-S. and Tsai, P.-W. (2009). 
Optimal two-level regular fractional factorial block and split-plot designs. 
\emph{Biometrika} \bold{96}, 83-93.

Huang, P., Chen, D. and Voelkel, J.O. (1998). Minimum-Aberration Two-Level Split-Plot Designs. 
\emph{Technometrics} \bold{40}, 314-326. 

}
\author{ Ulrike Groemping }

\seealso{ See Also \code{\link{FrF2}} for regular fractional factorials, 
\code{\link{catlg}} for the Chen, Sun, Wu catalogue of designs 
and some accessor functions, 
and \code{\link{block}} for the statistical aspects of blocked designs.}
\examples{
########## hard to change factors ####################
## example from Bingham and Sitter Technometrics 19999
## MotorSpeed, FeedMode,FeedSizing,MaterialType are hard to change
BS.ex <- FrF2(16,7,hard=4,
     factor.names=c("MotorSpeed", "FeedMode","FeedSizing","MaterialType",
                  "Gain","ScreenAngle","ScreenVibLevel"), 
     default.levels=c("-","+"))
design.info(BS.ex)
BS.ex
## NOTE: the design has 8 whole plots.
## The first hard-to-change factors have very few changes only 
## between whole plots. 
## A conscious and honest decision is required whether it is 
##    acceptable for the situation at hand not to reset them!
## A proper split-plot design with resetting all whole plot factors 
## for each whole plot would be strongly preferred from a 
## statistical point of view. 

########## automatic generation for split plot ##########
## 3 control factors, 5 noise factors, control factors are whole plot factors
## 8 plots desired in a total of 32 runs
## Bingham Sitter 2003
BS.ex2a <- FrF2(32, 8, WPs=8, nfac.WP=3, 
      factor.names=c(paste("C",1:3,sep=""), paste("N",1:5,sep="")),randomize=TRUE)

## manual generation of this same design
BS.ex2m <- FrF2(32, 8, generators=c("ABD","ACD","BCDE"),WPs=8, WPfacs=c("C1","C2","C3"), nfac.WP=3, 
      factor.names=c(paste("C",1:3,sep=""),paste("N",1:5,sep="")),randomize=TRUE)

## design with few whole plot factors
## 2 whole plot factors, 7 split plot factors
## 8 whole plots, i.e. one extra WP factor needed
BSS.cheese.exa <- FrF2(32, 9, WPs=8, nfac.WP=2, 
      factor.names=c("A","B","p","q","r","s","t","u","v"))
design.info(BSS.cheese.exa)
## manual generation of the design used by Bingham, Schoen and Sitter
## note that the generators include a generator for the 10th spplitting factor
    ## s= ABq, t = Apq, u = ABpr and v = Aqr, splitting factor rho=Apqr
BSS.cheese.exm <- FrF2(32, gen=list(c(1,2,4),c(1,3,4),c(1,2,3,5),c(1,4,5),c(1,3,4,5)), 
      WPs=8, nfac.WP=3, WPfacs=c(1,2,10),
      factor.names=c("A","B","p","q","r","s","t","u","v","rho"))
design.info(BSS.cheese.exm)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line

\name{IAPlot}
\alias{MEPlot}
\alias{MEPlot.design}
\alias{MEPlot.default}
\alias{IAPlot}
\alias{IAPlot.design}
\alias{IAPlot.default}
\alias{intfind}
\alias{check}
\alias{remodel}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Main Effects and Interaction Plots }
\description{
  Main effects plots and interaction plots are produced. The other documented 
  functions are not intended for users. 
}
\usage{
MEPlot(obj, ...)
\method{MEPlot}{design}(obj, ..., response = NULL)
\method{MEPlot}{default}(obj, main = paste("Main effects plot for", respnam), 
    pch = 15, cex.xax = par("cex.axis"), cex.yax = cex.xax, mgp.ylab = 4, 
    cex.title = 1.5, cex.main = par("cex.main"), 
    lwd = par("lwd"), las=par("las"), abbrev = 3, select = NULL, ...)

IAPlot(obj, ...)
\method{IAPlot}{design}(obj, ..., response = NULL)
\method{IAPlot}{default}(obj, main = paste("Interaction plot matrix for", respnam), 
    pch = c(15, 17), cex.lab = par("cex.lab"), cex = par("cex"), 
    cex.xax = par("cex.axis"), cex.yax = cex.xax, cex.title = 1.5, 
    lwd = par("lwd"), las=par("las"), abbrev = 4, select = NULL, show.alias = FALSE, ...)
         
intfind(i, j, mat)

check(obj)

remodel(obj)

}

\arguments{
  \item{obj}{ an experimental design of class \code{\link[DoE.base:class-design]{design}} 
              with the \code{type} element of the \code{design.info} attribute 
              containing \dQuote{FrF2} or \dQuote{pb}\cr
              OR\cr
              a linear model object with 2-level factors or numerical 
              2-level variables; \cr
              the structure must be such that effects are either fully 
              aliased or orthogonal, like in a regular fractional factorial 2-level 
              design;\cr
              note that \code{IAPlot} currently requires the response in \code{obj} to be 
              a pre-defined variable and not a calculated quantity}
  \item{\dots}{ further arguments to be passed to the default function; \cr
              \dots in the default method are not used, they have been added because of 
              formal requirements only }
  \item{response}{ character string that specifies response variable to be used, 
              must be an element of \code{response.names(obj)}; 
              if NULL, the first response from \code{response.names(obj)} is used}
  \item{main}{ overall title for the plot assembly }
  \item{pch}{ Plot symbol number \code{MEPlot}, 
              or vector of two plot symbol numbers for the lower 
              and higher level of the trace factor \code{iap}}
  \item{cex.xax}{ size of x-axis annotation, defaults to \code{cex.axis}-parameter }
  \item{cex.yax}{ size of y-axis annotation, defaults to cex.xax }
  \item{mgp.ylab}{horizontal placement of label of vertical axis in \code{MEPlot}}
  \item{cex.title}{multiplier for size of overall title 
              (cex.main is multiplied with this factor) } 
  \item{cex.main}{ size of individual plot titles in \code{MEPlot} } 
  \item{cex.lab}{ Size of variable names in diagonal panels of interaction plots
              produced by \code{IAPlot}.}
  \item{cex}{ size of plot symbols in interaction plots }
  \item{lwd}{ line width for plot lines and axes }
  \item{las}{ orientation for tick mark labels (\code{las=1} is recommended) }
  \item{abbrev}{ number of characters shown for factor levels }
  \item{select}{ vector with position numbers of the main effects to be displayed; \cr
              default: all main effects; the default implies the full interaction plot matrix 
              for \code{IAPlot}.\cr
              For \code{IAPlot}, the full interaction plot matrix for the selected 
              factors is displayed. Of course, at least two factors must be selected. 
              Furthermore, the linear model \code{obj} must at least contain one 
              interaction term among the selected variables. 
              For interactions that do not occur in the linear model, not plot is shown.
              An interaction plot matrix of data means can be obtained by 
              specifying the model with all possible 2-factor interactions (e.g. formula 
              \code{y~(.)^2} for a regular 2-level fractional factorial, for which 
              \code{y} is the only response and all other variables are 2-level factors).
              }
  \item{show.alias}{ if TRUE, the interaction plot shows the number 
              of the list entry from aliases(obj) (cf. \code{\link{aliases}}) in order to support immediate 
              diagnosis of which depicted interaction may be due to other 
              than the shown effect because of aliasing;\cr
              CAUTION: if the \code{select} option is used, 
              the model is reduced to the selected factors, i.e. aliases with 
              unselected factors are not shown!}
  \item{i}{integer, for internal use only}
  \item{j}{integer, for internal use only}
  \item{mat}{matrix, for internal use only}
}

\details{
    For functions \code{MEPlot} or \code{IAPlot}, 
    if \code{obj} is a design with at least one response variable 
    rather than a linear model fit, 
    the \code{lm}-method for class \code{design} is applied to it with the 
    required degree (1 or 2), 
    and the default method for the respective function is afterwards applied to the 
    resulting linear model.\cr 
    If the design contains a block factor, the plot functions show non-block effects only. 
    
  \describe{
  \item{MEPlot}{produces plots of all treatment main effects in the model, 
              or selected ones if \code{select} is specified}
  \item{IAPlot}{produces plots of all treatment interaction effects in the model,
              or selected ones if \code{select} is specified}
  \item{intfind}{is an internal function not directly useful for users}
  \item{check}{is an internal function for checking whether the model complies 
              with assumptions (fractional factorial of 2-level factors 
              with full or no aliasing, not partial aliasing; 
              this implies that Plackett-Burman designs with partial aliasing 
              of 2-factor interactions give an OK (=TRUE) in \code{check} for 
              pure main effects models only.)}
  \item{remodel}{is an internal function that redoes factor values into -1 and 1 
              coding, regardless of the contrasts that have been used for the 
              original factors;
              numerical data are transformed by subtracting the mean and dividing 
              by half the range (max-min), which also transforms them to -1 and 1 
              coding in the 2-level case (and leads to an error otherwise)}
  }
}
\value{
  \code{MEPlot} and \code{IAPlot} invisibly return the plotted effects (two-row 
           matrix or four-row matrix, respectively). If \code{show.alias=TRUE}, 
           the matrix returned by IAPlot has as the attribute \code{aliasgroups},
           which contains all alias groups (list element number corresponds to 
           number in the graphics tableau).
  
  The internal function \code{check} is used within other functions for checking 
           whether the model is a fractional factorial with 2-level factors and 
           no partial aliasing, as requested for the package to work.  
           It is applied to remodeled objects only and returns a logical.  
           If the returned value is FALSE, the calling function fails. 
              
  The internal function \code{intfind} returns an integer (length 1 or 0). 
           It is not useful for users.
  
  The internal function \code{remodel} is applied to a linear model object and 
  returns a list of two components:
      \item{model }{is the redone model with x-variables recoded to numeric 
              -1 and 1 notation and aov objects made into \dQuote{pure} lm objects}
      \item{labs }{is a list preserving the level information from original factors
               (levels are minus and plus for numerical variables)}
}

\references{ 
    Box G. E. P, Hunter, W. C. and Hunter, J. S. (2005)
    \emph{Statistics for Experimenters, 2nd edition}.
    New York: Wiley.
 }
\author{ Ulrike Groemping }

\seealso{ \code{\link{FrF2-package}} for examples }
\keyword{ design }

\name{makecatlg}
\alias{makecatlg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Function for creating a class catlg catalogue from a vector of generators }
\description{
  creates a class catlg catalogue with a single element for use in functions
  colpick or FrF2
}
\usage{
makecatlg(k, gen)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{k}{ number of base factors spanning a full factorial with the desired number of runs }
  \item{gen}{ generators as a numeric vector of Yates column numbers }
}
\details{
   If generators are available in a different format, they must be transformed to Yates column numbers.

   For a character vector \code{genc} with elements like \code{ABC}, \code{ADE}, etc.,
   a code for obtaining Yates columns with order preserved is
   \code{sapply(1:length(genc), function(obj) which(names(Yates)==genc[obj]))}
   (a solution with \code{which} applied to the entire vector at once does not preserve the order).

   Yet different formats like \code{123}, \code{145}, etc., can e.g.
   be preprocessed by picking the suitable elements from \code{\link[DoE.base:utilities]{Letters}}, e.g.
   \code{paste(Letters[as.numeric(unlist(strsplit("123","")))],collapse="")}.
}
\value{
   The function returns a list of class \code{catlg} with a single element.
   }
\author{ Ulrike Groemping }
\note{ This package is still under development, but does already provide useful and well-tested results.}
\seealso{ See also \code{\link{FrF2}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line
\examples{
## Xu's fraction 13-5.2
genXu <- c(127, 143, 179, 85, 150)
catXu <- makecatlg(k=8, genXu)
colpick(catXu, q=2) ## Godolphin blocking into blocks of size 4 yields 56 clear 2fis
FrF2(256, 13, blocks=64, alias.block.2fis=TRUE, select.catlg=catXu)
}

\name{estimable.2fis}
\alias{estimable.2fis}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Statistical and algorithmic aspects of requesting 2-factor interactions to be estimable in FrF2 }
\description{
  This help page documents the statistical and algorithmic details of requesting 2-factor interactions to be estimable in FrF2 
}
\details{
  The option \code{estimable} allows to specify 2-factor interactions (2fis) that 
  have to be estimable in the model. Whenever a resolution V or higher design is available, 
  this option is unnecessary, because all 2fis are estimable in the sense that they are 
  not aliased with any main effect or any other 2fi. If resolution V or higher is not affordable, 
  the option \code{estimable} can ensure that certain 2fis can nevertheless be estimated. 
  
  Per default, it is assumed that a resolution IV design is required, 
  as it is normally not reasonable to allow main effects to be 
  aliased with other 2-factor interactions in this situation. There are two types of 
  estimability that are distinguished by the setting of option \code{clear} in 
  function \code{\link{FrF2}} (cf. Groemping 2010).
  
  Let us first consider designs of at least resolution IV. 
  With option \code{clear=TRUE}, \code{FrF2} searches for 
  a model for which all main effects and all 2fis given in \code{estimable} are 
  clear of aliasing with any other 2fis. This is a weaker requirement than resolution V, 
  because 2fis outside those specified in \code{estimable} may be aliased with 
  each other. But it is much stronger than what is done in case of \code{clear=FALSE}: 
  For the latter, \code{FrF2} searches for a design that has a distinct column in 
  the model matrix for each main effect and each interaction requested 
  in \code{estimable}. 
  
  Users can explicitly permit that resolution III designs are included in the 
  search of designs for which the specified 2fis are estimable (by the \code{res3=TRUE} option). 
  In case of \code{clear=TRUE}, this leads to the somewhat 
  strange situation that main effects can be aliased with 2fis from outside 
  \code{estimable} while 2fis from inside \code{estimable} are not aliased with 
  any main effects or 2fis. 
  
  With \code{clear=TRUE}, the algorithm compares the requirement set to 
  catalogued sets of clear 2fis by a graph isomorphism algorithm from R-package 
  \pkg{igraph}. For details of this algorithm, 
  cf. Groemping (2012). With the catalogue \code{\link{catlg}} available in this package, 
  the best (minimum aberration) existing clear designs are guaranteed to be found 
  for up to 64 runs and have a good chance to be found for 128 runs. For 128 runs, 
  it is possible to load an additional large catalogue (package \pkg{\link[FrF2.catlg128]{FrF2.catlg128}}) 
  in order to also guarantee that the best clear design is found. For 256 and 512 runs, 
  only one or two resolution IV designs of each size are catalogued so that 
  option \code{estimable} can try to influence allocation of factors to columns, 
  but may fail although an appropriate clear design would exist outside the catalogued 
  designs.\cr
  The search for a clear design 
  is often fast. If it isn't, option \code{sort} 
  of function \code{\link{FrF2}} can help. For the occasional situation where 
  this doesn't help either, a manual search may help, see \code{\link{CIG}} 
  for an example of how to proceed.\cr
  Since version 2 of package \pkg{FrF2}, requesting 2fis to be clear is compatible 
  with blocking a design. The algorithm behind that functionality is based on 
  Godolphin (2021) and is described in Groemping (2021). 
  The default implementation strives for a guartanteed and best possible result. 
  Arguments \code{firsthit} and \code{useV} to function \code{\link{FrF2}} 
  can be used for trying to obtain a possibly not best result (\code{firsthit}) 
  faster or to use a (sometimes) faster algorithm that is not guaranteed 
  to deliver a result even though it might exist for resolution IV situations 
  (\code{useV=FALSE}). 
    
  With \code{clear=FALSE}, the algorithm loops through the eligible designs from 
  \code{catlg.select} from good to worse (in terms of MA) and, for each design, loops 
  through all eligible permutations of the experiment factors from \code{perms}. 
  If \code{perms} is omitted, the permutations are looped through in lexicographic 
  order starting from 1:nfac or \code{perm.start}. Especially in this case, 
  run times of the search algorithm can be very long. 
  The \code{max.time} option allows to limit this run time. 
  If the time limit is reached, the final situation (catalogued design and 
  current permutation of experiment factors) is printed so that the user can 
  decide to proceed later with this starting point (indicated by \code{catlg.select} 
  for the catalogued design(s) to be used and \code{perm.start} for the current 
  permutation of experiment factors). 

  With \code{clear=TRUE}, the algorithm loops through the eligible designs from 
  \code{catlg.select} from good to worse (in terms of MA) and, for each design, 
  uses a subgraph isomorphism check from package \code{igraph}. There are two such 
  algorithms, VF2 (the default, Cordella et al. 2001) and LAD (introduced with 
  version 1.7 of package \pkg{FrF2}, Solnon 2010), 
  which can be chosen with the  \code{method} option. 
  Run times of the subgraph isomorphism search are often fast, 
  but can also be very very slow in unlucky situations. 
  Where the VF2 algorithm is particularly slow, the LAD algorithm is often fast 
  (see Groemping 2014b). 
  Especially for the VF2 algorithm, run times may strongly depend on the ordering 
  of factors, which can be influenced by the option \code{sort}.
  As the slowness of the process is intrinsic to the subgraph isomorphism 
  search problem (which is NP-complete), a \code{max.time} option analogous to 
  the \code{clear=FALSE} situation would be of very limited use only and is 
  therefore not available. Instead, it is possible to have a look at the 
  number of the design that was in the process of being searched when the 
  process was interrupted (with the command \code{FrF2.currentlychecked()}).

  Note that - according to the structure of the catalogued designs and the lexicographic 
  order of checking permutations - the initial order of the factors has a strong influence 
  on the run time for larger or unlucky problems. For example, consider 
  an experiment in 32~runs and 11~factors, for six of which the pairwise interactions are to be estimable 
  (Example 1 in Wu and Chen 1992). \code{estimable} for this model can be specified as \cr
    \code{formula("~(F+G+H+J+K+L)^2")} \cr
  OR \cr
    \code{formula("~(A+B+C+D+E+F)^2")}.\cr
    The former runs a lot faster than the latter (I have not yet seen the latter finish 
  the first catalogued design, if \code{perms} is not specified). 
  The reason is that the latter needs more permutations of the experiment factors than 
  the former, since the factors with high positions 
  change place faster and more often than those with low positions. 
  
  For this particular design, it is very advisable to constrain the 
  permutations of the experiment factors to the different subset selections of six factors 
  from eleven, since permutations within the sets do not change the possibility of accomodating 
  a design. The required permutations for the second version of this example 
  can be obtained e.g. by the following code: 
  
  \code{perms.6 <- combn(11,6)}\cr
  \code{perms.full <- matrix(NA,ncol(perms.6),11)}\cr
  \code{for (i in 1:ncol(perms.6))}\cr
  \code{perms.full[i,] <- c(perms.6[,i],setdiff(1:11,perms.6[,i]))}\cr

  Handing perms.full to the procedure using the \code{perms} option makes the second version of the 
  requested interaction terms fast as well, since up to almost 40 Mio permutations of experiment 
  factors are reduced to at most 462. Thus, whenever possible, 
  one should try to limit the permutations necessary in case of \code{clear=FALSE}.
  
  In order to support relatively comfortable creation of distinct designs of some frequently-used types 
  of required interaction patterns, the function \code{\link{compromise}} has been 
  divised: it supports creation of the so-called compromise plans of classes 1 to 4 (cf. 
  e.g. Addelman 1962; Ke, Tang and Wu 2005; Groemping 2012). 
  The list it returns also contains a component \code{perms.full} that can be used as input 
  for the \code{perms} option.
  
  Please contact me with any suggestions for improvements.
}
\references{
Addelman, S. (1962). Symmetrical and asymmetrical fractional factorial plans. 
\emph{Technometrics} \bold{4}, 47-58.

Chen, J., Sun, D.X. and Wu, C.F.J. (1993). A catalogue of 2-level and 3-level 
orthogonal arrays. 
\emph{International Statistical Review} \bold{61}, 131-145. 

Cordella, L.P., Foggia, P., Sansone, C. and Vento, M. (2001). 
An improved algorithm for matching large graphs. \emph{Proc. of the 3rd 
IAPR TC-15 Workshop on Graphbased Representations in Pattern Recognition},
149--159.

Godolphin, J. (2021). Construction of Blocked Factorial Designs to Estimate 
   Main Effects and Selected Two-Factor Interactions. \emph{J. Royal Statistical Society} \bold{B 83}, 5-29. 
   \doi{10.1111/rssb.12397}.

Groemping, U. (2010). \dQuote{Clear} and \dQuote{Distinct}: two approaches for regular 
fractional factorial designs with estimability requirements. 
\emph{Reports in Mathematics, Physics and Chemistry}, report 02/2010, 
Department II, Beuth University of Applied Sciences Berlin.
\url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-002.pdf}.

Groemping, U. (2012). Creating clear designs: a graph-based algorithm and a 
catalogue of clear compromise plans. \emph{IIE Transactions} \bold{44}, 988--1001. 
Early preprint available at \url{http://www1.bht-berlin.de/FB_II/reports/Report-2010-005.pdf}.

    Groemping, U. (2014a). {R} Package {FrF2} for Creating and Analyzing Fractional 
    Factorial 2-Level Designs. \emph{Journal of Statistical Software}, \bold{56}, 
    Issue 1, 1-56. \url{https://www.jstatsoft.org/v56/i01/}.
    
    Groemping, U. (2014b). A Note on Dominating Fractional Factorial 
    Two-Level Designs With Clear Two-Factor Interactions. \emph{Technometrics} 
    \bold{56}, 42--45.

Groemping, U. (2021). An algorithm for blocking regular fractional factorial 
2-level designs with clear two-factor interactions. \emph{Computational Statistics and Data Analysis} 
\bold{153}, 1-18. \doi{10.1016/j.csda.2020.107059}. Preprint at 
\href{http://www1.bht-berlin.de/FB_II/reports/Report-2019-003.pdf}{Report 3/2019}.

Ke, W., Tang, B. and Wu, H. (2005). Compromise plans with clear two-factor interactions. 
\emph{Statistica Sinica} \bold{15}, 709-715.

Solnon, C. (2010). AllDifferent-based Filtering for Subgraph Isomorphism.
\emph{Artificial Intelligence} \bold{174}, 850--864.

Wu, C.F.J. and Chen, Y. (1992) 
A graph-aided method for planning two-level experiments when certain interactions 
are important. 
\emph{Technometrics} \bold{34}, 162-175. 
}

\author{ Ulrike Groemping }

\seealso{ See also \code{\link{FrF2}} for regular fractional factorials, 
\code{\link{catlg}} for the Chen, Sun, Wu (1993) and larger catalogues of designs 
and some accessor functions, and function \code{\link{compromise}} for a convenience 
function to handle estimability requests for compromise plans}
\examples{
########## usage of estimable ###########################
  ## design with all 2fis of factor A estimable on distinct columns in 16 runs
  FrF2(16, nfactors=6, estimable = rbind(rep(1,5),2:6), clear=FALSE)
  FrF2(16, nfactors=6, estimable = c("AB","AC","AD","AE","AF"), clear=FALSE)
  FrF2(16, nfactors=6, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=FALSE)
            ## formula would also accept self-defined factor names
            ## from factor.names instead of letters A, B, C, ...
            
  ## estimable does not need any other input
  FrF2(estimable=formula("~(A+B+C)^2+D+E"))

  ## estimable with factor names 
  ## resolution three must be permitted, as FrF2 first determines that 8 runs 
  ##     would be sufficient degrees of freedom to estimate all effects 
  ##     and then tries to accomodate the 2fis from the model clear of aliasing in 8 runs
  FrF2(estimable=formula("~one+two+three+four+two:three+two:four"), 
       factor.names=c("one","two","three","four"), res3=TRUE)
  ## clear=FALSE allows to allocate all effects on distinct columns in the 
  ##     8 run MA resolution IV design
  FrF2(estimable=formula("~one+two+three+four+two:three+two:four"), 
       factor.names=c("one","two","three","four"), clear=FALSE)

  ## 7 factors instead of 6, but no requirements for factor G
  FrF2(16, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=FALSE)
  ## larger design for handling this with all required effects clear
  FrF2(32, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=TRUE)
  ## 16 run design for handling this with required 2fis clear, but main effects aliased
  ## (does not usually make sense)
  FrF2(16, nfactors=7, estimable = formula("~A+B+C+D+E+F+A:(B+C+D+E+F)"), 
       clear=TRUE, res3=TRUE)

## example for necessity of perms for the clear=FALSE case
## based on Wu and Chen Example 1
  \dontrun{
  ## runs per default about max.time=60 seconds, before throwing error with 
  ##        interim results
  ## results could be used in select.catlg and perm.start for restarting with 
  ##       calculation of further possibilities
  FrF2(32, nfactors=11, estimable = formula("~(A+B+C+D+E+F)^2"), clear=FALSE)
  ## would run for a long long time (I have not yet been patient enough)
  FrF2(32, nfactors=11, estimable = formula("~(A+B+C+D+E+F)^2"), clear=FALSE, 
       max.time=Inf)
  }
  ## can be easily done with perms, 
  ## as only different subsets of six factors are non-isomorphic
  perms.6 <- combn(11,6)
  perms.full <- matrix(NA,ncol(perms.6),11)
  for (i in 1:ncol(perms.6))
     perms.full[i,] <- c(perms.6[,i],setdiff(1:11,perms.6[,i]))
  ## function compromise will calculate the necessary perms entries automatically
  compromise(11,1:6)$perms.full
  FrF2(32, nfactors=11, estimable = formula("~(A+B+C+D+E+F)^2"), clear=FALSE, 
      perms = perms.full )
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ array }
\keyword{ design }% __ONLY ONE__ keyword per line
